
/*************************************************************************


Copyright 1998, 1999, 2000 Gustavo Diaz-Jerez


This file is part of FractMus.

    FractMus is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FractMus is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FractMus; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


**************************************************************************/


// ChangesDlg.cpp : implementation file
//



#include "stdafx.h"
#include "FractMus.h"

#include "ChangesDlg.h"
#include "FractMusView.h"
#include "FractMusDoc.h"
#include "MorseThueDlg.h"
#include "LogMapDlg.h"
#include "HenonDlg.h"
#include "FnoiseDlg.h"
#include "LorentzDlg.h"
#include "HopalongMartinDlg.h"
#include "MartinDlg.h"
#include "GingerDlg.h"
#include "EarthWormDlg.h"
#include "RandomAlgorithmDlg.h"
#include "WolframDlg.h"
#include "HailstoneDlg.h"
#include "NoteValDlg.h"
#include "V1ModulDlg.h"

#include "defs.h"

extern int PercussionChannel;
extern const char* InstrumentNames[128];
extern const char* PercussionNames[47];

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CChangesDlg dialog


CChangesDlg::CChangesDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CChangesDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CChangesDlg)
	m_bAlgorithm = FALSE;
	m_bCoeff = FALSE;
	m_bDynamic = FALSE;
	m_bInstrument = FALSE;
	m_bInverted = FALSE;
	m_bMatch = FALSE;
	m_bMode = FALSE;
	m_bModulate = FALSE;
	m_Modulates = FALSE;
	m_bNote = FALSE;
	m_bNoteval = FALSE;
	m_bOctaves = FALSE;
	m_bPan = FALSE;
	m_Base = 2;
	m_Multi = 1;
	m_bScale = FALSE;
	m_Dynamic = -1;
	m_Pan = 0;
	m_Inverted = FALSE;
	m_MatchScale = FALSE;
	m_bPercussion = FALSE;
	m_voiceonoff = 2;
	m_AllActiveVoices = FALSE;
	//}}AFX_DATA_INIT


	m_ModulRandom = FALSE;
	m_ModulNum = 5;
	m_LogMapVar = 3.9f;
	m_FNoiseVar = .5f;
	m_HenonA = 1.7f;
	m_HenonB = 0.4f;
	m_HopalongVar = 3.14159f;
	m_MartinVar = 3.14159f;
	m_GingerX = -0.1f;
	m_GingerY = 0.0f;
	m_LorentzVar = 0.8f;
	m_EarthWormVar = 2;
	m_EarthWormMulti = 3;
	m_EarthWormTruncator = 6;
	m_RandomMin = 0.0;
	m_RandomMax = 1.0;
	m_WolframRule = 30;
	m_WolframInit = 1;
	m_WolframSize = 128;
	m_Hailstone = 27;
	m_othernoteval = 8;

	UsrScl.m_title = "User Scale";



	UserScale = NULL;
	IsSelected = NULL;
}

CChangesDlg::~CChangesDlg()
{
	if (UserScale)
	{
		delete UserScale;
		UserScale = NULL;
	}

	if (IsSelected)
	{
		delete IsSelected;
		IsSelected = NULL;
	}
	

}
void CChangesDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CChangesDlg)
	DDX_Check(pDX, IDC_ALGORITHM_CHK, m_bAlgorithm);
	DDX_Check(pDX, IDC_COEFFICIENT_CHK, m_bCoeff);
	DDX_Check(pDX, IDC_DYNAMIC_CHK, m_bDynamic);
	DDX_Check(pDX, IDC_INSTRUMENT_CHK, m_bInstrument);
	DDX_Check(pDX, IDC_INV_CHK, m_bInverted);
	DDX_Check(pDX, IDC_MATCH_CHK, m_bMatch);
	DDX_Check(pDX, IDC_MODE_CHK, m_bMode);
	DDX_Check(pDX, IDC_MODUL_CHK, m_bModulate);
	DDX_Check(pDX, IDC_MODULATE_CHK, m_Modulates);
	DDX_Check(pDX, IDC_NOTE_CHK, m_bNote);
	DDX_Check(pDX, IDC_NOTEVAL_CHK, m_bNoteval);
	DDX_Check(pDX, IDC_OCTAVES_CHK, m_bOctaves);
	DDX_Check(pDX, IDC_PAN_CHK, m_bPan);
	DDX_Check(pDX, IDC_SCALE_CHK, m_bScale);
	DDX_Radio(pDX, IDC_PIANISSIMO, m_Dynamic);
	DDX_Scroll(pDX, IDC_PAN, m_Pan);
	DDX_Check(pDX, IDC_INVERTED, m_Inverted);
	DDX_Check(pDX, IDC_MATCHSCL, m_MatchScale);
	DDX_Check(pDX, IDC_PERCUSSION_CHK, m_bPercussion);
	DDX_CBIndex(pDX, IDC_VOICEONOFF_COMBO, m_voiceonoff);
	DDX_Check(pDX, IDC_ALLACTIVEVOICES_CHK, m_AllActiveVoices);
	//}}AFX_DATA_MAP

	DDX_Check(pDX, IDC_VC1_CHK, m_VoiceOn[VOICE1]);
	DDX_Check(pDX, IDC_VC2_CHK, m_VoiceOn[VOICE2]);
	DDX_Check(pDX, IDC_VC3_CHK, m_VoiceOn[VOICE3]);
	DDX_Check(pDX, IDC_VC4_CHK, m_VoiceOn[VOICE4]);
	DDX_Check(pDX, IDC_VC5_CHK, m_VoiceOn[VOICE5]);
	DDX_Check(pDX, IDC_VC6_CHK, m_VoiceOn[VOICE6]);
	DDX_Check(pDX, IDC_VC7_CHK, m_VoiceOn[VOICE7]);
	DDX_Check(pDX, IDC_VC8_CHK, m_VoiceOn[VOICE8]);
	DDX_Check(pDX, IDC_VC9_CHK, m_VoiceOn[VOICE9]);
	DDX_Check(pDX, IDC_VC10_CHK, m_VoiceOn[VOICE10]);
	DDX_Check(pDX, IDC_VC11_CHK, m_VoiceOn[VOICE11]);
	DDX_Check(pDX, IDC_VC12_CHK, m_VoiceOn[VOICE12]);
	DDX_Check(pDX, IDC_VC13_CHK, m_VoiceOn[VOICE13]);
	DDX_Check(pDX, IDC_VC14_CHK, m_VoiceOn[VOICE14]);
	DDX_Check(pDX, IDC_VC15_CHK, m_VoiceOn[VOICE15]);
	DDX_Check(pDX, IDC_VC16_CHK, m_VoiceOn[VOICE16]);

	m_VoiceID[VOICE1] = IDC_VC1_CHK;
	m_VoiceID[VOICE2] = IDC_VC2_CHK;
	m_VoiceID[VOICE3] = IDC_VC3_CHK;
	m_VoiceID[VOICE4] = IDC_VC4_CHK;
	m_VoiceID[VOICE5] = IDC_VC5_CHK;
	m_VoiceID[VOICE6] = IDC_VC6_CHK;
	m_VoiceID[VOICE7] = IDC_VC7_CHK;
	m_VoiceID[VOICE8] = IDC_VC8_CHK;
	m_VoiceID[VOICE9] = IDC_VC9_CHK;
	m_VoiceID[VOICE10] = IDC_VC10_CHK;
	m_VoiceID[VOICE11] = IDC_VC11_CHK;
	m_VoiceID[VOICE12] = IDC_VC12_CHK;
	m_VoiceID[VOICE13] = IDC_VC13_CHK;
	m_VoiceID[VOICE14] = IDC_VC14_CHK;
	m_VoiceID[VOICE15] = IDC_VC15_CHK;
	m_VoiceID[VOICE16] = IDC_VC16_CHK;

}


BEGIN_MESSAGE_MAP(CChangesDlg, CDialog)
	//{{AFX_MSG_MAP(CChangesDlg)
	ON_BN_CLICKED(IDC_EVENTS, OnEvents)
	ON_BN_CLICKED(IDC_EVENT_ALL, OnEventAll)
	ON_BN_CLICKED(IDC_EVENT_NONE, OnEventNone)
	ON_BN_CLICKED(IDC_ALGORITHM_CHK, OnAlgorithmChk)
	ON_BN_CLICKED(IDC_COEFFICIENT_CHK, OnCoefficientChk)
	ON_BN_CLICKED(IDC_DYNAMIC_CHK, OnDynamicChk)
	ON_BN_CLICKED(IDC_INSTRUMENT_CHK, OnInstrumentChk)
	ON_BN_CLICKED(IDC_INV_CHK, OnInvChk)
	ON_BN_CLICKED(IDC_MATCH_CHK, OnMatchChk)
	ON_BN_CLICKED(IDC_MODE_CHK, OnModeChk)
	ON_BN_CLICKED(IDC_MODUL_CHK, OnModulChk)
	ON_BN_CLICKED(IDC_MODULATE_CHK, OnModulateChk)
	ON_BN_CLICKED(IDC_NOTE_CHK, OnNoteChk)
	ON_BN_CLICKED(IDC_NOTEVAL_CHK, OnNotevalChk)
	ON_BN_CLICKED(IDC_OCTAVES_CHK, OnOctavesChk)
	ON_BN_CLICKED(IDC_PAN_CHK, OnPanChk)
	ON_BN_CLICKED(IDC_SCALE_CHK, OnScaleChk)
	ON_BN_CLICKED(IDC_ALL, OnAll)
	ON_BN_CLICKED(IDC_NONE, OnNone)
	ON_BN_CLICKED(IDC_COEFF_BUTT, OnCoeffButt)
	ON_CBN_CLOSEUP(IDC_SCALE_COMBO, OnCloseupScaleCombo)
	ON_BN_CLICKED(IDC_SETTINGS, OnSettings)
	ON_CBN_CLOSEUP(IDC_NOTEVALUE_COMBO, OnCloseupNotevalueCombo)
	ON_BN_CLICKED(IDC_INVERTED, OnInverted)
	ON_WM_HSCROLL()
	ON_BN_CLICKED(IDC_PAN_ZERO, OnPanZero)
	ON_BN_CLICKED(IDC_PERCUSSION_CHK, OnPercussionChk)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CChangesDlg message handlers

void CChangesDlg::OnOK() 
{
}

BOOL CChangesDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

	CFractMusDoc* pDoc = pView->GetDocument();
	
	CListBox* pList = (CListBox*)GetDlgItem(IDC_EVENTLIST);

	if (pDoc->m_NumEvents>-1)
	{

		for (int i = 0; i<=pDoc->m_NumEvents; i++)
		{
			char s[55];

			itoa(i, s, 10);

			pList->AddString(s);
		}

		if(IsSelected)
		{
			delete IsSelected;
			IsSelected = NULL;
		}

		IsSelected = new BOOL [pDoc->m_NumEvents + 1];

		for (i=0; i<=pDoc->m_NumEvents; i++)
			IsSelected[i] = FALSE;
	}

	m_AllActiveVoices = FALSE;


	CScrollBar* pSC = (CScrollBar*)GetDlgItem(IDC_PAN);
	CStatic* pSt = (CStatic*)GetDlgItem(IDC_PAN_STATIC);
	char s[20];

	pSC->SetScrollRange(-64, 63);
	pSC->SetScrollPos(m_Pan);

	itoa(m_Pan, s ,10);
	pSt->SetWindowText(s);

	CComboBox* pCB = (CComboBox*)GetDlgItem(IDC_VOICEONOFF_COMBO);

	pCB->SetCurSel(2);


	GetDlgItem(IDC_EVENTS)->EnableWindow(pDoc->m_NumEvents>-1);
	GetDlgItem(IDC_EVENT_ALL)->EnableWindow(pDoc->m_NumEvents>-1);
	GetDlgItem(IDC_EVENT_NONE)->EnableWindow(pDoc->m_NumEvents>-1);

	GetDlgItem(IDC_MODULATE_CHK)->EnableWindow(m_bModulate);
	GetDlgItem(IDC_NUMOCTS_COMBO)->EnableWindow(m_bOctaves);
	GetDlgItem(IDC_STARTNOTE_COMBO)->EnableWindow(m_bNote);
	GetDlgItem(IDC_SCALE_COMBO)->EnableWindow(m_bScale);
	GetDlgItem(IDC_NOTEVALUE_COMBO)->EnableWindow(m_bNoteval);
	GetDlgItem(IDC_INSTRUMENT_COMBO)->EnableWindow(m_bInstrument&&(!m_bPercussion));
	GetDlgItem(IDC_MODE_COMBO)->EnableWindow(m_bMode);
	GetDlgItem(IDC_PAN)->EnableWindow(m_bPan);
	GetDlgItem(IDC_PAN_ZERO)->EnableWindow(m_bPan);
	GetDlgItem(IDC_PIANISSIMO)->EnableWindow(m_bDynamic);
	GetDlgItem(IDC_PIANO)->EnableWindow(m_bDynamic);
	GetDlgItem(IDC_MPIANO)->EnableWindow(m_bDynamic);
	GetDlgItem(IDC_MFORTE)->EnableWindow(m_bDynamic);
	GetDlgItem(IDC_FORTE)->EnableWindow(m_bDynamic);
	GetDlgItem(IDC_FORTISSIMO)->EnableWindow(m_bDynamic);
	GetDlgItem(IDC_INVERTED)->EnableWindow(m_bInverted);
	GetDlgItem(IDC_MATCHSCL)->EnableWindow(m_bInverted&&m_bMatch&&m_Inverted);
	GetDlgItem(IDC_ALGORITHM_COMBO)->EnableWindow(m_bAlgorithm);
	GetDlgItem(IDC_COEFF_BUTT)->EnableWindow(m_bCoeff);
	GetDlgItem(IDC_PERCUSSION_COMBO)->EnableWindow(m_bPercussion&&(!m_bInstrument));
	GetDlgItem(IDC_PERCUSSION_CHK)->EnableWindow(!m_bInstrument);
	GetDlgItem(IDC_INSTRUMENT_CHK)->EnableWindow(!m_bPercussion);
	GetDlgItem(IDC_VOICEONOFF_COMBO)->EnableWindow(pDoc->m_NumEvents>-1);
	GetDlgItem(IDC_VOO_ST)->EnableWindow(pDoc->m_NumEvents>-1);
	GetDlgItem(IDC_ALLACTIVEVOICES_CHK)->EnableWindow(pDoc->m_NumEvents>-1);
	GetDlgItem(IDC_EVENTNUM_ST)->SetWindowText("- - -");

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CChangesDlg::OnEvents() 
{
	if (!UpdateData(TRUE))
		return;

	int numv = 0;

	for (int g = VOICE1; g<MAXVOICES; g++)
		if (m_VoiceOn[g])
			++numv;

	if ((!numv)&&(!m_AllActiveVoices))
	{
		MessageBox("No voices selected!","Error",MB_OK+MB_ICONEXCLAMATION);
		return;
	}

	CFractMusDoc* pDoc = pView->GetDocument();	
	CListBox* pList = (CListBox*)GetDlgItem(IDC_EVENTLIST);
	CStatic* pSta = (CStatic*)GetDlgItem(IDC_EVENTNUM_ST);


	int numchoices = pList->GetSelCount();

	if(numchoices<1)
	{
		MessageBox("No events selected!","Error",MB_OK+MB_ICONEXCLAMATION);
		return;
	}

	for (int i =0; i<=pDoc->m_NumEvents; i++)
		IsSelected[i] = FALSE;


	int* selectedItems = new int[numchoices];

	pList->GetSelItems(numchoices, selectedItems);

	for(i =0; i<numchoices; i++)
		IsSelected[ selectedItems[i] ] = TRUE;


	///////////////////////

	CComboBox* pB;

	if (m_bOctaves)
	{
		pB = (CComboBox*)GetDlgItem(IDC_NUMOCTS_COMBO);

		if (pB->GetCurSel() == LB_ERR)
		{
			MessageBox("No Octaves selected!", "Error", MB_OK+MB_ICONEXCLAMATION);
			return;
		}

	}

	if (m_bNote)
	{
		pB = (CComboBox*)GetDlgItem(IDC_STARTNOTE_COMBO);

		if (pB->GetCurSel() == LB_ERR)
		{
			MessageBox("No Note selected!", "Error", MB_OK+MB_ICONEXCLAMATION);
			return;
		}

	}

	if (m_bScale)
	{
		pB = (CComboBox*)GetDlgItem(IDC_SCALE_COMBO);

		if (pB->GetCurSel() == LB_ERR)
		{
			MessageBox("No Scale selected!", "Error", MB_OK+MB_ICONEXCLAMATION);
			return;
		}

	}

	if (m_bNoteval)
	{
		pB = (CComboBox*)GetDlgItem(IDC_NOTEVALUE_COMBO);

		if (pB->GetCurSel() == LB_ERR)
		{
			MessageBox("No Note Value selected!", "Error", MB_OK+MB_ICONEXCLAMATION);
			return;
		}

	}

	if (m_bMode)
	{
		pB = (CComboBox*)GetDlgItem(IDC_MODE_COMBO);

		if (pB->GetCurSel() == LB_ERR)
		{
			MessageBox("No Mode selected!", "Error", MB_OK+MB_ICONEXCLAMATION);
			return;
		}

	}

	if (m_bInstrument)
	{
		pB = (CComboBox*)GetDlgItem(IDC_INSTRUMENT_COMBO);

		if (pB->GetCurSel() == LB_ERR)
		{
			MessageBox("No Instrument selected!", "Error", MB_OK+MB_ICONEXCLAMATION);
			return;
		}

	}

	if (m_bPercussion)
	{
		pB = (CComboBox*)GetDlgItem(IDC_PERCUSSION_COMBO);

		if (pB->GetCurSel() == LB_ERR)
		{
			MessageBox("No Percussion instrument selected!", "Error", MB_OK+MB_ICONEXCLAMATION);
			return;
		}

	}



	if (m_bDynamic)
	{
		
		if (m_Dynamic == -1)
		{
			MessageBox("No Dynamic selected!", "Error", MB_OK+MB_ICONEXCLAMATION);
			return;
		}

	}

	if (m_bAlgorithm)
	{
		pB = (CComboBox*)GetDlgItem(IDC_ALGORITHM_COMBO);

		if (pB->GetCurSel() == LB_ERR)
		{
			MessageBox("No Algorithm selected!", "Error", MB_OK+MB_ICONEXCLAMATION);
			return;
		}

	}

	//////////////////////

	CWaitCursor wait;


	if ((m_voiceonoff==0)||(m_voiceonoff==1))
	{
		for (int k = 0; k<=pDoc->m_NumEvents; k++)
			if (IsSelected[k])
			{

				for (int i = VOICE1; i<MAXVOICES; i++)
				{
					if (m_VoiceOn[i]||(pDoc->Event[k].IsActive[i]&&m_AllActiveVoices))
					{
						pDoc->Event[k].IsActive[i] = m_voiceonoff;
					}
				}
			}

	}


	for (int k = 0; k<=pDoc->m_NumEvents; k++)
		if (IsSelected[k])
		{
			CString m;
			m.Format("%d", k);
			pSta->SetWindowText(m);


			for (int i = VOICE1; i<MAXVOICES; i++)
			{
				if (m_VoiceOn[i]||(pDoc->Event[k].IsActive[i]&&m_AllActiveVoices))
				{


					if (m_bModulate)
					{
						pDoc->Event[k].Modulates[i] = m_Modulates;

						pDoc->Event[k].ModulNum[i] = m_ModulNum;
						pDoc->Event[k].ModulRandom[i] = m_ModulRandom;

					}

					if (m_bOctaves)
					{

						pB = (CComboBox*)GetDlgItem(IDC_NUMOCTS_COMBO);

						int sel = pB->GetCurSel();

						pDoc->Event[k].NumOctsID[i] = sel;

						switch(sel)
						{
							case ONE_OCT:
								pDoc->Event[k].NumOcts[i] = 1;
								break;
							case TWO_OCT:
								pDoc->Event[k].NumOcts[i] = 2;
								break;
							case THREE_OCT:
								pDoc->Event[k].NumOcts[i] = 3;
								break;
							case FOUR_OCT:
								pDoc->Event[k].NumOcts[i] = 4;
								break;
						}


					}

					if (m_bNote)
					{

						pB = (CComboBox*)GetDlgItem(IDC_STARTNOTE_COMBO);

						int sel = pB->GetCurSel();

						pDoc->Event[k].Note[i]  = sel;

					}

					if (m_bNoteval)
					{

						pB = (CComboBox*)GetDlgItem(IDC_NOTEVALUE_COMBO);

						int sel = pB->GetCurSel();

						pDoc->Event[k].NoteValueID[i] = sel;

						switch(sel)
						{
							case WHOLE:
								pDoc->Event[k].NoteValue[i] = 32.0f;
								break;
							case HALF:
								pDoc->Event[k].NoteValue[i] = 16.0f;
								break;
							case QUARTER:
								pDoc->Event[k].NoteValue[i] = 8.0f;
								break;
							case EIGHT:
								pDoc->Event[k].NoteValue[i] = 4.0f;
								break;
							case SIXTEENTH:
								pDoc->Event[k].NoteValue[i] = 2.0f;
								break;
							case THIRTYSECOND:
								pDoc->Event[k].NoteValue[i] = 1.0f;
								break;
							case WHOLE_DOTTED:
								pDoc->Event[k].NoteValue[i] = 48.00f;
								break;
							case HALF_DOTTED:
								pDoc->Event[k].NoteValue[i] = 24.00f;
								break;
							case QUARTER_DOTTED:
								pDoc->Event[k].NoteValue[i] = 12.00f;
								break;
							case EIGHTH_DOTTED:
								pDoc->Event[k].NoteValue[i] = 6.0f;
								break;
							case SIXTEENTH_DOTTED:
								pDoc->Event[k].NoteValue[i] = 3.0f;
								break;
							case OTHER:
								pDoc->Event[k].NoteValue[i] = (float)m_othernoteval;
								break;
						}

					}

					if (m_bInstrument)
					{

						pB = (CComboBox*)GetDlgItem(IDC_INSTRUMENT_COMBO);

						int sel = pB->GetCurSel();

						pDoc->Event[k].Instrument[i] = sel;
						pDoc->Event[k].Channel[i] = i;

					}

					if (m_bPercussion)
					{

						pB = (CComboBox*)GetDlgItem(IDC_PERCUSSION_COMBO);

						int sel = pB->GetCurSel();

						pDoc->Event[k].Percussion[i] = sel;
						pDoc->Event[k].Channel[i] = PercussionChannel;

					}

					if (m_bMode)
					{

						pB = (CComboBox*)GetDlgItem(IDC_MODE_COMBO);

						int sel = pB->GetCurSel();

						pDoc->Event[k].Mode[i] = sel;
					}

					if (m_bPan)
					{
						pDoc->Event[k].Pan[i] = m_Pan;
					}

					if (m_bDynamic)
					{

						pDoc->Event[k].DynChoice[i] = m_Dynamic;
						
						switch(m_Dynamic)
						{
							case PP:
								pDoc->Event[k].Dynamic[i] = PIANISSIMO;
								break;
							case P:
								pDoc->Event[k].Dynamic[i] = PIANO;
								break;
							case MP:
								pDoc->Event[k].Dynamic[i] = MEZZO_PIANO;
								break;
							case MF:
								pDoc->Event[k].Dynamic[i] = MEZZO_FORTE;
								break;
							case F:
								pDoc->Event[k].Dynamic[i] = FORTE;
								break;
							case FF:
								pDoc->Event[k].Dynamic[i] = FORTISSIMO;
								break;
						}
					}

					if (m_bInverted)
					{
						pDoc->Event[k].Inverted[i] = m_Inverted;

						if (m_bMatch)
						{
							pDoc->Event[k].MatchScale[i] = m_MatchScale;
						}
					}

					if (m_bAlgorithm)
					{

						pB = (CComboBox*)GetDlgItem(IDC_ALGORITHM_COMBO);

						int sel = pB->GetCurSel();

						pDoc->Event[k].AlgorithmNum[i] = sel;

						if (m_bCoeff)
						{
							switch (sel)
							{
								case MORSE_THUE:
									pDoc->Event[k].Base[i] = m_Base;
									pDoc->Event[k].Multi[i] = m_Multi;
									break;
								case LOGMAP:
									pDoc->Event[k].LogMapVar[i] = m_LogMapVar;
									break;
								case ONE_OVER_F_NOISE:
									pDoc->Event[k].FNoiseVar[i] = m_FNoiseVar;
									break;
								case HENON:
									pDoc->Event[k].HenonA[i] = m_HenonA;
									pDoc->Event[k].HenonB[i] = m_HenonB;
									break;
								case HOPALONG:
									pDoc->Event[k].HopalongVar[i] = m_HopalongVar;
									break;
								case MARTIN:
									pDoc->Event[k].MartinVar[i] = m_MartinVar;
									break;
								case GINGERBREADMAN:
									pDoc->Event[k].GingerX[i] = m_GingerX;
									pDoc->Event[k].GingerY[i] = m_GingerY;
									break;
								case LORENTZ:
									pDoc->Event[k].LorentzVar[i] = m_LorentzVar;
									break;
								case EARTHWORM:
									pDoc->Event[k].EarthWormVar[i] = m_EarthWormVar;
									pDoc->Event[k].EarthWormMulti[i] = m_EarthWormMulti;
									pDoc->Event[k].EarthWormTruncator[i] = m_EarthWormTruncator;
									break;
								case RANDOM:
									pDoc->Event[k].RandomMin[i] = m_RandomMin;
									pDoc->Event[k].RandomMax[i] = m_RandomMax;
									break;
								case HAILSTONE:
									pDoc->Event[k].Hailstone[i] = m_Hailstone;
									break;
								case WOLFRAM:
									pDoc->Event[k].WolframRule[i] = m_WolframRule;
									pDoc->Event[k].WolframInit[i] = m_WolframInit;
									pDoc->Event[k].WolframSize[i] = m_WolframSize;
									break;



							}
						}

					}

					if (m_bScale)
					{

						int j;

						pB = (CComboBox*)GetDlgItem(IDC_SCALE_COMBO);

						int sel = pB->GetCurSel();

						pDoc->Event[k].ScaleID[i] = sel;


				/////////////////////////////////////////////////////////////

						if (pDoc->Event[k].pScale[i])
						{
							delete pDoc->Event[k].pScale[i];
							pDoc->Event[k].pScale[i] = NULL;
						}


						switch(sel)
						{
							case MAJOR:
								if ((pDoc->Event[k].Inverted[i])&&(pDoc->Event[k].MatchScale[i]))
								{
									pDoc->Event[k].pScale[i] = new int[Phrygian[0] + 1];
									for(j =  0; j<=Phrygian[0]; j++)
										pDoc->Event[k].pScale[i][j] = Phrygian[j];
									break;
								}
								else
								{
									pDoc->Event[k].pScale[i] = new int[Major[0] + 1];
									for(j =  0; j<=Major[0]; j++)
										pDoc->Event[k].pScale[i][j] = Major[j];
									break;
								}
							case MINOR_NATURAL:
								if ((pDoc->Event[k].Inverted[i])&&(pDoc->Event[k].MatchScale[i]))
								{
									pDoc->Event[k].pScale[i] = new int[Mixolydian[0] + 1];
									for(j =  0; j<=Mixolydian[0]; j++)
										pDoc->Event[k].pScale[i][j] = Mixolydian[j];
									break;
								}
								else
								{		
									pDoc->Event[k].pScale[i] = new int[MinorNatural[0] + 1];
									for(j =  0; j<=MinorNatural[0]; j++)
										pDoc->Event[k].pScale[i][j] = MinorNatural[j];
									break;
								}
							case MINOR_HARMONIC:
								if ((pDoc->Event[k].Inverted[i])&&(pDoc->Event[k].MatchScale[i]))
								{
									pDoc->Event[k].pScale[i] = new int[7 + 1];
									pDoc->Event[k].pScale[i][0] = 7;
									pDoc->Event[k].pScale[i][1] = DO;
									pDoc->Event[k].pScale[i][2] = REb;
									pDoc->Event[k].pScale[i][3] = MI;
									pDoc->Event[k].pScale[i][4] = FA;
									pDoc->Event[k].pScale[i][5] = SOL;
									pDoc->Event[k].pScale[i][6] = LA;
									pDoc->Event[k].pScale[i][7] = SIb;

									break;
								}
								else
								{			
									pDoc->Event[k].pScale[i] = new int[MinorHarmonic[0] + 1];
									for(j =  0; j<=MinorHarmonic[0]; j++)
										pDoc->Event[k].pScale[i][j] = MinorHarmonic[j];
									break;
								}
							case MINOR_MELODIC:
								if ((pDoc->Event[k].Inverted[i])&&(pDoc->Event[k].MatchScale[i]))
								{
									pDoc->Event[k].pScale[i] = new int[7 + 1];
									pDoc->Event[k].pScale[i][0] = 7;
									pDoc->Event[k].pScale[i][1] = DO;
									pDoc->Event[k].pScale[i][2] = REb;
									pDoc->Event[k].pScale[i][3] = MIb;
									pDoc->Event[k].pScale[i][4] = FA;
									pDoc->Event[k].pScale[i][5] = SOL;
									pDoc->Event[k].pScale[i][6] = LA;
									pDoc->Event[k].pScale[i][7] = SIb;

									break;
								}
								else
								{			
									pDoc->Event[k].pScale[i] = new int[MinorMelodic[0] + 1];
									for(j =  0; j<=MinorMelodic[0]; j++)
										pDoc->Event[k].pScale[i][j] = MinorMelodic[j];
									break;
								}
							case LYDIAN:
								if ((pDoc->Event[k].Inverted[i])&&(pDoc->Event[k].MatchScale[i]))
								{
									pDoc->Event[k].pScale[i] = new int[Locrian[0] + 1];
									for(j =  0; j<=Locrian[0]; j++)
										pDoc->Event[k].pScale[i][j] = Locrian[j];
									break;
								}
								else
								{
									pDoc->Event[k].pScale[i] = new int[Lydian[0] + 1];
									for(j =  0; j<=Lydian[0]; j++)
										pDoc->Event[k].pScale[i][j] = Lydian[j];
									break;
								}		
							case MIXOLYDIAN:
								if ((pDoc->Event[k].Inverted[i])&&(pDoc->Event[k].MatchScale[i]))
								{
									pDoc->Event[k].pScale[i] = new int[MinorNatural[0] + 1];
									for(j =  0; j<=MinorNatural[0]; j++)
										pDoc->Event[k].pScale[i][j] = MinorNatural[j];
									break;
								}
								else
								{
									pDoc->Event[k].pScale[i] = new int[Mixolydian[0] + 1];
									for(j =  0; j<=Mixolydian[0]; j++)
										pDoc->Event[k].pScale[i][j] = Mixolydian[j];
									break;
								}		
							case DORIAN:
								pDoc->Event[k].pScale[i] = new int[Dorian[0] + 1];
								for(j =  0; j<=Dorian[0]; j++)
									pDoc->Event[k].pScale[i][j] = Dorian[j];
								break;
							case PHRYGIAN:
								if ((pDoc->Event[k].Inverted[i])&&(pDoc->Event[k].MatchScale[i]))
								{
									pDoc->Event[k].pScale[i] = new int[Major[0] + 1];
									for(j =  0; j<=Major[0]; j++)
										pDoc->Event[k].pScale[i][j] = Major[j];
									break;
								}
								else
								{
									pDoc->Event[k].pScale[i] = new int[Phrygian[0] + 1];
									for(j =  0; j<=Phrygian[0]; j++)
										pDoc->Event[k].pScale[i][j] = Phrygian[j];
									break;
								}		
							case LOCRIAN:
								if ((pDoc->Event[k].Inverted[i])&&(pDoc->Event[k].MatchScale[i]))
								{
									pDoc->Event[k].pScale[i] = new int[Lydian[0] + 1];
									for(j =  0; j<=Lydian[0]; j++)
										pDoc->Event[k].pScale[i][j] = Lydian[j];
									break;
								}
								else
								{
									pDoc->Event[k].pScale[i] = new int[Locrian[0] + 1];
									for(j =  0; j<=Locrian[0]; j++)
										pDoc->Event[k].pScale[i][j] = Locrian[j];
									break;
								}		
							case OCTATONIC1:
								if ((pDoc->Event[k].Inverted[i])&&(pDoc->Event[k].MatchScale[i]))
								{
									pDoc->Event[k].pScale[i] = new int[Octatonic2[0] + 1];
									for(j =  0; j<=Octatonic2[0]; j++)
										pDoc->Event[k].pScale[i][j] = Octatonic2[j];
									break;
								}
								else
								{
									pDoc->Event[k].pScale[i] = new int[Octatonic1[0] + 1];
									for(j =  0; j<=Octatonic1[0]; j++)
										pDoc->Event[k].pScale[i][j] = Octatonic1[j];
									break;
								}		
							case OCTATONIC2:
								if ((pDoc->Event[k].Inverted[i])&&(pDoc->Event[k].MatchScale[i]))
								{
									pDoc->Event[k].pScale[i] = new int[Octatonic1[0] + 1];
									for(j =  0; j<=Octatonic1[0]; j++)
										pDoc->Event[k].pScale[i][j] = Octatonic1[j];
									break;
								}
								else
								{
									pDoc->Event[k].pScale[i] = new int[Octatonic2[0] + 1];
									for(j =  0; j<=Octatonic2[0]; j++)
										pDoc->Event[k].pScale[i][j] = Octatonic2[j];
									break;
								}		
							case ACCOUSTIC:
								if ((pDoc->Event[k].Inverted[i])&&(pDoc->Event[k].MatchScale[i]))
								{
									pDoc->Event[k].pScale[i] = new int[7 + 1];
									pDoc->Event[k].pScale[i][0] = 7;
									pDoc->Event[k].pScale[i][1] = DO;
									pDoc->Event[k].pScale[i][2] = RE;
									pDoc->Event[k].pScale[i][3] = MIb;
									pDoc->Event[k].pScale[i][4] = FA;
									pDoc->Event[k].pScale[i][5] = SOLb;
									pDoc->Event[k].pScale[i][6] = LAb;
									pDoc->Event[k].pScale[i][7] = SIb;

									break;
								}
								else
								{			
									pDoc->Event[k].pScale[i] = new int[Accoustic[0] + 1];
									for(j =  0; j<=Accoustic[0]; j++)
										pDoc->Event[k].pScale[i][j] = Accoustic[j];
									break;
								}
							case WHOLETONE:
								pDoc->Event[k].pScale[i] = new int[Wholetone[0] + 1];
								for(j =  0; j<=Wholetone[0]; j++)
									pDoc->Event[k].pScale[i][j] = Wholetone[j];
								break;
							case PENTATONIC:
								if ((pDoc->Event[k].Inverted[i])&&(pDoc->Event[k].MatchScale[i]))
								{
									pDoc->Event[k].pScale[i] = new int[5 + 1];
									pDoc->Event[k].pScale[i][0] = 5;
									pDoc->Event[k].pScale[i][1] = DO;
									pDoc->Event[k].pScale[i][2] = MIb;
									pDoc->Event[k].pScale[i][3] = FA;
									pDoc->Event[k].pScale[i][4] = LAb;
									pDoc->Event[k].pScale[i][5] = SIb;
									break;
								}
								else
								{			
									pDoc->Event[k].pScale[i] = new int[Pentatonic[0] + 1];
									for(j =  0; j<=Pentatonic[0]; j++)
										pDoc->Event[k].pScale[i][j] = Pentatonic[j];
									break;
								}
							case CHROMATIC:
								pDoc->Event[k].pScale[i] = new int[Chromatic[0] + 1];
								for(j = 0; j<=Chromatic[0]; j++)
									pDoc->Event[k].pScale[i][j] = Chromatic[j];
								break;
							case USER_DEFINED:

								if (pDoc->Event[k].pUserScale[i])
								{
									delete pDoc->Event[k].pUserScale[i];
									pDoc->Event[k].pUserScale[i] = NULL;
								}

								if ((pDoc->Event[k].Inverted[i])&&(pDoc->Event[k].MatchScale[i]))
								{

									pDoc->Event[k].pScale[i] = new int[UserScale[0] + 1];
									pDoc->Event[k].pUserScale[i] = new int[UserScale[0] + 1];
									pDoc->Event[k].pScale[i][0] = pDoc->Event[k].pUserScale[i][0] = UserScale[0];
									for(j = 1; j<=UserScale[0]; j++)
									{
										pDoc->Event[k].pScale[i][j] = (12-UserScale[j])*(UserScale[j]!=0);
										pDoc->Event[k].pUserScale[i][j] = UserScale[j];
									}

									pView->SortScale(pDoc->Event[k].pScale[i], pDoc->Event[k].pScale[i][0]);
									pView->SortScale(pDoc->Event[k].pUserScale[i], pDoc->Event[k].pUserScale[i][0]);
									break;
								}
								else
								{

									pDoc->Event[k].pScale[i] = new int[UserScale[0] + 1];
									pDoc->Event[k].pUserScale[i] = new int[UserScale[0] + 1];
									for(j = 0; j<=UserScale[0]; j++)
									{
										pDoc->Event[k].pScale[i][j] = UserScale[j];
										pDoc->Event[k].pUserScale[i][j] = UserScale[j];
									}
									break;
								}
						} // switch(sel)
						
					} // if (m_bScale)

				} // if (m_VoiceOn[i])

			} // for (i = VOICE1 ... 

		} // if (IsSelected[k])

		pSta->SetWindowText("(Done)");

		delete selectedItems;

		selectedItems = NULL;

	
}

void CChangesDlg::OnEventAll() 
{
	CFractMusDoc* pDoc = pView->GetDocument();
	CListBox* pList = (CListBox*)GetDlgItem(IDC_EVENTLIST);

	pList->SelItemRange(TRUE, 0, pDoc->m_NumEvents);

	
}

void CChangesDlg::OnEventNone() 
{
	CFractMusDoc* pDoc = pView->GetDocument();
	CListBox* pList = (CListBox*)GetDlgItem(IDC_EVENTLIST);

	pList->SelItemRange(FALSE, 0, pDoc->m_NumEvents);
	
}

void CChangesDlg::OnAlgorithmChk() 
{
	UpdateData(TRUE);
	GetDlgItem(IDC_ALGORITHM_COMBO)->EnableWindow(m_bAlgorithm);
	
}

void CChangesDlg::OnCoefficientChk() 
{
	UpdateData(TRUE);
	GetDlgItem(IDC_COEFF_BUTT)->EnableWindow(m_bCoeff);
}

void CChangesDlg::OnDynamicChk() 
{
	UpdateData(TRUE);
	GetDlgItem(IDC_PIANISSIMO)->EnableWindow(m_bDynamic);
	GetDlgItem(IDC_PIANO)->EnableWindow(m_bDynamic);
	GetDlgItem(IDC_MPIANO)->EnableWindow(m_bDynamic);
	GetDlgItem(IDC_MFORTE)->EnableWindow(m_bDynamic);
	GetDlgItem(IDC_FORTE)->EnableWindow(m_bDynamic);
	GetDlgItem(IDC_FORTISSIMO)->EnableWindow(m_bDynamic);
}

void CChangesDlg::OnInstrumentChk() 
{
	UpdateData(TRUE);
	GetDlgItem(IDC_INSTRUMENT_COMBO)->EnableWindow(m_bInstrument);
	m_bPercussion = FALSE;
	CButton* pButt = (CButton*)GetDlgItem(IDC_PERCUSSION_CHK);
	pButt->SetCheck(m_bPercussion);
	GetDlgItem(IDC_PERCUSSION_COMBO)->EnableWindow(m_bPercussion);
	GetDlgItem(IDC_PERCUSSION_CHK)->EnableWindow(!m_bInstrument);
}

void CChangesDlg::OnInvChk() 
{
	UpdateData(TRUE);
	GetDlgItem(IDC_INVERTED)->EnableWindow(m_bInverted);
	GetDlgItem(IDC_MATCHSCL)->EnableWindow(m_bInverted&&m_bMatch&&m_Inverted);
}

void CChangesDlg::OnMatchChk() 
{
	UpdateData(TRUE);
	GetDlgItem(IDC_MATCHSCL)->EnableWindow(m_bInverted&&m_bMatch&&m_Inverted);
}

void CChangesDlg::OnModeChk() 
{
	UpdateData(TRUE);
	GetDlgItem(IDC_MODE_COMBO)->EnableWindow(m_bMode);
}

void CChangesDlg::OnModulChk() 
{
	UpdateData(TRUE);
	GetDlgItem(IDC_MODULATE_CHK)->EnableWindow(m_bModulate);
}

void CChangesDlg::OnModulateChk() 
{
	UpdateData(TRUE);

	if (m_Modulates)
	{
		CV1ModulDlg dlg;

		dlg.m_title = "Modulate";
		dlg.m_V1ModulDlgNum = m_ModulNum;
		dlg.m_V1ModulDlgChoice = m_ModulRandom;

		if ( dlg.DoModal() == IDOK)
		{
			m_ModulNum = dlg.m_V1ModulDlgNum;
			m_ModulRandom = dlg.m_V1ModulDlgChoice;
		}
		else
		{
			m_Modulates = FALSE;
			UpdateData(FALSE);
		}
	}
	
}

void CChangesDlg::OnNoteChk() 
{
	UpdateData(TRUE);
	GetDlgItem(IDC_STARTNOTE_COMBO)->EnableWindow(m_bNote);
}

void CChangesDlg::OnNotevalChk() 
{
	UpdateData(TRUE);
	GetDlgItem(IDC_NOTEVALUE_COMBO)->EnableWindow(m_bNoteval);
}

void CChangesDlg::OnOctavesChk() 
{
	UpdateData(TRUE);
	GetDlgItem(IDC_NUMOCTS_COMBO)->EnableWindow(m_bOctaves);
}

void CChangesDlg::OnPanChk() 
{
	UpdateData(TRUE);
	GetDlgItem(IDC_PAN)->EnableWindow(m_bPan);
	GetDlgItem(IDC_PAN_ZERO)->EnableWindow(m_bPan);
}


void CChangesDlg::OnScaleChk() 
{
	UpdateData(TRUE);
	GetDlgItem(IDC_SCALE_COMBO)->EnableWindow(m_bScale);

	
}

void CChangesDlg::OnAll() 
{

	UpdateData(TRUE);

	m_bAlgorithm = TRUE;
	m_bCoeff = TRUE;
	m_bDynamic = TRUE;
	m_bInstrument = TRUE;
	m_bInverted = TRUE;
	m_bMatch = TRUE;
	m_bMode = TRUE;
	m_bModulate = TRUE;
	m_bNote = TRUE;
	m_bNoteval = TRUE;
	m_bOctaves = TRUE;
	m_bPan = TRUE;
	m_bScale = TRUE;
	m_bPercussion = FALSE;

	UpdateData(FALSE);

	GetDlgItem(IDC_MODULATE_CHK)->EnableWindow(m_bModulate);
	GetDlgItem(IDC_NUMOCTS_COMBO)->EnableWindow(m_bOctaves);
	GetDlgItem(IDC_STARTNOTE_COMBO)->EnableWindow(m_bNote);
	GetDlgItem(IDC_SCALE_COMBO)->EnableWindow(m_bScale);
	GetDlgItem(IDC_NOTEVALUE_COMBO)->EnableWindow(m_bNoteval);
	GetDlgItem(IDC_INSTRUMENT_COMBO)->EnableWindow(m_bInstrument);
	GetDlgItem(IDC_PERCUSSION_COMBO)->EnableWindow(m_bPercussion);
	GetDlgItem(IDC_MODE_COMBO)->EnableWindow(m_bMode);
	GetDlgItem(IDC_PAN)->EnableWindow(m_bPan);
	GetDlgItem(IDC_PAN_ZERO)->EnableWindow(m_bPan);
	GetDlgItem(IDC_PIANISSIMO)->EnableWindow(m_bDynamic);
	GetDlgItem(IDC_PIANO)->EnableWindow(m_bDynamic);
	GetDlgItem(IDC_MPIANO)->EnableWindow(m_bDynamic);
	GetDlgItem(IDC_MFORTE)->EnableWindow(m_bDynamic);
	GetDlgItem(IDC_FORTE)->EnableWindow(m_bDynamic);
	GetDlgItem(IDC_FORTISSIMO)->EnableWindow(m_bDynamic);
	GetDlgItem(IDC_INVERTED)->EnableWindow(m_bInverted);
	GetDlgItem(IDC_MATCHSCL)->EnableWindow(m_bInverted&&m_bMatch&&m_Inverted);
	GetDlgItem(IDC_ALGORITHM_COMBO)->EnableWindow(m_bAlgorithm);
	GetDlgItem(IDC_COEFF_BUTT)->EnableWindow(m_bCoeff);
	GetDlgItem(IDC_PERCUSSION_CHK)->EnableWindow(m_bPercussion);
	GetDlgItem(IDC_INSTRUMENT_CHK)->EnableWindow(m_bInstrument);
	GetDlgItem(IDC_PERCUSSION_CHK)->EnableWindow(m_bPercussion);
	GetDlgItem(IDC_INSTRUMENT_CHK)->EnableWindow(m_bInstrument);
	
}

void CChangesDlg::OnNone() 
{

	UpdateData(TRUE);
	m_bAlgorithm = FALSE;
	m_bCoeff = FALSE;
	m_bDynamic = FALSE;
	m_bInstrument = FALSE;
	m_bInverted = FALSE;
	m_bMatch = FALSE;
	m_bMode = FALSE;
	m_bModulate = FALSE;
	m_bNote = FALSE;
	m_bNoteval = FALSE;
	m_bOctaves = FALSE;
	m_bPan = FALSE;
	m_bScale = FALSE;
	m_bPercussion = FALSE;

	UpdateData(FALSE);	

	GetDlgItem(IDC_MODULATE_CHK)->EnableWindow(m_bModulate);
	GetDlgItem(IDC_NUMOCTS_COMBO)->EnableWindow(m_bOctaves);
	GetDlgItem(IDC_STARTNOTE_COMBO)->EnableWindow(m_bNote);
	GetDlgItem(IDC_SCALE_COMBO)->EnableWindow(m_bScale);
	GetDlgItem(IDC_NOTEVALUE_COMBO)->EnableWindow(m_bNoteval);
	GetDlgItem(IDC_INSTRUMENT_COMBO)->EnableWindow(m_bInstrument);
	GetDlgItem(IDC_MODE_COMBO)->EnableWindow(m_bMode);
	GetDlgItem(IDC_PAN)->EnableWindow(m_bPan);
	GetDlgItem(IDC_PAN_ZERO)->EnableWindow(m_bPan);
	GetDlgItem(IDC_PIANISSIMO)->EnableWindow(m_bDynamic);
	GetDlgItem(IDC_PIANO)->EnableWindow(m_bDynamic);
	GetDlgItem(IDC_MPIANO)->EnableWindow(m_bDynamic);
	GetDlgItem(IDC_MFORTE)->EnableWindow(m_bDynamic);
	GetDlgItem(IDC_FORTE)->EnableWindow(m_bDynamic);
	GetDlgItem(IDC_FORTISSIMO)->EnableWindow(m_bDynamic);
	GetDlgItem(IDC_INVERTED)->EnableWindow(m_bInverted);
	GetDlgItem(IDC_MATCHSCL)->EnableWindow(m_bInverted&&m_bMatch&&m_Inverted);
	GetDlgItem(IDC_ALGORITHM_COMBO)->EnableWindow(m_bAlgorithm);
	GetDlgItem(IDC_COEFF_BUTT)->EnableWindow(m_bCoeff);
	GetDlgItem(IDC_PERCUSSION_COMBO)->EnableWindow(m_bPercussion);
	GetDlgItem(IDC_PERCUSSION_CHK)->EnableWindow(!m_bInstrument);
	GetDlgItem(IDC_INSTRUMENT_CHK)->EnableWindow(!m_bPercussion);


	
	
}

void CChangesDlg::OnCoeffButt() 
{
	
	UpdateData(TRUE);

	CComboBox* pB = (CComboBox*)GetDlgItem(IDC_ALGORITHM_COMBO);

	int sel = pB->GetCurSel();

	if ( sel == LB_ERR )
		return;

	switch (sel)
	{
		case MORSE_THUE:
			{
			CMorseThueDlg dlg0;

			dlg0.title = "Morse-Thue";
			dlg0.m_base = m_Base;
			dlg0.m_multi = m_Multi;
			dlg0.pView = pView;
			dlg0.voicenum = 0;

			if ( dlg0.DoModal() == IDOK )
			{
				m_Base = dlg0.m_base;
				m_Multi = dlg0.m_multi;
			}

			break;
			}

		case LOGMAP:
			{
			
			CLogMapDlg dlg1;

			dlg1.title = "LogMap";
			dlg1.m_LogMapVar = m_LogMapVar;
			dlg1.pView = pView;
			dlg1.voicenum = 0;

			if ( dlg1.DoModal() == IDOK )
				m_LogMapVar = dlg1.m_LogMapVar;

			break;
			}

		case ONE_OVER_F_NOISE:
			{
			
			CFNoiseDlg dlg2;

			dlg2.title = "1/f Noise";
			dlg2.m_fnoisevar = m_FNoiseVar;
			dlg2.pView = pView;
			dlg2.voicenum = 0;

			if ( dlg2.DoModal() == IDOK )
				m_FNoiseVar = dlg2.m_fnoisevar;

			break;
			}

		case HENON:
			{
			
			CHenonDlg dlg3;

			dlg3.title = "Henon";
			dlg3.m_henonA = m_HenonA;
			dlg3.m_henonB = m_HenonB;
			dlg3.pView = pView;
			dlg3.voicenum = 0;

			if ( dlg3.DoModal() == IDOK )
			{
				m_HenonA = dlg3.m_henonA;
				m_HenonB = dlg3.m_henonB;
			}
			break;
			}

		case HOPALONG:
			{
			
			CHopalongMartinDlg dlg4;

			dlg4.title = "Hopalong";
			dlg4.m_angle = m_HopalongVar;
			dlg4.pView = pView;
			dlg4.voicenum = 0;

			if ( dlg4.DoModal() == IDOK )
				m_HopalongVar = dlg4.m_angle;

			break;
			}

		case MARTIN:
			{
			
			CMartinDlg dlg5;

			dlg5.title = "Martin";
			dlg5.m_angle = m_MartinVar;
			dlg5.pView = pView;
			dlg5.voicenum = 0;

			if ( dlg5.DoModal() == IDOK )
				m_MartinVar = dlg5.m_angle;

			break;
			}

		case GINGERBREADMAN:
			{
			
			CGingerDlg dlg6;

			dlg6.title = "Ginger";
			dlg6.m_gingerx = m_GingerX;
			dlg6.m_gingery = m_GingerY;
			dlg6.pView = pView;
			dlg6.voicenum = 0;

			if ( dlg6.DoModal() == IDOK )
			{
				m_GingerX = dlg6.m_gingerx;
				m_GingerY = dlg6.m_gingery;
			}
			break;
			}

		case LORENTZ:
			{
			
			CLorentzDlg dlg7;

			dlg7.title = "Lorenz";
			dlg7.m_lorentz = m_LorentzVar;
			dlg7.pView = pView;
			dlg7.voicenum = 0;

			if ( dlg7.DoModal() == IDOK )
				m_LorentzVar = dlg7.m_lorentz;

			break;
			}
		case EARTHWORM:
		{

			CEarthWormDlg dlg8;

			dlg8.title = "Earthworm";
			dlg8.m_xvalue = m_EarthWormVar;
			dlg8.m_multi = m_EarthWormMulti;
			dlg8.m_truncator = m_EarthWormTruncator;
			dlg8.pView = pView;
			dlg8.voicenum = 0;

			if (dlg8.DoModal() == IDOK)
			{
				m_EarthWormVar = dlg8.m_xvalue;
				m_EarthWormMulti = dlg8.m_multi;
				m_EarthWormTruncator = dlg8.m_truncator;
			}

			break;

		}

		case RANDOM:
		{

			CRandomAlgorithmDlg dlg9;

			dlg9.title = "Random";
			dlg9.m_ranmax = m_RandomMax;
			dlg9.m_ranmin = m_RandomMin;
			dlg9.pView = pView;
			dlg9.voicenum = 0;

			if (dlg9.DoModal() == IDOK)
			{
				m_RandomMax = dlg9.m_ranmax;
				m_RandomMin = dlg9.m_ranmin;
			}

			break;

		}

		case HAILSTONE:
		{

			CHailstoneDlg dlg10;

			dlg10.title = "3n+1 numbers";
			dlg10.m_hailstone = m_Hailstone;
			dlg10.pView = pView;
			dlg10.voicenum = 0;


			if (dlg10.DoModal() == IDOK)
			{
				m_Hailstone = dlg10.m_hailstone;
			}

			break;

		}

		case WOLFRAM:
		{

			CWolframDlg dlg11;

			dlg11.title = "Wolfram CA";
			dlg11.m_rules = m_WolframRule;
			dlg11.m_initconfig = m_WolframInit;
			dlg11.m_universesize = m_WolframSize;
			if (dlg11.DoModal() == IDOK)
			{
				m_WolframRule = dlg11.m_rules;
				m_WolframInit = dlg11.m_initconfig;
				m_WolframSize = dlg11.m_universesize;
			}

			break;

		}



	}


}
void CChangesDlg::OnCloseupScaleCombo() 
{
	CFractMusDoc* pDoc = pView->GetDocument();
	
	CComboBox* pB = (CComboBox*)GetDlgItem(IDC_SCALE_COMBO);

	int sel = pB->GetCurSel();

	if (sel == LB_ERR)
		return;

	if (sel == USER_DEFINED)
	{
		int j = 1;

		InitUserScale();
		SetUserScale();
			
		UsrScl.DoModal();

		if (UserScale)
		{
			delete UserScale;
			UserScale = NULL;
		}

		UserScale = new int[UsrScl.m_numnotes + 1];
		UserScale[0] = UsrScl.m_numnotes;
		for(int i = 0; i<12; i++)
			if (UsrScl.m_bUserScale[i])
				UserScale[j++] = i;
		pView->SortScale(UserScale, UserScale[0]);
		pView->TransposeToC(UserScale, UserScale[0]);
	}	
}

void CChangesDlg::OnSettings() 
{
	CFractMusDoc* pDoc = pView->GetDocument();
	
	CComboBox* pB;

	if (!UpdateData())
		return;

	int numv = 0;

	for (int g = VOICE1; g<MAXVOICES; g++)
		if (m_VoiceOn[g])
			++numv;

	if (!numv)
	{
		MessageBox("No voices selected!","Error",MB_OK+MB_ICONEXCLAMATION);
		return;
	}

	if (m_bOctaves)
	{
		pB = (CComboBox*)GetDlgItem(IDC_NUMOCTS_COMBO);

		if (pB->GetCurSel() == LB_ERR)
		{
			MessageBox("No Octaves selected!", "Error", MB_OK+MB_ICONEXCLAMATION);
			return;
		}

	}

	if (m_bNote)
	{
		pB = (CComboBox*)GetDlgItem(IDC_STARTNOTE_COMBO);

		if (pB->GetCurSel() == LB_ERR)
		{
			MessageBox("No Note selected!", "Error", MB_OK+MB_ICONEXCLAMATION);
			return;
		}

	}

	if (m_bScale)
	{
		pB = (CComboBox*)GetDlgItem(IDC_SCALE_COMBO);

		if (pB->GetCurSel() == LB_ERR)
		{
			MessageBox("No Scale selected!", "Error", MB_OK+MB_ICONEXCLAMATION);
			return;
		}

	}

	if (m_bNoteval)
	{
		pB = (CComboBox*)GetDlgItem(IDC_NOTEVALUE_COMBO);

		if (pB->GetCurSel() == LB_ERR)
		{
			MessageBox("No Note Value selected!", "Error", MB_OK+MB_ICONEXCLAMATION);
			return;
		}

	}



	if (m_bMode)
	{
		pB = (CComboBox*)GetDlgItem(IDC_MODE_COMBO);

		if (pB->GetCurSel() == LB_ERR)
		{
			MessageBox("No Mode selected!", "Error", MB_OK+MB_ICONEXCLAMATION);
			return;
		}

	}

	if (m_bInstrument)
	{
		pB = (CComboBox*)GetDlgItem(IDC_INSTRUMENT_COMBO);

		if (pB->GetCurSel() == LB_ERR)
		{
			MessageBox("No Instrument selected!", "Error", MB_OK+MB_ICONEXCLAMATION);
			return;
		}

	}


	if (m_bPercussion)
	{
		pB = (CComboBox*)GetDlgItem(IDC_PERCUSSION_COMBO);

		if (pB->GetCurSel() == LB_ERR)
		{
			MessageBox("No Percussion instrument selected!", "Error", MB_OK+MB_ICONEXCLAMATION);
			return;
		}

	}

	if (m_bDynamic)
	{
		
		if (m_Dynamic == -1)
		{
			MessageBox("No Dynamic selected!", "Error", MB_OK+MB_ICONEXCLAMATION);
			return;
		}

	}

	if (m_bAlgorithm)
	{
		pB = (CComboBox*)GetDlgItem(IDC_ALGORITHM_COMBO);

		if (pB->GetCurSel() == LB_ERR)
		{
			MessageBox("No Algorithm selected!", "Error", MB_OK+MB_ICONEXCLAMATION);
			return;
		}

	}

	for (int i = VOICE1; i<MAXVOICES; i++)
	{
		if (m_VoiceOn[i])
		{


			if (m_bModulate)
			{
				pDoc->m_Modulates[i] = pView->m_Modulates[i] = m_Modulates;

				pDoc->m_ModulNum[i] = m_ModulNum;
				pDoc->m_ModulRandom[i] = m_ModulRandom;

			}

			if (m_bOctaves)
			{

				pB = (CComboBox*)GetDlgItem(IDC_NUMOCTS_COMBO);

				int sel = pB->GetCurSel();

				pDoc->m_NumOctsID[i] = pView->m_NumOctsID[i] = sel;

				switch(sel)
				{
					case ONE_OCT:
						pDoc->m_NumOcts[i] = 1;
						break;
					case TWO_OCT:
						pDoc->m_NumOcts[i] = 2;
						break;
					case THREE_OCT:
						pDoc->m_NumOcts[i] = 3;
						break;
					case FOUR_OCT:
						pDoc->m_NumOcts[i] = 4;
						break;
				}

				pB = (CComboBox*)(pView->GetDlgItem(pView->NumOctsID[i]));

				pB->SetCurSel(sel);

			}

			if (m_bNote)
			{

				pB = (CComboBox*)GetDlgItem(IDC_STARTNOTE_COMBO);

				int sel = pB->GetCurSel();

				pDoc->m_Note[i]  = pView->m_Note[i] = sel;

				pB = (CComboBox*)(pView->GetDlgItem(pView->NoteID[i]));

				pB->SetCurSel(sel);

			}

			if (m_bNoteval)
			{

				pB = (CComboBox*)GetDlgItem(IDC_NOTEVALUE_COMBO);

				int sel = pB->GetCurSel();

				pDoc->m_NoteValueID[i] = pView->m_NoteValueID[i] = sel;

				pB = (CComboBox*)(pView->GetDlgItem(pView->NoteValueID[i]));


				switch(sel)
				{
					case WHOLE:
						pDoc->m_NoteValue[i] = 32.0f;
						pB->SetCurSel(sel);
						break;
					case HALF:
						pDoc->m_NoteValue[i] = 16.0f;
						pB->SetCurSel(sel);
						break;
					case QUARTER:
						pDoc->m_NoteValue[i] = 8.0f;
						pB->SetCurSel(sel);
						break;
					case EIGHT:
						pDoc->m_NoteValue[i] = 4.0f;
						pB->SetCurSel(sel);
						break;
					case SIXTEENTH:
						pDoc->m_NoteValue[i] = 2.0f;
						pB->SetCurSel(sel);
						break;
					case THIRTYSECOND:
						pDoc->m_NoteValue[i] = 1.0f;
						pB->SetCurSel(sel);
						break;
					case WHOLE_DOTTED:
						pDoc->m_NoteValue[i] = 48.00f;
						pB->SetCurSel(sel);
						break;
					case HALF_DOTTED:
						pDoc->m_NoteValue[i] = 24.00f;
						pB->SetCurSel(sel);
						break;
					case QUARTER_DOTTED:
						pDoc->m_NoteValue[i] = 12.00f;
						pB->SetCurSel(sel);
						break;
					case EIGHTH_DOTTED:
						pDoc->m_NoteValue[i] = 6.0f;
						pB->SetCurSel(sel);
						break;
					case SIXTEENTH_DOTTED:
						pDoc->m_NoteValue[i] = 3.0f;
						pB->SetCurSel(sel);
						break;
					case OTHER:
						CString nval;
						nval.Format("Other (%d)", m_othernoteval);
						pB->DeleteString(OTHER);
						pB->AddString(nval);
						pDoc->m_NoteValue[i] = (float)m_othernoteval;
						pB->SetCurSel(OTHER);
						break;
				}

			}

			if (m_bInstrument)
			{
				CButton* pButt = (CButton*)(pView->GetDlgItem(pView->ActiveID[i]));

				pB = (CComboBox*)GetDlgItem(IDC_INSTRUMENT_COMBO);

				int sel = pB->GetCurSel();

				pDoc->m_Instrument[i] = sel;
				pDoc->m_Channel[i] = i;

				if (pDoc->m_Channel[i] == PercussionChannel)
				{
					pButt->SetWindowText(PercussionNames[ pDoc->m_Percussion[i] ]);
				}
				else
					pButt->SetWindowText(InstrumentNames[ pDoc->m_Instrument[i] ]);

			}

			if (m_bPercussion)
			{

				CButton* pButt = (CButton*)(pView->GetDlgItem(pView->ActiveID[i]));
				pB = (CComboBox*)GetDlgItem(IDC_PERCUSSION_COMBO);

				int sel = pB->GetCurSel();

				pDoc->m_Percussion[i] = sel;
				pDoc->m_Channel[i] = PercussionChannel;
				pButt->SetWindowText(PercussionNames[ pDoc->m_Percussion[i] ]);

			}

			if (m_bMode)
			{

				pB = (CComboBox*)GetDlgItem(IDC_MODE_COMBO);

				int sel = pB->GetCurSel();

				pDoc->m_mode[i] = sel;
			}

			if (m_bPan)
			{
				pDoc->m_Pan[i] = m_Pan;
			}

			if (m_bDynamic)
			{

				pDoc->m_DynChoice[i] = m_Dynamic;
				
				switch(m_Dynamic)
				{
					case PP:
						pDoc->m_Dynamic[i] = PIANISSIMO;
						break;
					case P:
						pDoc->m_Dynamic[i] = PIANO;
						break;
					case MP:
						pDoc->m_Dynamic[i] = MEZZO_PIANO;
						break;
					case MF:
						pDoc->m_Dynamic[i] = MEZZO_FORTE;
						break;
					case F:
						pDoc->m_Dynamic[i] = FORTE;
						break;
					case FF:
						pDoc->m_Dynamic[i] = FORTISSIMO;
						break;
				}
			}

			if (m_bInverted)
			{
				pDoc->m_Inverted[i] = m_Inverted;

				if (m_bMatch)
				{
					pDoc->m_MatchScale[i] = m_MatchScale;
				}
			}

			if (m_bAlgorithm)
			{

				pB = (CComboBox*)GetDlgItem(IDC_ALGORITHM_COMBO);

				int sel = pB->GetCurSel();

				pDoc->AlgorithmNum[i] = sel;

				if (m_bCoeff)
				{
					switch (sel)
					{
						case MORSE_THUE:
							pDoc->m_Base[i] = m_Base;
							pDoc->m_Multi[i] = m_Multi;
							break;
						case LOGMAP:
							pDoc->m_LogMapVar[i] = m_LogMapVar;
							break;
						case ONE_OVER_F_NOISE:
							pDoc->m_FNoiseVar[i] = m_FNoiseVar;
							break;
						case HENON:
							pDoc->m_HenonA[i] = m_HenonA;
							pDoc->m_HenonB[i] = m_HenonB;
							break;
						case HOPALONG:
							pDoc->m_HopalongVar[i] = m_HopalongVar;
							break;
						case MARTIN:
							pDoc->m_MartinVar[i] = m_MartinVar;
							break;
						case GINGERBREADMAN:
							pDoc->m_GingerX[i] = m_GingerX;
							pDoc->m_GingerY[i] = m_GingerY;
							break;
						case LORENTZ:
							pDoc->m_LorentzVar[i] = m_LorentzVar;
							break;
						case EARTHWORM:
							pDoc->m_EarthWormVar[i] = m_EarthWormVar;
							pDoc->m_EarthWormMulti[i] = m_EarthWormMulti;
							pDoc->m_EarthWormTruncator[i] = m_EarthWormTruncator;
							break;
						case RANDOM:
							pDoc->m_RandomMin[i] = m_RandomMin;
							pDoc->m_RandomMax[i] = m_RandomMax;
							break;
						case HAILSTONE:
							pDoc->m_Hailstone[i] = m_Hailstone;
							break;
						case WOLFRAM:
							pDoc->m_WolframRule[i] = m_WolframRule;
							pDoc->m_WolframInit[i] = m_WolframInit;
							pDoc->m_WolframSize[i] = m_WolframSize;
							break;
					}
				}

			}

			if (m_bScale)
			{

				int j;

				pB = (CComboBox*)GetDlgItem(IDC_SCALE_COMBO);

				int sel = pB->GetCurSel();

				pDoc->m_ScaleID[i] = pView->m_ScaleID[i] = sel;


		/////////////////////////////////////////////////////////////

				if (pDoc->m_pScale[i])
				{
					delete pDoc->m_pScale[i];
					pDoc->m_pScale[i] = NULL;
				}


				switch(sel)
				{
					case MAJOR:
						if ((pDoc->m_Inverted[i])&&(pDoc->m_MatchScale[i]))
						{
							pDoc->m_pScale[i] = new int[Phrygian[0] + 1];
							for(j =  0; j<=Phrygian[0]; j++)
								pDoc->m_pScale[i][j] = Phrygian[j];
							break;
						}
						else
						{
							pDoc->m_pScale[i] = new int[Major[0] + 1];
							for(j =  0; j<=Major[0]; j++)
								pDoc->m_pScale[i][j] = Major[j];
							break;
						}
					case MINOR_NATURAL:
						if ((pDoc->m_Inverted[i])&&(pDoc->m_MatchScale[i]))
						{
							pDoc->m_pScale[i] = new int[Mixolydian[0] + 1];
							for(j =  0; j<=Mixolydian[0]; j++)
								pDoc->m_pScale[i][j] = Mixolydian[j];
							break;
						}
						else
						{		
							pDoc->m_pScale[i] = new int[MinorNatural[0] + 1];
							for(j =  0; j<=MinorNatural[0]; j++)
								pDoc->m_pScale[i][j] = MinorNatural[j];
							break;
						}
					case MINOR_HARMONIC:
						if ((pDoc->m_Inverted[i])&&(pDoc->m_MatchScale[i]))
						{
							pDoc->m_pScale[i] = new int[7 + 1];
							pDoc->m_pScale[i][0] = 7;
							pDoc->m_pScale[i][1] = DO;
							pDoc->m_pScale[i][2] = REb;
							pDoc->m_pScale[i][3] = MI;
							pDoc->m_pScale[i][4] = FA;
							pDoc->m_pScale[i][5] = SOL;
							pDoc->m_pScale[i][6] = LA;
							pDoc->m_pScale[i][7] = SIb;

							break;
						}
						else
						{			
							pDoc->m_pScale[i] = new int[MinorHarmonic[0] + 1];
							for(j =  0; j<=MinorHarmonic[0]; j++)
								pDoc->m_pScale[i][j] = MinorHarmonic[j];
							break;
						}
					case MINOR_MELODIC:
						if ((pDoc->m_Inverted[i])&&(pDoc->m_MatchScale[i]))
						{
							pDoc->m_pScale[i] = new int[7 + 1];
							pDoc->m_pScale[i][0] = 7;
							pDoc->m_pScale[i][1] = DO;
							pDoc->m_pScale[i][2] = REb;
							pDoc->m_pScale[i][3] = MIb;
							pDoc->m_pScale[i][4] = FA;
							pDoc->m_pScale[i][5] = SOL;
							pDoc->m_pScale[i][6] = LA;
							pDoc->m_pScale[i][7] = SIb;

							break;
						}
						else
						{			
							pDoc->m_pScale[i] = new int[MinorMelodic[0] + 1];
							for(j =  0; j<=MinorMelodic[0]; j++)
								pDoc->m_pScale[i][j] = MinorMelodic[j];
							break;
						}
					case LYDIAN:
						if ((pDoc->m_Inverted[i])&&(pDoc->m_MatchScale[i]))
						{
							pDoc->m_pScale[i] = new int[Locrian[0] + 1];
							for(j =  0; j<=Locrian[0]; j++)
								pDoc->m_pScale[i][j] = Locrian[j];
							break;
						}
						else
						{
							pDoc->m_pScale[i] = new int[Lydian[0] + 1];
							for(j =  0; j<=Lydian[0]; j++)
								pDoc->m_pScale[i][j] = Lydian[j];
							break;
						}		
					case MIXOLYDIAN:
						if ((pDoc->m_Inverted[i])&&(pDoc->m_MatchScale[i]))
						{
							pDoc->m_pScale[i] = new int[MinorNatural[0] + 1];
							for(j =  0; j<=MinorNatural[0]; j++)
								pDoc->m_pScale[i][j] = MinorNatural[j];
							break;
						}
						else
						{
							pDoc->m_pScale[i] = new int[Mixolydian[0] + 1];
							for(j =  0; j<=Mixolydian[0]; j++)
								pDoc->m_pScale[i][j] = Mixolydian[j];
							break;
						}		
					case DORIAN:
						pDoc->m_pScale[i] = new int[Dorian[0] + 1];
						for(j =  0; j<=Dorian[0]; j++)
							pDoc->m_pScale[i][j] = Dorian[j];
						break;
					case PHRYGIAN:
						if ((pDoc->m_Inverted[i])&&(pDoc->m_MatchScale[i]))
						{
							pDoc->m_pScale[i] = new int[Major[0] + 1];
							for(j =  0; j<=Major[0]; j++)
								pDoc->m_pScale[i][j] = Major[j];
							break;
						}
						else
						{
							pDoc->m_pScale[i] = new int[Phrygian[0] + 1];
							for(j =  0; j<=Phrygian[0]; j++)
								pDoc->m_pScale[i][j] = Phrygian[j];
							break;
						}		
					case LOCRIAN:
						if ((pDoc->m_Inverted[i])&&(pDoc->m_MatchScale[i]))
						{
							pDoc->m_pScale[i] = new int[Lydian[0] + 1];
							for(j =  0; j<=Lydian[0]; j++)
								pDoc->m_pScale[i][j] = Lydian[j];
							break;
						}
						else
						{
							pDoc->m_pScale[i] = new int[Locrian[0] + 1];
							for(j =  0; j<=Locrian[0]; j++)
								pDoc->m_pScale[i][j] = Locrian[j];
							break;
						}		
					case OCTATONIC1:
						if ((pDoc->m_Inverted[i])&&(pDoc->m_MatchScale[i]))
						{
							pDoc->m_pScale[i] = new int[Octatonic2[0] + 1];
							for(j =  0; j<=Octatonic2[0]; j++)
								pDoc->m_pScale[i][j] = Octatonic2[j];
							break;
						}
						else
						{
							pDoc->m_pScale[i] = new int[Octatonic1[0] + 1];
							for(j =  0; j<=Octatonic1[0]; j++)
								pDoc->m_pScale[i][j] = Octatonic1[j];
							break;
						}		
					case OCTATONIC2:
						if ((pDoc->m_Inverted[i])&&(pDoc->m_MatchScale[i]))
						{
							pDoc->m_pScale[i] = new int[Octatonic1[0] + 1];
							for(j =  0; j<=Octatonic1[0]; j++)
								pDoc->m_pScale[i][j] = Octatonic1[j];
							break;
						}
						else
						{
							pDoc->m_pScale[i] = new int[Octatonic2[0] + 1];
							for(j =  0; j<=Octatonic2[0]; j++)
								pDoc->m_pScale[i][j] = Octatonic2[j];
							break;
						}		
					case ACCOUSTIC:
						if ((pDoc->m_Inverted[i])&&(pDoc->m_MatchScale[i]))
						{
							pDoc->m_pScale[i] = new int[7 + 1];
							pDoc->m_pScale[i][0] = 7;
							pDoc->m_pScale[i][1] = DO;
							pDoc->m_pScale[i][2] = RE;
							pDoc->m_pScale[i][3] = MIb;
							pDoc->m_pScale[i][4] = FA;
							pDoc->m_pScale[i][5] = SOLb;
							pDoc->m_pScale[i][6] = LAb;
							pDoc->m_pScale[i][7] = SIb;

							break;
						}
						else
						{			
							pDoc->m_pScale[i] = new int[Accoustic[0] + 1];
							for(j =  0; j<=Accoustic[0]; j++)
								pDoc->m_pScale[i][j] = Accoustic[j];
							break;
						}
					case WHOLETONE:
						pDoc->m_pScale[i] = new int[Wholetone[0] + 1];
						for(j =  0; j<=Wholetone[0]; j++)
							pDoc->m_pScale[i][j] = Wholetone[j];
						break;
					case PENTATONIC:
						if ((pDoc->m_Inverted[i])&&(pDoc->m_MatchScale[i]))
						{
							pDoc->m_pScale[i] = new int[5 + 1];
							pDoc->m_pScale[i][0] = 5;
							pDoc->m_pScale[i][1] = DO;
							pDoc->m_pScale[i][2] = MIb;
							pDoc->m_pScale[i][3] = FA;
							pDoc->m_pScale[i][4] = LAb;
							pDoc->m_pScale[i][5] = SIb;
							break;
						}
						else
						{			
							pDoc->m_pScale[i] = new int[Pentatonic[0] + 1];
							for(j =  0; j<=Pentatonic[0]; j++)
								pDoc->m_pScale[i][j] = Pentatonic[j];
							break;
						}
					case CHROMATIC:
						pDoc->m_pScale[i] = new int[Chromatic[0] + 1];
						for(j = 0; j<=Chromatic[0]; j++)
							pDoc->m_pScale[i][j] = Chromatic[j];
						break;
					case USER_DEFINED:

						if (pDoc->m_pUserScale[i])
						{
							delete pDoc->m_pUserScale[i];
							pDoc->m_pUserScale[i] = NULL;
						}

						if ((pDoc->m_Inverted[i])&&(pDoc->m_MatchScale[i]))
						{

							pDoc->m_pScale[i] = new int[UserScale[0] + 1];
							pDoc->m_pUserScale[i] = new int[UserScale[0] + 1];
							pDoc->m_pScale[i][0] = pDoc->m_pUserScale[i][0] = UserScale[0];
							for(j = 1; j<=UserScale[0]; j++)
							{
								pDoc->m_pScale[i][j] = (12-UserScale[j])*(UserScale[j]!=0);
								pDoc->m_pUserScale[i][j] = UserScale[j];
							}

							pView->SortScale(pDoc->m_pScale[i], pDoc->m_pScale[i][0]);
							pView->SortScale(pDoc->m_pUserScale[i], pDoc->m_pUserScale[i][0]);
							break;
						}
						else
						{

							pDoc->m_pScale[i] = new int[UserScale[0] + 1];
							pDoc->m_pUserScale[i] = new int[UserScale[0] + 1];
							for(j = 0; j<=UserScale[0]; j++)
							{
								pDoc->m_pScale[i][j] = UserScale[j];
								pDoc->m_pUserScale[i][j] = UserScale[j];
							}
							break;
						}
				}

				pB = (CComboBox*)(pView->GetDlgItem(pView->ScaleID[i]));

				pB->SetCurSel(sel);

				
			}

		}

	}

	pView->UpdateData(FALSE);

	pDoc->SetModifiedFlag();



}

void CChangesDlg::OnCloseupNotevalueCombo() 
{
	CFractMusDoc* pDoc = pView->GetDocument();
	
	CComboBox* pB = (CComboBox*)GetDlgItem(IDC_NOTEVALUE_COMBO);

	int sel = pB->GetCurSel();

	if (sel == LB_ERR)
		return;

	if (sel == OTHER)
	{
		CNoteValDlg dlg;

		dlg.title = "Other value";
		dlg.m_NewVal = m_othernoteval;

		dlg.DoModal();

		m_othernoteval = dlg.m_NewVal;

	}


}

void CChangesDlg::OnInverted() 
{
	UpdateData(TRUE);
	GetDlgItem(IDC_MATCHSCL)->EnableWindow(m_bInverted&&m_bMatch&&m_Inverted);
	
}


void CChangesDlg::InitUserScale()
{
	UsrScl.m_numnotes = 0;
	for(int i=0; i<12; i++)
		UsrScl.m_bUserScale[i] = FALSE;

	UsrScl.m_C_ON = FALSE;
	UsrScl.m_Db_ON = FALSE;
	UsrScl.m_D_ON = FALSE;
	UsrScl.m_Eb_ON = FALSE;
	UsrScl.m_E_ON = FALSE;
	UsrScl.m_F_ON = FALSE;
	UsrScl.m_Gb_ON = FALSE;
	UsrScl.m_G_ON = FALSE;
	UsrScl.m_Ab_ON = FALSE;
	UsrScl.m_A_ON = FALSE;
	UsrScl.m_Bb_ON = FALSE;
	UsrScl.m_B_ON = FALSE;
}

void CChangesDlg::SetUserScale()
{

	UsrScl.m_numnotes = UserScale[0];

	for(int i=1; i<=UserScale[0]; i++)
		UsrScl.m_bUserScale[UserScale[i]] = TRUE;

	UsrScl.m_C_ON = UsrScl.m_bUserScale[0];
	UsrScl.m_Db_ON =UsrScl.m_bUserScale[1];
	UsrScl.m_D_ON = UsrScl.m_bUserScale[2];
	UsrScl.m_Eb_ON =UsrScl.m_bUserScale[3];
	UsrScl.m_E_ON = UsrScl.m_bUserScale[4];
	UsrScl.m_F_ON = UsrScl.m_bUserScale[5];
	UsrScl.m_Gb_ON =UsrScl.m_bUserScale[6];
	UsrScl.m_G_ON = UsrScl.m_bUserScale[7];
	UsrScl.m_Ab_ON =UsrScl.m_bUserScale[8];
	UsrScl.m_A_ON = UsrScl.m_bUserScale[9];
	UsrScl.m_Bb_ON =UsrScl.m_bUserScale[10];
	UsrScl.m_B_ON = UsrScl.m_bUserScale[11];


}



void CChangesDlg::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	CScrollBar* pSC = (CScrollBar*)GetDlgItem(IDC_PAN);
	CStatic* pSt = (CStatic*)GetDlgItem(IDC_PAN_STATIC);
	char s[20];

	if (pScrollBar != (CScrollBar*)pSC)
		return;

	int pos = pSC->GetScrollPos();

	switch(nSBCode)
	{
		case SB_LINERIGHT:
			pos++;
			if(pos>63) pos = 63;
			break;
		case SB_LINELEFT:
			pos--;
			if(pos<-64) pos = -64;
			break;
		case SB_THUMBPOSITION:
		case SB_THUMBTRACK:
			pos = nPos;
			break;
		case SB_PAGERIGHT:
			pos +=5;
			if(pos>63) pos = 63;
			break;
		case SB_PAGELEFT:
			pos -= 5;
			if(pos<-64) pos = -64;
			break;
	}

	pSC->SetScrollPos(pos);

	m_Pan = pos;
	itoa(m_Pan, s ,10);
	pSt->SetWindowText(s);
	
	CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
}

void CChangesDlg::OnPanZero() 
{
	CScrollBar* pSC = (CScrollBar*)GetDlgItem(IDC_PAN);
	CStatic* pSt = (CStatic*)GetDlgItem(IDC_PAN_STATIC);
	char s[20];


	pSC->SetScrollPos(0);

	m_Pan = 0;
	itoa(m_Pan, s ,10);
	pSt->SetWindowText(s);
	
}

void CChangesDlg::OnPercussionChk() 
{
	UpdateData(TRUE);
	GetDlgItem(IDC_PERCUSSION_COMBO)->EnableWindow(m_bPercussion);
	m_bInstrument = FALSE;
	CButton* pButt = (CButton*)GetDlgItem(IDC_INSTRUMENT_CHK);
	pButt->SetCheck(m_bInstrument);
	GetDlgItem(IDC_INSTRUMENT_COMBO)->EnableWindow(m_bInstrument);
	GetDlgItem(IDC_INSTRUMENT_CHK)->EnableWindow(!m_bPercussion);
	
}
