
/*************************************************************************


Copyright 1998, 1999, 2000 Gustavo Diaz-Jerez


This file is part of FractMus.

    FractMus is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FractMus is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FractMus; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


**************************************************************************/


#ifndef _FRACTMUSVIEW_H
#define _FRACTMUSVIEW_H

// FractMusView.h : interface of the CFractMusView class
//
/////////////////////////////////////////////////////////////////////////////


#include "V1MidiSetDlg.h"
#include "V1ModulDlg.h"
#include "V1UsrSclDlg.h"
#include "EventsDlg.h"
#include "GraphWnd.h"
#include "InfoDlg.h"
#include "NoteValDlg.h"
#include "RandDlg.h"
#include "ChangesDlg.h"
#include "GetNoteDlg.h"
#include "TransposerDlg.h"
#include "InstrumentationDlg.h"

#include "FractMusDoc.h"

#include "defs.h"


class CFractMusView : public CFormView
{
protected: // create from serialization only
	CFractMusView();
	DECLARE_DYNCREATE(CFractMusView)


public:


	void EventsDone();

	void UpdateStartNum(int voice);
	void UpdateScaleCombo(int voice);
	void UpdateNoteValueCombo(int voice);
	void UpdateNumOctsCombo(int voice);
	void UpdateNoteCombo(int voice);
	void UpdateMidi(int voice);
	void UpdateModulation(int voice);
	void UpdateActive(int voice);
	void UpdateAlgorithm(int voice, short alg);

	void InitUserScale(int voice);
	void SetUserScale(int voice);
	void SetScale(int voice, int choice);
	void SetNoteValue(int voice, int choice);
	void SetNumOcts(int voice, int choice);
	void TransposeToC (int* inscale, int num);
	void SortScale (int* inscale, int limit);

	 //threads

	friend UINT PlayComposition(LPVOID);
	friend UINT PlayScript(LPVOID);



public:

	CEventsDlg*		m_pEvents;
	CGraphWnd*		m_pGraph;
	CV1UsrSclDlg	m_V1UsrScl[MAXVOICES];
	CV1MidiSetDlg	m_V1MidiSet[MAXVOICES];
	CV1ModulDlg		m_V1Modul[MAXVOICES];
	CNoteValDlg		m_NoteVal[MAXVOICES];

	int		m_TempoMarking; 
	long	GeneralCounter;
	long	StartCounter;
	BOOL	LockGeneralCounter;
	short	PlayChoice;
	BOOL	UpdateSettings;

	


public:
	//{{AFX_DATA(CFractMusView)
	enum { IDD = IDD_FRACTMUS_FORM };
	long	m_StartFrom;
	//}}AFX_DATA

	BOOL	m_IsActive[MAXVOICES];
	BOOL	m_Modulates[MAXVOICES];
	long	m_StartNum[MAXVOICES];
	
	int		m_ScaleID[MAXVOICES];
	int		m_NumOctsID[MAXVOICES];
	int		m_Note[MAXVOICES];
	int		m_NoteValueID[MAXVOICES];

	UINT	StartNumID[MAXVOICES];
	UINT	ScaleID[MAXVOICES];
	UINT	NumOctsID[MAXVOICES];
	UINT	NoteID[MAXVOICES];
	UINT	ActiveID[MAXVOICES];
	UINT	NoteValueID[MAXVOICES];
	UINT	ModulatesID[MAXVOICES];
	UINT	MidiButtonID[MAXVOICES];

	CRandDlg			m_randDlg;
	CChangesDlg			m_changesDlg;
	CTransposerDlg		m_transposerDlg;
	CInstrumentationDlg m_instrumentationDlg;
	

// Attributes
public:
	CFractMusDoc* GetDocument();

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CFractMusView)
	public:
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	virtual void OnInitialUpdate();
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	virtual void OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CFractMusView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	//{{AFX_MSG(CFractMusView)
	afx_msg void OnVoice1MidiButton();
	afx_msg void OnVoice1ModulateChk();
	afx_msg void OnCloseupVoice1NotevalueCombo();
	afx_msg void OnCloseupVoice1NumoctsCombo();
	afx_msg void OnVoice1OnChk();
	afx_msg void OnCloseupVoice1ScaleCombo();
	afx_msg void OnCloseupVoice1StartnoteCombo();
	afx_msg void OnKillfocusVoice1StartnumEdit();
	afx_msg void OnVoice2MidiButton();
	afx_msg void OnVoice2ModulateChk();
	afx_msg void OnCloseupVoice2NotevalueCombo();
	afx_msg void OnCloseupVoice2NumoctsCombo();
	afx_msg void OnVoice2OnChk();
	afx_msg void OnCloseupVoice2ScaleCombo();
	afx_msg void OnCloseupVoice2StartnoteCombo();
	afx_msg void OnKillfocusVoice2StartnumEdit();
	afx_msg void OnVoice3MidiButton();
	afx_msg void OnVoice3ModulateChk();
	afx_msg void OnCloseupVoice3NotevalueCombo();
	afx_msg void OnCloseupVoice3NumoctsCombo();
	afx_msg void OnVoice3OnChk();
	afx_msg void OnCloseupVoice3ScaleCombo();
	afx_msg void OnCloseupVoice3StartnoteCombo();
	afx_msg void OnKillfocusVoice3StartnumEdit();
	afx_msg void OnVoice4MidiButton();
	afx_msg void OnVoice4ModulateChk();
	afx_msg void OnCloseupVoice4NotevalueCombo();
	afx_msg void OnCloseupVoice4NumoctsCombo();
	afx_msg void OnVoice4OnChk();
	afx_msg void OnCloseupVoice4ScaleCombo();
	afx_msg void OnCloseupVoice4StartnoteCombo();
	afx_msg void OnKillfocusVoice4StartnumEdit();
	afx_msg void OnVoice5MidiButton();
	afx_msg void OnVoice5ModulateChk();
	afx_msg void OnCloseupVoice5NotevalueCombo();
	afx_msg void OnCloseupVoice5NumoctsCombo();
	afx_msg void OnVoice5OnChk();
	afx_msg void OnCloseupVoice5ScaleCombo();
	afx_msg void OnCloseupVoice5StartnoteCombo();
	afx_msg void OnKillfocusVoice5StartnumEdit();
	afx_msg void OnVoice6MidiButton();
	afx_msg void OnVoice6ModulateChk();
	afx_msg void OnCloseupVoice6NotevalueCombo();
	afx_msg void OnCloseupVoice6NumoctsCombo();
	afx_msg void OnVoice6OnChk();
	afx_msg void OnCloseupVoice6ScaleCombo();
	afx_msg void OnCloseupVoice6StartnoteCombo();
	afx_msg void OnKillfocusVoice6StartnumEdit();
	afx_msg void OnVoice7MidiButton();
	afx_msg void OnVoice7ModulateChk();
	afx_msg void OnCloseupVoice7NotevalueCombo();
	afx_msg void OnCloseupVoice7NumoctsCombo();
	afx_msg void OnVoice7OnChk();
	afx_msg void OnCloseupVoice7ScaleCombo();
	afx_msg void OnCloseupVoice7StartnoteCombo();
	afx_msg void OnKillfocusVoice7StartnumEdit();
	afx_msg void OnVoice8MidiButton();
	afx_msg void OnVoice8ModulateChk();
	afx_msg void OnCloseupVoice8NotevalueCombo();
	afx_msg void OnCloseupVoice8NumoctsCombo();
	afx_msg void OnVoice8OnChk();
	afx_msg void OnCloseupVoice8ScaleCombo();
	afx_msg void OnCloseupVoice8StartnoteCombo();
	afx_msg void OnKillfocusVoice8StartnumEdit();
	afx_msg void OnUpdateAppExit(CCmdUI* pCmdUI);
	afx_msg void OnUpdateFileNew(CCmdUI* pCmdUI);
	afx_msg void OnUpdateFileOpen(CCmdUI* pCmdUI);
	afx_msg void OnUpdateFileSave(CCmdUI* pCmdUI);
	afx_msg void OnUpdateFileSaveAs(CCmdUI* pCmdUI);
	afx_msg void OnUpdateFileSavemidi(CCmdUI* pCmdUI);
	afx_msg void OnPlayPlay();
	afx_msg void OnUpdatePlayPlay(CCmdUI* pCmdUI);
	afx_msg void OnPlayStop();
	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnFileSavemidi();
	afx_msg void OnOptsEvents();
	afx_msg void OnUpdateOptsEvents(CCmdUI* pCmdUI);
	afx_msg void OnResetGencounter();
	afx_msg void OnKillfocusGencounterEd();
	afx_msg void OnAlgorithmsVoice1Morsethue();
	afx_msg void OnUpdateAlgorithmsVoice1Morsethue(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice2Morsethue();
	afx_msg void OnUpdateAlgorithmsVoice2Morsethue(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice3Morsethue();
	afx_msg void OnUpdateAlgorithmsVoice3Morsethue(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice4Morsethue();
	afx_msg void OnUpdateAlgorithmsVoice4Morsethue(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice5Morsethue();
	afx_msg void OnUpdateAlgorithmsVoice5Morsethue(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice6Morsethue();
	afx_msg void OnUpdateAlgorithmsVoice6Morsethue(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice7Morsethue();
	afx_msg void OnUpdateAlgorithmsVoice7Morsethue(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice8Morsethue();
	afx_msg void OnUpdateAlgorithmsVoice8Morsethue(CCmdUI* pCmdUI);
	afx_msg void OnCurrent();
	afx_msg void OnUpdateCurrent(CCmdUI* pCmdUI);
	afx_msg void OnComposition();
	afx_msg void OnUpdateComposition(CCmdUI* pCmdUI);
	afx_msg void OnRound();
	afx_msg void OnAlgorithmsVoice1Logmap();
	afx_msg void OnUpdateAlgorithmsVoice1Logmap(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice2Logmap();
	afx_msg void OnUpdateAlgorithmsVoice2Logmap(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice3Logmap();
	afx_msg void OnUpdateAlgorithmsVoice3Logmap(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice4Logmap();
	afx_msg void OnUpdateAlgorithmsVoice4Logmap(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice5Logmap();
	afx_msg void OnUpdateAlgorithmsVoice5Logmap(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice6Logmap();
	afx_msg void OnUpdateAlgorithmsVoice6Logmap(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice7Logmap();
	afx_msg void OnUpdateAlgorithmsVoice7Logmap(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice8Logmap();
	afx_msg void OnUpdateAlgorithmsVoice8Logmap(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice1Oneoverf();
	afx_msg void OnUpdateAlgorithmsVoice1Oneoverf(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice2Oneoverf();
	afx_msg void OnUpdateAlgorithmsVoice2Oneoverf(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice3Oneoverf();
	afx_msg void OnUpdateAlgorithmsVoice3Oneoverf(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice4Oneoverf();
	afx_msg void OnUpdateAlgorithmsVoice4Oneoverf(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice5Oneoverf();
	afx_msg void OnUpdateAlgorithmsVoice5Oneoverf(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice6Oneoverf();
	afx_msg void OnUpdateAlgorithmsVoice6Oneoverf(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice7Oneoverf();
	afx_msg void OnUpdateAlgorithmsVoice7Oneoverf(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice8Oneoverf();
	afx_msg void OnUpdateAlgorithmsVoice8Oneoverf(CCmdUI* pCmdUI);
	afx_msg void OnOptsDevice();
	afx_msg void OnUpdateOptsDevice(CCmdUI* pCmdUI);
	afx_msg void OnVoice10MidiButton();
	afx_msg void OnVoice10ModulateChk();
	afx_msg void OnCloseupVoice10NotevalueCombo();
	afx_msg void OnCloseupVoice10NumoctsCombo();
	afx_msg void OnVoice10OnChk();
	afx_msg void OnCloseupVoice10ScaleCombo();
	afx_msg void OnCloseupVoice10StartnoteCombo();
	afx_msg void OnKillfocusVoice10StartnumEdit();
	afx_msg void OnVoice11MidiButton();
	afx_msg void OnVoice11ModulateChk();
	afx_msg void OnCloseupVoice11NotevalueCombo();
	afx_msg void OnCloseupVoice11NumoctsCombo();
	afx_msg void OnVoice11OnChk();
	afx_msg void OnCloseupVoice11ScaleCombo();
	afx_msg void OnCloseupVoice11StartnoteCombo();
	afx_msg void OnKillfocusVoice11StartnumEdit();
	afx_msg void OnVoice12MidiButton();
	afx_msg void OnVoice12ModulateChk();
	afx_msg void OnCloseupVoice12NotevalueCombo();
	afx_msg void OnCloseupVoice12NumoctsCombo();
	afx_msg void OnVoice12OnChk();
	afx_msg void OnCloseupVoice12ScaleCombo();
	afx_msg void OnCloseupVoice12StartnoteCombo();
	afx_msg void OnKillfocusVoice12StartnumEdit();
	afx_msg void OnVoice13MidiButton();
	afx_msg void OnVoice13ModulateChk();
	afx_msg void OnCloseupVoice13NotevalueCombo();
	afx_msg void OnCloseupVoice13NumoctsCombo();
	afx_msg void OnVoice13OnChk();
	afx_msg void OnCloseupVoice13ScaleCombo();
	afx_msg void OnCloseupVoice13StartnoteCombo();
	afx_msg void OnKillfocusVoice13StartnumEdit();
	afx_msg void OnVoice14MidiButton();
	afx_msg void OnVoice14ModulateChk();
	afx_msg void OnCloseupVoice14NotevalueCombo();
	afx_msg void OnCloseupVoice14NumoctsCombo();
	afx_msg void OnVoice14OnChk();
	afx_msg void OnCloseupVoice14ScaleCombo();
	afx_msg void OnCloseupVoice14StartnoteCombo();
	afx_msg void OnKillfocusVoice14StartnumEdit();
	afx_msg void OnVoice15MidiButton();
	afx_msg void OnVoice15ModulateChk();
	afx_msg void OnCloseupVoice15NotevalueCombo();
	afx_msg void OnCloseupVoice15NumoctsCombo();
	afx_msg void OnVoice15OnChk();
	afx_msg void OnCloseupVoice15ScaleCombo();
	afx_msg void OnCloseupVoice15StartnoteCombo();
	afx_msg void OnKillfocusVoice15StartnumEdit();
	afx_msg void OnVoice16MidiButton();
	afx_msg void OnVoice16ModulateChk();
	afx_msg void OnCloseupVoice16NotevalueCombo();
	afx_msg void OnCloseupVoice16NumoctsCombo();
	afx_msg void OnVoice16OnChk();
	afx_msg void OnCloseupVoice16ScaleCombo();
	afx_msg void OnCloseupVoice16StartnoteCombo();
	afx_msg void OnKillfocusVoice16StartnumEdit();
	afx_msg void OnVoice9MidiButton();
	afx_msg void OnVoice9ModulateChk();
	afx_msg void OnCloseupVoice9NotevalueCombo();
	afx_msg void OnCloseupVoice9NumoctsCombo();
	afx_msg void OnVoice9OnChk();
	afx_msg void OnCloseupVoice9ScaleCombo();
	afx_msg void OnCloseupVoice9StartnoteCombo();
	afx_msg void OnKillfocusVoice9StartnumEdit();
	afx_msg void OnCloseupTimesignumCmb();
	afx_msg void OnCloseupTimesigdenCmb();
	afx_msg void OnAlgorithmsVoice10Logmap();
	afx_msg void OnUpdateAlgorithmsVoice10Logmap(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice10Morsethue();
	afx_msg void OnUpdateAlgorithmsVoice10Morsethue(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice10Oneoverf();
	afx_msg void OnUpdateAlgorithmsVoice10Oneoverf(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice11Logmap();
	afx_msg void OnUpdateAlgorithmsVoice11Logmap(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice11Morsethue();
	afx_msg void OnUpdateAlgorithmsVoice11Morsethue(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice11Oneoverf();
	afx_msg void OnUpdateAlgorithmsVoice11Oneoverf(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice12Logmap();
	afx_msg void OnUpdateAlgorithmsVoice12Logmap(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice12Morsethue();
	afx_msg void OnUpdateAlgorithmsVoice12Morsethue(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice12Oneoverf();
	afx_msg void OnUpdateAlgorithmsVoice12Oneoverf(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice13Logmap();
	afx_msg void OnUpdateAlgorithmsVoice13Logmap(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice13Morsethue();
	afx_msg void OnUpdateAlgorithmsVoice13Morsethue(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice13Oneoverf();
	afx_msg void OnUpdateAlgorithmsVoice13Oneoverf(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice14Logmap();
	afx_msg void OnUpdateAlgorithmsVoice14Logmap(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice14Morsethue();
	afx_msg void OnUpdateAlgorithmsVoice14Morsethue(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice14Oneoverf();
	afx_msg void OnUpdateAlgorithmsVoice14Oneoverf(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice15Logmap();
	afx_msg void OnUpdateAlgorithmsVoice15Logmap(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice15Morsethue();
	afx_msg void OnUpdateAlgorithmsVoice15Morsethue(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice15Oneoverf();
	afx_msg void OnUpdateAlgorithmsVoice15Oneoverf(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice16Logmap();
	afx_msg void OnUpdateAlgorithmsVoice16Logmap(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice16Morsethue();
	afx_msg void OnUpdateAlgorithmsVoice16Morsethue(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice16Oneoverf();
	afx_msg void OnUpdateAlgorithmsVoice16Oneoverf(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice9Logmap();
	afx_msg void OnUpdateAlgorithmsVoice9Logmap(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice9Morsethue();
	afx_msg void OnUpdateAlgorithmsVoice9Morsethue(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice9Oneoverf();
	afx_msg void OnUpdateAlgorithmsVoice9Oneoverf(CCmdUI* pCmdUI);
	afx_msg void OnUpdatesettings();
	afx_msg void OnUpdateUpdatesettings(CCmdUI* pCmdUI);
	afx_msg void OnCompInfo();
	afx_msg void OnUpdateCompInfo(CCmdUI* pCmdUI);
	afx_msg void OnRandomize();
	afx_msg void OnUpdateRandomize(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice1Henon();
	afx_msg void OnUpdateAlgorithmsVoice1Henon(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice2Henon();
	afx_msg void OnAlgorithmsVoice3Henon();
	afx_msg void OnAlgorithmsVoice4Henon();
	afx_msg void OnAlgorithmsVoice5Henon();
	afx_msg void OnAlgorithmsVoice6Henon();
	afx_msg void OnAlgorithmsVoice7Henon();
	afx_msg void OnAlgorithmsVoice8Henon();
	afx_msg void OnAlgorithmsVoice9Henon();
	afx_msg void OnAlgorithmsVoice10Henon();
	afx_msg void OnAlgorithmsVoice11Henon();
	afx_msg void OnAlgorithmsVoice12Henon();
	afx_msg void OnAlgorithmsVoice13Henon();
	afx_msg void OnAlgorithmsVoice14Henon();
	afx_msg void OnAlgorithmsVoice15Henon();
	afx_msg void OnAlgorithmsVoice16Henon();
	afx_msg void OnUpdateAlgorithmsVoice2Henon(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice3Henon(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice4Henon(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice5Henon(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice6Henon(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice7Henon(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice8Henon(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice9Henon(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice10Henon(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice11Henon(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice12Henon(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice13Henon(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice14Henon(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice15Henon(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice16Henon(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice1Ginger();
	afx_msg void OnAlgorithmsVoice1Hopalong();
	afx_msg void OnAlgorithmsVoice1Martin();
	afx_msg void OnUpdateAlgorithmsVoice1Ginger(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice1Hopalong(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice1Martin(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice2Ginger();
	afx_msg void OnAlgorithmsVoice2Hopalong();
	afx_msg void OnAlgorithmsVoice2Martin();
	afx_msg void OnUpdateAlgorithmsVoice2Ginger(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice2Hopalong(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice2Martin(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice3Ginger();
	afx_msg void OnAlgorithmsVoice4Ginger();
	afx_msg void OnAlgorithmsVoice5Ginger();
	afx_msg void OnAlgorithmsVoice6Ginger();
	afx_msg void OnAlgorithmsVoice7Ginger();
	afx_msg void OnAlgorithmsVoice8Ginger();
	afx_msg void OnAlgorithmsVoice9Ginger();
	afx_msg void OnAlgorithmsVoice10Ginger();
	afx_msg void OnAlgorithmsVoice11Ginger();
	afx_msg void OnAlgorithmsVoice12Ginger();
	afx_msg void OnAlgorithmsVoice13Ginger();
	afx_msg void OnAlgorithmsVoice14Ginger();
	afx_msg void OnAlgorithmsVoice15Ginger();
	afx_msg void OnAlgorithmsVoice16Ginger();
	afx_msg void OnAlgorithmsVoice3Hopalong();
	afx_msg void OnAlgorithmsVoice4Hopalong();
	afx_msg void OnAlgorithmsVoice5Hopalong();
	afx_msg void OnAlgorithmsVoice6Hopalong();
	afx_msg void OnAlgorithmsVoice7Hopalong();
	afx_msg void OnAlgorithmsVoice8Hopalong();
	afx_msg void OnAlgorithmsVoice9Hopalong();
	afx_msg void OnAlgorithmsVoice10Hopalong();
	afx_msg void OnAlgorithmsVoice11Hopalong();
	afx_msg void OnAlgorithmsVoice12Hopalong();
	afx_msg void OnAlgorithmsVoice13Hopalong();
	afx_msg void OnAlgorithmsVoice14Hopalong();
	afx_msg void OnAlgorithmsVoice15Hopalong();
	afx_msg void OnAlgorithmsVoice16Hopalong();
	afx_msg void OnAlgorithmsVoice3Martin();
	afx_msg void OnAlgorithmsVoice4Martin();
	afx_msg void OnAlgorithmsVoice5Martin();
	afx_msg void OnAlgorithmsVoice6Martin();
	afx_msg void OnAlgorithmsVoice7Martin();
	afx_msg void OnAlgorithmsVoice8Martin();
	afx_msg void OnAlgorithmsVoice9Martin();
	afx_msg void OnAlgorithmsVoice10Martin();
	afx_msg void OnAlgorithmsVoice11Martin();
	afx_msg void OnAlgorithmsVoice12Martin();
	afx_msg void OnAlgorithmsVoice13Martin();
	afx_msg void OnAlgorithmsVoice14Martin();
	afx_msg void OnAlgorithmsVoice15Martin();
	afx_msg void OnAlgorithmsVoice16Martin();
	afx_msg void OnUpdateAlgorithmsVoice3Ginger(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice4Ginger(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice5Ginger(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice6Ginger(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice7Ginger(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice8Ginger(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice9Ginger(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice10Ginger(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice11Ginger(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice12Ginger(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice13Ginger(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice14Ginger(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice15Ginger(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice16Ginger(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice3Hopalong(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice4Hopalong(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice5Hopalong(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice6Hopalong(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice7Hopalong(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice8Hopalong(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice9Hopalong(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice10Hopalong(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice11Hopalong(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice12Hopalong(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice13Hopalong(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice14Hopalong(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice15Hopalong(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice16Hopalong(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice3Martin(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice4Martin(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice5Martin(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice6Martin(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice7Martin(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice8Martin(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice9Martin(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice10Martin(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice11Martin(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice12Martin(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice13Martin(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice14Martin(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice15Martin(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice16Martin(CCmdUI* pCmdUI);
	afx_msg void OnPlayfromBt();
	afx_msg void OnAlgorithmsVoice1Lorentz();
	afx_msg void OnUpdateAlgorithmsVoice1Lorentz(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice2Lorentz();
	afx_msg void OnAlgorithmsVoice3Lorentz();
	afx_msg void OnAlgorithmsVoice4Lorentz();
	afx_msg void OnAlgorithmsVoice5Lorentz();
	afx_msg void OnAlgorithmsVoice6Lorentz();
	afx_msg void OnAlgorithmsVoice7Lorentz();
	afx_msg void OnAlgorithmsVoice8Lorentz();
	afx_msg void OnAlgorithmsVoice9Lorentz();
	afx_msg void OnAlgorithmsVoice10Lorentz();
	afx_msg void OnAlgorithmsVoice11Lorentz();
	afx_msg void OnAlgorithmsVoice12Lorentz();
	afx_msg void OnAlgorithmsVoice13Lorentz();
	afx_msg void OnAlgorithmsVoice14Lorentz();
	afx_msg void OnAlgorithmsVoice15Lorentz();
	afx_msg void OnAlgorithmsVoice16Lorentz();
	afx_msg void OnUpdateAlgorithmsVoice2Lorentz(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice3Lorentz(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice4Lorentz(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice5Lorentz(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice6Lorentz(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice7Lorentz(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice8Lorentz(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice9Lorentz(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice10Lorentz(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice11Lorentz(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice12Lorentz(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice13Lorentz(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice14Lorentz(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice15Lorentz(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice16Lorentz(CCmdUI* pCmdUI);
	afx_msg void OnOptsGraph();
	afx_msg void OnUpdateOptsGraph(CCmdUI* pCmdUI);
	afx_msg void OnKillfocusStartcounterEd();
	afx_msg void OnOptsGetnote();
	afx_msg void OnUpdateOptsGetnote(CCmdUI* pCmdUI);
	afx_msg void OnGlobaltempoBt();
	afx_msg void OnGlobalChanges();
	afx_msg void OnUpdateGlobalChanges(CCmdUI* pCmdUI);
	afx_msg void OnFilePlaymidi();
	afx_msg void OnUpdateFilePlaymidi(CCmdUI* pCmdUI);
	afx_msg void OnCalculator();
	afx_msg void OnOptsPercussion();
	afx_msg void OnUpdateOptsPercussion(CCmdUI* pCmdUI);
	afx_msg void OnGlobaltimesigBt();
	afx_msg void OnTransposer();
	afx_msg void OnUpdateTransposer(CCmdUI* pCmdUI);
	afx_msg void OnChangePassword();
	afx_msg void OnUpdateChangePassword(CCmdUI* pCmdUI);
	afx_msg void OnUnlockDocument();
	afx_msg void OnUpdateUnlockDocument(CCmdUI* pCmdUI);
	afx_msg void OnRoundEvents();
	afx_msg void OnUpdateRoundEvents(CCmdUI* pCmdUI);
	afx_msg void OnInstrumentation();
	afx_msg void OnUpdateInstrumentation(CCmdUI* pCmdUI);
	afx_msg void OnComments();
	afx_msg void OnVoice1Earthworm();
	afx_msg void OnVoice2Earthworm();
	afx_msg void OnVoice3Earthworm();
	afx_msg void OnVoice4Earthworm();
	afx_msg void OnVoice5Earthworm();
	afx_msg void OnVoice6Earthworm();
	afx_msg void OnVoice7Earthworm();
	afx_msg void OnVoice8Earthworm();
	afx_msg void OnVoice9Earthworm();
	afx_msg void OnVoice10Earthworm();
	afx_msg void OnVoice11Earthworm();
	afx_msg void OnVoice12Earthworm();
	afx_msg void OnVoice13Earthworm();
	afx_msg void OnVoice14Earthworm();
	afx_msg void OnVoice15Earthworm();
	afx_msg void OnVoice16Earthworm();
	afx_msg void OnUpdateVoice1Earthworm(CCmdUI* pCmdUI);
	afx_msg void OnUpdateVoice2Earthworm(CCmdUI* pCmdUI);
	afx_msg void OnUpdateVoice3Earthworm(CCmdUI* pCmdUI);
	afx_msg void OnUpdateVoice4Earthworm(CCmdUI* pCmdUI);
	afx_msg void OnUpdateVoice5Earthworm(CCmdUI* pCmdUI);
	afx_msg void OnUpdateVoice6Earthworm(CCmdUI* pCmdUI);
	afx_msg void OnUpdateVoice7Earthworm(CCmdUI* pCmdUI);
	afx_msg void OnUpdateVoice8Earthworm(CCmdUI* pCmdUI);
	afx_msg void OnUpdateVoice9Earthworm(CCmdUI* pCmdUI);
	afx_msg void OnUpdateVoice10Earthworm(CCmdUI* pCmdUI);
	afx_msg void OnUpdateVoice11Earthworm(CCmdUI* pCmdUI);
	afx_msg void OnUpdateVoice12Earthworm(CCmdUI* pCmdUI);
	afx_msg void OnUpdateVoice13Earthworm(CCmdUI* pCmdUI);
	afx_msg void OnUpdateVoice14Earthworm(CCmdUI* pCmdUI);
	afx_msg void OnUpdateVoice15Earthworm(CCmdUI* pCmdUI);
	afx_msg void OnUpdateVoice16Earthworm(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice1Random();
	afx_msg void OnAlgorithmsVoice2Random();
	afx_msg void OnAlgorithmsVoice3Random();
	afx_msg void OnAlgorithmsVoice4Random();
	afx_msg void OnAlgorithmsVoice5Random();
	afx_msg void OnAlgorithmsVoice6Random();
	afx_msg void OnAlgorithmsVoice7Random();
	afx_msg void OnAlgorithmsVoice8Random();
	afx_msg void OnAlgorithmsVoice9Random();
	afx_msg void OnAlgorithmsVoice10Random();
	afx_msg void OnAlgorithmsVoice11Random();
	afx_msg void OnAlgorithmsVoice12Random();
	afx_msg void OnAlgorithmsVoice13Random();
	afx_msg void OnAlgorithmsVoice14Random();
	afx_msg void OnAlgorithmsVoice15Random();
	afx_msg void OnAlgorithmsVoice16Random();
	afx_msg void OnUpdateAlgorithmsVoice1Random(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice2Random(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice3Random(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice4Random(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice5Random(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice6Random(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice7Random(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice8Random(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice9Random(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice10Random(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice11Random(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice13Random(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice14Random(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice15Random(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice16Random(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice12Random(CCmdUI* pCmdUI);
	afx_msg void OnAlgorithmsVoice1Hailstone();
	afx_msg void OnAlgorithmsVoice2Hailstone();
	afx_msg void OnAlgorithmsVoice3Hailstone();
	afx_msg void OnAlgorithmsVoice4Hailstone();
	afx_msg void OnAlgorithmsVoice5Hailstone();
	afx_msg void OnAlgorithmsVoice6Hailstone();
	afx_msg void OnAlgorithmsVoice7Hailstone();
	afx_msg void OnAlgorithmsVoice8Hailstone();
	afx_msg void OnAlgorithmsVoice9Hailstone();
	afx_msg void OnAlgorithmsVoice10Hailstone();
	afx_msg void OnAlgorithmsVoice11Hailstone();
	afx_msg void OnAlgorithmsVoice12Hailstone();
	afx_msg void OnAlgorithmsVoice13Hailstone();
	afx_msg void OnAlgorithmsVoice14Hailstone();
	afx_msg void OnAlgorithmsVoice15Hailstone();
	afx_msg void OnAlgorithmsVoice16Hailstone();
	afx_msg void OnAlgorithmsVoice1Wolfram();
	afx_msg void OnAlgorithmsVoice2Wolfram();
	afx_msg void OnAlgorithmsVoice3Wolfram();
	afx_msg void OnAlgorithmsVoice4Wolfram();
	afx_msg void OnAlgorithmsVoice5Wolfram();
	afx_msg void OnAlgorithmsVoice6Wolfram();
	afx_msg void OnAlgorithmsVoice7Wolfram();
	afx_msg void OnAlgorithmsVoice8Wolfram();
	afx_msg void OnAlgorithmsVoice9Wolfram();
	afx_msg void OnAlgorithmsVoice10Wolfram();
	afx_msg void OnAlgorithmsVoice11Wolfram();
	afx_msg void OnAlgorithmsVoice12Wolfram();
	afx_msg void OnAlgorithmsVoice13Wolfram();
	afx_msg void OnAlgorithmsVoice14Wolfram();
	afx_msg void OnAlgorithmsVoice15Wolfram();
	afx_msg void OnAlgorithmsVoice16Wolfram();
	afx_msg void OnUpdateAlgorithmsVoice1Hailstone(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice2Hailstone(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice3Hailstone(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice4Hailstone(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice5Hailstone(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice6Hailstone(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice7Hailstone(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice8Hailstone(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice9Hailstone(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice10Hailstone(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice11Hailstone(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice12Hailstone(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice13Hailstone(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice14Hailstone(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice15Hailstone(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice16Hailstone(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice1Wolfram(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice2Wolfram(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice3Wolfram(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice4Wolfram(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice5Wolfram(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice6Wolfram(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice7Wolfram(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice8Wolfram(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice9Wolfram(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice10Wolfram(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice11Wolfram(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice12Wolfram(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice13Wolfram(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice14Wolfram(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice15Wolfram(CCmdUI* pCmdUI);
	afx_msg void OnUpdateAlgorithmsVoice16Wolfram(CCmdUI* pCmdUI);
	afx_msg void OnViewTips();
	afx_msg void OnUpdateViewTips(CCmdUI* pCmdUI);
	//}}AFX_MSG

	afx_msg BOOL OnToolTipNotify( UINT id, NMHDR * pNMHDR, LRESULT * pResult );
public:
	afx_msg void OnLockGencounter();
protected:
	DECLARE_MESSAGE_MAP()
};

#ifndef _DEBUG  // debug version in FractMusView.cpp
inline CFractMusDoc* CFractMusView::GetDocument()
   { return (CFractMusDoc*)m_pDocument; }
#endif

#endif

/////////////////////////////////////////////////////////////////////////////
