
/*************************************************************************


Copyright 1998, 1999, 2000 Gustavo Diaz-Jerez


This file is part of FractMus.

    FractMus is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FractMus is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FractMus; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


**************************************************************************/


// MartinDlg.cpp : implementation file
//

#include "stdafx.h"
#include "FractMus.h"
#include "MartinDlg.h"
#include "MartinGraph.h"
#include "FractMusView.h"
#include "FractMusDoc.h"
#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


UINT ShowMartin(LPVOID pObj);
extern start;
extern running;
extern int count6[MAXVOICES];
extern WORD noteOn(BYTE , BYTE , BYTE );
extern WORD noteOff(BYTE, BYTE, BYTE);
extern WORD progChange(BYTE , BYTE);
extern WORD modeChange(BYTE , BYTE , BYTE);
extern HMIDIOUT midiOut;
extern int DevChoice;
extern int PercussionChannel;
extern float MartinX[MAXVOICES];
extern float MartinY[MAXVOICES];



extern int __fastcall Martin(int voice, long counter, long test, const int* scale, 
				int numocts,long base, long startnum, 
				int startnote,BOOL modulate, BOOL modulrand, 
				int modulnum, BOOL inverted, float logmapvar, float oneoverf,
				float henonA, float henonB, float hopalong,
				float martin, float lorentz, unsigned long earthwormvar,
				unsigned long earthwormmulti, unsigned earthwormtruncator,
				double ranmin, double ranmax, BYTE wolframrule,
				int wolframsize, long hailstonestart);


/////////////////////////////////////////////////////////////////////////////
// CMartinDlg dialog


CMartinDlg::CMartinDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CMartinDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CMartinDlg)
	m_angle = 0.0f;
	//}}AFX_DATA_INIT

	m_pGraph = NULL;
	start = TRUE;

}


void CMartinDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMartinDlg)
	DDX_Text(pDX, IDC_ANGLE_ED, m_angle);
	DDV_MinMaxFloat(pDX, m_angle, -6.28319f, 6.28319f);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CMartinDlg, CDialog)
	//{{AFX_MSG_MAP(CMartinDlg)
	ON_BN_CLICKED(IDC_VISUALIZE, OnVisualize)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMartinDlg message handlers

void CMartinDlg::OnVisualize() 
{
	if (!UpdateData())
		return;

	running = !running;

	if (m_pGraph!=NULL)
	{
		m_pGraph->SetForegroundWindow();
		if (running)
		{
			GetDlgItem(IDC_VISUALIZE)->SetWindowText("Stop");
			AfxBeginThread(ShowMartin, this);
		}


		if (!running)
			GetDlgItem(IDC_VISUALIZE)->SetWindowText("Run");

		return;
	}

	start = TRUE;

	int maxX = 600 + ::GetSystemMetrics(SM_CXSIZEFRAME)*2;
	int maxY = 310 + ::GetSystemMetrics(SM_CYCAPTION)+::GetSystemMetrics(SM_CXSIZEFRAME);


	CClientDC dc(m_pGraph);
	m_pGraph = new CMartinGraph;
	CRect r(0,0,maxX,maxY);
	/*LOGBRUSH lb;

	lb.lbStyle = BS_SOLID;
	lb.lbColor = RGB(0,0,0);
	lb.lbHatch = HS_CROSS;*/


	m_pGraph->m_memDC.CreateCompatibleDC(&dc);
	m_pGraph->m_bmp.CreateCompatibleBitmap(&dc, 600, 310);
	m_pGraph->m_memDC.SelectObject(&(m_pGraph->m_bmp));
	m_pGraph->m_bkbrush.CreateSolidBrush(RGB(0,0,0));
	m_pGraph->m_memDC.SelectObject(&(m_pGraph->m_bkbrush));
	m_pGraph->m_Pen.CreatePen(PS_SOLID, 0, RGB(255,0,0));
	m_pGraph->m_memDC.SelectObject(&(m_pGraph->m_Pen));
	m_pGraph->m_memDC.PatBlt(0,0,600,310,PATCOPY);
	m_pGraph->m_pParent = this;
	m_pGraph->Create(	NULL, "",
						WS_CAPTION|WS_SYSMENU|WS_VISIBLE, r
						   );



	GetDlgItem(IDOK)->EnableWindow(FALSE);
	GetDlgItem(IDCANCEL)->EnableWindow(FALSE);

	running = TRUE;
	GetDlgItem(IDC_VISUALIZE)->SetWindowText("Stop");
	

	AfxBeginThread(ShowMartin, this);
	
}

void CMartinDlg::OnOK() 
{
	if(!UpdateData(TRUE))
		return;
	
	CDialog::OnOK();
}

BOOL CMartinDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	SetWindowText(title);
	
	return TRUE;  // return TRUE unless you set the focus to a control
}



UINT ShowMartin(LPVOID pObj)
{
	CMartinDlg* Wp = (CMartinDlg*)pObj;
	CFractMusDoc* pDoc= Wp->pView->GetDocument();

	static counter=304;
	static long count;
	static dracula;
	int result=0;
	int lastnote =0, note =0;
	float x = 0.0, y = 0.0, tempx, tempy, a = Wp->m_angle;
	int cx, cy;
	float  min = 1000000.0, max = -1000000.0;


	UINT res = midiOutOpen(&midiOut, DevChoice, NULL, NULL, CALLBACK_NULL);

	if( res!=MMSYSERR_NOERROR )
	{
		switch(result)
		{

		case MMSYSERR_ALLOCATED:
			AfxMessageBox("The device is already in use, free it and try again", MB_OK+MB_ICONHAND);
			break;
		case MMSYSERR_NOMEM:
			AfxMessageBox("Could not allocate memory for the device", MB_OK+MB_ICONHAND);
			break;
		case MMSYSERR_INVALPARAM:
			AfxMessageBox("Invalid pointer!", MB_OK+MB_ICONHAND);
			break;
		}

	}

		if( res!=MMSYSERR_NOERROR )
			return 0;

		progChange(0xC0+pDoc->m_Channel[Wp->voicenum], (BYTE)pDoc->m_Instrument[Wp->voicenum]);


	if (start)
	{
		counter =304;
		count6[Wp->voicenum]=0;
		count = pDoc->dStartCounter;
		dracula = int(pDoc->m_NoteValue[Wp->voicenum]);

		if (::IsWindow(Wp->m_pGraph->m_hWnd))
		{
			Wp->m_pGraph->m_memDC.FillSolidRect(0, 0, 300, 310, RGB(224,224,224));
			Wp->m_pGraph->m_memDC.FillSolidRect(301, 0, 303, 310, RGB(0,0,255));
			Wp->m_pGraph->m_memDC.FillSolidRect(304, 0, 600, 310, RGB(0,0,0));
			Wp->m_pGraph->InvalidateRect(NULL);
		}


		for (int h = 0; h<1000; h++)
		{

			tempx = y - sin(x);
			tempy = a - x;

			if (tempx<min)
				min = tempx;
			if (tempx>max)
				max = tempx;
			x = tempx;
			y = tempy;

		}


		x = 0.0f, y = 0.0f;

		for (h = 0; h<5000; h++)
		{
			tempx = y - sin(x);
			tempy = a - x;

			//cx = int( 103.8 + (tempx*23.0769) );
			//cy = int( 196.2 - (tempy*23.0769) );

			cx = int( 150 + ( tempx* (300/40 /*( fabs(max)+fabs(min) )*/ ) ) );
			cy = int( 150 - ( tempy* (300/40 /*( fabs(max)+fabs(min) )*/ ) ) );

			if ((cx>=0)&&(cx<300)&&(cy>=0)&&(cy<300))
			{
				if (::IsWindow(Wp->m_pGraph->m_hWnd))
					Wp->m_pGraph->m_memDC.SetPixel(cx,cy,RGB(0,0,0));
			}

			x = tempx;
			y = tempy;


		}

			MartinX[Wp->voicenum] = MartinX[Wp->voicenum] = 0.0f;

			for(int j = 0; j<count+(pDoc->m_StartNum[Wp->voicenum]*dracula); j++)
			{
				if(!((pDoc->m_StartNum[Wp->voicenum]*dracula+j)%dracula))
				{


				result = Martin(Wp->voicenum, (pDoc->m_StartNum[Wp->voicenum]*dracula+j)/dracula, 0,
									pDoc->m_pScale[Wp->voicenum], 
									pDoc->m_NumOcts[Wp->voicenum], 
									0, 
									pDoc->m_StartNum[Wp->voicenum],
									pDoc->m_Note[Wp->voicenum], 
									pDoc->m_Modulates[Wp->voicenum],
									pDoc->m_ModulRandom[Wp->voicenum], 
									pDoc->m_ModulNum[Wp->voicenum],
									pDoc->m_Inverted[Wp->voicenum],
									0,
									0,
									0,
									0,
									0,
									Wp->m_angle,
									0,
									0,
									0,
									0,
									0,
									0,
									0,
									0,
									0
									
					);
				}
			}



		if (::IsWindow(Wp->m_pGraph->m_hWnd))
				Wp->m_pGraph->m_memDC.MoveTo(304,150);
	}

	while (running)
	{
			if((pDoc->m_StartNum[Wp->voicenum]*dracula+count)>=0)
			{
			if(!((pDoc->m_StartNum[Wp->voicenum]*dracula+count)%dracula))
			{
				result = Martin(Wp->voicenum, (pDoc->m_StartNum[Wp->voicenum]*dracula+count)/dracula, 0,
									pDoc->m_pScale[Wp->voicenum], 
									pDoc->m_NumOcts[Wp->voicenum], 
									0, 
									pDoc->m_StartNum[Wp->voicenum],
									pDoc->m_Note[Wp->voicenum], 
									pDoc->m_Modulates[Wp->voicenum],
									pDoc->m_ModulRandom[Wp->voicenum], 
									pDoc->m_ModulNum[Wp->voicenum],
									pDoc->m_Inverted[Wp->voicenum],
									0,
									0,
									0,
									0,
									0,
									Wp->m_angle,
									0,
									0,
									0,
									0,
									0,
									0,
									0,
									0,
									0
									
					);

				if (result < 0)
					result =0;
				if (result>127)
					result = 127;

				note = result;

				result -= pDoc->m_Note[Wp->voicenum];

				COLORREF color = RGB(255,128,64);

				if (::IsWindow(Wp->m_pGraph->m_hWnd))
					Wp->m_pGraph->m_memDC.LineTo(counter, 150-(result*2));


				if ((note!=lastnote)||pDoc->m_mode[Wp->voicenum]==0)
				{
					if (pDoc->m_Channel[Wp->voicenum] == PercussionChannel)
						noteOff(0x80+pDoc->m_Channel[Wp->voicenum], (BYTE)pDoc->m_Percussion[Wp->voicenum]+35, (BYTE)0);
					else
						noteOff(0x80+pDoc->m_Channel[Wp->voicenum], (BYTE)lastnote, (BYTE)0);

					if (pDoc->m_Channel[Wp->voicenum] == PercussionChannel)
						noteOn(0x90+pDoc->m_Channel[Wp->voicenum], (BYTE)pDoc->m_Percussion[Wp->voicenum]+35, (BYTE)(pDoc->m_Dynamic[Wp->voicenum]/*pDoc->m_IsActive[i]*/));
					else
						noteOn(0x90+pDoc->m_Channel[Wp->voicenum], (BYTE)note, (BYTE)(pDoc->m_Dynamic[Wp->voicenum]/*pDoc->m_IsActive[i]*/));

					lastnote=note;

				}




				if (::IsWindow(Wp->m_pGraph->m_hWnd))
					Wp->m_pGraph->RedrawWindow(NULL,NULL,RDW_INVALIDATE|RDW_UPDATENOW|RDW_NOERASE);	
				counter += 3;
				Sleep(50);
				if (counter>599)
				{
					if (::IsWindow(Wp->m_pGraph->m_hWnd))
						Wp->m_pGraph->m_memDC.FillSolidRect(304, 0, 600, 310, RGB(0,0,0));
					counter =304;
					if (::IsWindow(Wp->m_pGraph->m_hWnd))
						Wp->m_pGraph->m_memDC.MoveTo(304,150-(result*2));
				}
			}
			}
			count++;

	}


	start = FALSE;

	midiOutReset(midiOut);
	midiOutClose(midiOut);


	return 0;



}
