
/*************************************************************************


Copyright 1998, 1999, 2000 Gustavo Diaz-Jerez


This file is part of FractMus.

    FractMus is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FractMus is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FractMus; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


**************************************************************************/


// RandDlg.cpp : implementation file
//

#include "stdafx.h"
#include "FractMus.h"
#include "FractMusView.h"
#include "FractMusDoc.h"
#include "RandDlg.h"
#include "P1RangeDlg.h"
#include "InstrumentsDlg.h"
#include "PercussionInstrumentDlg.h"
#include "StartNoteDlg.h"
#include "V1UsrSclDlg.h"

#include "math.h"

#include "defs.h"

extern int PercussionChannel;
extern const char* InstrumentNames[128];
extern const char* PercussionNames[47];



#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CRandDlg dialog

extern float RanDev();
extern int Tempo;




CRandDlg::CRandDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CRandDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CRandDlg)
	m_exact = 6;
	m_atmost = 12;
	m_exactORatmost = 1;
	//}}AFX_DATA_INIT

	for (int i=VOICE1; i<MAXVOICES; i++)
		m_VoiceOn[i] = FALSE;

	m_P1From = 2;
	m_P1To = 100;
	m_P2From = 1;
	m_P2To = 100;

	m_algorithm = TRUE;
	m_dynamic = TRUE;
	m_instrument = TRUE;
	m_AllActiveVoices = TRUE;
	m_percussion = TRUE;
	m_inverted = TRUE;
	m_mode = TRUE;
	m_modulate = TRUE;
	m_note = TRUE;
	m_noteval = TRUE;
	m_octaves = TRUE;
	m_pan = TRUE;
	m_param1 = TRUE;
	m_param2 = TRUE;
	m_scale = TRUE;
	m_tempo = TRUE;
	m_numberOfEvents = 10;
	m_counterMin = 50;
	m_counterMax = 200;
	m_allowSilence = FALSE;
	m_coeff = TRUE;
	m_onoff = FALSE;
	m_userscl = TRUE;
	m_onlyusrscl = FALSE;
	m_matchscl = TRUE;
	m_minOnVoices = 0;
	m_CountersChoice = 0;
	m_CountersRandomMin = 0;
	m_CountersRandomMax = 1000;
	m_sequential = TRUE;
	m_Composition = FALSE;

	IsSelected = NULL;

}


CRandDlg::~CRandDlg()
{
	if (IsSelected)
	{
		delete IsSelected;
		IsSelected = NULL;
	}

}


void CRandDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CRandDlg)
	DDX_Text(pDX, IDC_EXACT_ED, m_exact);
	DDV_MinMaxInt(pDX, m_exact, 1, 12);
	DDX_Text(pDX, IDC_ATMOST_ED, m_atmost);
	DDV_MinMaxInt(pDX, m_atmost, 1, 12);
	DDX_Radio(pDX, IDC_EXACT_RD, m_exactORatmost);
	//}}AFX_DATA_MAP

	DDX_Check(pDX, IDC_ALLACTIVEVOICES_CHK, m_AllActiveVoices);
	DDX_Check(pDX, IDC_ALGORITHM_CHK, m_algorithm);
	DDX_Check(pDX, IDC_DYNAMIC_CHK, m_dynamic);
	DDX_Check(pDX, IDC_INSTRUMENT_CHK, m_instrument);
	DDX_Check(pDX, IDC_PERCUSSION_CHK, m_percussion);
	DDX_Check(pDX, IDC_INVERTED_CHK, m_inverted);
	DDX_Check(pDX, IDC_MODE_CHK, m_mode);
	DDX_Check(pDX, IDC_MODULATE_CHK, m_modulate);
	DDX_Check(pDX, IDC_NOTE_CHK, m_note);
	DDX_Check(pDX, IDC_NOTEVAL_CHK, m_noteval);
	DDX_Check(pDX, IDC_OCTAVES_CHK, m_octaves);
	DDX_Check(pDX, IDC_PAN_CHK, m_pan);
	DDX_Check(pDX, IDC_PARAM1_CHK, m_param1);
	DDX_Check(pDX, IDC_PARAM2_CHK, m_param2);
	DDX_Check(pDX, IDC_SCALE_CHK, m_scale);
	DDX_Check(pDX, IDC_TEMPO_CHK, m_tempo);
	DDX_Text(pDX, IDC_EVENTS_NUMBER, m_numberOfEvents);
	DDX_Text(pDX, IDC_COUNTER_MIN, m_counterMin);
	DDX_Text(pDX, IDC_COUNTER_MAX, m_counterMax);
	DDX_Check(pDX, IDC_NOSILENCE, m_allowSilence);
	DDX_Check(pDX, IDC_COEFF_CHK, m_coeff);
	DDX_Check(pDX, IDC_ONOFF_CHK, m_onoff);
	DDX_Check(pDX, IDC_USERSCL, m_userscl);
	DDX_Check(pDX, IDC_ONLYUSERSCL, m_onlyusrscl);
	DDX_Check(pDX, IDC_MATCHSCL_CHK, m_matchscl);
	DDX_Text(pDX, IDC_NUMVOICES_ED, m_minOnVoices);
	DDX_Radio(pDX, IDC_APPEND_RD, m_CountersChoice);
	DDX_Text(pDX, IDC_RANDMIN_ED, m_CountersRandomMin);
	DDX_Text(pDX, IDC_RANDMAX_ED, m_CountersRandomMax);
	DDX_Check(pDX, IDC_RANDSEQ, m_sequential);

	DDX_Check(pDX, IDC_V1_CHK, m_VoiceOn[VOICE1]);
	DDX_Check(pDX, IDC_V2_CHK, m_VoiceOn[VOICE2]);
	DDX_Check(pDX, IDC_V3_CHK, m_VoiceOn[VOICE3]);
	DDX_Check(pDX, IDC_V4_CHK, m_VoiceOn[VOICE4]);
	DDX_Check(pDX, IDC_V5_CHK, m_VoiceOn[VOICE5]);
	DDX_Check(pDX, IDC_V6_CHK, m_VoiceOn[VOICE6]);
	DDX_Check(pDX, IDC_V7_CHK, m_VoiceOn[VOICE7]);
	DDX_Check(pDX, IDC_V8_CHK, m_VoiceOn[VOICE8]);
	DDX_Check(pDX, IDC_V9_CHK, m_VoiceOn[VOICE9]);
	DDX_Check(pDX, IDC_V10_CHK, m_VoiceOn[VOICE10]);
	DDX_Check(pDX, IDC_V11_CHK, m_VoiceOn[VOICE11]);
	DDX_Check(pDX, IDC_V12_CHK, m_VoiceOn[VOICE12]);
	DDX_Check(pDX, IDC_V13_CHK, m_VoiceOn[VOICE13]);
	DDX_Check(pDX, IDC_V14_CHK, m_VoiceOn[VOICE14]);
	DDX_Check(pDX, IDC_V15_CHK, m_VoiceOn[VOICE15]);
	DDX_Check(pDX, IDC_V16_CHK, m_VoiceOn[VOICE16]);

	m_VoiceID[VOICE1] = IDC_V1_CHK;
	m_VoiceID[VOICE2] = IDC_V2_CHK;
	m_VoiceID[VOICE3] = IDC_V3_CHK;
	m_VoiceID[VOICE4] = IDC_V4_CHK;
	m_VoiceID[VOICE5] = IDC_V5_CHK;
	m_VoiceID[VOICE6] = IDC_V6_CHK;
	m_VoiceID[VOICE7] = IDC_V7_CHK;
	m_VoiceID[VOICE8] = IDC_V8_CHK;
	m_VoiceID[VOICE9] = IDC_V9_CHK;
	m_VoiceID[VOICE10] = IDC_V10_CHK;
	m_VoiceID[VOICE11] = IDC_V11_CHK;
	m_VoiceID[VOICE12] = IDC_V12_CHK;
	m_VoiceID[VOICE13] = IDC_V13_CHK;
	m_VoiceID[VOICE14] = IDC_V14_CHK;
	m_VoiceID[VOICE15] = IDC_V15_CHK;
	m_VoiceID[VOICE16] = IDC_V16_CHK;


}


BEGIN_MESSAGE_MAP(CRandDlg, CDialog)
	//{{AFX_MSG_MAP(CRandDlg)
	ON_BN_CLICKED(IDC_RAND, OnRand)
	ON_BN_CLICKED(IDC_PARAM1_CHK, OnParam1Chk)
	ON_BN_CLICKED(IDC_PARAM2_CHK, OnParam2Chk)
	ON_BN_CLICKED(IDC_SELECTALL, OnSelectall)
	ON_BN_CLICKED(IDC_SELECTNONE, OnSelectnone)
	ON_EN_KILLFOCUS(IDC_COUNTER_MAX, OnKillfocusCounterMax)
	ON_EN_KILLFOCUS(IDC_COUNTER_MIN, OnKillfocusCounterMin)
	ON_EN_KILLFOCUS(IDC_EVENTS_NUMBER, OnKillfocusEventsNumber)
	ON_BN_CLICKED(IDC_CREATE_RANDOM_COMPOSITION, OnCreateRandomComposition)
	ON_BN_CLICKED(IDC_NOTE_CHK, OnNoteChk)
	ON_BN_CLICKED(IDC_ONOFF_CHK, OnOnoffChk)
	ON_BN_CLICKED(IDC_SCALE_CHK, OnScaleChk)
	ON_BN_CLICKED(IDC_USERSCL, OnUserscl)
	ON_BN_CLICKED(IDC_INVERTED_CHK, OnInvertedChk)
	ON_BN_CLICKED(IDC_NOSILENCE, OnNosilence)
	ON_BN_CLICKED(IDC_ALGORITHM_CHK, OnAlgorithmChk)
	ON_BN_CLICKED(IDC_NOTEVAL_CHK, OnNotevalChk)
	ON_BN_CLICKED(IDC_INSTRUMENT_CHK, OnInstrumentChk)
	ON_BN_CLICKED(IDC_OCTAVES_CHK, OnOctavesChk)
	ON_BN_CLICKED(IDC_DYNAMIC_CHK, OnDynamicChk)
	ON_BN_CLICKED(IDC_RANDOMIZESELECTEDEVENTS, OnRandomizeselectedevents)
	ON_BN_CLICKED(IDC_SELECTALLEVENTS, OnSelectallevents)
	ON_BN_CLICKED(IDC_SELECTNONEEVENTS, OnSelectnoneevents)
	ON_BN_CLICKED(IDC_ALLACTIVEVOICES_CHK, OnAllactivevoicesChk)
	ON_BN_CLICKED(IDC_PERCUSSION_CHK, OnPercussionChk)
	ON_BN_CLICKED(IDC_PRESETS, OnPresets)
	ON_BN_CLICKED(IDC_ATMOST_RD, OnAtmostRd)
	ON_BN_CLICKED(IDC_EXACT_RD, OnExactRd)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRandDlg message handlers

void CRandDlg::OnOK() 
{
	
}

BOOL CRandDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

	for(int i = VOICE1; i<MAXVOICES; i++)
		m_VoiceOn[i] = m_Parent->m_IsActive[i];

	m_minOnVoices = 1;
	m_AllActiveVoices = FALSE;

	UpdateData(FALSE);

	m_Composition = FALSE;

	ScalesChoice.pRand = this;
	RandomizerParameters.pRand = this;
	RandomizerParameters.WindowTitle = "Randomizer Presets";

	CFractMusDoc* pDoc = m_Parent->GetDocument();

	CListBox* pList = (CListBox*)GetDlgItem(IDC_EVENTS_LIST);

	if (pDoc->m_NumEvents>-1)
	{

		for (int i = 0; i<=pDoc->m_NumEvents; i++)
		{
			char s[55];

			itoa(i, s, 10);

			pList->AddString(s);
		}

		if(IsSelected)
		{
			delete IsSelected;
			IsSelected = NULL;
		}

		IsSelected = new BOOL [pDoc->m_NumEvents + 1];

		for (i=0; i<=pDoc->m_NumEvents; i++)
			IsSelected[i] = FALSE;
	}


	
	GetDlgItem(IDC_NOSILENCE)->EnableWindow(m_onoff);
	GetDlgItem(IDC_NUMVOICES_ED)->EnableWindow(m_onoff&&(!m_allowSilence));
	GetDlgItem(IDC_NUMVOICES_ST)->EnableWindow(m_onoff&&(!m_allowSilence));
	GetDlgItem(IDC_USERSCL)->EnableWindow(m_scale&&ScalesChoice.m_user);
	GetDlgItem(IDC_ONLYUSERSCL)->EnableWindow(m_scale&&m_userscl);
	GetDlgItem(IDC_EXACT_RD)->EnableWindow(m_scale&&ScalesChoice.m_user&&m_userscl);
	GetDlgItem(IDC_EXACT_ED)->EnableWindow(m_scale&&ScalesChoice.m_user&&(m_exactORatmost == 0)&&m_userscl);
	GetDlgItem(IDC_EXACT_ST)->EnableWindow(m_scale&&ScalesChoice.m_user&&(m_exactORatmost == 0)&&m_userscl);
	GetDlgItem(IDC_ATMOST_RD)->EnableWindow(m_scale&&ScalesChoice.m_user&&m_userscl);
	GetDlgItem(IDC_ATMOST_ED)->EnableWindow(m_scale&&ScalesChoice.m_user&&(m_exactORatmost == 1)&&m_userscl);
	GetDlgItem(IDC_ATMOST_ST)->EnableWindow(m_scale&&ScalesChoice.m_user&&(m_exactORatmost == 1)&&m_userscl);
	GetDlgItem(IDC_MATCHSCL_CHK)->EnableWindow(m_inverted);
	GetDlgItem(IDC_SELECTALLEVENTS)->EnableWindow(pDoc->m_NumEvents>-1);
	GetDlgItem(IDC_SELECTNONEEVENTS)->EnableWindow(pDoc->m_NumEvents>-1);
	GetDlgItem(IDC_RANDOMIZESELECTEDEVENTS)->EnableWindow(pDoc->m_NumEvents>-1);
	GetDlgItem(IDC_ALLACTIVEVOICES_CHK)->EnableWindow(pDoc->m_NumEvents>-1);

	GetDlgItem(IDC_EVENTNUM_ST)->SetWindowText("- - -");
	GetDlgItem(IDC_CREATEEVENT_ST)->SetWindowText("- - -");


	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CRandDlg::OnRand() 
{
	CFractMusDoc* pDoc = m_Parent->GetDocument();
	CComboBox* pB;

	UpdateData(TRUE);

	for(int i = VOICE1; i<MAXVOICES; i++)
	{
		if (m_VoiceOn[i])
		{
			if (m_algorithm)
			{
				int al;

				do
				{
					al = int(RanDev()*12.0f);
				}
				while(	((!Algorithms.m_morsethue)&&(al == MORSE_THUE))||
						((!Algorithms.m_logmap)&&(al == LOGMAP))||
						((!Algorithms.m_noise)&&(al == ONE_OVER_F_NOISE))||
						((!Algorithms.m_henon)&&(al == HENON))||
						((!Algorithms.m_hopalong)&&(al == HOPALONG))||
						((!Algorithms.m_martin)&&(al == MARTIN))||
						((!Algorithms.m_ginger)&&(al == GINGERBREADMAN))||
						((!Algorithms.m_lorentz)&&(al == LORENTZ))||
						((!Algorithms.m_earthworm)&&(al == EARTHWORM))||
						((!Algorithms.m_random)&&(al == RANDOM))||
						((!Algorithms.m_hailstone)&&(al == HAILSTONE))||
						((!Algorithms.m_wolfram)&&(al == WOLFRAM))
					 );

				pDoc->AlgorithmNum[i] = al;

			}

			if ((m_param1)&&(pDoc->AlgorithmNum[i]==MORSE_THUE))
			{
				long base = long((RanDev()*(m_P1To-m_P1From))+m_P1From);
				pDoc->m_Base[i] = base;
			}

			if ((m_param2)&&(pDoc->AlgorithmNum[i]==MORSE_THUE))
			{
				long multi = long((RanDev()*(m_P2To-m_P2From))+m_P2From);
				pDoc->m_Multi[i] = multi;
			}

			if ((m_coeff)&&(pDoc->AlgorithmNum[i]==LOGMAP))
				pDoc->m_LogMapVar[i] = (RanDev()*0.5f)+3.5f;

			if ((m_coeff)&&(pDoc->AlgorithmNum[i]==ONE_OVER_F_NOISE))
				pDoc->m_FNoiseVar[i] = RanDev();

			if ((m_coeff)&&(pDoc->AlgorithmNum[i]==HENON))
			{
				pDoc->m_HenonA[i] = RanDev()+1.0f;
				pDoc->m_HenonB[i] = RanDev();
			}

			if ((m_coeff)&&(pDoc->AlgorithmNum[i]==HOPALONG))
			{
				pDoc->m_HopalongVar[i] = (RanDev()* 12.56637061436f)- 6.28318530718f;
			}

			if ((m_coeff)&&(pDoc->AlgorithmNum[i]==MARTIN))
			{
				pDoc->m_MartinVar[i] = (RanDev()* 12.56637061436f)- 6.28318530718f;
			}

			if ((m_coeff)&&(pDoc->AlgorithmNum[i]==GINGERBREADMAN))
			{
				pDoc->m_GingerX[i] = (RanDev()*40.0f)-20.0f;
				pDoc->m_GingerY[i] = (RanDev()*40.0f)-20.0f;
			}

			if ((m_coeff)&&(pDoc->AlgorithmNum[i]==LORENTZ))
			{
				pDoc->m_LorentzVar[i] = (RanDev()* 0.3f)+ 0.7f;
			}

			if ((m_coeff)&&(pDoc->AlgorithmNum[i]==EARTHWORM))
			{
				pDoc->m_EarthWormMulti[i] = unsigned long((RanDev()* 999997.0f)+ 2.0f);
				pDoc->m_EarthWormVar[i] = unsigned long((RanDev()* 999997.0f)+ 2.0f);
				pDoc->m_EarthWormTruncator[i] = unsigned((RanDev()* 9.0f)+ 1.0f);
			}
			if ((m_coeff)&&(pDoc->AlgorithmNum[i]==RANDOM))
			{
				pDoc->m_RandomMin[i] = (RanDev()* 0.5f);
				pDoc->m_RandomMax[i] = (RanDev()* 0.5f)+ 0.5f;
			}
			if ((m_coeff)&&(pDoc->AlgorithmNum[i]==HAILSTONE))
			{
				pDoc->m_Hailstone[i] = (long)(RanDev()* 9999999.0f)+ 1.0f;
			}
			if ((m_coeff)&&(pDoc->AlgorithmNum[i]==WOLFRAM))
			{
				pDoc->m_WolframRule[i] = BYTE((RanDev()* 254.0f)+ 1.0f);
				pDoc->m_WolframInit[i] = unsigned long((RanDev()* 999999999.0f)+ 1.0f);
				pDoc->m_WolframSize[i] = int((RanDev()* 384.0f)+ 128.0f);
			}

			if (m_modulate)
			{
				if (RanDev()>0.5f)
				{
					pDoc->m_Modulates[i] = m_Parent->m_Modulates[i] = TRUE;


					if (RanDev()<=0.5)
					{
						pDoc->m_ModulRandom[i] = FALSE;
						pDoc->m_ModulNum[i] = int((RanDev()*49)+1);
					}
					else
						pDoc->m_ModulRandom[i] = TRUE;
				}
				else
					pDoc->m_Modulates[i] = m_Parent->m_Modulates[i] = FALSE;
			}

			if (m_octaves)
			{
				
				int oct;

				do
				{
					oct = int(RanDev()*4.0f);
				}
				while(	((!Octaves.m_one)&&(oct == ONE_OCT))||
						((!Octaves.m_two)&&(oct == TWO_OCT))||
						((!Octaves.m_three)&&(oct == THREE_OCT))||
						((!Octaves.m_four)&&(oct == FOUR_OCT))
					);

				pDoc->m_NumOcts[i] = oct+1;
				pDoc->m_NumOctsID[i] = m_Parent->m_NumOctsID[i] = oct;

				if(!m_Composition)
				{
					pB = (CComboBox*)m_Parent->GetDlgItem(m_Parent->NumOctsID[i]);	
					pB->SetCurSel(oct);
				}

				
			}

			if (m_note)
			{
				int note;

				do
				{
					note = int(RanDev()*97.0f);
				}
				while (	((!Notes[0])&&(note == 0))||
						((!Notes[1])&&(note == 1))||
						((!Notes[2])&&(note == 2))||
						((!Notes[3])&&(note == 3))||
						((!Notes[4])&&(note == 4))||
						((!Notes[5])&&(note == 5))||
						((!Notes[6])&&(note == 6))||
						((!Notes[7])&&(note == 7))||
						((!Notes[8])&&(note == 8))||
						((!Notes[9])&&(note == 9))||
						((!Notes[10])&&(note == 10))||
						((!Notes[11])&&(note == 11))||
						((!Notes[12])&&(note == 12))||
						((!Notes[13])&&(note == 13))||
						((!Notes[14])&&(note == 14))||
						((!Notes[15])&&(note == 15))||
						((!Notes[16])&&(note == 16))||
						((!Notes[17])&&(note == 17))||
						((!Notes[18])&&(note == 18))||
						((!Notes[19])&&(note == 19))||
						((!Notes[20])&&(note == 20))||
						((!Notes[21])&&(note == 21))||
						((!Notes[22])&&(note == 22))||
						((!Notes[23])&&(note == 23))||
						((!Notes[24])&&(note == 24))||
						((!Notes[25])&&(note == 25))||
						((!Notes[26])&&(note == 26))||
						((!Notes[27])&&(note == 27))||
						((!Notes[28])&&(note == 28))||
						((!Notes[29])&&(note == 29))||
						((!Notes[30])&&(note == 30))||
						((!Notes[31])&&(note == 31))||
						((!Notes[32])&&(note == 32))||
						((!Notes[33])&&(note == 33))||
						((!Notes[34])&&(note == 34))||
						((!Notes[35])&&(note == 35))||
						((!Notes[36])&&(note == 36))||
						((!Notes[37])&&(note == 37))||
						((!Notes[38])&&(note == 38))||
						((!Notes[39])&&(note == 39))||
						((!Notes[40])&&(note == 40))||
						((!Notes[41])&&(note == 41))||
						((!Notes[42])&&(note == 42))||
						((!Notes[43])&&(note == 43))||
						((!Notes[44])&&(note == 44))||
						((!Notes[45])&&(note == 45))||
						((!Notes[46])&&(note == 46))||
						((!Notes[47])&&(note == 47))||
						((!Notes[48])&&(note == 48))||
						((!Notes[49])&&(note == 49))||
						((!Notes[50])&&(note == 50))||
						((!Notes[51])&&(note == 51))||
						((!Notes[52])&&(note == 52))||
						((!Notes[53])&&(note == 53))||
						((!Notes[54])&&(note == 54))||
						((!Notes[55])&&(note == 55))||
						((!Notes[56])&&(note == 56))||
						((!Notes[57])&&(note == 57))||
						((!Notes[58])&&(note == 58))||
						((!Notes[59])&&(note == 59))||
						((!Notes[60])&&(note == 60))||
						((!Notes[61])&&(note == 61))||
						((!Notes[62])&&(note == 62))||
						((!Notes[63])&&(note == 63))||
						((!Notes[64])&&(note == 64))||
						((!Notes[65])&&(note == 65))||
						((!Notes[66])&&(note == 66))||
						((!Notes[67])&&(note == 67))||
						((!Notes[68])&&(note == 68))||
						((!Notes[69])&&(note == 69))||
						((!Notes[70])&&(note == 70))||
						((!Notes[71])&&(note == 71))||
						((!Notes[72])&&(note == 72))||
						((!Notes[73])&&(note == 73))||
						((!Notes[74])&&(note == 74))||
						((!Notes[75])&&(note == 75))||
						((!Notes[76])&&(note == 76))||
						((!Notes[77])&&(note == 77))||
						((!Notes[78])&&(note == 78))||
						((!Notes[79])&&(note == 79))||
						((!Notes[80])&&(note == 80))||
						((!Notes[81])&&(note == 81))||
						((!Notes[82])&&(note == 82))||
						((!Notes[83])&&(note == 83))||
						((!Notes[84])&&(note == 84))||
						((!Notes[85])&&(note == 85))||
						((!Notes[86])&&(note == 86))||
						((!Notes[87])&&(note == 87))||
						((!Notes[88])&&(note == 88))||
						((!Notes[89])&&(note == 89))||
						((!Notes[90])&&(note == 90))||
						((!Notes[91])&&(note == 91))||
						((!Notes[92])&&(note == 92))||
						((!Notes[93])&&(note == 93))||
						((!Notes[94])&&(note == 94))||
						((!Notes[95])&&(note == 95))||
						((!Notes[96])&&(note == 96))
					);


				pDoc->m_Note[i] = m_Parent->m_Note[i] = note;

				if(!m_Composition)
				{
					pB = (CComboBox*)m_Parent->GetDlgItem(m_Parent->NoteID[i]);
					pB->SetCurSel(note);
				}
			}

			if (m_inverted)
			{
				if (RanDev()<0.5f)
					pDoc->m_Inverted[i] = FALSE;
				else
					pDoc->m_Inverted[i] = TRUE;

				if ((m_matchscl)&&(pDoc->m_Inverted[i]))
				{
					if (RanDev()<0.5f)
						pDoc->m_MatchScale[i] = FALSE;
					else
						pDoc->m_MatchScale[i] = TRUE;
				}
			}


			if (m_scale)
			{
				int scale;
				int j;

				if (m_onlyusrscl)
					scale = USER_DEFINED;
				else
				{

					do
					{
						scale = int(RanDev()*(15.0f+float(ScalesChoice.m_user)));
					}
					while ( ((!Scales[0])&&(scale == MAJOR))||
							((!Scales[1])&&(scale == MINOR_NATURAL))||
							((!Scales[2])&&(scale == MINOR_HARMONIC))||
							((!Scales[3])&&(scale == MINOR_MELODIC))||
							((!Scales[4])&&(scale == LYDIAN))||
							((!Scales[5])&&(scale == MIXOLYDIAN))||
							((!Scales[6])&&(scale == DORIAN))||
							((!Scales[7])&&(scale == PHRYGIAN))||
							((!Scales[8])&&(scale == LOCRIAN))||
							((!Scales[9])&&(scale == OCTATONIC1))||
							((!Scales[10])&&(scale == OCTATONIC2))||
							((!Scales[11])&&(scale == ACCOUSTIC))||
							((!Scales[12])&&(scale == PENTATONIC))||
							((!Scales[13])&&(scale == WHOLETONE))||
							((!Scales[14])&&(scale == CHROMATIC))||
							((!ScalesChoice.m_user)&&(scale == USER_DEFINED))
						  );
				}


				
				pDoc->m_ScaleID[i] = m_Parent->m_ScaleID[i] = scale;

				if (pDoc->m_pScale[i])
				{
					delete pDoc->m_pScale[i];
					pDoc->m_pScale[i] = NULL;
				}


				switch(scale)
				{
					case MAJOR:
						if ((pDoc->m_Inverted[i])&&(pDoc->m_MatchScale[i]))
						{
							pDoc->m_pScale[i] = new int[Phrygian[0] + 1];
							for(j = 0; j<=Phrygian[0]; j++)
								pDoc->m_pScale[i][j] = Phrygian[j];
							break;
						}
						else
						{
							pDoc->m_pScale[i] = new int[Major[0] + 1];
							for(j = 0; j<=Major[0]; j++)
								pDoc->m_pScale[i][j] = Major[j];
							break;
						}
					case MINOR_NATURAL:
						if ((pDoc->m_Inverted[i])&&(pDoc->m_MatchScale[i]))
						{
							pDoc->m_pScale[i] = new int[Mixolydian[0] + 1];
							for(j = 0; j<=Mixolydian[0]; j++)
								pDoc->m_pScale[i][j] = Mixolydian[j];
							break;
						}
						else
						{		
							pDoc->m_pScale[i] = new int[MinorNatural[0] + 1];
							for(j = 0; j<=MinorNatural[0]; j++)
								pDoc->m_pScale[i][j] = MinorNatural[j];
							break;
						}
					case MINOR_HARMONIC:
						if ((pDoc->m_Inverted[i])&&(pDoc->m_MatchScale[i]))
						{
							pDoc->m_pScale[i] = new int[7 + 1];
							pDoc->m_pScale[i][0] = 7;
							pDoc->m_pScale[i][1] = DO;
							pDoc->m_pScale[i][2] = REb;
							pDoc->m_pScale[i][3] = MI;
							pDoc->m_pScale[i][4] = FA;
							pDoc->m_pScale[i][5] = SOL;
							pDoc->m_pScale[i][6] = LA;
							pDoc->m_pScale[i][7] = SIb;

							break;
						}
						else
						{			
							pDoc->m_pScale[i] = new int[MinorHarmonic[0] + 1];
							for(j = 0; j<=MinorHarmonic[0]; j++)
								pDoc->m_pScale[i][j] = MinorHarmonic[j];
							break;
						}
					case MINOR_MELODIC:
						if ((pDoc->m_Inverted[i])&&(pDoc->m_MatchScale[i]))
						{
							pDoc->m_pScale[i] = new int[7 + 1];
							pDoc->m_pScale[i][0] = 7;
							pDoc->m_pScale[i][1] = DO;
							pDoc->m_pScale[i][2] = REb;
							pDoc->m_pScale[i][3] = MIb;
							pDoc->m_pScale[i][4] = FA;
							pDoc->m_pScale[i][5] = SOL;
							pDoc->m_pScale[i][6] = LA;
							pDoc->m_pScale[i][7] = SIb;

							break;
						}
						else
						{			
							pDoc->m_pScale[i] = new int[MinorMelodic[0] + 1];
							for(j = 0; j<=MinorMelodic[0]; j++)
								pDoc->m_pScale[i][j] = MinorMelodic[j];
							break;
						}
					case LYDIAN:
						if ((pDoc->m_Inverted[i])&&(pDoc->m_MatchScale[i]))
						{
							pDoc->m_pScale[i] = new int[Locrian[0] + 1];
							for(j = 0; j<=Locrian[0]; j++)
								pDoc->m_pScale[i][j] = Locrian[j];
							break;
						}
						else
						{
							pDoc->m_pScale[i] = new int[Lydian[0] + 1];
							for(j = 0; j<=Lydian[0]; j++)
								pDoc->m_pScale[i][j] = Lydian[j];
							break;
						}		
					case MIXOLYDIAN:
						if ((pDoc->m_Inverted[i])&&(pDoc->m_MatchScale[i]))
						{
							pDoc->m_pScale[i] = new int[MinorNatural[0] + 1];
							for(j = 0; j<=MinorNatural[0]; j++)
								pDoc->m_pScale[i][j] = MinorNatural[j];
							break;
						}
						else
						{
							pDoc->m_pScale[i] = new int[Mixolydian[0] + 1];
							for(j = 0; j<=Mixolydian[0]; j++)
								pDoc->m_pScale[i][j] = Mixolydian[j];
							break;
						}		
					case DORIAN:
						pDoc->m_pScale[i] = new int[Dorian[0] + 1];
						for(j = 0; j<=Dorian[0]; j++)
							pDoc->m_pScale[i][j] = Dorian[j];
						break;
					case PHRYGIAN:
						if ((pDoc->m_Inverted[i])&&(pDoc->m_MatchScale[i]))
						{
							pDoc->m_pScale[i] = new int[Major[0] + 1];
							for(j = 0; j<=Major[0]; j++)
								pDoc->m_pScale[i][j] = Major[j];
							break;
						}
						else
						{
							pDoc->m_pScale[i] = new int[Phrygian[0] + 1];
							for(j = 0; j<=Phrygian[0]; j++)
								pDoc->m_pScale[i][j] = Phrygian[j];
							break;
						}		
					case LOCRIAN:
						if ((pDoc->m_Inverted[i])&&(pDoc->m_MatchScale[i]))
						{
							pDoc->m_pScale[i] = new int[Lydian[0] + 1];
							for(j = 0; j<=Lydian[0]; j++)
								pDoc->m_pScale[i][j] = Lydian[j];
							break;
						}
						else
						{
							pDoc->m_pScale[i] = new int[Locrian[0] + 1];
							for(j = 0; j<=Locrian[0]; j++)
								pDoc->m_pScale[i][j] = Locrian[j];
							break;
						}		
					case OCTATONIC1:
						if ((pDoc->m_Inverted[i])&&(pDoc->m_MatchScale[i]))
						{
							pDoc->m_pScale[i] = new int[Octatonic2[0] + 1];
							for(j = 0; j<=Octatonic2[0]; j++)
								pDoc->m_pScale[i][j] = Octatonic2[j];
							break;
						}
						else
						{
							pDoc->m_pScale[i] = new int[Octatonic1[0] + 1];
							for(j = 0; j<=Octatonic1[0]; j++)
								pDoc->m_pScale[i][j] = Octatonic1[j];
							break;
						}		
					case OCTATONIC2:
						if ((pDoc->m_Inverted[i])&&(pDoc->m_MatchScale[i]))
						{
							pDoc->m_pScale[i] = new int[Octatonic1[0] + 1];
							for(j = 0; j<=Octatonic1[0]; j++)
								pDoc->m_pScale[i][j] = Octatonic1[j];
							break;
						}
						else
						{
							pDoc->m_pScale[i] = new int[Octatonic2[0] + 1];
							for(j = 0; j<=Octatonic2[0]; j++)
								pDoc->m_pScale[i][j] = Octatonic2[j];
							break;
						}		
					case ACCOUSTIC:
						if ((pDoc->m_Inverted[i])&&(pDoc->m_MatchScale[i]))
						{
							pDoc->m_pScale[i] = new int[7 + 1];
							pDoc->m_pScale[i][0] = 7;
							pDoc->m_pScale[i][1] = DO;
							pDoc->m_pScale[i][2] = RE;
							pDoc->m_pScale[i][3] = MIb;
							pDoc->m_pScale[i][4] = FA;
							pDoc->m_pScale[i][5] = SOLb;
							pDoc->m_pScale[i][6] = LAb;
							pDoc->m_pScale[i][7] = SIb;

							break;
						}
						else
						{			
							pDoc->m_pScale[i] = new int[Accoustic[0] + 1];
							for(j = 0; j<=Accoustic[0]; j++)
								pDoc->m_pScale[i][j] = Accoustic[j];
							break;
						}
					case WHOLETONE:
						pDoc->m_pScale[i] = new int[Wholetone[0] + 1];
						for(j = 0; j<=Wholetone[0]; j++)
							pDoc->m_pScale[i][j] = Wholetone[j];
						break;
					case PENTATONIC:
						if ((pDoc->m_Inverted[i])&&(pDoc->m_MatchScale[i]))
						{
							pDoc->m_pScale[i] = new int[5 + 1];
							pDoc->m_pScale[i][0] = 5;
							pDoc->m_pScale[i][1] = DO;
							pDoc->m_pScale[i][2] = MIb;
							pDoc->m_pScale[i][3] = FA;
							pDoc->m_pScale[i][4] = LAb;
							pDoc->m_pScale[i][5] = SIb;
							break;
						}
						else
						{			
							pDoc->m_pScale[i] = new int[Pentatonic[0] + 1];
							for(j = 0; j<=Pentatonic[0]; j++)
								pDoc->m_pScale[i][j] = Pentatonic[j];
							break;
						}
					case CHROMATIC:
						pDoc->m_pScale[i] = new int[Chromatic[0] + 1];
						for(j = 0; j<=Chromatic[0]; j++)
							pDoc->m_pScale[i][j] = Chromatic[j];
						break;
					case USER_DEFINED:

						int x = 1;

						if (m_userscl)
						{

							if (m_exactORatmost == 0)
							{

							do 
							{
								m_Parent->InitUserScale(i);
								if (RanDev()>=0.5f)
								{
									m_Parent->m_V1UsrScl[i].m_C_ON = TRUE;
									m_Parent->m_V1UsrScl[i].m_bUserScale[0] = TRUE;
									++m_Parent->m_V1UsrScl[i].m_numnotes;
								}
								if (RanDev()>=0.5f)
								{
									m_Parent->m_V1UsrScl[i].m_Db_ON = TRUE;
									m_Parent->m_V1UsrScl[i].m_bUserScale[1] = TRUE;
									++m_Parent->m_V1UsrScl[i].m_numnotes;
								}
								if (RanDev()>=0.5f)
								{
									m_Parent->m_V1UsrScl[i].m_D_ON = TRUE;
									m_Parent->m_V1UsrScl[i].m_bUserScale[2] = TRUE;
									++m_Parent->m_V1UsrScl[i].m_numnotes;
								}
								if (RanDev()>=0.5f)
								{
									m_Parent->m_V1UsrScl[i].m_Eb_ON = TRUE;
									m_Parent->m_V1UsrScl[i].m_bUserScale[3] = TRUE;
									++m_Parent->m_V1UsrScl[i].m_numnotes;
								}
								if (RanDev()>=0.5f)
								{
									m_Parent->m_V1UsrScl[i].m_E_ON = TRUE;
									m_Parent->m_V1UsrScl[i].m_bUserScale[4] = TRUE;
									++m_Parent->m_V1UsrScl[i].m_numnotes;
								}
								if (RanDev()>=0.5f)
								{
									m_Parent->m_V1UsrScl[i].m_F_ON = TRUE;
									m_Parent->m_V1UsrScl[i].m_bUserScale[5] = TRUE;
									++m_Parent->m_V1UsrScl[i].m_numnotes;
								}
								if (RanDev()>=0.5f)
								{
									m_Parent->m_V1UsrScl[i].m_Gb_ON = TRUE;
									m_Parent->m_V1UsrScl[i].m_bUserScale[6] = TRUE;
									++m_Parent->m_V1UsrScl[i].m_numnotes;
								}
								if (RanDev()>=0.5f)
								{
									m_Parent->m_V1UsrScl[i].m_G_ON = TRUE;
									m_Parent->m_V1UsrScl[i].m_bUserScale[7] = TRUE;
									++m_Parent->m_V1UsrScl[i].m_numnotes;
								}
								if (RanDev()>=0.5f)
								{
									m_Parent->m_V1UsrScl[i].m_Ab_ON = TRUE;
									m_Parent->m_V1UsrScl[i].m_bUserScale[8] = TRUE;
									++m_Parent->m_V1UsrScl[i].m_numnotes;
								}
								if (RanDev()>=0.5f)
								{
									m_Parent->m_V1UsrScl[i].m_A_ON = TRUE;
									m_Parent->m_V1UsrScl[i].m_bUserScale[9] = TRUE;
									++m_Parent->m_V1UsrScl[i].m_numnotes;
								}
								if (RanDev()>=0.5f)
								{
									m_Parent->m_V1UsrScl[i].m_Bb_ON = TRUE;
									m_Parent->m_V1UsrScl[i].m_bUserScale[10] = TRUE;
									++m_Parent->m_V1UsrScl[i].m_numnotes;
								}
								if (RanDev()>=0.5f)
								{
									m_Parent->m_V1UsrScl[i].m_B_ON = TRUE;
									m_Parent->m_V1UsrScl[i].m_bUserScale[11] = TRUE;
									++m_Parent->m_V1UsrScl[i].m_numnotes;
								}
							}
							while ((m_Parent->m_V1UsrScl[i].m_numnotes<m_exact)||(m_Parent->m_V1UsrScl[i].m_numnotes>m_exact));

							}

							if (m_exactORatmost == 1)
							{

							do 
							{
								m_Parent->InitUserScale(i);
								if (RanDev()>=0.5f)
								{
									m_Parent->m_V1UsrScl[i].m_C_ON = TRUE;
									m_Parent->m_V1UsrScl[i].m_bUserScale[0] = TRUE;
									++m_Parent->m_V1UsrScl[i].m_numnotes;
								}
								if (RanDev()>=0.5f)
								{
									m_Parent->m_V1UsrScl[i].m_Db_ON = TRUE;
									m_Parent->m_V1UsrScl[i].m_bUserScale[1] = TRUE;
									++m_Parent->m_V1UsrScl[i].m_numnotes;
								}
								if (RanDev()>=0.5f)
								{
									m_Parent->m_V1UsrScl[i].m_D_ON = TRUE;
									m_Parent->m_V1UsrScl[i].m_bUserScale[2] = TRUE;
									++m_Parent->m_V1UsrScl[i].m_numnotes;
								}
								if (RanDev()>=0.5f)
								{
									m_Parent->m_V1UsrScl[i].m_Eb_ON = TRUE;
									m_Parent->m_V1UsrScl[i].m_bUserScale[3] = TRUE;
									++m_Parent->m_V1UsrScl[i].m_numnotes;
								}
								if (RanDev()>=0.5f)
								{
									m_Parent->m_V1UsrScl[i].m_E_ON = TRUE;
									m_Parent->m_V1UsrScl[i].m_bUserScale[4] = TRUE;
									++m_Parent->m_V1UsrScl[i].m_numnotes;
								}
								if (RanDev()>=0.5f)
								{
									m_Parent->m_V1UsrScl[i].m_F_ON = TRUE;
									m_Parent->m_V1UsrScl[i].m_bUserScale[5] = TRUE;
									++m_Parent->m_V1UsrScl[i].m_numnotes;
								}
								if (RanDev()>=0.5f)
								{
									m_Parent->m_V1UsrScl[i].m_Gb_ON = TRUE;
									m_Parent->m_V1UsrScl[i].m_bUserScale[6] = TRUE;
									++m_Parent->m_V1UsrScl[i].m_numnotes;
								}
								if (RanDev()>=0.5f)
								{
									m_Parent->m_V1UsrScl[i].m_G_ON = TRUE;
									m_Parent->m_V1UsrScl[i].m_bUserScale[7] = TRUE;
									++m_Parent->m_V1UsrScl[i].m_numnotes;
								}
								if (RanDev()>=0.5f)
								{
									m_Parent->m_V1UsrScl[i].m_Ab_ON = TRUE;
									m_Parent->m_V1UsrScl[i].m_bUserScale[8] = TRUE;
									++m_Parent->m_V1UsrScl[i].m_numnotes;
								}
								if (RanDev()>=0.5f)
								{
									m_Parent->m_V1UsrScl[i].m_A_ON = TRUE;
									m_Parent->m_V1UsrScl[i].m_bUserScale[9] = TRUE;
									++m_Parent->m_V1UsrScl[i].m_numnotes;
								}
								if (RanDev()>=0.5f)
								{
									m_Parent->m_V1UsrScl[i].m_Bb_ON = TRUE;
									m_Parent->m_V1UsrScl[i].m_bUserScale[10] = TRUE;
									++m_Parent->m_V1UsrScl[i].m_numnotes;
								}
								if (RanDev()>=0.5f)
								{
									m_Parent->m_V1UsrScl[i].m_B_ON = TRUE;
									m_Parent->m_V1UsrScl[i].m_bUserScale[11] = TRUE;
									++m_Parent->m_V1UsrScl[i].m_numnotes;
								}
							}
							while ((m_Parent->m_V1UsrScl[i].m_numnotes<1)||(m_Parent->m_V1UsrScl[i].m_numnotes>m_atmost));

							}





							if (pDoc->m_pUserScale[i])
							{
								delete pDoc->m_pUserScale[i];
								pDoc->m_pUserScale[i] = NULL;
							}
							pDoc->m_pUserScale[i] = new int[m_Parent->m_V1UsrScl[i].m_numnotes + 1];
							pDoc->m_pUserScale[i][0] = m_Parent->m_V1UsrScl[i].m_numnotes;
							for(j = 0; j<12; j++)
								if (m_Parent->m_V1UsrScl[i].m_bUserScale[j])
									pDoc->m_pUserScale[i][x++] = j;
							m_Parent->SortScale(pDoc->m_pUserScale[i], pDoc->m_pUserScale[i][0]);
							m_Parent->TransposeToC(pDoc->m_pUserScale[i], pDoc->m_pUserScale[i][0]);
						}

						if ((pDoc->m_Inverted[i])&&(pDoc->m_MatchScale[i]))
						{
							pDoc->m_pScale[i] = new int[pDoc->m_pUserScale[i][0] + 1];
							pDoc->m_pScale[i][0] = pDoc->m_pUserScale[i][0];
							for(j = 1; j<=pDoc->m_pUserScale[i][0]; j++)
								pDoc->m_pScale[i][j] = (12-pDoc->m_pUserScale[i][j])*(pDoc->m_pUserScale[i][j]!=0);
							m_Parent->SortScale(pDoc->m_pScale[i], pDoc->m_pScale[i][0]);
							break;
						}
						else
						{
							pDoc->m_pScale[i] = new int[pDoc->m_pUserScale[i][0] + 1];
							pDoc->m_pScale[i][0] = pDoc->m_pUserScale[i][0];
							for(j = 1; j<=pDoc->m_pUserScale[i][0]; j++)
								pDoc->m_pScale[i][j] = pDoc->m_pUserScale[i][j];
							break;
						}




				}

				if(!m_Composition)

				{
					pB = (CComboBox*)m_Parent->GetDlgItem(m_Parent->ScaleID[i]);
					pB->SetCurSel(scale);
				}

			}


			if (m_noteval)
			{

				int noteval;

				do
				{
					noteval = int(RanDev()*12.0f);
				}
				while(	((!NoteValues.m_whole)&&(noteval == WHOLE))||
						((!NoteValues.m_half)&&(noteval == HALF))||
						((!NoteValues.m_quarter)&&(noteval == QUARTER))||
						((!NoteValues.m_8)&&(noteval == EIGHT))||
						((!NoteValues.m_16)&&(noteval == SIXTEENTH))||
						((!NoteValues.m_32)&&(noteval == THIRTYSECOND))||
						((!NoteValues.m_dottedwhole)&&(noteval == WHOLE_DOTTED))||
						((!NoteValues.m_dottedhalf)&&(noteval == HALF_DOTTED))||
						((!NoteValues.m_dottedquarter)&&(noteval == QUARTER_DOTTED))||
						((!NoteValues.m_dotted8)&&(noteval == EIGHTH_DOTTED))||
						((!NoteValues.m_dotted16)&&(noteval == SIXTEENTH_DOTTED))||
						((!NoteValues.m_other)&&(noteval == OTHER))
					 );


				pDoc->m_NoteValueID[i] = m_Parent->m_NoteValueID[i] = noteval;

				switch(noteval)
				{
					case WHOLE:
						pDoc->m_NoteValue[i] = 32.0f;
						break;
					case HALF:
						pDoc->m_NoteValue[i] = 16.0f;
						break;
					case QUARTER:
						pDoc->m_NoteValue[i] = 8.0f;
						break;
					case EIGHT:
						pDoc->m_NoteValue[i] = 4.0f;
						break;
					case SIXTEENTH:
						pDoc->m_NoteValue[i] = 2.0f;
						break;
					case THIRTYSECOND:
						pDoc->m_NoteValue[i] = 1.0f;
						break;
					case WHOLE_DOTTED:
						pDoc->m_NoteValue[i] = 48.00f;
						break;
					case HALF_DOTTED:
						pDoc->m_NoteValue[i] = 24.00f;
						break;
					case QUARTER_DOTTED:
						pDoc->m_NoteValue[i] = 12.00f;
						break;
					case EIGHTH_DOTTED:
						pDoc->m_NoteValue[i] = 6.0f;
						break;
					case SIXTEENTH_DOTTED:
						pDoc->m_NoteValue[i] = 3.0f;
						break;
					case OTHER:
						CString nval;
						int newval = int((RanDev()*(NoteValues.othermax-NoteValues.othermin))+NoteValues.othermin);
						m_Parent->m_NoteVal[i].m_NewVal = (long)newval;
						nval.Format("Other (%d)", newval);
						pB = (CComboBox*)m_Parent->GetDlgItem(m_Parent->NoteValueID[i]);
						pB->DeleteString(OTHER);
						pB->AddString(nval);
						pDoc->m_NoteValue[i] = (float)newval;
						break;

				}
				if(!m_Composition)
				{
					pB = (CComboBox*)m_Parent->GetDlgItem(m_Parent->NoteValueID[i]);
					pB->SetCurSel(noteval);
				}
			}

			if (m_instrument)
			{
				int ins;

				do
				{
					ins = int(RanDev()*128.0f);
				}
				while (	((!Instrument[0])&&(ins == 0))||
						((!Instrument[1])&&(ins == 1))||
						((!Instrument[2])&&(ins == 2))||
						((!Instrument[3])&&(ins == 3))||
						((!Instrument[4])&&(ins == 4))||
						((!Instrument[5])&&(ins == 5))||
						((!Instrument[6])&&(ins == 6))||
						((!Instrument[7])&&(ins == 7))||
						((!Instrument[8])&&(ins == 8))||
						((!Instrument[9])&&(ins == 9))||
						((!Instrument[10])&&(ins == 10))||
						((!Instrument[11])&&(ins == 11))||
						((!Instrument[12])&&(ins == 12))||
						((!Instrument[13])&&(ins == 13))||
						((!Instrument[14])&&(ins == 14))||
						((!Instrument[15])&&(ins == 15))||
						((!Instrument[16])&&(ins == 16))||
						((!Instrument[17])&&(ins == 17))||
						((!Instrument[18])&&(ins == 18))||
						((!Instrument[19])&&(ins == 19))||
						((!Instrument[20])&&(ins == 20))||
						((!Instrument[21])&&(ins == 21))||
						((!Instrument[22])&&(ins == 22))||
						((!Instrument[23])&&(ins == 23))||
						((!Instrument[24])&&(ins == 24))||
						((!Instrument[25])&&(ins == 25))||
						((!Instrument[26])&&(ins == 26))||
						((!Instrument[27])&&(ins == 27))||
						((!Instrument[28])&&(ins == 28))||
						((!Instrument[29])&&(ins == 29))||
						((!Instrument[30])&&(ins == 30))||
						((!Instrument[31])&&(ins == 31))||
						((!Instrument[32])&&(ins == 32))||
						((!Instrument[33])&&(ins == 33))||
						((!Instrument[34])&&(ins == 34))||
						((!Instrument[35])&&(ins == 35))||
						((!Instrument[36])&&(ins == 36))||
						((!Instrument[37])&&(ins == 37))||
						((!Instrument[38])&&(ins == 38))||
						((!Instrument[39])&&(ins == 39))||
						((!Instrument[40])&&(ins == 40))||
						((!Instrument[41])&&(ins == 41))||
						((!Instrument[42])&&(ins == 42))||
						((!Instrument[43])&&(ins == 43))||
						((!Instrument[44])&&(ins == 44))||
						((!Instrument[45])&&(ins == 45))||
						((!Instrument[46])&&(ins == 46))||
						((!Instrument[47])&&(ins == 47))||
						((!Instrument[48])&&(ins == 48))||
						((!Instrument[49])&&(ins == 49))||
						((!Instrument[50])&&(ins == 50))||
						((!Instrument[51])&&(ins == 51))||
						((!Instrument[52])&&(ins == 52))||
						((!Instrument[53])&&(ins == 53))||
						((!Instrument[54])&&(ins == 54))||
						((!Instrument[55])&&(ins == 55))||
						((!Instrument[56])&&(ins == 56))||
						((!Instrument[57])&&(ins == 57))||
						((!Instrument[58])&&(ins == 58))||
						((!Instrument[59])&&(ins == 59))||
						((!Instrument[60])&&(ins == 60))||
						((!Instrument[61])&&(ins == 61))||
						((!Instrument[62])&&(ins == 62))||
						((!Instrument[63])&&(ins == 63))||
						((!Instrument[64])&&(ins == 64))||
						((!Instrument[65])&&(ins == 65))||
						((!Instrument[66])&&(ins == 66))||
						((!Instrument[67])&&(ins == 67))||
						((!Instrument[68])&&(ins == 68))||
						((!Instrument[69])&&(ins == 69))||
						((!Instrument[70])&&(ins == 70))||
						((!Instrument[71])&&(ins == 71))||
						((!Instrument[72])&&(ins == 72))||
						((!Instrument[73])&&(ins == 73))||
						((!Instrument[74])&&(ins == 74))||
						((!Instrument[75])&&(ins == 75))||
						((!Instrument[76])&&(ins == 76))||
						((!Instrument[77])&&(ins == 77))||
						((!Instrument[78])&&(ins == 78))||
						((!Instrument[79])&&(ins == 79))||
						((!Instrument[80])&&(ins == 80))||
						((!Instrument[81])&&(ins == 81))||
						((!Instrument[82])&&(ins == 82))||
						((!Instrument[83])&&(ins == 83))||
						((!Instrument[84])&&(ins == 84))||
						((!Instrument[85])&&(ins == 85))||
						((!Instrument[86])&&(ins == 86))||
						((!Instrument[87])&&(ins == 87))||
						((!Instrument[88])&&(ins == 88))||
						((!Instrument[89])&&(ins == 89))||
						((!Instrument[90])&&(ins == 90))||
						((!Instrument[91])&&(ins == 91))||
						((!Instrument[92])&&(ins == 92))||
						((!Instrument[93])&&(ins == 93))||
						((!Instrument[94])&&(ins == 94))||
						((!Instrument[95])&&(ins == 95))||
						((!Instrument[96])&&(ins == 96))||
						((!Instrument[97])&&(ins == 97))||
						((!Instrument[98])&&(ins == 98))||
						((!Instrument[99])&&(ins == 99))||
						((!Instrument[100])&&(ins == 100 ))||
						((!Instrument[101])&&(ins == 101))||
						((!Instrument[102])&&(ins == 102))||
						((!Instrument[103])&&(ins == 103))||
						((!Instrument[104])&&(ins == 104))||
						((!Instrument[105])&&(ins == 105))||
						((!Instrument[106])&&(ins == 106))||
						((!Instrument[107])&&(ins == 107))||
						((!Instrument[108])&&(ins == 108))||
						((!Instrument[109])&&(ins == 109))||
						((!Instrument[110])&&(ins == 110))||
						((!Instrument[111])&&(ins == 111))||
						((!Instrument[112])&&(ins == 112))||
						((!Instrument[113])&&(ins == 113))||
						((!Instrument[114])&&(ins == 114))||
						((!Instrument[115])&&(ins == 115))||
						((!Instrument[116])&&(ins == 116))||
						((!Instrument[117])&&(ins == 117))||
						((!Instrument[118])&&(ins == 118))||
						((!Instrument[119])&&(ins == 119))||
						((!Instrument[120])&&(ins == 120))||
						((!Instrument[121])&&(ins == 121))||
						((!Instrument[122])&&(ins == 122))||
						((!Instrument[123])&&(ins == 123))||
						((!Instrument[124])&&(ins == 124))||
						((!Instrument[125])&&(ins == 125))||
						((!Instrument[126])&&(ins == 126))||
						((!Instrument[127])&&(ins == 127))
					);
						
				pDoc->m_Instrument[i] = ins;
				pDoc->m_Channel[i] = i;
			}


			/////////////////////////////////

			if (m_percussion)
			{
				if ((!m_instrument)||(RanDev()<0.3333333333))
				{
					int perc;

					do
					{
						perc = int(RanDev()*47.0f);
					}
					while (	((!PercussionInstrument[0])&&(perc == 0))||
							((!PercussionInstrument[1])&&(perc == 1))||
							((!PercussionInstrument[2])&&(perc == 2))||
							((!PercussionInstrument[3])&&(perc == 3))||
							((!PercussionInstrument[4])&&(perc == 4))||
							((!PercussionInstrument[5])&&(perc == 5))||
							((!PercussionInstrument[6])&&(perc == 6))||
							((!PercussionInstrument[7])&&(perc == 7))||
							((!PercussionInstrument[8])&&(perc == 8))||
							((!PercussionInstrument[9])&&(perc == 9))||
							((!PercussionInstrument[10])&&(perc == 10))||
							((!PercussionInstrument[11])&&(perc == 11))||
							((!PercussionInstrument[12])&&(perc == 12))||
							((!PercussionInstrument[13])&&(perc == 13))||
							((!PercussionInstrument[14])&&(perc == 14))||
							((!PercussionInstrument[15])&&(perc == 15))||
							((!PercussionInstrument[16])&&(perc == 16))||
							((!PercussionInstrument[17])&&(perc == 17))||
							((!PercussionInstrument[18])&&(perc == 18))||
							((!PercussionInstrument[19])&&(perc == 19))||
							((!PercussionInstrument[20])&&(perc == 20))||
							((!PercussionInstrument[21])&&(perc == 21))||
							((!PercussionInstrument[22])&&(perc == 22))||
							((!PercussionInstrument[23])&&(perc == 23))||
							((!PercussionInstrument[24])&&(perc == 24))||
							((!PercussionInstrument[25])&&(perc == 25))||
							((!PercussionInstrument[26])&&(perc == 26))||
							((!PercussionInstrument[27])&&(perc == 27))||
							((!PercussionInstrument[28])&&(perc == 28))||
							((!PercussionInstrument[29])&&(perc == 29))||
							((!PercussionInstrument[30])&&(perc == 30))||
							((!PercussionInstrument[31])&&(perc == 31))||
							((!PercussionInstrument[32])&&(perc == 32))||
							((!PercussionInstrument[33])&&(perc == 33))||
							((!PercussionInstrument[34])&&(perc == 34))||
							((!PercussionInstrument[35])&&(perc == 35))||
							((!PercussionInstrument[36])&&(perc == 36))||
							((!PercussionInstrument[37])&&(perc == 37))||
							((!PercussionInstrument[38])&&(perc == 38))||
							((!PercussionInstrument[39])&&(perc == 39))||
							((!PercussionInstrument[40])&&(perc == 40))||
							((!PercussionInstrument[41])&&(perc == 41))||
							((!PercussionInstrument[42])&&(perc == 42))||
							((!PercussionInstrument[43])&&(perc == 43))||
							((!PercussionInstrument[44])&&(perc == 44))||
							((!PercussionInstrument[45])&&(perc == 45))||
							((!PercussionInstrument[46])&&(perc == 46))
						);
							
					pDoc->m_Percussion[i] = perc;
					pDoc->m_Channel[i] = PercussionChannel;
				}
			}


			/////////////////////



			if (m_mode)
			{
				if (RanDev()<0.5f)
					pDoc->m_mode[i] = 0;
				else
					pDoc->m_mode[i] = 1;
			}

			if (m_dynamic)
			{
				int dyn;

				do
				{
					dyn = int(RanDev()*6.0f);
				}
				while(	((!Dynamics.m_pp)&&(dyn == PP))||
						((!Dynamics.m_p)&&(dyn == P))||
						((!Dynamics.m_mp)&&(dyn == MP))||
						((!Dynamics.m_mf)&&(dyn == MF))||
						((!Dynamics.m_f)&&(dyn == F))||
						((!Dynamics.m_ff)&&(dyn == FF))
					);

				pDoc->m_DynChoice[i] = dyn;

				switch(dyn)
				{
					case PP:
						pDoc->m_Dynamic[i] = PIANISSIMO;
						break;
					case P:
						pDoc->m_Dynamic[i] = PIANO;
						break;
					case MP:
						pDoc->m_Dynamic[i] = MEZZO_PIANO;
						break;
					case MF:
						pDoc->m_Dynamic[i] = MEZZO_FORTE;
						break;
					case F:
						pDoc->m_Dynamic[i] = FORTE;
						break;
					case FF:
						pDoc->m_Dynamic[i] = FORTISSIMO;
						break;
				}
			}


			if (m_pan)
			{
				float pan = RanDev();
				if ((pan>=0.0f)&&(pan<0.333333f))
					pDoc->m_Pan[i] = -64;
				if ((pan>=0.333333f)&&(pan<0.666666f))
					pDoc->m_Pan[i] = 0;
				if ((pan>=0.666666f)&&(pan<=1.0f))
					pDoc->m_Pan[i] = 63;
			}



		} 


	}

	if (m_tempo)
	{
		int tpo = int(RanDev()*320.0f)+20;
		pDoc->m_TempoMarking = Tempo = m_Parent->m_TempoMarking = tpo;
		if(!m_Composition)
		{
			char s[20];
			CScrollBar* pSC = (CScrollBar*)m_Parent->GetDlgItem(IDC_TEMPO_SLIDER2);
			CStatic* pSt = (CStatic*)m_Parent->GetDlgItem(IDC_TEMPO_STATIC);

			pSC->SetScrollPos(tpo);

			itoa(tpo, s ,10);
			pSt->SetWindowText(s);
		}
	}


	if(!m_Composition)
	{

		for (i = VOICE1; i<MAXVOICES; i++)
		{
			if (m_VoiceOn[i])
			{

				m_Parent->m_IsActive[i] = pDoc->m_IsActive[i] = m_VoiceOn[i];
				m_Parent->GetDlgItem(m_Parent->StartNumID[i])->EnableWindow(m_Parent->m_IsActive[i]);
				m_Parent->GetDlgItem(m_Parent->ModulatesID[i])->EnableWindow(m_Parent->m_IsActive[i]);
				m_Parent->GetDlgItem(m_Parent->NumOctsID[i])->EnableWindow(m_Parent->m_IsActive[i]);
				m_Parent->GetDlgItem(m_Parent->NoteID[i])->EnableWindow(m_Parent->m_IsActive[i]);
				m_Parent->GetDlgItem(m_Parent->ScaleID[i])->EnableWindow(m_Parent->m_IsActive[i]);
				m_Parent->GetDlgItem(m_Parent->NoteValueID[i])->EnableWindow(m_Parent->m_IsActive[i]);
				m_Parent->GetDlgItem(m_Parent->MidiButtonID[i])->EnableWindow(m_Parent->m_IsActive[i]);
				
				CButton* pButt;

				pButt = (CButton*)(m_Parent->GetDlgItem(m_Parent->ActiveID[i]));

				if (pDoc->m_Channel[i] == PercussionChannel)
				{
					pButt->SetWindowText(PercussionNames[ pDoc->m_Percussion[i] ]);
				}
				else
					pButt->SetWindowText(InstrumentNames[ pDoc->m_Instrument[i] ]);
			}


		}
		m_Parent->UpdateData(FALSE);

	}






		
	pDoc->SetModifiedFlag();

	
}

void CRandDlg::OnParam1Chk() 
{
	UpdateData(TRUE);

	if (m_param1)
	{

		CP1RangeDlg RndDlg;

		RndDlg.title = "Param1 Range";

		RndDlg.m_IsParam1 = TRUE;
		RndDlg.m_IsParam2 = FALSE;
		RndDlg.m_IsNoteVal = FALSE;

		RndDlg.m_from = m_P1From;
		RndDlg.m_to = m_P1To;

		RndDlg.DoModal();

		m_P1From  = RndDlg.m_from;
		m_P1To = RndDlg.m_to;

	}

	
}

void CRandDlg::OnParam2Chk() 
{
	UpdateData(TRUE);

	if (m_param2)
	{

		CP1RangeDlg RndDlg;

		RndDlg.title = "Param2 Range";

		RndDlg.m_IsParam1 = FALSE;
		RndDlg.m_IsParam2 = TRUE;
		RndDlg.m_IsNoteVal = FALSE;

		RndDlg.m_from = m_P2From;
		RndDlg.m_to = m_P2To;

		RndDlg.DoModal();

		m_P2From  = RndDlg.m_from;
		m_P2To = RndDlg.m_to;
	}
}

void CRandDlg::OnSelectall() 
{

	UpdateData(TRUE);

	m_algorithm = TRUE;
	m_dynamic = TRUE;
	m_instrument = TRUE;
	m_percussion = TRUE;
	m_inverted = TRUE;
	m_mode = TRUE;
	m_modulate = TRUE;
	m_note = TRUE;
	m_noteval = TRUE;
	m_octaves = TRUE;
	m_pan = TRUE;
	m_param1 = TRUE;
	m_param2 = TRUE;
	m_coeff = TRUE;
	m_scale = TRUE;
	m_tempo = TRUE;
	//m_onoff = TRUE;
	m_matchscl = TRUE;

	GetDlgItem(IDC_NOSILENCE)->EnableWindow(m_onoff);
	GetDlgItem(IDC_NUMVOICES_ED)->EnableWindow(m_onoff&&(!m_allowSilence));
	GetDlgItem(IDC_NUMVOICES_ST)->EnableWindow(m_onoff&&(!m_allowSilence));
	GetDlgItem(IDC_USERSCL)->EnableWindow(m_scale&&ScalesChoice.m_user);
	GetDlgItem(IDC_EXACT_RD)->EnableWindow(m_scale&&ScalesChoice.m_user&&m_userscl);
	GetDlgItem(IDC_EXACT_ED)->EnableWindow(m_scale&&ScalesChoice.m_user&&(m_exactORatmost == 0)&&m_userscl);
	GetDlgItem(IDC_EXACT_ST)->EnableWindow(m_scale&&ScalesChoice.m_user&&(m_exactORatmost == 0)&&m_userscl);
	GetDlgItem(IDC_ATMOST_RD)->EnableWindow(m_scale&&ScalesChoice.m_user&&m_userscl);
	GetDlgItem(IDC_ATMOST_ED)->EnableWindow(m_scale&&ScalesChoice.m_user&&(m_exactORatmost == 1)&&m_userscl);
	GetDlgItem(IDC_ATMOST_ST)->EnableWindow(m_scale&&ScalesChoice.m_user&&(m_exactORatmost == 1)&&m_userscl);
	GetDlgItem(IDC_ONLYUSERSCL)->EnableWindow(m_userscl);
	GetDlgItem(IDC_MATCHSCL_CHK)->EnableWindow(TRUE);
	UpdateData(FALSE);


}

void CRandDlg::OnSelectnone() 
{
	UpdateData(TRUE);

	m_algorithm = FALSE;
	m_dynamic = FALSE;
	m_instrument = FALSE;
	m_percussion = FALSE;
	m_inverted = FALSE;
	m_mode = FALSE;
	m_modulate = FALSE;
	m_note = FALSE;
	m_noteval = FALSE;
	m_octaves = FALSE;
	m_pan = FALSE;
	m_param1 = FALSE;
	m_param2 = FALSE;
	m_coeff = FALSE;
	m_scale = FALSE;
	m_tempo = FALSE;
	//m_onoff = FALSE;
	m_matchscl = FALSE;

	GetDlgItem(IDC_NOSILENCE)->EnableWindow(m_onoff);
	GetDlgItem(IDC_NUMVOICES_ED)->EnableWindow(m_onoff&&(!m_allowSilence));
	GetDlgItem(IDC_NUMVOICES_ST)->EnableWindow(m_onoff&&(!m_allowSilence));
	GetDlgItem(IDC_EXACT_RD)->EnableWindow(m_scale&&ScalesChoice.m_user&&m_userscl);
	GetDlgItem(IDC_EXACT_ED)->EnableWindow(m_scale&&ScalesChoice.m_user&&(m_exactORatmost == 0)&&m_userscl);
	GetDlgItem(IDC_EXACT_ST)->EnableWindow(m_scale&&ScalesChoice.m_user&&(m_exactORatmost == 0)&&m_userscl);
	GetDlgItem(IDC_ATMOST_RD)->EnableWindow(m_scale&&ScalesChoice.m_user&&m_userscl);
	GetDlgItem(IDC_ATMOST_ED)->EnableWindow(m_scale&&ScalesChoice.m_user&&(m_exactORatmost == 1)&&m_userscl);
	GetDlgItem(IDC_ATMOST_ST)->EnableWindow(m_scale&&ScalesChoice.m_user&&(m_exactORatmost == 1)&&m_userscl);
	GetDlgItem(IDC_USERSCL)->EnableWindow(FALSE);
	GetDlgItem(IDC_ONLYUSERSCL)->EnableWindow(FALSE);
	GetDlgItem(IDC_MATCHSCL_CHK)->EnableWindow(m_inverted);

	UpdateData(FALSE);

	
}

void CRandDlg::OnCancel() 
{
	UpdateData(TRUE);
	
	CDialog::OnCancel();
}

void CRandDlg::OnKillfocusCounterMax() 
{
	UpdateData(TRUE);
	
}

void CRandDlg::OnKillfocusCounterMin() 
{
	UpdateData(TRUE);
	
}

void CRandDlg::OnKillfocusEventsNumber() 
{
	UpdateData(TRUE);

	CFractMusDoc* pDoc = m_Parent->GetDocument();

}

void CRandDlg::OnCreateRandomComposition() 
{
	if(!UpdateData(TRUE))
		return;

	CFractMusDoc* pDoc = m_Parent->GetDocument();


	int maxevs = 999 - pDoc->m_NumEvents;

	CString msg;
	int oldcnt, cnt, prevcnt = 0;
	int actives;
	int numvc = 0;




	if ((m_numberOfEvents<1)||(m_numberOfEvents>(UINT)maxevs)||
		(m_counterMin<1)||(m_counterMax>1000)||(m_counterMax<=m_counterMin))

	{
		msg.Format("Maximum number of Events is %d.\n", maxevs);
		msg = msg + "Minimum number of Events of 1.\n";
		msg = msg + "Minimum counter value is 1.\n";
		msg = msg + "Maximum counter value is 1000\n";
		msg = msg + "Minimum counter value must be less than Maximum counter value.";

		MessageBox(msg, "Error", MB_OK+MB_ICONEXCLAMATION);

		return;

	}

	if ((m_CountersRandomMin<0)||(m_CountersRandomMin>49000)||
		(m_CountersRandomMax<0)||(m_CountersRandomMax>49000)||
		(m_CountersRandomMin>=m_CountersRandomMax))

	{
		msg = msg + "Minimum values for Random Counters is 0.\n";
		msg = msg + "Maximum values for Random Counters is 49000\n";
		msg = msg + "Minimum counter value must be less than Maximum counter value.";

		MessageBox(msg, "Error", MB_OK+MB_ICONEXCLAMATION);

		return;

	}


	BOOL novoices  = TRUE;

	for (int i = VOICE1; i<MAXVOICES; i++)
		if (m_VoiceOn[i])
		{
			novoices = FALSE;
			numvc++;
		}

	if (novoices)
	{
		MessageBox("Please select some voices", "FractMus", MB_OK+MB_ICONEXCLAMATION);
		return;
	}

	if ((m_minOnVoices<1)||(m_minOnVoices>(numvc)))
	{
		msg.Format("Minimum Voices On range is 1 to %d", numvc);
		MessageBox(msg, "Error", MB_OK+MB_ICONEXCLAMATION);
		return;
	}



	CWaitCursor wait;
	m_Composition = TRUE;


	if (m_Parent->m_pEvents ==  NULL)
	{
		m_Parent->m_pEvents = new CEventsDlg(m_Parent);
		m_Parent->m_pEvents->Create();

	}

	m_Parent->m_pEvents->SetForegroundWindow();
	m_Parent->m_pEvents->SetActiveWindow();


	BOOL oldAlgorithm = m_algorithm;
	BOOL oldDynamic = m_dynamic;
	BOOL oldInstrument = m_instrument;
	BOOL oldPercussionInstrument = m_percussion;
	BOOL oldInverted = m_inverted;
	BOOL oldMode = m_mode;
	BOOL oldModulate  = m_modulate;
	BOOL oldNote = m_note;
	BOOL oldNoteval = m_noteval;
	BOOL oldOctaves = m_octaves;
	BOOL oldPan = m_pan;
	BOOL oldParam1 = m_param1;
	BOOL oldParam2 = m_param2;
	BOOL oldCoeff = m_coeff;
	BOOL oldScale = m_scale;
	BOOL oldTempo = m_tempo;
	BOOL oldOnoff = m_onoff;
	BOOL oldMatchscl = m_matchscl;

	BOOL oldVoiceOn[MAXVOICES];
	BOOL IsMutableVoiceOn[MAXVOICES];

	for (i = VOICE1; i<MAXVOICES; i++)
		oldVoiceOn[i] = m_VoiceOn[i];

	for (i = VOICE1; i<MAXVOICES; i++)
		IsMutableVoiceOn[i] = m_VoiceOn[i];

	BOOL IsMutableAlgorithm = m_algorithm;
	BOOL IsMutableDynamic = m_dynamic;
	BOOL IsMutableInstrument = m_instrument;
	BOOL IsMutablePercussionInstrument = m_instrument;
	BOOL IsMutableInverted = m_inverted;
	BOOL IsMutableMode = m_mode;
	BOOL IsMutableModulate  = m_modulate;
	BOOL IsMutableNote = m_note;
	BOOL IsMutableNoteval = m_noteval;
	BOOL IsMutableOctaves = m_octaves;
	BOOL IsMutablePan = m_pan;
	BOOL IsMutableParam1 = m_param1;
	BOOL IsMutableParam2 = m_param2;
	BOOL IsMutableCoeff = m_coeff;
	BOOL IsMutableScale = m_scale;
	BOOL IsMutableTempo = m_tempo;
	BOOL IsMutableMatchscl = m_matchscl;

	CEdit* pE = (CEdit*)(m_Parent->GetDlgItem(IDC_GENCOUNTER_ED));
	CStatic* pSta = (CStatic*)GetDlgItem(IDC_CREATEEVENT_ST);
	CString m;

	int RandomStartCounter = int(RanDev()*(float(m_CountersRandomMax-m_CountersRandomMin)))+m_CountersRandomMin;


	for (int j = 0; j<(int)m_numberOfEvents; j++)
	{
		m.Format("%d", j);
		pSta->SetWindowText(m);

		switch(m_CountersChoice)
		{
			case 0:
			{

				if (pDoc->m_NumEvents<0)
					oldcnt = m_Parent->StartCounter = pDoc->dStartCounter = 0;
				else
					oldcnt = m_Parent->StartCounter = pDoc->dStartCounter = pDoc->Event[pDoc->m_NumEvents].genCount;


				cnt = int((RanDev()*(m_counterMax-m_counterMin))+m_counterMin);

				if((oldcnt + cnt)>50000)
				{
					MessageBox("Maximum counter value reached","FractMus",MB_OK+MB_ICONEXCLAMATION);
					OnCancel();
					return;
				}

				m_Parent->GeneralCounter = pDoc->dGeneralCounter = oldcnt + cnt;
				break;
			}

			case 1:
			{
				
				if (!j)
					oldcnt = m_Parent->StartCounter = pDoc->dStartCounter = 0;
				else
					oldcnt = m_Parent->StartCounter = pDoc->dStartCounter = pDoc->dGeneralCounter;


				cnt = int((RanDev()*(m_counterMax-m_counterMin))+m_counterMin);

				if((oldcnt + cnt)>50000)
				{
					MessageBox("Maximum counter value reached!","FractMus",MB_OK+MB_ICONEXCLAMATION);
					OnCancel();
					return;
				}

				m_Parent->GeneralCounter = pDoc->dGeneralCounter = oldcnt + cnt;
				break;
			}

			case 2:
			{

				if (m_sequential)
				{
					if (!j)
						oldcnt = m_Parent->StartCounter = pDoc->dStartCounter = RandomStartCounter;
					else
						oldcnt = m_Parent->StartCounter = pDoc->dStartCounter = pDoc->dGeneralCounter;


					cnt = int((RanDev()*(m_counterMax-m_counterMin))+m_counterMin);

					if((oldcnt + cnt)>50000)
					{
						MessageBox("Maximum counter value reached!","FractMus",MB_OK+MB_ICONEXCLAMATION);
						OnCancel();
						return;
					}

					m_Parent->GeneralCounter = pDoc->dGeneralCounter = oldcnt + cnt;
					break;
				}
				else
				{

					RandomStartCounter = int(RanDev()*(float(m_CountersRandomMax-m_CountersRandomMin)))+m_CountersRandomMin;

					m_Parent->StartCounter = pDoc->dStartCounter = RandomStartCounter;

					cnt = int((RanDev()*(m_counterMax-m_counterMin))+m_counterMin);

					m_Parent->GeneralCounter = pDoc->dGeneralCounter = RandomStartCounter + cnt;

					break;
				}
			}
		}



		if (numvc<2)
		{

			do
			{

				actives = 0;

				for (i = VOICE1; i<MAXVOICES; i++)
				{
					

					if (IsMutableVoiceOn[i])
					{
						if (RanDev()<0.5f)
							m_VoiceOn[i] = pDoc->m_IsActive[i] = m_Parent->m_IsActive[i] = !m_onoff;
						else
						{
							m_VoiceOn[i] = pDoc->m_IsActive[i] = m_Parent->m_IsActive[i] = TRUE;
						}
						if (m_VoiceOn[i])
							++actives;

					}
				}
			}
			while (!(m_allowSilence)&&(!actives));

		}
		else
		{

			do
			{

				actives = 0;

				for (i = VOICE1; i<MAXVOICES; i++)
				{
					

					if (IsMutableVoiceOn[i])
					{
						if (RanDev()<0.5f)
							m_VoiceOn[i] = pDoc->m_IsActive[i] = m_Parent->m_IsActive[i] = !m_onoff;
						else
						{
							m_VoiceOn[i] = pDoc->m_IsActive[i] = m_Parent->m_IsActive[i] = TRUE;
						}

						if (m_VoiceOn[i])
							++actives;
					}
				}
			}
			while (!(m_allowSilence)&&(actives<m_minOnVoices));
		}

		if (IsMutableAlgorithm)
		{
			if (RanDev()<0.5f)
				m_algorithm = TRUE;
			else 
				m_algorithm = FALSE;
		}

		if (IsMutableDynamic)
		{
			if (RanDev()<0.5f)
				m_dynamic = TRUE;
			else 
				m_dynamic = FALSE;
		}

		if (IsMutableInstrument)
		{
			if (RanDev()<0.5f)
				m_instrument = TRUE;
			else 
				m_instrument = FALSE;
		}

		if (IsMutablePercussionInstrument)
		{
			if (RanDev()<0.5f)
				m_percussion = TRUE;
			else 
				m_percussion = FALSE;
		}

		if (IsMutableInverted)
		{
			if (RanDev()<0.5f)
				m_inverted = TRUE;
			else 
				m_inverted = FALSE;

			if(m_inverted&&IsMutableMatchscl)
			{
				if (RanDev()<0.5f)
					m_matchscl = TRUE;
				else 
					m_matchscl = FALSE;
			}

		}

		if (IsMutableMode)
		{
			if (RanDev()<0.5f)
				m_mode = TRUE;
			else 
				m_mode = FALSE;
		}

		if (IsMutableModulate)
		{
			if (RanDev()<0.5f)
				m_modulate = TRUE;
			else 
				m_modulate = FALSE;
		}

		if (IsMutableNote)
		{
			if (RanDev()<0.5f)
				m_note = TRUE;
			else 
				m_note = FALSE;
		}


		if (IsMutableNoteval)
		{
			if (RanDev()<0.5f)
				m_noteval = TRUE;
			else 
				m_noteval = FALSE;
		}

		if (IsMutableOctaves)
		{
			if (RanDev()<0.5f)
				m_octaves = TRUE;
			else 
				m_octaves = FALSE;
		}

		if (IsMutableParam1)
		{
			if (RanDev()<0.5f)
				m_param1 = TRUE;
			else 
				m_param1 = FALSE;
		}


		if (IsMutableParam2)
		{
			if (RanDev()<0.5f)
				m_param2 = TRUE;
			else 
				m_param2 = FALSE;
		}

		if (IsMutableCoeff)
		{
			if (RanDev()<0.5f)
				m_coeff = TRUE;
			else 
				m_coeff = FALSE;
		}

		if (IsMutableScale)
		{
			if (RanDev()<0.5f)
				m_scale = TRUE;
			else 
				m_scale = FALSE;
		}

		if (IsMutableTempo)
		{
			if (RanDev()<0.5f)
				m_tempo = TRUE;
			else 
				m_tempo = FALSE;
		}


		OnRand();
		m_Parent->m_pEvents->OnAdd();

	}


	m_algorithm = oldAlgorithm;
	m_dynamic = oldDynamic;
	m_instrument = oldInstrument;
	m_percussion = oldPercussionInstrument;
	m_inverted = oldInverted;
	m_mode = oldMode;
	m_modulate = oldModulate;
	m_note = oldNote;
	m_noteval = oldNoteval;
	m_octaves = oldOctaves;
	m_pan = oldPan;
	m_param1 = oldParam1;
	m_param2 = oldParam2;
	m_coeff = oldCoeff;
	m_scale = oldScale;
	m_tempo = oldTempo;
	m_onoff = oldOnoff;
	m_matchscl = oldMatchscl;

	for (i = VOICE1; i<MAXVOICES; i++)
		m_VoiceOn[i] = oldVoiceOn[i];

	UpdateData(FALSE);

	pSta->SetWindowText("(Done)");

	CComboBox* pB;
	CScrollBar* pS;
	CStatic* pSt = (CStatic*)(m_Parent->GetDlgItem(IDC_TEMPO_STATIC));


	Tempo = m_Parent->m_TempoMarking = pDoc->m_TempoMarking = pDoc->Event[0].Tempo;
	m_Parent->GeneralCounter = pDoc->dGeneralCounter = pDoc->Event[0].genCount;
	m_Parent->StartCounter = pDoc->dStartCounter = pDoc->Event[0].count;
	m_Parent->m_StartFrom = pDoc->Event[0].count;
	pS = (CScrollBar*)(m_Parent->GetDlgItem(IDC_TEMPO_SLIDER2));
	pS->SetScrollPos(Tempo);



	char s[10];

	itoa(Tempo, s, 10);
	pSt->SetWindowText(s);

	pDoc->m_TimeSigNum = pDoc->Event[0].TimeSigNum;
	pDoc->m_TimeSigDen = pDoc->Event[0].TimeSigDen;
	pB = (CComboBox*)(m_Parent->GetDlgItem(IDC_TIMESIGNUM_CMB));
	pB->SetCurSel((pDoc->m_TimeSigNum)-1);
	pB = (CComboBox*)(m_Parent->GetDlgItem(IDC_TIMESIGDEN_CMB));
	pB->SetCurSel(int(log10(double(pDoc->m_TimeSigDen))/log10((double)2)));

	for(int p = VOICE1; p<MAXVOICES; p++)
	{
		pDoc->AlgorithmNum[p] = pDoc->Event[0].AlgorithmNum[p];
		pDoc->m_IsActive[p] = m_Parent->m_IsActive[p] = pDoc->Event[0].IsActive[p];
		pDoc->m_Base[p] =  pDoc->Event[0].Base[p];
		pDoc->m_Multi[p] = pDoc->Event[0].Multi[p];
		pDoc->m_StartNum[p] = m_Parent->m_StartNum[p] = pDoc->Event[0].StartNum[p];
		pDoc->m_NumOcts[p] = pDoc->Event[0].NumOcts[p];
		pDoc->m_Modulates[p] = m_Parent->m_Modulates[p] = pDoc->Event[0].Modulates[p] ;
		pDoc->m_ModulRandom[p] = m_Parent->m_V1Modul[p].m_V1ModulDlgChoice = pDoc->Event[0].ModulRandom[p] ;
		pDoc->m_ModulNum[p] = m_Parent->m_V1Modul[p].m_V1ModulDlgNum = pDoc->Event[0].ModulNum[p] ;
		pDoc->m_Note[p] = m_Parent->m_Note[p] = pDoc->Event[0].Note[p] ;
		pDoc->m_NoteValue[p] = pDoc->Event[0].NoteValue[p] ;
		pDoc->m_Instrument[p] = m_Parent->m_V1MidiSet[p].m_instrument  = pDoc->Event[0].Instrument[p];
		pDoc->m_Channel[p] = pDoc->Event[0].Channel[p];
		pDoc->m_mode[p] = m_Parent->m_V1MidiSet[p].m_mode = pDoc->Event[0].Mode[p];
		pDoc->m_ScaleID[p] = m_Parent->m_ScaleID[p] = pDoc->Event[0].ScaleID[p];
		pDoc->m_NumOctsID[p] = m_Parent->m_NumOctsID[p] = pDoc->Event[0].NumOctsID[p];
		pDoc->m_NoteValueID[p] = m_Parent->m_NoteValueID[p] = pDoc->Event[0].NoteValueID[p];
		pDoc->m_Inverted[p] = pDoc->Event[0].Inverted[p];
		pDoc->m_MatchScale[p] = pDoc->Event[0].MatchScale[p];
		pDoc->m_Dynamic[p] = pDoc->Event[0].Dynamic[p];
		pDoc->m_DynChoice[p] = pDoc->Event[0].DynChoice[p];
		pDoc->m_LogMapVar[p] = pDoc->Event[0].LogMapVar[p];
		pDoc->m_FNoiseVar[p] = pDoc->Event[0].FNoiseVar[p];
		pDoc->m_LorentzVar[p] = pDoc->Event[0].LorentzVar[p];
		pDoc->m_HenonA[p] = pDoc->Event[0].HenonA[p];
		pDoc->m_HenonB[p] = pDoc->Event[0].HenonB[p];
		pDoc->m_HopalongVar[p] = pDoc->Event[0].HopalongVar[p];
		pDoc->m_MartinVar[p] = pDoc->Event[0].MartinVar[p];
		pDoc->m_GingerX[p] = pDoc->Event[0].GingerX[p];
		pDoc->m_GingerY[p] = pDoc->Event[0].GingerY[p];
		pDoc->m_Pan[p] = pDoc->Event[0].Pan[p];
		pDoc->m_Percussion[p] = pDoc->Event[0].Percussion[p];
	}

	m_Parent->UpdateData(FALSE);

	for(p = VOICE1; p<MAXVOICES; p++)
	{
		pB = (CComboBox*)(m_Parent->GetDlgItem(m_Parent->NumOctsID[p]));
		pB->SetCurSel(m_Parent->m_NumOctsID[p]);
		pB = (CComboBox*)(m_Parent->GetDlgItem(m_Parent->NoteID[p]));
		pB->SetCurSel(m_Parent->m_Note[p]);
		pB = (CComboBox*)(m_Parent->GetDlgItem(m_Parent->ScaleID[p]));
		pB->SetCurSel(m_Parent->m_ScaleID[p]);
		pB = (CComboBox*)(m_Parent->GetDlgItem(m_Parent->NoteValueID[p]));
		pB->SetCurSel(m_Parent->m_NoteValueID[p]);
	}


	for (p = VOICE1; p<MAXVOICES; p++)
	{
		CString nval;

		if (m_Parent->m_NoteValueID[p] == OTHER)
		{
			nval.Format("Other (%d)", (long)pDoc->m_NoteValue[p]);
			pB = (CComboBox*)(m_Parent->GetDlgItem(m_Parent->NoteValueID[p]));
			pB->DeleteString(OTHER);
			pB->AddString(nval);
			pB->SetCurSel(OTHER);
		}
	}



	for(p = VOICE1; p<MAXVOICES; p++)
	{
		if (pDoc->m_pUserScale[p])
		{
			delete pDoc->m_pUserScale[p];
			pDoc->m_pUserScale[p] = NULL;
		}
		pDoc->m_pUserScale[p] = new int[pDoc->Event[0].pUserScale[p][0] + 1];
		pDoc->m_pUserScale[p][0] = pDoc->Event[0].pUserScale[p][0];
		for(int h = 1; h<=pDoc->Event[0].pUserScale[p][0]; h++)
			pDoc->m_pUserScale[p][h] = pDoc->Event[0].pUserScale[p][h];
	}


	for(p = VOICE1; p<MAXVOICES; p++)
	{
		if (pDoc->m_pScale[p])
		{
			delete pDoc->m_pScale[p];
			pDoc->m_pScale[p] = NULL;
		}
		pDoc->m_pScale[p] = new int[pDoc->Event[0].pScale[p][0] + 1];
		pDoc->m_pScale[p][0] = pDoc->Event[0].pScale[p][0];
		for(int h = 1; h<=pDoc->Event[0].pScale[p][0]; h++)
			pDoc->m_pScale[p][h] = pDoc->Event[0].pScale[p][h];
		if (pDoc->m_ScaleID[p] == USER_DEFINED)
		{
			m_Parent->InitUserScale(p);
			m_Parent->SetUserScale(p);
		}
	}

	pB = (CComboBox*)m_Parent->GetDlgItem(IDC_PLAYFROM_CMB);
	pB->SetCurSel(0);
	CButton* pButt;


	for(i = VOICE1; i<MAXVOICES; i++)
	{

		pE = (CEdit*)(m_Parent->GetDlgItem(m_Parent->StartNumID[i]))->EnableWindow(m_Parent->m_IsActive[i]);
		pButt = (CButton*)(m_Parent->GetDlgItem(m_Parent->ModulatesID[i]))->EnableWindow(m_Parent->m_IsActive[i]);
		pB = (CComboBox*)(m_Parent->GetDlgItem(m_Parent->NumOctsID[i]));
		pB->SetCurSel(m_Parent->m_NumOctsID[i]);
		pB = (CComboBox*)(m_Parent->GetDlgItem(m_Parent->NumOctsID[i]))->EnableWindow(m_Parent->m_IsActive[i]);
		pB = (CComboBox*)(m_Parent->GetDlgItem(m_Parent->NoteID[i]));
		pB->SetCurSel(m_Parent->m_Note[i]);
		pB = (CComboBox*)(m_Parent->GetDlgItem(m_Parent->NoteID[i]))->EnableWindow(m_Parent->m_IsActive[i]);
		pB = (CComboBox*)(m_Parent->GetDlgItem(m_Parent->ScaleID[i]));
		pB->SetCurSel(m_Parent->m_ScaleID[i]);
		pB = (CComboBox*)(m_Parent->GetDlgItem(m_Parent->ScaleID[i]))->EnableWindow(m_Parent->m_IsActive[i]);
		pB = (CComboBox*)(m_Parent->GetDlgItem(m_Parent->NoteValueID[i]));
		pB->SetCurSel(m_Parent->m_NoteValueID[i]);
		pB = (CComboBox*)(m_Parent->GetDlgItem(m_Parent->NoteValueID[i]))->EnableWindow(m_Parent->m_IsActive[i]);
		pButt = (CButton*)(m_Parent->GetDlgItem(m_Parent->MidiButtonID[i]))->EnableWindow(m_Parent->m_IsActive[i]);
		pButt = (CButton*)(m_Parent->GetDlgItem(m_Parent->ActiveID[i]));

		if (pDoc->m_Channel[i] == PercussionChannel)
		{
			pButt->SetWindowText(PercussionNames[ pDoc->m_Percussion[i] ]);
		}
		else
			pButt->SetWindowText(InstrumentNames[ pDoc->m_Instrument[i] ]);
	}


	m_Composition = FALSE;

	m_Parent->PlayChoice = COMPOSITION;

	m_Parent->GetDlgItem(IDC_PLAYFROM_CMB)->EnableWindow(TRUE);
	m_Parent->GetDlgItem(IDC_PLAYFROM_BT)->EnableWindow(TRUE);

	OnCancel();

		
}

void CRandDlg::OnNoteChk() 
{

	UpdateData(TRUE);

	if(m_note)
	{
		CStartNoteDlg dlg;

		dlg.pRand = this;

		dlg.DoModal();

		for(int i = 0; i<97; i++)
			Notes[i] = FALSE;

		for(i = 0; i<97; i++)
			Notes[i] = dlg.IsSelected[i];
	}

	
}

void CRandDlg::OnOnoffChk() 
{
	CFractMusDoc* pDoc = m_Parent->GetDocument();
	UpdateData(TRUE);
	GetDlgItem(IDC_NOSILENCE)->EnableWindow(m_onoff);
	GetDlgItem(IDC_NUMVOICES_ED)->EnableWindow(m_onoff&&(!m_allowSilence));
	GetDlgItem(IDC_NUMVOICES_ST)->EnableWindow(m_onoff&&(!m_allowSilence));

	if (pDoc->m_NumEvents>-1)
	{
		if (m_onoff)
		{
			m_AllActiveVoices = FALSE;
			UpdateData(FALSE);
		}
	}

}

void CRandDlg::OnScaleChk() 
{
	UpdateData(TRUE);

	if (m_scale)
	{
		ScalesChoice.DoModal();

		for(int i = 0; i<15; i++)
			Scales[i] = FALSE;

		for(i = 0; i<15; i++)
			Scales[i] = ScalesChoice.IsSelected[i];
	}


	GetDlgItem(IDC_USERSCL)->EnableWindow(m_scale&&ScalesChoice.m_user);

	if (!ScalesChoice.m_user)
		m_userscl = FALSE;

	GetDlgItem(IDC_ONLYUSERSCL)->EnableWindow(m_scale&&m_userscl);

	GetDlgItem(IDC_EXACT_RD)->EnableWindow(m_scale&&ScalesChoice.m_user&&m_userscl);
	GetDlgItem(IDC_EXACT_ED)->EnableWindow(m_scale&&ScalesChoice.m_user&&(m_exactORatmost == 0)&&m_userscl);
	GetDlgItem(IDC_EXACT_ST)->EnableWindow(m_scale&&ScalesChoice.m_user&&(m_exactORatmost == 0)&&m_userscl);
	GetDlgItem(IDC_ATMOST_RD)->EnableWindow(m_scale&&ScalesChoice.m_user&&m_userscl);
	GetDlgItem(IDC_ATMOST_ED)->EnableWindow(m_scale&&ScalesChoice.m_user&&(m_exactORatmost == 1)&&m_userscl);
	GetDlgItem(IDC_ATMOST_ST)->EnableWindow(m_scale&&ScalesChoice.m_user&&(m_exactORatmost == 1)&&m_userscl);

	if (!m_userscl)
		m_onlyusrscl = FALSE;

	UpdateData(FALSE);
}

void CRandDlg::OnUserscl() 
{
	UpdateData(TRUE);
	GetDlgItem(IDC_ONLYUSERSCL)->EnableWindow(m_userscl);
	GetDlgItem(IDC_EXACT_RD)->EnableWindow(m_scale&&ScalesChoice.m_user&&m_userscl);
	GetDlgItem(IDC_EXACT_ED)->EnableWindow(m_scale&&ScalesChoice.m_user&&(m_exactORatmost == 0)&&m_userscl);
	GetDlgItem(IDC_EXACT_ST)->EnableWindow(m_scale&&ScalesChoice.m_user&&(m_exactORatmost == 0)&&m_userscl);
	GetDlgItem(IDC_ATMOST_RD)->EnableWindow(m_scale&&ScalesChoice.m_user&&m_userscl);
	GetDlgItem(IDC_ATMOST_ED)->EnableWindow(m_scale&&ScalesChoice.m_user&&(m_exactORatmost == 1)&&m_userscl);
	GetDlgItem(IDC_ATMOST_ST)->EnableWindow(m_scale&&ScalesChoice.m_user&&(m_exactORatmost == 1)&&m_userscl);
	if (!m_userscl)
		m_onlyusrscl = FALSE;
	UpdateData(FALSE);
}

void CRandDlg::OnInvertedChk() 
{
	UpdateData(TRUE);
	GetDlgItem(IDC_MATCHSCL_CHK)->EnableWindow(m_inverted);
	
}

void CRandDlg::OnNosilence() 
{

	UpdateData(TRUE);

	GetDlgItem(IDC_NUMVOICES_ED)->EnableWindow(m_onoff&&(!m_allowSilence));
	GetDlgItem(IDC_NUMVOICES_ST)->EnableWindow(m_onoff&&(!m_allowSilence));
	
}

void CRandDlg::OnAlgorithmChk() 
{
	UpdateData(TRUE);

	if (m_algorithm)
		Algorithms.DoModal();	

}

void CRandDlg::OnNotevalChk() 
{
	UpdateData(TRUE);

	if (m_noteval)
		NoteValues.DoModal();
	
}

void CRandDlg::OnInstrumentChk() 
{
	UpdateData(TRUE);

	if(m_instrument)
	{
		CInstrumentsDlg dlg;

		dlg.pRand = this;

		dlg.DoModal();

		for(int i = 0; i<128; i++)
			Instrument[i] = FALSE;

		for(i = 0; i<128; i++)
			Instrument[i] = dlg.IsSelected[i];
	}
	
}

void CRandDlg::OnOctavesChk() 
{
	UpdateData(TRUE);

	if (m_octaves)
		Octaves.DoModal();
	
}

void CRandDlg::OnDynamicChk() 
{
	UpdateData(TRUE);

	if(m_dynamic)
		Dynamics.DoModal();
	
}

void CRandDlg::OnRandomizeselectedevents() 
{
	if (!UpdateData(TRUE))
		return;

	int numv = 0;

	for (int g = VOICE1; g<MAXVOICES; g++)
		if (m_VoiceOn[g])
			++numv;

	if ((!numv)&&(!m_AllActiveVoices))
	{
		MessageBox("No voices selected!","Error",MB_OK+MB_ICONEXCLAMATION);
		return;
	}

	CFractMusDoc* pDoc = m_Parent->GetDocument();	
	CListBox* pList = (CListBox*)GetDlgItem(IDC_EVENTS_LIST);
	CComboBox* pB;

	int numchoices = pList->GetSelCount();

	if(numchoices<1)
	{
		MessageBox("No events selected!","Error",MB_OK+MB_ICONEXCLAMATION);
		return;
	}

	for (int i =0; i<=pDoc->m_NumEvents; i++)
		IsSelected[i] = FALSE;


	int* selectedItems = new int[numchoices];

	pList->GetSelItems(numchoices, selectedItems);

	for(i =0; i<numchoices; i++)
		IsSelected[ selectedItems[i] ] = TRUE;

	CWaitCursor wait;

	CStatic* pSt = (CStatic*)GetDlgItem(IDC_EVENTNUM_ST);

/////////////////////////////////////////////////////////////////////////////

	BOOL WillBeActive[MAXVOICES];

	for (int a = VOICE1; a<MAXVOICES; a++)
		WillBeActive[a] = FALSE;


	if(m_onoff)
	{

		int numvc = 0;

		for(int a = VOICE1; a<MAXVOICES; a++)
			if (m_VoiceOn[a])
				++numvc;

		if ((m_minOnVoices<1)||(m_minOnVoices>(numvc)))
		{
			CString msg;
			msg.Format("Minimum Voices On range is 1 to %d", numvc);
			MessageBox(msg, "Error", MB_OK+MB_ICONEXCLAMATION);
			return;
		}
	}





	for (int l = 0; l<=pDoc->m_NumEvents; l++)
		if (IsSelected[l])
		{
			CString m;
			m.Format("%d", l);
			pSt->SetWindowText(m);

			if (m_onoff)
			{

					int actives;
					do
					{




						for (int h  = VOICE1; h<MAXVOICES; h++)
							pDoc->Event[l].IsActive[h] = FALSE;

						actives = 0;

						for (h  = VOICE1; h<MAXVOICES; h++)
						{
							

							if (m_VoiceOn[h])
							{
								if (RanDev()<0.5f)
									pDoc->Event[l].IsActive[h] = FALSE;
								else
								{
									pDoc->Event[l].IsActive[h] = TRUE;
								}

								if (pDoc->Event[l].IsActive[h])
									++actives;
							}
						}
					}
					while (!(m_allowSilence)&&(actives<m_minOnVoices));


			}

			for(int i = VOICE1; i<MAXVOICES; i++)
			{
				if (m_VoiceOn[i]||(pDoc->Event[l].IsActive[i]&&m_AllActiveVoices))
				{

					if (m_algorithm)
					{
						int al;

						do
						{
							al = int(RanDev()*12.0f);
						}
						while(	((!Algorithms.m_morsethue)&&(al == MORSE_THUE))||
								((!Algorithms.m_logmap)&&(al == LOGMAP))||
								((!Algorithms.m_noise)&&(al == ONE_OVER_F_NOISE))||
								((!Algorithms.m_henon)&&(al == HENON))||
								((!Algorithms.m_hopalong)&&(al == HOPALONG))||
								((!Algorithms.m_martin)&&(al == MARTIN))||
								((!Algorithms.m_ginger)&&(al == GINGERBREADMAN))||
								((!Algorithms.m_lorentz)&&(al == LORENTZ))||
								((!Algorithms.m_earthworm)&&(al == EARTHWORM))||
								((!Algorithms.m_random)&&(al == RANDOM))||
								((!Algorithms.m_hailstone)&&(al == HAILSTONE))||
								((!Algorithms.m_wolfram)&&(al == WOLFRAM))
							 );

						pDoc->Event[l].AlgorithmNum[i] = al;

					}

					if ((m_param1)&&(pDoc->Event[l].AlgorithmNum[i]==MORSE_THUE))
					{
						long base = long((RanDev()*(m_P1To-m_P1From))+m_P1From);
						pDoc->Event[l].Base[i] = base;
					}

					if ((m_param2)&&(pDoc->Event[l].AlgorithmNum[i]==MORSE_THUE))
					{
						long multi = long((RanDev()*(m_P2To-m_P2From))+m_P2From);
						pDoc->Event[l].Multi[i] = multi;
					}

					if ((m_coeff)&&(pDoc->Event[l].AlgorithmNum[i]==LOGMAP))
						pDoc->Event[l].LogMapVar[i] = (RanDev()*0.5f)+3.5f;

					if ((m_coeff)&&(pDoc->Event[l].AlgorithmNum[i]==ONE_OVER_F_NOISE))
						pDoc->Event[l].FNoiseVar[i] = RanDev();

					if ((m_coeff)&&(pDoc->Event[l].AlgorithmNum[i]==HENON))
					{
						pDoc->Event[l].HenonA[i] = RanDev()+1.0f;
						pDoc->Event[l].HenonB[i] = RanDev();
					}

					if ((m_coeff)&&(pDoc->Event[l].AlgorithmNum[i]==HOPALONG))
					{
						pDoc->Event[l].HopalongVar[i] = (RanDev()* 12.56637061436f)- 6.28318530718f;
					}

					if ((m_coeff)&&(pDoc->Event[l].AlgorithmNum[i]==MARTIN))
					{
						pDoc->Event[l].MartinVar[i] = (RanDev()* 12.56637061436f)- 6.28318530718f;
					}

					if ((m_coeff)&&(pDoc->Event[l].AlgorithmNum[i]==GINGERBREADMAN))
					{
						pDoc->Event[l].GingerX[i] = (RanDev()*40.0f)-20.0f;
						pDoc->Event[l].GingerY[i] = (RanDev()*40.0f)-20.0f;
					}

					if ((m_coeff)&&(pDoc->Event[l].AlgorithmNum[i]==LORENTZ))
					{
						pDoc->Event[l].LorentzVar[i] = (RanDev()* 0.3f)+ 0.7f;
					}

					if ((m_coeff)&&(pDoc->Event[l].AlgorithmNum[i]==EARTHWORM))
					{
						pDoc->Event[l].EarthWormMulti[i] = unsigned long((RanDev()* 999997.0f)+ 2.0f);
						pDoc->Event[l].EarthWormVar[i] = unsigned long((RanDev()* 999997.0f)+ 2.0f);
						pDoc->Event[l].EarthWormTruncator[i] = unsigned((RanDev()* 9.0f)+ 1.0f);
					}
					if ((m_coeff)&&(pDoc->Event[l].AlgorithmNum[i]==RANDOM))
					{
						pDoc->Event[l].RandomMin[i] = (RanDev()* 0.5f);
						pDoc->Event[l].RandomMax[i] = (RanDev()* 0.5f)+ 0.5f;
					}
					if ((m_coeff)&&(pDoc->Event[l].AlgorithmNum[i]==HAILSTONE))
					{
						pDoc->Event[l].Hailstone[i] = (long)(RanDev()* 9999999.0f)+ 1.0f;
					}
					if ((m_coeff)&&(pDoc->Event[l].AlgorithmNum[i]==WOLFRAM))
					{
						pDoc->Event[l].WolframRule[i] = BYTE((RanDev()* 254.0f)+ 1.0f);
						pDoc->Event[l].WolframInit[i] = unsigned long((RanDev()* 999999999.0f)+ 1.0f);
						pDoc->Event[l].WolframSize[i] = int((RanDev()* 384.0f)+ 128.0f);
					}

					if (m_modulate)
					{
						if (RanDev()>0.5f)
						{
							pDoc->Event[l].Modulates[i] = TRUE;


							if (RanDev()<=0.5)
							{
								pDoc->Event[l].ModulRandom[i] = FALSE;
								pDoc->Event[l].ModulNum[i] = int((RanDev()*49)+1);
							}
							else
								pDoc->Event[l].ModulRandom[i] = TRUE;
						}
						else
							pDoc->Event[l].Modulates[i] = FALSE;
					}

					if (m_octaves)
					{
						
						int oct;

						do
						{
							oct = int(RanDev()*4.0f);
						}
						while(	((!Octaves.m_one)&&(oct == ONE_OCT))||
								((!Octaves.m_two)&&(oct == TWO_OCT))||
								((!Octaves.m_three)&&(oct == THREE_OCT))||
								((!Octaves.m_four)&&(oct == FOUR_OCT))
							);

						pDoc->Event[l].NumOcts[i] = oct+1;
						pDoc->Event[l].NumOctsID[i] = oct;

						/*if(!m_Composition)
						{
							pB = (CComboBox*)m_Parent->GetDlgItem(m_Parent->NumOctsID[i]);	
							pB->SetCurSel(oct);
						}*/

						
					}

					if (m_note)
					{
						int note;

						do
						{
							note = int(RanDev()*97.0f);
						}
						while (	((!Notes[0])&&(note == 0))||
								((!Notes[1])&&(note == 1))||
								((!Notes[2])&&(note == 2))||
								((!Notes[3])&&(note == 3))||
								((!Notes[4])&&(note == 4))||
								((!Notes[5])&&(note == 5))||
								((!Notes[6])&&(note == 6))||
								((!Notes[7])&&(note == 7))||
								((!Notes[8])&&(note == 8))||
								((!Notes[9])&&(note == 9))||
								((!Notes[10])&&(note == 10))||
								((!Notes[11])&&(note == 11))||
								((!Notes[12])&&(note == 12))||
								((!Notes[13])&&(note == 13))||
								((!Notes[14])&&(note == 14))||
								((!Notes[15])&&(note == 15))||
								((!Notes[16])&&(note == 16))||
								((!Notes[17])&&(note == 17))||
								((!Notes[18])&&(note == 18))||
								((!Notes[19])&&(note == 19))||
								((!Notes[20])&&(note == 20))||
								((!Notes[21])&&(note == 21))||
								((!Notes[22])&&(note == 22))||
								((!Notes[23])&&(note == 23))||
								((!Notes[24])&&(note == 24))||
								((!Notes[25])&&(note == 25))||
								((!Notes[26])&&(note == 26))||
								((!Notes[27])&&(note == 27))||
								((!Notes[28])&&(note == 28))||
								((!Notes[29])&&(note == 29))||
								((!Notes[30])&&(note == 30))||
								((!Notes[31])&&(note == 31))||
								((!Notes[32])&&(note == 32))||
								((!Notes[33])&&(note == 33))||
								((!Notes[34])&&(note == 34))||
								((!Notes[35])&&(note == 35))||
								((!Notes[36])&&(note == 36))||
								((!Notes[37])&&(note == 37))||
								((!Notes[38])&&(note == 38))||
								((!Notes[39])&&(note == 39))||
								((!Notes[40])&&(note == 40))||
								((!Notes[41])&&(note == 41))||
								((!Notes[42])&&(note == 42))||
								((!Notes[43])&&(note == 43))||
								((!Notes[44])&&(note == 44))||
								((!Notes[45])&&(note == 45))||
								((!Notes[46])&&(note == 46))||
								((!Notes[47])&&(note == 47))||
								((!Notes[48])&&(note == 48))||
								((!Notes[49])&&(note == 49))||
								((!Notes[50])&&(note == 50))||
								((!Notes[51])&&(note == 51))||
								((!Notes[52])&&(note == 52))||
								((!Notes[53])&&(note == 53))||
								((!Notes[54])&&(note == 54))||
								((!Notes[55])&&(note == 55))||
								((!Notes[56])&&(note == 56))||
								((!Notes[57])&&(note == 57))||
								((!Notes[58])&&(note == 58))||
								((!Notes[59])&&(note == 59))||
								((!Notes[60])&&(note == 60))||
								((!Notes[61])&&(note == 61))||
								((!Notes[62])&&(note == 62))||
								((!Notes[63])&&(note == 63))||
								((!Notes[64])&&(note == 64))||
								((!Notes[65])&&(note == 65))||
								((!Notes[66])&&(note == 66))||
								((!Notes[67])&&(note == 67))||
								((!Notes[68])&&(note == 68))||
								((!Notes[69])&&(note == 69))||
								((!Notes[70])&&(note == 70))||
								((!Notes[71])&&(note == 71))||
								((!Notes[72])&&(note == 72))||
								((!Notes[73])&&(note == 73))||
								((!Notes[74])&&(note == 74))||
								((!Notes[75])&&(note == 75))||
								((!Notes[76])&&(note == 76))||
								((!Notes[77])&&(note == 77))||
								((!Notes[78])&&(note == 78))||
								((!Notes[79])&&(note == 79))||
								((!Notes[80])&&(note == 80))||
								((!Notes[81])&&(note == 81))||
								((!Notes[82])&&(note == 82))||
								((!Notes[83])&&(note == 83))||
								((!Notes[84])&&(note == 84))||
								((!Notes[85])&&(note == 85))||
								((!Notes[86])&&(note == 86))||
								((!Notes[87])&&(note == 87))||
								((!Notes[88])&&(note == 88))||
								((!Notes[89])&&(note == 89))||
								((!Notes[90])&&(note == 90))||
								((!Notes[91])&&(note == 91))||
								((!Notes[92])&&(note == 92))||
								((!Notes[93])&&(note == 93))||
								((!Notes[94])&&(note == 94))||
								((!Notes[95])&&(note == 95))||
								((!Notes[96])&&(note == 96))
							);


						pDoc->Event[l].Note[i] = note;

						/*if(!m_Composition)
						{
							pB = (CComboBox*)m_Parent->GetDlgItem(m_Parent->NoteID[i]);
							pB->SetCurSel(note);
						}*/
					}

					if (m_inverted)
					{
						if (RanDev()<0.5f)
							pDoc->Event[l].Inverted[i] = FALSE;
						else
							pDoc->Event[l].Inverted[i] = TRUE;

						if ((m_matchscl)&&(pDoc->Event[l].Inverted[i]))
						{
							if (RanDev()<0.5f)
								pDoc->Event[l].MatchScale[i] = FALSE;
							else
								pDoc->Event[l].MatchScale[i] = TRUE;
						}
					}


					if (m_scale)
					{
						int scale;
						int j;

						if (m_onlyusrscl)
							scale = USER_DEFINED;
						else
						{

							do
							{
								scale = int(RanDev()*(15.0f+float(ScalesChoice.m_user)));
							}
							while ( ((!Scales[0])&&(scale == MAJOR))||
									((!Scales[1])&&(scale == MINOR_NATURAL))||
									((!Scales[2])&&(scale == MINOR_HARMONIC))||
									((!Scales[3])&&(scale == MINOR_MELODIC))||
									((!Scales[4])&&(scale == LYDIAN))||
									((!Scales[5])&&(scale == MIXOLYDIAN))||
									((!Scales[6])&&(scale == DORIAN))||
									((!Scales[7])&&(scale == PHRYGIAN))||
									((!Scales[8])&&(scale == LOCRIAN))||
									((!Scales[9])&&(scale == OCTATONIC1))||
									((!Scales[10])&&(scale == OCTATONIC2))||
									((!Scales[11])&&(scale == ACCOUSTIC))||
									((!Scales[12])&&(scale == PENTATONIC))||
									((!Scales[13])&&(scale == WHOLETONE))||
									((!Scales[14])&&(scale == CHROMATIC))||
									((!ScalesChoice.m_user)&&(scale == USER_DEFINED))
								  );
						}


						
						pDoc->Event[l].ScaleID[i] = scale;

						if (pDoc->Event[l].pScale[i])
						{
							delete pDoc->Event[l].pScale[i];
							pDoc->Event[l].pScale[i] = NULL;
						}


						switch(scale)
						{
							case MAJOR:
								if ((pDoc->Event[l].Inverted[i])&&(pDoc->Event[l].MatchScale[i]))
								{
									pDoc->Event[l].pScale[i] = new int[Phrygian[0] + 1];
									for(j = 0; j<=Phrygian[0]; j++)
										pDoc->Event[l].pScale[i][j] = Phrygian[j];
									break;
								}
								else
								{
									pDoc->Event[l].pScale[i] = new int[Major[0] + 1];
									for(j = 0; j<=Major[0]; j++)
										pDoc->Event[l].pScale[i][j] = Major[j];
									break;
								}
							case MINOR_NATURAL:
								if ((pDoc->Event[l].Inverted[i])&&(pDoc->Event[l].MatchScale[i]))
								{
									pDoc->Event[l].pScale[i] = new int[Mixolydian[0] + 1];
									for(j = 0; j<=Mixolydian[0]; j++)
										pDoc->Event[l].pScale[i][j] = Mixolydian[j];
									break;
								}
								else
								{		
									pDoc->Event[l].pScale[i] = new int[MinorNatural[0] + 1];
									for(j = 0; j<=MinorNatural[0]; j++)
										pDoc->Event[l].pScale[i][j] = MinorNatural[j];
									break;
								}
							case MINOR_HARMONIC:
								if ((pDoc->Event[l].Inverted[i])&&(pDoc->Event[l].MatchScale[i]))
								{
									pDoc->Event[l].pScale[i] = new int[7 + 1];
									pDoc->Event[l].pScale[i][0] = 7;
									pDoc->Event[l].pScale[i][1] = DO;
									pDoc->Event[l].pScale[i][2] = REb;
									pDoc->Event[l].pScale[i][3] = MI;
									pDoc->Event[l].pScale[i][4] = FA;
									pDoc->Event[l].pScale[i][5] = SOL;
									pDoc->Event[l].pScale[i][6] = LA;
									pDoc->Event[l].pScale[i][7] = SIb;

									break;
								}
								else
								{			
									pDoc->Event[l].pScale[i] = new int[MinorHarmonic[0] + 1];
									for(j = 0; j<=MinorHarmonic[0]; j++)
										pDoc->Event[l].pScale[i][j] = MinorHarmonic[j];
									break;
								}
							case MINOR_MELODIC:
								if ((pDoc->Event[l].Inverted[i])&&(pDoc->Event[l].MatchScale[i]))
								{
									pDoc->Event[l].pScale[i] = new int[7 + 1];
									pDoc->Event[l].pScale[i][0] = 7;
									pDoc->Event[l].pScale[i][1] = DO;
									pDoc->Event[l].pScale[i][2] = REb;
									pDoc->Event[l].pScale[i][3] = MIb;
									pDoc->Event[l].pScale[i][4] = FA;
									pDoc->Event[l].pScale[i][5] = SOL;
									pDoc->Event[l].pScale[i][6] = LA;
									pDoc->Event[l].pScale[i][7] = SIb;

									break;
								}
								else
								{			
									pDoc->Event[l].pScale[i] = new int[MinorMelodic[0] + 1];
									for(j = 0; j<=MinorMelodic[0]; j++)
										pDoc->Event[l].pScale[i][j] = MinorMelodic[j];
									break;
								}
							case LYDIAN:
								if ((pDoc->Event[l].Inverted[i])&&(pDoc->Event[l].MatchScale[i]))
								{
									pDoc->Event[l].pScale[i] = new int[Locrian[0] + 1];
									for(j = 0; j<=Locrian[0]; j++)
										pDoc->Event[l].pScale[i][j] = Locrian[j];
									break;
								}
								else
								{
									pDoc->Event[l].pScale[i] = new int[Lydian[0] + 1];
									for(j = 0; j<=Lydian[0]; j++)
										pDoc->Event[l].pScale[i][j] = Lydian[j];
									break;
								}		
							case MIXOLYDIAN:
								if ((pDoc->Event[l].Inverted[i])&&(pDoc->Event[l].MatchScale[i]))
								{
									pDoc->Event[l].pScale[i] = new int[MinorNatural[0] + 1];
									for(j = 0; j<=MinorNatural[0]; j++)
										pDoc->Event[l].pScale[i][j] = MinorNatural[j];
									break;
								}
								else
								{
									pDoc->Event[l].pScale[i] = new int[Mixolydian[0] + 1];
									for(j = 0; j<=Mixolydian[0]; j++)
										pDoc->Event[l].pScale[i][j] = Mixolydian[j];
									break;
								}		
							case DORIAN:
								pDoc->Event[l].pScale[i] = new int[Dorian[0] + 1];
								for(j = 0; j<=Dorian[0]; j++)
									pDoc->Event[l].pScale[i][j] = Dorian[j];
								break;
							case PHRYGIAN:
								if ((pDoc->Event[l].Inverted[i])&&(pDoc->Event[l].MatchScale[i]))
								{
									pDoc->Event[l].pScale[i] = new int[Major[0] + 1];
									for(j = 0; j<=Major[0]; j++)
										pDoc->Event[l].pScale[i][j] = Major[j];
									break;
								}
								else
								{
									pDoc->Event[l].pScale[i] = new int[Phrygian[0] + 1];
									for(j = 0; j<=Phrygian[0]; j++)
										pDoc->Event[l].pScale[i][j] = Phrygian[j];
									break;
								}		
							case LOCRIAN:
								if ((pDoc->Event[l].Inverted[i])&&(pDoc->Event[l].MatchScale[i]))
								{
									pDoc->Event[l].pScale[i] = new int[Lydian[0] + 1];
									for(j = 0; j<=Lydian[0]; j++)
										pDoc->Event[l].pScale[i][j] = Lydian[j];
									break;
								}
								else
								{
									pDoc->Event[l].pScale[i] = new int[Locrian[0] + 1];
									for(j = 0; j<=Locrian[0]; j++)
										pDoc->Event[l].pScale[i][j] = Locrian[j];
									break;
								}		
							case OCTATONIC1:
								if ((pDoc->Event[l].Inverted[i])&&(pDoc->Event[l].MatchScale[i]))
								{
									pDoc->Event[l].pScale[i] = new int[Octatonic2[0] + 1];
									for(j = 0; j<=Octatonic2[0]; j++)
										pDoc->Event[l].pScale[i][j] = Octatonic2[j];
									break;
								}
								else
								{
									pDoc->Event[l].pScale[i] = new int[Octatonic1[0] + 1];
									for(j = 0; j<=Octatonic1[0]; j++)
										pDoc->Event[l].pScale[i][j] = Octatonic1[j];
									break;
								}		
							case OCTATONIC2:
								if ((pDoc->Event[l].Inverted[i])&&(pDoc->Event[l].MatchScale[i]))
								{
									pDoc->Event[l].pScale[i] = new int[Octatonic1[0] + 1];
									for(j = 0; j<=Octatonic1[0]; j++)
										pDoc->Event[l].pScale[i][j] = Octatonic1[j];
									break;
								}
								else
								{
									pDoc->Event[l].pScale[i] = new int[Octatonic2[0] + 1];
									for(j = 0; j<=Octatonic2[0]; j++)
										pDoc->Event[l].pScale[i][j] = Octatonic2[j];
									break;
								}		
							case ACCOUSTIC:
								if ((pDoc->Event[l].Inverted[i])&&(pDoc->Event[l].MatchScale[i]))
								{
									pDoc->Event[l].pScale[i] = new int[7 + 1];
									pDoc->Event[l].pScale[i][0] = 7;
									pDoc->Event[l].pScale[i][1] = DO;
									pDoc->Event[l].pScale[i][2] = RE;
									pDoc->Event[l].pScale[i][3] = MIb;
									pDoc->Event[l].pScale[i][4] = FA;
									pDoc->Event[l].pScale[i][5] = SOLb;
									pDoc->Event[l].pScale[i][6] = LAb;
									pDoc->Event[l].pScale[i][7] = SIb;

									break;
								}
								else
								{			
									pDoc->Event[l].pScale[i] = new int[Accoustic[0] + 1];
									for(j = 0; j<=Accoustic[0]; j++)
										pDoc->Event[l].pScale[i][j] = Accoustic[j];
									break;
								}
							case WHOLETONE:
								pDoc->Event[l].pScale[i] = new int[Wholetone[0] + 1];
								for(j = 0; j<=Wholetone[0]; j++)
									pDoc->Event[l].pScale[i][j] = Wholetone[j];
								break;
							case PENTATONIC:
								if ((pDoc->Event[l].Inverted[i])&&(pDoc->Event[l].MatchScale[i]))
								{
									pDoc->Event[l].pScale[i] = new int[5 + 1];
									pDoc->Event[l].pScale[i][0] = 5;
									pDoc->Event[l].pScale[i][1] = DO;
									pDoc->Event[l].pScale[i][2] = MIb;
									pDoc->Event[l].pScale[i][3] = FA;
									pDoc->Event[l].pScale[i][4] = LAb;
									pDoc->Event[l].pScale[i][5] = SIb;
									break;
								}
								else
								{			
									pDoc->Event[l].pScale[i] = new int[Pentatonic[0] + 1];
									for(j = 0; j<=Pentatonic[0]; j++)
										pDoc->Event[l].pScale[i][j] = Pentatonic[j];
									break;
								}
							case CHROMATIC:
								pDoc->Event[l].pScale[i] = new int[Chromatic[0] + 1];
								for(j = 0; j<=Chromatic[0]; j++)
									pDoc->Event[l].pScale[i][j] = Chromatic[j];
								break;
							case USER_DEFINED:

								int x = 1;

								if (m_userscl)
								{
									CV1UsrSclDlg V1UsrScl;

									if (m_exactORatmost == 0)
									{

									do 
									{

										for(int pinga=0; pinga<12; pinga++)
											V1UsrScl.m_bUserScale[pinga] = FALSE;

										V1UsrScl.m_numnotes = 0;

										V1UsrScl.m_C_ON = FALSE;
										V1UsrScl.m_Db_ON = FALSE;
										V1UsrScl.m_D_ON = FALSE;
										V1UsrScl.m_Eb_ON = FALSE;
										V1UsrScl.m_E_ON = FALSE;
										V1UsrScl.m_F_ON = FALSE;
										V1UsrScl.m_Gb_ON = FALSE;
										V1UsrScl.m_G_ON = FALSE;
										V1UsrScl.m_Ab_ON = FALSE;
										V1UsrScl.m_A_ON = FALSE;
										V1UsrScl.m_Bb_ON = FALSE;
										V1UsrScl.m_B_ON = FALSE;

										if (RanDev()>=0.5f)
										{
											V1UsrScl.m_C_ON = TRUE;
											V1UsrScl.m_bUserScale[0] = TRUE;
											V1UsrScl.m_numnotes++;
										}
										if (RanDev()>=0.5f)
										{
											V1UsrScl.m_Db_ON = TRUE;
											V1UsrScl.m_bUserScale[1] = TRUE;
											V1UsrScl.m_numnotes++;
										}
										if (RanDev()>=0.5f)
										{
											V1UsrScl.m_D_ON = TRUE;
											V1UsrScl.m_bUserScale[2] = TRUE;
											V1UsrScl.m_numnotes++;
										}
										if (RanDev()>=0.5f)
										{
											V1UsrScl.m_Eb_ON = TRUE;
											V1UsrScl.m_bUserScale[3] = TRUE;
											V1UsrScl.m_numnotes++;
										}
										if (RanDev()>=0.5f)
										{
											V1UsrScl.m_E_ON = TRUE;
											V1UsrScl.m_bUserScale[4] = TRUE;
											V1UsrScl.m_numnotes++;
										}
										if (RanDev()>=0.5f)
										{
											V1UsrScl.m_F_ON = TRUE;
											V1UsrScl.m_bUserScale[5] = TRUE;
											V1UsrScl.m_numnotes++;
										}
										if (RanDev()>=0.5f)
										{
											V1UsrScl.m_Gb_ON = TRUE;
											V1UsrScl.m_bUserScale[6] = TRUE;
											V1UsrScl.m_numnotes++;
										}
										if (RanDev()>=0.5f)
										{
											V1UsrScl.m_G_ON = TRUE;
											V1UsrScl.m_bUserScale[7] = TRUE;
											V1UsrScl.m_numnotes++;
										}
										if (RanDev()>=0.5f)
										{
											V1UsrScl.m_Ab_ON = TRUE;
											V1UsrScl.m_bUserScale[8] = TRUE;
											V1UsrScl.m_numnotes++;
										}
										if (RanDev()>=0.5f)
										{
											V1UsrScl.m_A_ON = TRUE;
											V1UsrScl.m_bUserScale[9] = TRUE;
											V1UsrScl.m_numnotes++;
										}
										if (RanDev()>=0.5f)
										{
											V1UsrScl.m_Bb_ON = TRUE;
											V1UsrScl.m_bUserScale[10] = TRUE;
											V1UsrScl.m_numnotes++;
										}
										if (RanDev()>=0.5f)
										{
											V1UsrScl.m_B_ON = TRUE;
											V1UsrScl.m_bUserScale[11] = TRUE;
											V1UsrScl.m_numnotes++;
										}
									}
									while ((V1UsrScl.m_numnotes<m_exact)||(V1UsrScl.m_numnotes>m_exact));
									}

									if (m_exactORatmost == 1)
									{

									do 
									{

										for(int pinga=0; pinga<12; pinga++)
											V1UsrScl.m_bUserScale[pinga] = FALSE;

										V1UsrScl.m_numnotes = 0;

										V1UsrScl.m_C_ON = FALSE;
										V1UsrScl.m_Db_ON = FALSE;
										V1UsrScl.m_D_ON = FALSE;
										V1UsrScl.m_Eb_ON = FALSE;
										V1UsrScl.m_E_ON = FALSE;
										V1UsrScl.m_F_ON = FALSE;
										V1UsrScl.m_Gb_ON = FALSE;
										V1UsrScl.m_G_ON = FALSE;
										V1UsrScl.m_Ab_ON = FALSE;
										V1UsrScl.m_A_ON = FALSE;
										V1UsrScl.m_Bb_ON = FALSE;
										V1UsrScl.m_B_ON = FALSE;

										if (RanDev()>=0.5f)
										{
											V1UsrScl.m_C_ON = TRUE;
											V1UsrScl.m_bUserScale[0] = TRUE;
											V1UsrScl.m_numnotes++;
										}
										if (RanDev()>=0.5f)
										{
											V1UsrScl.m_Db_ON = TRUE;
											V1UsrScl.m_bUserScale[1] = TRUE;
											V1UsrScl.m_numnotes++;
										}
										if (RanDev()>=0.5f)
										{
											V1UsrScl.m_D_ON = TRUE;
											V1UsrScl.m_bUserScale[2] = TRUE;
											V1UsrScl.m_numnotes++;
										}
										if (RanDev()>=0.5f)
										{
											V1UsrScl.m_Eb_ON = TRUE;
											V1UsrScl.m_bUserScale[3] = TRUE;
											V1UsrScl.m_numnotes++;
										}
										if (RanDev()>=0.5f)
										{
											V1UsrScl.m_E_ON = TRUE;
											V1UsrScl.m_bUserScale[4] = TRUE;
											V1UsrScl.m_numnotes++;
										}
										if (RanDev()>=0.5f)
										{
											V1UsrScl.m_F_ON = TRUE;
											V1UsrScl.m_bUserScale[5] = TRUE;
											V1UsrScl.m_numnotes++;
										}
										if (RanDev()>=0.5f)
										{
											V1UsrScl.m_Gb_ON = TRUE;
											V1UsrScl.m_bUserScale[6] = TRUE;
											V1UsrScl.m_numnotes++;
										}
										if (RanDev()>=0.5f)
										{
											V1UsrScl.m_G_ON = TRUE;
											V1UsrScl.m_bUserScale[7] = TRUE;
											V1UsrScl.m_numnotes++;
										}
										if (RanDev()>=0.5f)
										{
											V1UsrScl.m_Ab_ON = TRUE;
											V1UsrScl.m_bUserScale[8] = TRUE;
											V1UsrScl.m_numnotes++;
										}
										if (RanDev()>=0.5f)
										{
											V1UsrScl.m_A_ON = TRUE;
											V1UsrScl.m_bUserScale[9] = TRUE;
											V1UsrScl.m_numnotes++;
										}
										if (RanDev()>=0.5f)
										{
											V1UsrScl.m_Bb_ON = TRUE;
											V1UsrScl.m_bUserScale[10] = TRUE;
											V1UsrScl.m_numnotes++;
										}
										if (RanDev()>=0.5f)
										{
											V1UsrScl.m_B_ON = TRUE;
											V1UsrScl.m_bUserScale[11] = TRUE;
											V1UsrScl.m_numnotes++;
										}
									}
									while ((V1UsrScl.m_numnotes<1)||(V1UsrScl.m_numnotes>m_atmost));
									}



									if (pDoc->Event[l].pUserScale[i])
									{
										delete pDoc->Event[l].pUserScale[i];
										pDoc->Event[l].pUserScale[i] = NULL;
									}
									pDoc->Event[l].pUserScale[i] = new int[V1UsrScl.m_numnotes + 1];
									pDoc->Event[l].pUserScale[i][0] = V1UsrScl.m_numnotes;
									for(j = 0; j<12; j++)
										if (V1UsrScl.m_bUserScale[j])
											pDoc->Event[l].pUserScale[i][x++] = j;
									m_Parent->SortScale(pDoc->Event[l].pUserScale[i], pDoc->Event[l].pUserScale[i][0]);
									m_Parent->TransposeToC(pDoc->Event[l].pUserScale[i], pDoc->Event[l].pUserScale[i][0]);
								}

								if ((pDoc->Event[l].Inverted[i])&&(pDoc->Event[l].MatchScale[i]))
								{
									pDoc->Event[l].pScale[i] = new int[pDoc->Event[l].pUserScale[i][0] + 1];
									pDoc->Event[l].pScale[i][0] = pDoc->Event[l].pUserScale[i][0];
									for(j = 1; j<=pDoc->Event[l].pUserScale[i][0]; j++)
										pDoc->Event[l].pScale[i][j] = (12-pDoc->Event[l].pUserScale[i][j])*(pDoc->Event[l].pUserScale[i][j]!=0);
									m_Parent->SortScale(pDoc->m_pScale[i], pDoc->m_pScale[i][0]);
									break;
								}
								else
								{
									pDoc->Event[l].pScale[i] = new int[pDoc->Event[l].pUserScale[i][0] + 1];
									pDoc->Event[l].pScale[i][0] = pDoc->Event[l].pUserScale[i][0];
									for(j = 1; j<=pDoc->Event[l].pUserScale[i][0]; j++)
										pDoc->Event[l].pScale[i][j] = pDoc->Event[l].pUserScale[i][j];
									break;
								}




						}

						/*if(!m_Composition)

						{
							pB = (CComboBox*)m_Parent->GetDlgItem(m_Parent->ScaleID[i]);
							pB->SetCurSel(scale);
						}*/

					}


					if (m_noteval)
					{

						int noteval;

						do
						{
							noteval = int(RanDev()*12.0f);
						}
						while(	((!NoteValues.m_whole)&&(noteval == WHOLE))||
								((!NoteValues.m_half)&&(noteval == HALF))||
								((!NoteValues.m_quarter)&&(noteval == QUARTER))||
								((!NoteValues.m_8)&&(noteval == EIGHT))||
								((!NoteValues.m_16)&&(noteval == SIXTEENTH))||
								((!NoteValues.m_32)&&(noteval == THIRTYSECOND))||
								((!NoteValues.m_dottedwhole)&&(noteval == WHOLE_DOTTED))||
								((!NoteValues.m_dottedhalf)&&(noteval == HALF_DOTTED))||
								((!NoteValues.m_dottedquarter)&&(noteval == QUARTER_DOTTED))||
								((!NoteValues.m_dotted8)&&(noteval == EIGHTH_DOTTED))||
								((!NoteValues.m_dotted16)&&(noteval == SIXTEENTH_DOTTED))||
								((!NoteValues.m_other)&&(noteval == OTHER))
							 );


						pDoc->Event[l].NoteValueID[i] = noteval;

						switch(noteval)
						{
							case WHOLE:
								pDoc->Event[l].NoteValue[i] = 32.0f;
								break;
							case HALF:
								pDoc->Event[l].NoteValue[i] = 16.0f;
								break;
							case QUARTER:
								pDoc->Event[l].NoteValue[i] = 8.0f;
								break;
							case EIGHT:
								pDoc->Event[l].NoteValue[i] = 4.0f;
								break;
							case SIXTEENTH:
								pDoc->Event[l].NoteValue[i] = 2.0f;
								break;
							case THIRTYSECOND:
								pDoc->Event[l].NoteValue[i] = 1.0f;
								break;
							case WHOLE_DOTTED:
								pDoc->Event[l].NoteValue[i] = 48.00f;
								break;
							case HALF_DOTTED:
								pDoc->Event[l].NoteValue[i] = 24.00f;
								break;
							case QUARTER_DOTTED:
								pDoc->Event[l].NoteValue[i] = 12.00f;
								break;
							case EIGHTH_DOTTED:
								pDoc->Event[l].NoteValue[i] = 6.0f;
								break;
							case SIXTEENTH_DOTTED:
								pDoc->Event[l].NoteValue[i] = 3.0f;
								break;
							case OTHER:
								//CString nval;
								int newval = int((RanDev()*(NoteValues.othermax-NoteValues.othermin))+NoteValues.othermin);
								//m_Parent->m_NoteVal[i].m_NewVal = (long)newval;
								//nval.Format("Other (%d)", newval);
								//pB = (CComboBox*)m_Parent->GetDlgItem(m_Parent->NoteValueID[i]);
								//pB->DeleteString(OTHER);
								//pB->AddString(nval);
								pDoc->Event[l].NoteValue[i] = (float)newval;
								break;

						}
						/*if(!m_Composition)
						{
							pB = (CComboBox*)m_Parent->GetDlgItem(m_Parent->NoteValueID[i]);
							pB->SetCurSel(noteval);
						}*/
					}

					if (m_instrument)
					{
						int ins;

						do
						{
							ins = int(RanDev()*128.0f);
						}
						while (	((!Instrument[0])&&(ins == 0))||
								((!Instrument[1])&&(ins == 1))||
								((!Instrument[2])&&(ins == 2))||
								((!Instrument[3])&&(ins == 3))||
								((!Instrument[4])&&(ins == 4))||
								((!Instrument[5])&&(ins == 5))||
								((!Instrument[6])&&(ins == 6))||
								((!Instrument[7])&&(ins == 7))||
								((!Instrument[8])&&(ins == 8))||
								((!Instrument[9])&&(ins == 9))||
								((!Instrument[10])&&(ins == 10))||
								((!Instrument[11])&&(ins == 11))||
								((!Instrument[12])&&(ins == 12))||
								((!Instrument[13])&&(ins == 13))||
								((!Instrument[14])&&(ins == 14))||
								((!Instrument[15])&&(ins == 15))||
								((!Instrument[16])&&(ins == 16))||
								((!Instrument[17])&&(ins == 17))||
								((!Instrument[18])&&(ins == 18))||
								((!Instrument[19])&&(ins == 19))||
								((!Instrument[20])&&(ins == 20))||
								((!Instrument[21])&&(ins == 21))||
								((!Instrument[22])&&(ins == 22))||
								((!Instrument[23])&&(ins == 23))||
								((!Instrument[24])&&(ins == 24))||
								((!Instrument[25])&&(ins == 25))||
								((!Instrument[26])&&(ins == 26))||
								((!Instrument[27])&&(ins == 27))||
								((!Instrument[28])&&(ins == 28))||
								((!Instrument[29])&&(ins == 29))||
								((!Instrument[30])&&(ins == 30))||
								((!Instrument[31])&&(ins == 31))||
								((!Instrument[32])&&(ins == 32))||
								((!Instrument[33])&&(ins == 33))||
								((!Instrument[34])&&(ins == 34))||
								((!Instrument[35])&&(ins == 35))||
								((!Instrument[36])&&(ins == 36))||
								((!Instrument[37])&&(ins == 37))||
								((!Instrument[38])&&(ins == 38))||
								((!Instrument[39])&&(ins == 39))||
								((!Instrument[40])&&(ins == 40))||
								((!Instrument[41])&&(ins == 41))||
								((!Instrument[42])&&(ins == 42))||
								((!Instrument[43])&&(ins == 43))||
								((!Instrument[44])&&(ins == 44))||
								((!Instrument[45])&&(ins == 45))||
								((!Instrument[46])&&(ins == 46))||
								((!Instrument[47])&&(ins == 47))||
								((!Instrument[48])&&(ins == 48))||
								((!Instrument[49])&&(ins == 49))||
								((!Instrument[50])&&(ins == 50))||
								((!Instrument[51])&&(ins == 51))||
								((!Instrument[52])&&(ins == 52))||
								((!Instrument[53])&&(ins == 53))||
								((!Instrument[54])&&(ins == 54))||
								((!Instrument[55])&&(ins == 55))||
								((!Instrument[56])&&(ins == 56))||
								((!Instrument[57])&&(ins == 57))||
								((!Instrument[58])&&(ins == 58))||
								((!Instrument[59])&&(ins == 59))||
								((!Instrument[60])&&(ins == 60))||
								((!Instrument[61])&&(ins == 61))||
								((!Instrument[62])&&(ins == 62))||
								((!Instrument[63])&&(ins == 63))||
								((!Instrument[64])&&(ins == 64))||
								((!Instrument[65])&&(ins == 65))||
								((!Instrument[66])&&(ins == 66))||
								((!Instrument[67])&&(ins == 67))||
								((!Instrument[68])&&(ins == 68))||
								((!Instrument[69])&&(ins == 69))||
								((!Instrument[70])&&(ins == 70))||
								((!Instrument[71])&&(ins == 71))||
								((!Instrument[72])&&(ins == 72))||
								((!Instrument[73])&&(ins == 73))||
								((!Instrument[74])&&(ins == 74))||
								((!Instrument[75])&&(ins == 75))||
								((!Instrument[76])&&(ins == 76))||
								((!Instrument[77])&&(ins == 77))||
								((!Instrument[78])&&(ins == 78))||
								((!Instrument[79])&&(ins == 79))||
								((!Instrument[80])&&(ins == 80))||
								((!Instrument[81])&&(ins == 81))||
								((!Instrument[82])&&(ins == 82))||
								((!Instrument[83])&&(ins == 83))||
								((!Instrument[84])&&(ins == 84))||
								((!Instrument[85])&&(ins == 85))||
								((!Instrument[86])&&(ins == 86))||
								((!Instrument[87])&&(ins == 87))||
								((!Instrument[88])&&(ins == 88))||
								((!Instrument[89])&&(ins == 89))||
								((!Instrument[90])&&(ins == 90))||
								((!Instrument[91])&&(ins == 91))||
								((!Instrument[92])&&(ins == 92))||
								((!Instrument[93])&&(ins == 93))||
								((!Instrument[94])&&(ins == 94))||
								((!Instrument[95])&&(ins == 95))||
								((!Instrument[96])&&(ins == 96))||
								((!Instrument[97])&&(ins == 97))||
								((!Instrument[98])&&(ins == 98))||
								((!Instrument[99])&&(ins == 99))||
								((!Instrument[100])&&(ins == 100 ))||
								((!Instrument[101])&&(ins == 101))||
								((!Instrument[102])&&(ins == 102))||
								((!Instrument[103])&&(ins == 103))||
								((!Instrument[104])&&(ins == 104))||
								((!Instrument[105])&&(ins == 105))||
								((!Instrument[106])&&(ins == 106))||
								((!Instrument[107])&&(ins == 107))||
								((!Instrument[108])&&(ins == 108))||
								((!Instrument[109])&&(ins == 109))||
								((!Instrument[110])&&(ins == 110))||
								((!Instrument[111])&&(ins == 111))||
								((!Instrument[112])&&(ins == 112))||
								((!Instrument[113])&&(ins == 113))||
								((!Instrument[114])&&(ins == 114))||
								((!Instrument[115])&&(ins == 115))||
								((!Instrument[116])&&(ins == 116))||
								((!Instrument[117])&&(ins == 117))||
								((!Instrument[118])&&(ins == 118))||
								((!Instrument[119])&&(ins == 119))||
								((!Instrument[120])&&(ins == 120))||
								((!Instrument[121])&&(ins == 121))||
								((!Instrument[122])&&(ins == 122))||
								((!Instrument[123])&&(ins == 123))||
								((!Instrument[124])&&(ins == 124))||
								((!Instrument[125])&&(ins == 125))||
								((!Instrument[126])&&(ins == 126))||
								((!Instrument[127])&&(ins == 127))
							);
								
						pDoc->Event[l].Instrument[i] = ins;
						pDoc->Event[l].Channel[i] = i;
					}

					if (m_percussion)
					{
						if ((!m_instrument)||(RanDev()<0.3333333333))
						{
							int perc;

							do
							{
								perc = int(RanDev()*47.0f);
							}
							while (	((!PercussionInstrument[0])&&(perc == 0))||
									((!PercussionInstrument[1])&&(perc == 1))||
									((!PercussionInstrument[2])&&(perc == 2))||
									((!PercussionInstrument[3])&&(perc == 3))||
									((!PercussionInstrument[4])&&(perc == 4))||
									((!PercussionInstrument[5])&&(perc == 5))||
									((!PercussionInstrument[6])&&(perc == 6))||
									((!PercussionInstrument[7])&&(perc == 7))||
									((!PercussionInstrument[8])&&(perc == 8))||
									((!PercussionInstrument[9])&&(perc == 9))||
									((!PercussionInstrument[10])&&(perc == 10))||
									((!PercussionInstrument[11])&&(perc == 11))||
									((!PercussionInstrument[12])&&(perc == 12))||
									((!PercussionInstrument[13])&&(perc == 13))||
									((!PercussionInstrument[14])&&(perc == 14))||
									((!PercussionInstrument[15])&&(perc == 15))||
									((!PercussionInstrument[16])&&(perc == 16))||
									((!PercussionInstrument[17])&&(perc == 17))||
									((!PercussionInstrument[18])&&(perc == 18))||
									((!PercussionInstrument[19])&&(perc == 19))||
									((!PercussionInstrument[20])&&(perc == 20))||
									((!PercussionInstrument[21])&&(perc == 21))||
									((!PercussionInstrument[22])&&(perc == 22))||
									((!PercussionInstrument[23])&&(perc == 23))||
									((!PercussionInstrument[24])&&(perc == 24))||
									((!PercussionInstrument[25])&&(perc == 25))||
									((!PercussionInstrument[26])&&(perc == 26))||
									((!PercussionInstrument[27])&&(perc == 27))||
									((!PercussionInstrument[28])&&(perc == 28))||
									((!PercussionInstrument[29])&&(perc == 29))||
									((!PercussionInstrument[30])&&(perc == 30))||
									((!PercussionInstrument[31])&&(perc == 31))||
									((!PercussionInstrument[32])&&(perc == 32))||
									((!PercussionInstrument[33])&&(perc == 33))||
									((!PercussionInstrument[34])&&(perc == 34))||
									((!PercussionInstrument[35])&&(perc == 35))||
									((!PercussionInstrument[36])&&(perc == 36))||
									((!PercussionInstrument[37])&&(perc == 37))||
									((!PercussionInstrument[38])&&(perc == 38))||
									((!PercussionInstrument[39])&&(perc == 39))||
									((!PercussionInstrument[40])&&(perc == 40))||
									((!PercussionInstrument[41])&&(perc == 41))||
									((!PercussionInstrument[42])&&(perc == 42))||
									((!PercussionInstrument[43])&&(perc == 43))||
									((!PercussionInstrument[44])&&(perc == 44))||
									((!PercussionInstrument[45])&&(perc == 45))||
									((!PercussionInstrument[46])&&(perc == 46))
								);
									
							pDoc->Event[l].Percussion[i] = perc;
							pDoc->Event[l].Channel[i] = PercussionChannel;
						}
					}


					if (m_mode)
					{
						if (RanDev()<0.5f)
							pDoc->Event[l].Mode[i] = 0;
						else
							pDoc->Event[l].Mode[i] = 1;
					}

					if (m_dynamic)
					{
						int dyn;

						do
						{
							dyn = int(RanDev()*6.0f);
						}
						while(	((!Dynamics.m_pp)&&(dyn == PP))||
								((!Dynamics.m_p)&&(dyn == P))||
								((!Dynamics.m_mp)&&(dyn == MP))||
								((!Dynamics.m_mf)&&(dyn == MF))||
								((!Dynamics.m_f)&&(dyn == F))||
								((!Dynamics.m_ff)&&(dyn == FF))
							);

						pDoc->m_DynChoice[i] = dyn;

						switch(dyn)
						{
							case PP:
								pDoc->Event[l].Dynamic[i] = PIANISSIMO;
								break;
							case P:
								pDoc->Event[l].Dynamic[i] = PIANO;
								break;
							case MP:
								pDoc->Event[l].Dynamic[i] = MEZZO_PIANO;
								break;
							case MF:
								pDoc->Event[l].Dynamic[i] = MEZZO_FORTE;
								break;
							case F:
								pDoc->Event[l].Dynamic[i] = FORTE;
								break;
							case FF:
								pDoc->Event[l].Dynamic[i] = FORTISSIMO;
								break;
						}
					}


					if (m_pan)
					{
						float pan = RanDev();
						if ((pan>=0.0f)&&(pan<0.333333f))
							pDoc->Event[l].Pan[i] = -64;
						if ((pan>=0.333333f)&&(pan<0.666666f))
							pDoc->Event[l].Pan[i] = 0;
						if ((pan>=0.666666f)&&(pan<=1.0f))
							pDoc->Event[l].Pan[i] = 63;
					}

				} //if (m_VoiceOn[i]||(pDoc->Event[l].IsActive[i]&&m_AllActiveVoices))

			}//for(int i = VOICE1; i<MAXVOICES; i++)
			if (m_tempo)
			{
				int tpo = int(RanDev()*320.0f)+20;
				pDoc->Event[l].Tempo = tpo;
			}
		}//if (IsSelected[l])

		pSt->SetWindowText("(Done)");

		delete selectedItems;

        selectedItems = NULL;

	
}

void CRandDlg::OnSelectallevents() 
{
	CFractMusDoc* pDoc = m_Parent->GetDocument();
	CListBox* pList = (CListBox*)GetDlgItem(IDC_EVENTS_LIST);

	pList->SelItemRange(TRUE, 0, pDoc->m_NumEvents);
	
}

void CRandDlg::OnSelectnoneevents() 
{
	CFractMusDoc* pDoc = m_Parent->GetDocument();
	CListBox* pList = (CListBox*)GetDlgItem(IDC_EVENTS_LIST);

	pList->SelItemRange(FALSE, 0, pDoc->m_NumEvents);
	
}

void CRandDlg::OnAllactivevoicesChk() 
{
	UpdateData(TRUE);

	if (m_AllActiveVoices)
	{
		m_onoff = FALSE;
		UpdateData(FALSE);
		GetDlgItem(IDC_NOSILENCE)->EnableWindow(m_onoff);
		GetDlgItem(IDC_NUMVOICES_ED)->EnableWindow(m_onoff&&(!m_allowSilence));
		GetDlgItem(IDC_NUMVOICES_ST)->EnableWindow(m_onoff&&(!m_allowSilence));
	}
	
}

void CRandDlg::OnPercussionChk() 
{
	UpdateData(TRUE);

	if(m_percussion)
	{
		CPercussionInstrumentDlg dlg;

		dlg.pRand = this;

		dlg.DoModal();

		for(int i = 0; i<47; i++)
			PercussionInstrument[i] = FALSE;

		for(i = 0; i<47; i++)
			PercussionInstrument[i] = dlg.IsSelected[i];
	}	
}

void CRandDlg::OnPresets() 
{
	if(!UpdateData())
		return;

	RandomizerParameters.DoModal();
	
}

void CRandDlg::OnAtmostRd() 
{
    UpdateData(TRUE);
	
	GetDlgItem(IDC_EXACT_RD)->EnableWindow(m_scale&&ScalesChoice.m_user&&m_userscl);
	GetDlgItem(IDC_EXACT_ED)->EnableWindow(m_scale&&ScalesChoice.m_user&&(m_exactORatmost == 0)&&m_userscl);
	GetDlgItem(IDC_EXACT_ST)->EnableWindow(m_scale&&ScalesChoice.m_user&&(m_exactORatmost == 0)&&m_userscl);
	GetDlgItem(IDC_ATMOST_RD)->EnableWindow(m_scale&&ScalesChoice.m_user&&m_userscl);
	GetDlgItem(IDC_ATMOST_ED)->EnableWindow(m_scale&&ScalesChoice.m_user&&(m_exactORatmost == 1)&&m_userscl);
	GetDlgItem(IDC_ATMOST_ST)->EnableWindow(m_scale&&ScalesChoice.m_user&&(m_exactORatmost == 1)&&m_userscl);
	
}

void CRandDlg::OnExactRd() 
{
    UpdateData(TRUE);
	
	GetDlgItem(IDC_EXACT_RD)->EnableWindow(m_scale&&ScalesChoice.m_user&&m_userscl);
	GetDlgItem(IDC_EXACT_ED)->EnableWindow(m_scale&&ScalesChoice.m_user&&(m_exactORatmost == 0)&&m_userscl);
	GetDlgItem(IDC_EXACT_ST)->EnableWindow(m_scale&&ScalesChoice.m_user&&(m_exactORatmost == 0)&&m_userscl);
	GetDlgItem(IDC_ATMOST_RD)->EnableWindow(m_scale&&ScalesChoice.m_user&&m_userscl);
	GetDlgItem(IDC_ATMOST_ED)->EnableWindow(m_scale&&ScalesChoice.m_user&&(m_exactORatmost == 1)&&m_userscl);
	GetDlgItem(IDC_ATMOST_ST)->EnableWindow(m_scale&&ScalesChoice.m_user&&(m_exactORatmost == 1)&&m_userscl);
}
