
/*************************************************************************


Copyright 1998, 1999, 2000 Gustavo Diaz-Jerez


This file is part of FractMus.

    FractMus is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FractMus is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FractMus; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


**************************************************************************/


////////////////////////////////////////////
// scales.h definition of notes and scales//
////////////////////////////////////////////

#ifndef DEFS_H
#define DEFS_H


#define MAXVOICES 16



#define MAJOR			0
#define MINOR_NATURAL	1
#define MINOR_HARMONIC	2
#define MINOR_MELODIC	3
#define LYDIAN			4
#define MIXOLYDIAN		5
#define DORIAN			6
#define PHRYGIAN		7
#define LOCRIAN			8
#define OCTATONIC1		9
#define OCTATONIC2		10
#define ACCOUSTIC		11
#define WHOLETONE		12
#define PENTATONIC		13
#define CHROMATIC		14
#define USER_DEFINED	15


#define WHOLE				0
#define HALF				1
#define QUARTER				2
#define EIGHT				3
#define SIXTEENTH			4
#define THIRTYSECOND		5
#define WHOLE_DOTTED		6
#define HALF_DOTTED			7
#define QUARTER_DOTTED		8
#define EIGHTH_DOTTED		9
#define SIXTEENTH_DOTTED	10
#define OTHER				11

#define ONE_OCT			0
#define TWO_OCT			1
#define THREE_OCT		2
#define FOUR_OCT		3

#define ROUND_UP		0
#define ROUND_DOWN		1



#define VOICE1			0
#define VOICE2			1
#define VOICE3			2
#define VOICE4			3
#define VOICE5			4
#define VOICE6			5
#define VOICE7			6
#define VOICE8			7
#define VOICE9			8
#define VOICE10			9
#define VOICE11			10
#define VOICE12			11
#define VOICE13			12
#define VOICE14			13
#define VOICE15			14
#define VOICE16			15


#define MORSE_THUE			0
#define LOGMAP				1
#define ONE_OVER_F_NOISE	2
#define HENON				3
#define HOPALONG			4
#define MARTIN				5
#define GINGERBREADMAN		6
#define LORENTZ				7
#define EARTHWORM			8
#define RANDOM				9
#define HAILSTONE			10
#define WOLFRAM				11


#define CURRENT_SETTINGS	0
#define COMPOSITION			1

#define FORTISSIMO		127
#define FORTE			110
#define MEZZO_FORTE		88
#define MEZZO_PIANO		66
#define PIANO			44
#define PIANISSIMO		22

#define PP				0
#define P				1
#define MP				2
#define MF				3
#define F				4
#define FF				5


 
enum { DO, REb, RE, MIb, MI, FA, SOLb, SOL, LAb, LA, SIb, SI };


const int Major[] =			{7, DO, RE, MI, FA, SOL, LA, SI};
const int MinorNatural[] =	{7, DO, RE, MIb, FA, SOL, LAb, SIb};
const int MinorHarmonic[] =	{7, DO, RE, MIb, FA, SOL, LAb, SI};
const int MinorMelodic[] =	{7, DO, RE, MIb, FA, SOL, LA, SI};
const int Lydian[] =		{7, DO, RE, MI, SOLb, SOL, LA, SI};
const int Mixolydian[] =	{7, DO, RE, MI, FA, SOL, LA, SIb};
const int Dorian[] =		{7, DO, RE, MIb, FA, SOL, LA, SIb};
const int Phrygian[] =		{7, DO, REb, MIb, FA, SOL, LAb, SIb};
const int Locrian[] =		{7, DO, REb, MIb, FA, SOLb, LAb, SIb};
const int Octatonic1[] =	{8, DO, RE, MIb, FA, SOLb, LAb, LA, SI};
const int Octatonic2[] =	{8, DO, REb, MIb, MI, SOLb, SOL, LA, SIb};
const int Accoustic[] =		{7, DO, RE, MI, SOLb, SOL, LA, SIb};
const int Wholetone[] =		{6, DO, RE, MI, SOLb, LAb, SIb};
const int Pentatonic[] =	{5, DO, RE, MI, SOL, LA};
const int Chromatic[] =		{12, DO, REb, RE, MIb, MI, FA, SOLb, SOL,
								 LAb, LA, SIb, SI};

const long IM1 = 2147483563L;
const long IM2 = 2147483399L;
const long IMM1 = IM1 - 1L;
const long IA1 = 40014L;
const long IA2 = 40692L;
const long IQ1 = 53668L;
const long IQ2 = 52774L;
const long IR1 = 12211L;
const long IR2 = 3791L;
const long NTAB = 32L;
const long NDIV = 1L+IMM1/long(NTAB);

const float RNMX = 1.0f - 0.0000001f;
const float AM = 1.0f/2147483563.0f;




#endif
