
/*************************************************************************


Copyright 1998, 1999, 2000 Gustavo Diaz-Jerez


This file is part of FractMus.

    FractMus is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FractMus is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FractMus; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


**************************************************************************/


// FractMusView.cpp : implementation of the CFractMusView class
//

#include "stdafx.h"
#include "FractMus.h"
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>

#include "FractMusDoc.h"
#include "FractMusView.h"
#include "midifiles.h"
#include "MidiDevDlg.h"
#include "RoundDlg.h"
#include "MorseThueDlg.h"
#include "LogMapDlg.h"
#include "FNoiseDlg.h"
//#include "RandDlg.h"
#include "HenonDlg.h"
#include "HopalongMartinDlg.h"
#include "MartinDlg.h"
#include "GingerDlg.h"
#include "LorentzDlg.h"
#include "EarthWormDlg.h"
#include "RandomAlgorithmDlg.h"
#include "WolframDlg.h"
#include "HailstoneDlg.h"
#include "PercussionDlg.h"
//#include "ChangesDlg.h"
#include "ChangePassword.h"
#include "UnlockDocumentDlg.h"
#include "RoundEventCountersDlg.h"
#include "InstrumentationDlg.h"
#include "CommentsDlg.h"


#include <math.h>
#include <time.h>
//#include <string.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CFractMusView

extern BOOL m_Playing;
BOOL IsDocumentLocked;
BOOL ShowTips = TRUE;
const BOOL FractMusREGISTERED = TRUE;  ///// Change to FALSE for UNREGISTERED

extern HMIDIOUT midiOut;
extern int DevChoice;

extern const char* InstrumentNames[128];
extern const char* PercussionNames[47];
extern const char* dyn[6];
extern const char* AlgorithmNames[12];
extern const char* Pitches[12];
int Tempo;

int PercussionChannel = 9;

long delay = 0L;
BOOL waiting = FALSE;
long Seed;



static long Mf_numbyteswritten = 0L;


FILE* SaveMidiFile;


int myputc(int c) {return(putc(c,SaveMidiFile));}


//void mf_write_track_chunk(int which_track,FILE* fp);

void mf_write_header_chunk(int format,int ntracks,int division);

int mf_write_midi_event(unsigned long delta_time,
						unsigned int type,
						unsigned int chan,
						unsigned char *data,
						unsigned long size);

int mf_write_meta_event(unsigned long delta_time,
						unsigned char type,
						unsigned char *data,
						unsigned long size);

void mf_write_tempo(unsigned long tempo, unsigned long delta_time);

void WriteVarLen(unsigned long value);

void write32bit(unsigned long data);

void write16bit(int data);

int eputc(unsigned char c);




float LogMap[MAXVOICES];
float LogMap2[MAXVOICES];
float FNoise[MAXVOICES];
float HenonX[MAXVOICES];
float HenonY[MAXVOICES];
float HopalongX[MAXVOICES];
float HopalongY[MAXVOICES];
float MartinX[MAXVOICES];
float MartinY[MAXVOICES];
float GingerX[MAXVOICES];
float GingerY[MAXVOICES];
float LorentzX[MAXVOICES];
unsigned long EarthWorm[MAXVOICES];

BYTE* WolframArray1[MAXVOICES];
BYTE* WolframArray2[MAXVOICES];

__int64 HailStoneNumber[MAXVOICES];

int count1[MAXVOICES];
int count2[MAXVOICES];
int count3[MAXVOICES];
int count4[MAXVOICES];
int count5[MAXVOICES];
int count6[MAXVOICES];
int count7[MAXVOICES];
int count8[MAXVOICES];
int count9[MAXVOICES];
int count10[MAXVOICES];
int count11[MAXVOICES];
int count12[MAXVOICES];

int __fastcall Morse_Thue(int voice, long counter, long test, const int* scale, 
				int numocts,long base, long startnum, 
				int startnote,BOOL modulate, BOOL modulrand, 
				int modulnum, BOOL inverted, float logmapvar, float oneoverf,
				float henonA, float henonB, float hopalong,
				float martin, float lorentz, unsigned long earthwormvar,
				unsigned long earthwormmulti, unsigned earthwormtruncator,
				double ranmin, double ranmax, BYTE wolframrule,
				int wolframsize, long hailstonestart);

int __fastcall LogisticMap(int voice, long counter, long test, const int* scale, 
				int numocts,long base, long startnum, 
				int startnote,BOOL modulate, BOOL modulrand, 
				int modulnum, BOOL inverted, float logmapvar, float oneoverf,
				float henonA, float henonB, float hopalong,
				float martin, float lorentz, unsigned long earthwormvar,
				unsigned long earthwormmulti, unsigned earthwormtruncator,
				double ranmin, double ranmax, BYTE wolframrule,
				int wolframsize, long hailstonestart);

int __fastcall OneOverF(int voice, long counter, long test, const int* scale, 
				int numocts,long base, long startnum, 
				int startnote,BOOL modulate, BOOL modulrand, 
				int modulnum, BOOL inverted, float logmapvar, float oneoverf,
				float henonA, float henonB, float hopalong,
				float martin, float lorentz, unsigned long earthwormvar,
				unsigned long earthwormmulti, unsigned earthwormtruncator,
				double ranmin, double ranmax, BYTE wolframrule,
				int wolframsize, long hailstonestart);

int __fastcall Henon(int voice, long counter, long test, const int* scale, 
				int numocts,long base, long startnum, 
				int startnote,BOOL modulate, BOOL modulrand, 
				int modulnum, BOOL inverted, float logmapvar, float oneoverf,
				float henonA, float henonB, float hopalong,
				float martin, float lorentz, unsigned long earthwormvar,
				unsigned long earthwormmulti, unsigned earthwormtruncator,
				double ranmin, double ranmax, BYTE wolframrule,
				int wolframsize, long hailstonestart);

int __fastcall Hopalong(int voice, long counter, long test, const int* scale, 
				int numocts,long base, long startnum, 
				int startnote,BOOL modulate, BOOL modulrand, 
				int modulnum, BOOL inverted, float logmapvar, float oneoverf,
				float henonA, float henonB, float hopalong,
				float martin, float lorentz, unsigned long earthwormvar,
				unsigned long earthwormmulti, unsigned earthwormtruncator,
				double ranmin, double ranmax, BYTE wolframrule,
				int wolframsize, long hailstonestart);

int __fastcall Martin(int voice, long counter, long test, const int* scale, 
				int numocts,long base, long startnum, 
				int startnote,BOOL modulate, BOOL modulrand, 
				int modulnum, BOOL inverted, float logmapvar, float oneoverf,
				float henonA, float henonB, float hopalong,
				float martin, float lorentz, unsigned long earthwormvar,
				unsigned long earthwormmulti, unsigned earthwormtruncator,
				double ranmin, double ranmax, BYTE wolframrule,
				int wolframsize, long hailstonestart);

int __fastcall Gingerbreadman(int voice, long counter, long test, const int* scale, 
				int numocts,long base, long startnum, 
				int startnote,BOOL modulate, BOOL modulrand, 
				int modulnum, BOOL inverted, float logmapvar, float oneoverf,
				float henonA, float henonB, float hopalong,
				float martin, float lorentz, unsigned long earthwormvar,
				unsigned long earthwormmulti, unsigned earthwormtruncator,
				double ranmin, double ranmax, BYTE wolframrule,
				int wolframsize, long hailstonestart);

int __fastcall Lorentz(int voice, long counter, long test, const int* scale, 
				int numocts,long base, long startnum, 
				int startnote,BOOL modulate, BOOL modulrand, 
				int modulnum, BOOL inverted, float logmapvar, float oneoverf,
				float henonA, float henonB, float hopalong,
				float martin, float lorentz, unsigned long earthwormvar,
				unsigned long earthwormmulti, unsigned earthwormtruncator,
				double ranmin, double ranmax, BYTE wolframrule,
				int wolframsize, long hailstonestart);

int __fastcall Earthworm(int voice, long counter, long test, const int* scale, 
				int numocts,long base, long startnum, 
				int startnote,BOOL modulate, BOOL modulrand, 
				int modulnum, BOOL inverted, float logmapvar, float oneoverf,
				float henonA, float henonB, float hopalong,
				float martin, float lorentz, unsigned long earthwormvar,
				unsigned long earthwormmulti, unsigned earthwormtruncator,
				double ranmin, double ranmax, BYTE wolframrule,
				int wolframsize, long hailstonestart);


int __fastcall RandomAlgorithm(int voice, long counter, long test, const int* scale, 
				int numocts,long base, long startnum, 
				int startnote,BOOL modulate, BOOL modulrand, 
				int modulnum, BOOL inverted, float logmapvar, float oneoverf,
				float henonA, float henonB, float hopalong,
				float martin, float lorentz, unsigned long earthwormvar,
				unsigned long earthwormmulti, unsigned earthwormtruncator,
				double ranmin, double ranmax, BYTE wolframrule,
				int wolframsize, long hailstonestart);

int __fastcall Hailstone(int voice, long counter, long test, const int* scale, 
				int numocts,long base, long startnum, 
				int startnote,BOOL modulate, BOOL modulrand, 
				int modulnum, BOOL inverted, float logmapvar, float oneoverf,
				float henonA, float henonB, float hopalong,
				float martin, float lorentz, unsigned long earthwormvar,
				unsigned long earthwormmulti, unsigned earthwormtruncator,
				double ranmin, double ranmax, BYTE wolframrule,
				int wolframsize, long hailstonestart);

int __fastcall Wolfram(int voice, long counter, long test, const int* scale, 
				int numocts,long base, long startnum, 
				int startnote,BOOL modulate, BOOL modulrand, 
				int modulnum, BOOL inverted, float logmapvar, float oneoverf,
				float henonA, float henonB, float hopalong,
				float martin, float lorentz, unsigned long earthwormvar,
				unsigned long earthwormmulti, unsigned earthwormtruncator,
				double ranmin, double ranmax, BYTE wolframrule,
				int wolframsize, long hailstonestart);



void ComputeTotal(long test, long base, long* total);
long Duration(double val);
WORD noteOn(BYTE , BYTE , BYTE );
WORD noteOff(BYTE, BYTE, BYTE);
WORD progChange(BYTE , BYTE);
WORD modeChange(BYTE , BYTE , BYTE);

float RanDev();


UINT Wait(LPVOID p);

int  (__fastcall*(Algorithm[12]))(int , long , long , const int* , 
				int ,long , long , 
				int ,BOOL , BOOL , 
				int, BOOL, float, float, float, float, float,
				float, float, unsigned long, unsigned long,
				unsigned, double, double, BYTE, int, long) = 
											{	Morse_Thue, 
												LogisticMap,
												OneOverF,
												Henon,
												Hopalong,
												Martin,
												Gingerbreadman,
												Lorentz,
												Earthworm,
												RandomAlgorithm,
												Hailstone,
												Wolfram
												
											};




IMPLEMENT_DYNCREATE(CFractMusView, CFormView)

BEGIN_MESSAGE_MAP(CFractMusView, CFormView)
	//{{AFX_MSG_MAP(CFractMusView)
	ON_BN_CLICKED(IDC_VOICE1_MIDI_BUTTON, OnVoice1MidiButton)
	ON_BN_CLICKED(IDC_VOICE1_MODULATE_CHK, OnVoice1ModulateChk)
	ON_CBN_CLOSEUP(IDC_VOICE1_NOTEVALUE_COMBO, OnCloseupVoice1NotevalueCombo)
	ON_CBN_CLOSEUP(IDC_VOICE1_NUMOCTS_COMBO, OnCloseupVoice1NumoctsCombo)
	ON_BN_CLICKED(IDC_VOICE1_ON_CHK, OnVoice1OnChk)
	ON_CBN_CLOSEUP(IDC_VOICE1_SCALE_COMBO, OnCloseupVoice1ScaleCombo)
	ON_CBN_CLOSEUP(IDC_VOICE1_STARTNOTE_COMBO, OnCloseupVoice1StartnoteCombo)
	ON_EN_KILLFOCUS(IDC_VOICE1_STARTNUM_EDIT, OnKillfocusVoice1StartnumEdit)
	ON_BN_CLICKED(IDC_VOICE2_MIDI_BUTTON, OnVoice2MidiButton)
	ON_BN_CLICKED(IDC_VOICE2_MODULATE_CHK, OnVoice2ModulateChk)
	ON_CBN_CLOSEUP(IDC_VOICE2_NOTEVALUE_COMBO, OnCloseupVoice2NotevalueCombo)
	ON_CBN_CLOSEUP(IDC_VOICE2_NUMOCTS_COMBO, OnCloseupVoice2NumoctsCombo)
	ON_BN_CLICKED(IDC_VOICE2_ON_CHK, OnVoice2OnChk)
	ON_CBN_CLOSEUP(IDC_VOICE2_SCALE_COMBO, OnCloseupVoice2ScaleCombo)
	ON_CBN_CLOSEUP(IDC_VOICE2_STARTNOTE_COMBO, OnCloseupVoice2StartnoteCombo)
	ON_EN_KILLFOCUS(IDC_VOICE2_STARTNUM_EDIT, OnKillfocusVoice2StartnumEdit)
	ON_BN_CLICKED(IDC_VOICE3_MIDI_BUTTON, OnVoice3MidiButton)
	ON_BN_CLICKED(IDC_VOICE3_MODULATE_CHK, OnVoice3ModulateChk)
	ON_CBN_CLOSEUP(IDC_VOICE3_NOTEVALUE_COMBO, OnCloseupVoice3NotevalueCombo)
	ON_CBN_CLOSEUP(IDC_VOICE3_NUMOCTS_COMBO, OnCloseupVoice3NumoctsCombo)
	ON_BN_CLICKED(IDC_VOICE3_ON_CHK, OnVoice3OnChk)
	ON_CBN_CLOSEUP(IDC_VOICE3_SCALE_COMBO, OnCloseupVoice3ScaleCombo)
	ON_CBN_CLOSEUP(IDC_VOICE3_STARTNOTE_COMBO, OnCloseupVoice3StartnoteCombo)
	ON_EN_KILLFOCUS(IDC_VOICE3_STARTNUM_EDIT, OnKillfocusVoice3StartnumEdit)
	ON_BN_CLICKED(IDC_VOICE4_MIDI_BUTTON, OnVoice4MidiButton)
	ON_BN_CLICKED(IDC_VOICE4_MODULATE_CHK, OnVoice4ModulateChk)
	ON_CBN_CLOSEUP(IDC_VOICE4_NOTEVALUE_COMBO, OnCloseupVoice4NotevalueCombo)
	ON_CBN_CLOSEUP(IDC_VOICE4_NUMOCTS_COMBO, OnCloseupVoice4NumoctsCombo)
	ON_BN_CLICKED(IDC_VOICE4_ON_CHK, OnVoice4OnChk)
	ON_CBN_CLOSEUP(IDC_VOICE4_SCALE_COMBO, OnCloseupVoice4ScaleCombo)
	ON_CBN_CLOSEUP(IDC_VOICE4_STARTNOTE_COMBO, OnCloseupVoice4StartnoteCombo)
	ON_EN_KILLFOCUS(IDC_VOICE4_STARTNUM_EDIT, OnKillfocusVoice4StartnumEdit)
	ON_BN_CLICKED(IDC_VOICE5_MIDI_BUTTON, OnVoice5MidiButton)
	ON_BN_CLICKED(IDC_VOICE5_MODULATE_CHK, OnVoice5ModulateChk)
	ON_CBN_CLOSEUP(IDC_VOICE5_NOTEVALUE_COMBO, OnCloseupVoice5NotevalueCombo)
	ON_CBN_CLOSEUP(IDC_VOICE5_NUMOCTS_COMBO, OnCloseupVoice5NumoctsCombo)
	ON_BN_CLICKED(IDC_VOICE5_ON_CHK, OnVoice5OnChk)
	ON_CBN_CLOSEUP(IDC_VOICE5_SCALE_COMBO, OnCloseupVoice5ScaleCombo)
	ON_CBN_CLOSEUP(IDC_VOICE5_STARTNOTE_COMBO, OnCloseupVoice5StartnoteCombo)
	ON_EN_KILLFOCUS(IDC_VOICE5_STARTNUM_EDIT, OnKillfocusVoice5StartnumEdit)
	ON_BN_CLICKED(IDC_VOICE6_MIDI_BUTTON, OnVoice6MidiButton)
	ON_BN_CLICKED(IDC_VOICE6_MODULATE_CHK, OnVoice6ModulateChk)
	ON_CBN_CLOSEUP(IDC_VOICE6_NOTEVALUE_COMBO, OnCloseupVoice6NotevalueCombo)
	ON_CBN_CLOSEUP(IDC_VOICE6_NUMOCTS_COMBO, OnCloseupVoice6NumoctsCombo)
	ON_BN_CLICKED(IDC_VOICE6_ON_CHK, OnVoice6OnChk)
	ON_CBN_CLOSEUP(IDC_VOICE6_SCALE_COMBO, OnCloseupVoice6ScaleCombo)
	ON_CBN_CLOSEUP(IDC_VOICE6_STARTNOTE_COMBO, OnCloseupVoice6StartnoteCombo)
	ON_EN_KILLFOCUS(IDC_VOICE6_STARTNUM_EDIT, OnKillfocusVoice6StartnumEdit)
	ON_BN_CLICKED(IDC_VOICE7_MIDI_BUTTON, OnVoice7MidiButton)
	ON_BN_CLICKED(IDC_VOICE7_MODULATE_CHK, OnVoice7ModulateChk)
	ON_CBN_CLOSEUP(IDC_VOICE7_NOTEVALUE_COMBO, OnCloseupVoice7NotevalueCombo)
	ON_CBN_CLOSEUP(IDC_VOICE7_NUMOCTS_COMBO, OnCloseupVoice7NumoctsCombo)
	ON_BN_CLICKED(IDC_VOICE7_ON_CHK, OnVoice7OnChk)
	ON_CBN_CLOSEUP(IDC_VOICE7_SCALE_COMBO, OnCloseupVoice7ScaleCombo)
	ON_CBN_CLOSEUP(IDC_VOICE7_STARTNOTE_COMBO, OnCloseupVoice7StartnoteCombo)
	ON_EN_KILLFOCUS(IDC_VOICE7_STARTNUM_EDIT, OnKillfocusVoice7StartnumEdit)
	ON_BN_CLICKED(IDC_VOICE8_MIDI_BUTTON, OnVoice8MidiButton)
	ON_BN_CLICKED(IDC_VOICE8_MODULATE_CHK, OnVoice8ModulateChk)
	ON_CBN_CLOSEUP(IDC_VOICE8_NOTEVALUE_COMBO, OnCloseupVoice8NotevalueCombo)
	ON_CBN_CLOSEUP(IDC_VOICE8_NUMOCTS_COMBO, OnCloseupVoice8NumoctsCombo)
	ON_BN_CLICKED(IDC_VOICE8_ON_CHK, OnVoice8OnChk)
	ON_CBN_CLOSEUP(IDC_VOICE8_SCALE_COMBO, OnCloseupVoice8ScaleCombo)
	ON_CBN_CLOSEUP(IDC_VOICE8_STARTNOTE_COMBO, OnCloseupVoice8StartnoteCombo)
	ON_EN_KILLFOCUS(IDC_VOICE8_STARTNUM_EDIT, OnKillfocusVoice8StartnumEdit)
	ON_UPDATE_COMMAND_UI(ID_APP_EXIT, OnUpdateAppExit)
	ON_UPDATE_COMMAND_UI(ID_FILE_NEW, OnUpdateFileNew)
	ON_UPDATE_COMMAND_UI(ID_FILE_OPEN, OnUpdateFileOpen)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE, OnUpdateFileSave)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE_AS, OnUpdateFileSaveAs)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVEMIDI, OnUpdateFileSavemidi)
	ON_COMMAND(ID_PLAY_PLAY, OnPlayPlay)
	ON_UPDATE_COMMAND_UI(ID_PLAY_PLAY, OnUpdatePlayPlay)
	ON_COMMAND(ID_PLAY_STOP, OnPlayStop)
	ON_WM_HSCROLL()
	ON_COMMAND(ID_FILE_SAVEMIDI, OnFileSavemidi)
	ON_COMMAND(IDC_OPTS_EVENTS, OnOptsEvents)
	ON_UPDATE_COMMAND_UI(IDC_OPTS_EVENTS, OnUpdateOptsEvents)
	ON_BN_CLICKED(IDC_RESET_GENCOUNTER, OnResetGencounter)
	ON_EN_KILLFOCUS(IDC_GENCOUNTER_ED, OnKillfocusGencounterEd)
	ON_COMMAND(ID_ALGORITHMS_VOICE1_MORSETHUE, OnAlgorithmsVoice1Morsethue)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE1_MORSETHUE, OnUpdateAlgorithmsVoice1Morsethue)
	ON_COMMAND(ID_ALGORITHMS_VOICE2_MORSETHUE, OnAlgorithmsVoice2Morsethue)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE2_MORSETHUE, OnUpdateAlgorithmsVoice2Morsethue)
	ON_COMMAND(ID_ALGORITHMS_VOICE3_MORSETHUE, OnAlgorithmsVoice3Morsethue)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE3_MORSETHUE, OnUpdateAlgorithmsVoice3Morsethue)
	ON_COMMAND(ID_ALGORITHMS_VOICE4_MORSETHUE, OnAlgorithmsVoice4Morsethue)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE4_MORSETHUE, OnUpdateAlgorithmsVoice4Morsethue)
	ON_COMMAND(ID_ALGORITHMS_VOICE5_MORSETHUE, OnAlgorithmsVoice5Morsethue)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE5_MORSETHUE, OnUpdateAlgorithmsVoice5Morsethue)
	ON_COMMAND(ID_ALGORITHMS_VOICE6_MORSETHUE, OnAlgorithmsVoice6Morsethue)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE6_MORSETHUE, OnUpdateAlgorithmsVoice6Morsethue)
	ON_COMMAND(ID_ALGORITHMS_VOICE7_MORSETHUE, OnAlgorithmsVoice7Morsethue)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE7_MORSETHUE, OnUpdateAlgorithmsVoice7Morsethue)
	ON_COMMAND(ID_ALGORITHMS_VOICE8_MORSETHUE, OnAlgorithmsVoice8Morsethue)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE8_MORSETHUE, OnUpdateAlgorithmsVoice8Morsethue)
	ON_COMMAND(ID_CURRENT, OnCurrent)
	ON_UPDATE_COMMAND_UI(ID_CURRENT, OnUpdateCurrent)
	ON_COMMAND(ID_COMPOSITION, OnComposition)
	ON_UPDATE_COMMAND_UI(ID_COMPOSITION, OnUpdateComposition)
	ON_BN_CLICKED(IDC_ROUND, OnRound)
	ON_COMMAND(ID_ALGORITHMS_VOICE1_LOGMAP, OnAlgorithmsVoice1Logmap)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE1_LOGMAP, OnUpdateAlgorithmsVoice1Logmap)
	ON_COMMAND(ID_ALGORITHMS_VOICE2_LOGMAP, OnAlgorithmsVoice2Logmap)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE2_LOGMAP, OnUpdateAlgorithmsVoice2Logmap)
	ON_COMMAND(ID_ALGORITHMS_VOICE3_LOGMAP, OnAlgorithmsVoice3Logmap)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE3_LOGMAP, OnUpdateAlgorithmsVoice3Logmap)
	ON_COMMAND(ID_ALGORITHMS_VOICE4_LOGMAP, OnAlgorithmsVoice4Logmap)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE4_LOGMAP, OnUpdateAlgorithmsVoice4Logmap)
	ON_COMMAND(ID_ALGORITHMS_VOICE5_LOGMAP, OnAlgorithmsVoice5Logmap)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE5_LOGMAP, OnUpdateAlgorithmsVoice5Logmap)
	ON_COMMAND(ID_ALGORITHMS_VOICE6_LOGMAP, OnAlgorithmsVoice6Logmap)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE6_LOGMAP, OnUpdateAlgorithmsVoice6Logmap)
	ON_COMMAND(ID_ALGORITHMS_VOICE7_LOGMAP, OnAlgorithmsVoice7Logmap)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE7_LOGMAP, OnUpdateAlgorithmsVoice7Logmap)
	ON_COMMAND(ID_ALGORITHMS_VOICE8_LOGMAP, OnAlgorithmsVoice8Logmap)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE8_LOGMAP, OnUpdateAlgorithmsVoice8Logmap)
	ON_COMMAND(ID_ALGORITHMS_VOICE1_ONEOVERF, OnAlgorithmsVoice1Oneoverf)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE1_ONEOVERF, OnUpdateAlgorithmsVoice1Oneoverf)
	ON_COMMAND(ID_ALGORITHMS_VOICE2_ONEOVERF, OnAlgorithmsVoice2Oneoverf)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE2_ONEOVERF, OnUpdateAlgorithmsVoice2Oneoverf)
	ON_COMMAND(ID_ALGORITHMS_VOICE3_ONEOVERF, OnAlgorithmsVoice3Oneoverf)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE3_ONEOVERF, OnUpdateAlgorithmsVoice3Oneoverf)
	ON_COMMAND(ID_ALGORITHMS_VOICE4_ONEOVERF, OnAlgorithmsVoice4Oneoverf)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE4_ONEOVERF, OnUpdateAlgorithmsVoice4Oneoverf)
	ON_COMMAND(ID_ALGORITHMS_VOICE5_ONEOVERF, OnAlgorithmsVoice5Oneoverf)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE5_ONEOVERF, OnUpdateAlgorithmsVoice5Oneoverf)
	ON_COMMAND(ID_ALGORITHMS_VOICE6_ONEOVERF, OnAlgorithmsVoice6Oneoverf)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE6_ONEOVERF, OnUpdateAlgorithmsVoice6Oneoverf)
	ON_COMMAND(ID_ALGORITHMS_VOICE7_ONEOVERF, OnAlgorithmsVoice7Oneoverf)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE7_ONEOVERF, OnUpdateAlgorithmsVoice7Oneoverf)
	ON_COMMAND(ID_ALGORITHMS_VOICE8_ONEOVERF, OnAlgorithmsVoice8Oneoverf)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE8_ONEOVERF, OnUpdateAlgorithmsVoice8Oneoverf)
	ON_COMMAND(IDC_OPTS_DEVICE, OnOptsDevice)
	ON_UPDATE_COMMAND_UI(IDC_OPTS_DEVICE, OnUpdateOptsDevice)
	ON_BN_CLICKED(IDC_VOICE10_MIDI_BUTTON, OnVoice10MidiButton)
	ON_BN_CLICKED(IDC_VOICE10_MODULATE_CHK, OnVoice10ModulateChk)
	ON_CBN_CLOSEUP(IDC_VOICE10_NOTEVALUE_COMBO, OnCloseupVoice10NotevalueCombo)
	ON_CBN_CLOSEUP(IDC_VOICE10_NUMOCTS_COMBO, OnCloseupVoice10NumoctsCombo)
	ON_BN_CLICKED(IDC_VOICE10_ON_CHK, OnVoice10OnChk)
	ON_CBN_CLOSEUP(IDC_VOICE10_SCALE_COMBO, OnCloseupVoice10ScaleCombo)
	ON_CBN_CLOSEUP(IDC_VOICE10_STARTNOTE_COMBO, OnCloseupVoice10StartnoteCombo)
	ON_EN_KILLFOCUS(IDC_VOICE10_STARTNUM_EDIT, OnKillfocusVoice10StartnumEdit)
	ON_BN_CLICKED(IDC_VOICE11_MIDI_BUTTON, OnVoice11MidiButton)
	ON_BN_CLICKED(IDC_VOICE11_MODULATE_CHK, OnVoice11ModulateChk)
	ON_CBN_CLOSEUP(IDC_VOICE11_NOTEVALUE_COMBO, OnCloseupVoice11NotevalueCombo)
	ON_CBN_CLOSEUP(IDC_VOICE11_NUMOCTS_COMBO, OnCloseupVoice11NumoctsCombo)
	ON_BN_CLICKED(IDC_VOICE11_ON_CHK, OnVoice11OnChk)
	ON_CBN_CLOSEUP(IDC_VOICE11_SCALE_COMBO, OnCloseupVoice11ScaleCombo)
	ON_CBN_CLOSEUP(IDC_VOICE11_STARTNOTE_COMBO, OnCloseupVoice11StartnoteCombo)
	ON_EN_KILLFOCUS(IDC_VOICE11_STARTNUM_EDIT, OnKillfocusVoice11StartnumEdit)
	ON_BN_CLICKED(IDC_VOICE12_MIDI_BUTTON, OnVoice12MidiButton)
	ON_BN_CLICKED(IDC_VOICE12_MODULATE_CHK, OnVoice12ModulateChk)
	ON_CBN_CLOSEUP(IDC_VOICE12_NOTEVALUE_COMBO, OnCloseupVoice12NotevalueCombo)
	ON_CBN_CLOSEUP(IDC_VOICE12_NUMOCTS_COMBO, OnCloseupVoice12NumoctsCombo)
	ON_BN_CLICKED(IDC_VOICE12_ON_CHK, OnVoice12OnChk)
	ON_CBN_CLOSEUP(IDC_VOICE12_SCALE_COMBO, OnCloseupVoice12ScaleCombo)
	ON_CBN_CLOSEUP(IDC_VOICE12_STARTNOTE_COMBO, OnCloseupVoice12StartnoteCombo)
	ON_EN_KILLFOCUS(IDC_VOICE12_STARTNUM_EDIT, OnKillfocusVoice12StartnumEdit)
	ON_BN_CLICKED(IDC_VOICE13_MIDI_BUTTON, OnVoice13MidiButton)
	ON_BN_CLICKED(IDC_VOICE13_MODULATE_CHK, OnVoice13ModulateChk)
	ON_CBN_CLOSEUP(IDC_VOICE13_NOTEVALUE_COMBO, OnCloseupVoice13NotevalueCombo)
	ON_CBN_CLOSEUP(IDC_VOICE13_NUMOCTS_COMBO, OnCloseupVoice13NumoctsCombo)
	ON_BN_CLICKED(IDC_VOICE13_ON_CHK, OnVoice13OnChk)
	ON_CBN_CLOSEUP(IDC_VOICE13_SCALE_COMBO, OnCloseupVoice13ScaleCombo)
	ON_CBN_CLOSEUP(IDC_VOICE13_STARTNOTE_COMBO, OnCloseupVoice13StartnoteCombo)
	ON_EN_KILLFOCUS(IDC_VOICE13_STARTNUM_EDIT, OnKillfocusVoice13StartnumEdit)
	ON_BN_CLICKED(IDC_VOICE14_MIDI_BUTTON, OnVoice14MidiButton)
	ON_BN_CLICKED(IDC_VOICE14_MODULATE_CHK, OnVoice14ModulateChk)
	ON_CBN_CLOSEUP(IDC_VOICE14_NOTEVALUE_COMBO, OnCloseupVoice14NotevalueCombo)
	ON_CBN_CLOSEUP(IDC_VOICE14_NUMOCTS_COMBO, OnCloseupVoice14NumoctsCombo)
	ON_BN_CLICKED(IDC_VOICE14_ON_CHK, OnVoice14OnChk)
	ON_CBN_CLOSEUP(IDC_VOICE14_SCALE_COMBO, OnCloseupVoice14ScaleCombo)
	ON_CBN_CLOSEUP(IDC_VOICE14_STARTNOTE_COMBO, OnCloseupVoice14StartnoteCombo)
	ON_EN_KILLFOCUS(IDC_VOICE14_STARTNUM_EDIT, OnKillfocusVoice14StartnumEdit)
	ON_BN_CLICKED(IDC_VOICE15_MIDI_BUTTON, OnVoice15MidiButton)
	ON_BN_CLICKED(IDC_VOICE15_MODULATE_CHK, OnVoice15ModulateChk)
	ON_CBN_CLOSEUP(IDC_VOICE15_NOTEVALUE_COMBO, OnCloseupVoice15NotevalueCombo)
	ON_CBN_CLOSEUP(IDC_VOICE15_NUMOCTS_COMBO, OnCloseupVoice15NumoctsCombo)
	ON_BN_CLICKED(IDC_VOICE15_ON_CHK, OnVoice15OnChk)
	ON_CBN_CLOSEUP(IDC_VOICE15_SCALE_COMBO, OnCloseupVoice15ScaleCombo)
	ON_CBN_CLOSEUP(IDC_VOICE15_STARTNOTE_COMBO, OnCloseupVoice15StartnoteCombo)
	ON_EN_KILLFOCUS(IDC_VOICE15_STARTNUM_EDIT, OnKillfocusVoice15StartnumEdit)
	ON_BN_CLICKED(IDC_VOICE16_MIDI_BUTTON, OnVoice16MidiButton)
	ON_BN_CLICKED(IDC_VOICE16_MODULATE_CHK, OnVoice16ModulateChk)
	ON_CBN_CLOSEUP(IDC_VOICE16_NOTEVALUE_COMBO, OnCloseupVoice16NotevalueCombo)
	ON_CBN_CLOSEUP(IDC_VOICE16_NUMOCTS_COMBO, OnCloseupVoice16NumoctsCombo)
	ON_BN_CLICKED(IDC_VOICE16_ON_CHK, OnVoice16OnChk)
	ON_CBN_CLOSEUP(IDC_VOICE16_SCALE_COMBO, OnCloseupVoice16ScaleCombo)
	ON_CBN_CLOSEUP(IDC_VOICE16_STARTNOTE_COMBO, OnCloseupVoice16StartnoteCombo)
	ON_EN_KILLFOCUS(IDC_VOICE16_STARTNUM_EDIT, OnKillfocusVoice16StartnumEdit)
	ON_BN_CLICKED(IDC_VOICE9_MIDI_BUTTON, OnVoice9MidiButton)
	ON_BN_CLICKED(IDC_VOICE9_MODULATE_CHK, OnVoice9ModulateChk)
	ON_CBN_CLOSEUP(IDC_VOICE9_NOTEVALUE_COMBO, OnCloseupVoice9NotevalueCombo)
	ON_CBN_CLOSEUP(IDC_VOICE9_NUMOCTS_COMBO, OnCloseupVoice9NumoctsCombo)
	ON_BN_CLICKED(IDC_VOICE9_ON_CHK, OnVoice9OnChk)
	ON_CBN_CLOSEUP(IDC_VOICE9_SCALE_COMBO, OnCloseupVoice9ScaleCombo)
	ON_CBN_CLOSEUP(IDC_VOICE9_STARTNOTE_COMBO, OnCloseupVoice9StartnoteCombo)
	ON_EN_KILLFOCUS(IDC_VOICE9_STARTNUM_EDIT, OnKillfocusVoice9StartnumEdit)
	ON_CBN_CLOSEUP(IDC_TIMESIGNUM_CMB, OnCloseupTimesignumCmb)
	ON_CBN_CLOSEUP(IDC_TIMESIGDEN_CMB, OnCloseupTimesigdenCmb)
	ON_COMMAND(ID_ALGORITHMS_VOICE10_LOGMAP, OnAlgorithmsVoice10Logmap)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE10_LOGMAP, OnUpdateAlgorithmsVoice10Logmap)
	ON_COMMAND(ID_ALGORITHMS_VOICE10_MORSETHUE, OnAlgorithmsVoice10Morsethue)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE10_MORSETHUE, OnUpdateAlgorithmsVoice10Morsethue)
	ON_COMMAND(ID_ALGORITHMS_VOICE10_ONEOVERF, OnAlgorithmsVoice10Oneoverf)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE10_ONEOVERF, OnUpdateAlgorithmsVoice10Oneoverf)
	ON_COMMAND(ID_ALGORITHMS_VOICE11_LOGMAP, OnAlgorithmsVoice11Logmap)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE11_LOGMAP, OnUpdateAlgorithmsVoice11Logmap)
	ON_COMMAND(ID_ALGORITHMS_VOICE11_MORSETHUE, OnAlgorithmsVoice11Morsethue)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE11_MORSETHUE, OnUpdateAlgorithmsVoice11Morsethue)
	ON_COMMAND(ID_ALGORITHMS_VOICE11_ONEOVERF, OnAlgorithmsVoice11Oneoverf)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE11_ONEOVERF, OnUpdateAlgorithmsVoice11Oneoverf)
	ON_COMMAND(ID_ALGORITHMS_VOICE12_LOGMAP, OnAlgorithmsVoice12Logmap)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE12_LOGMAP, OnUpdateAlgorithmsVoice12Logmap)
	ON_COMMAND(ID_ALGORITHMS_VOICE12_MORSETHUE, OnAlgorithmsVoice12Morsethue)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE12_MORSETHUE, OnUpdateAlgorithmsVoice12Morsethue)
	ON_COMMAND(ID_ALGORITHMS_VOICE12_ONEOVERF, OnAlgorithmsVoice12Oneoverf)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE12_ONEOVERF, OnUpdateAlgorithmsVoice12Oneoverf)
	ON_COMMAND(ID_ALGORITHMS_VOICE13_LOGMAP, OnAlgorithmsVoice13Logmap)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE13_LOGMAP, OnUpdateAlgorithmsVoice13Logmap)
	ON_COMMAND(ID_ALGORITHMS_VOICE13_MORSETHUE, OnAlgorithmsVoice13Morsethue)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE13_MORSETHUE, OnUpdateAlgorithmsVoice13Morsethue)
	ON_COMMAND(ID_ALGORITHMS_VOICE13_ONEOVERF, OnAlgorithmsVoice13Oneoverf)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE13_ONEOVERF, OnUpdateAlgorithmsVoice13Oneoverf)
	ON_COMMAND(ID_ALGORITHMS_VOICE14_LOGMAP, OnAlgorithmsVoice14Logmap)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE14_LOGMAP, OnUpdateAlgorithmsVoice14Logmap)
	ON_COMMAND(ID_ALGORITHMS_VOICE14_MORSETHUE, OnAlgorithmsVoice14Morsethue)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE14_MORSETHUE, OnUpdateAlgorithmsVoice14Morsethue)
	ON_COMMAND(ID_ALGORITHMS_VOICE14_ONEOVERF, OnAlgorithmsVoice14Oneoverf)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE14_ONEOVERF, OnUpdateAlgorithmsVoice14Oneoverf)
	ON_COMMAND(ID_ALGORITHMS_VOICE15_LOGMAP, OnAlgorithmsVoice15Logmap)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE15_LOGMAP, OnUpdateAlgorithmsVoice15Logmap)
	ON_COMMAND(ID_ALGORITHMS_VOICE15_MORSETHUE, OnAlgorithmsVoice15Morsethue)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE15_MORSETHUE, OnUpdateAlgorithmsVoice15Morsethue)
	ON_COMMAND(ID_ALGORITHMS_VOICE15_ONEOVERF, OnAlgorithmsVoice15Oneoverf)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE15_ONEOVERF, OnUpdateAlgorithmsVoice15Oneoverf)
	ON_COMMAND(ID_ALGORITHMS_VOICE16_LOGMAP, OnAlgorithmsVoice16Logmap)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE16_LOGMAP, OnUpdateAlgorithmsVoice16Logmap)
	ON_COMMAND(ID_ALGORITHMS_VOICE16_MORSETHUE, OnAlgorithmsVoice16Morsethue)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE16_MORSETHUE, OnUpdateAlgorithmsVoice16Morsethue)
	ON_COMMAND(ID_ALGORITHMS_VOICE16_ONEOVERF, OnAlgorithmsVoice16Oneoverf)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE16_ONEOVERF, OnUpdateAlgorithmsVoice16Oneoverf)
	ON_COMMAND(ID_ALGORITHMS_VOICE9_LOGMAP, OnAlgorithmsVoice9Logmap)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE9_LOGMAP, OnUpdateAlgorithmsVoice9Logmap)
	ON_COMMAND(ID_ALGORITHMS_VOICE9_MORSETHUE, OnAlgorithmsVoice9Morsethue)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE9_MORSETHUE, OnUpdateAlgorithmsVoice9Morsethue)
	ON_COMMAND(ID_ALGORITHMS_VOICE9_ONEOVERF, OnAlgorithmsVoice9Oneoverf)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE9_ONEOVERF, OnUpdateAlgorithmsVoice9Oneoverf)
	ON_COMMAND(IDC_UPDATESETTINGS, OnUpdatesettings)
	ON_UPDATE_COMMAND_UI(IDC_UPDATESETTINGS, OnUpdateUpdatesettings)
	ON_COMMAND(IDC_COMP_INFO, OnCompInfo)
	ON_UPDATE_COMMAND_UI(IDC_COMP_INFO, OnUpdateCompInfo)
	ON_COMMAND(IDC_RANDOMIZE, OnRandomize)
	ON_UPDATE_COMMAND_UI(IDC_RANDOMIZE, OnUpdateRandomize)
	ON_COMMAND(ID_ALGORITHMS_VOICE1_HENON, OnAlgorithmsVoice1Henon)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE1_HENON, OnUpdateAlgorithmsVoice1Henon)
	ON_COMMAND(ID_ALGORITHMS_VOICE2_HENON, OnAlgorithmsVoice2Henon)
	ON_COMMAND(ID_ALGORITHMS_VOICE3_HENON, OnAlgorithmsVoice3Henon)
	ON_COMMAND(ID_ALGORITHMS_VOICE4_HENON, OnAlgorithmsVoice4Henon)
	ON_COMMAND(ID_ALGORITHMS_VOICE5_HENON, OnAlgorithmsVoice5Henon)
	ON_COMMAND(ID_ALGORITHMS_VOICE6_HENON, OnAlgorithmsVoice6Henon)
	ON_COMMAND(ID_ALGORITHMS_VOICE7_HENON, OnAlgorithmsVoice7Henon)
	ON_COMMAND(ID_ALGORITHMS_VOICE8_HENON, OnAlgorithmsVoice8Henon)
	ON_COMMAND(ID_ALGORITHMS_VOICE9_HENON, OnAlgorithmsVoice9Henon)
	ON_COMMAND(ID_ALGORITHMS_VOICE10_HENON, OnAlgorithmsVoice10Henon)
	ON_COMMAND(ID_ALGORITHMS_VOICE11_HENON, OnAlgorithmsVoice11Henon)
	ON_COMMAND(ID_ALGORITHMS_VOICE12_HENON, OnAlgorithmsVoice12Henon)
	ON_COMMAND(ID_ALGORITHMS_VOICE13_HENON, OnAlgorithmsVoice13Henon)
	ON_COMMAND(ID_ALGORITHMS_VOICE14_HENON, OnAlgorithmsVoice14Henon)
	ON_COMMAND(ID_ALGORITHMS_VOICE15_HENON, OnAlgorithmsVoice15Henon)
	ON_COMMAND(ID_ALGORITHMS_VOICE16_HENON, OnAlgorithmsVoice16Henon)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE2_HENON, OnUpdateAlgorithmsVoice2Henon)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE3_HENON, OnUpdateAlgorithmsVoice3Henon)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE4_HENON, OnUpdateAlgorithmsVoice4Henon)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE5_HENON, OnUpdateAlgorithmsVoice5Henon)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE6_HENON, OnUpdateAlgorithmsVoice6Henon)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE7_HENON, OnUpdateAlgorithmsVoice7Henon)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE8_HENON, OnUpdateAlgorithmsVoice8Henon)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE9_HENON, OnUpdateAlgorithmsVoice9Henon)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE10_HENON, OnUpdateAlgorithmsVoice10Henon)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE11_HENON, OnUpdateAlgorithmsVoice11Henon)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE12_HENON, OnUpdateAlgorithmsVoice12Henon)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE13_HENON, OnUpdateAlgorithmsVoice13Henon)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE14_HENON, OnUpdateAlgorithmsVoice14Henon)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE15_HENON, OnUpdateAlgorithmsVoice15Henon)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE16_HENON, OnUpdateAlgorithmsVoice16Henon)
	ON_COMMAND(ID_ALGORITHMS_VOICE1_GINGER, OnAlgorithmsVoice1Ginger)
	ON_COMMAND(ID_ALGORITHMS_VOICE1_HOPALONG, OnAlgorithmsVoice1Hopalong)
	ON_COMMAND(ID_ALGORITHMS_VOICE1_MARTIN, OnAlgorithmsVoice1Martin)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE1_GINGER, OnUpdateAlgorithmsVoice1Ginger)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE1_HOPALONG, OnUpdateAlgorithmsVoice1Hopalong)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE1_MARTIN, OnUpdateAlgorithmsVoice1Martin)
	ON_COMMAND(ID_ALGORITHMS_VOICE2_GINGER, OnAlgorithmsVoice2Ginger)
	ON_COMMAND(ID_ALGORITHMS_VOICE2_HOPALONG, OnAlgorithmsVoice2Hopalong)
	ON_COMMAND(ID_ALGORITHMS_VOICE2_MARTIN, OnAlgorithmsVoice2Martin)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE2_GINGER, OnUpdateAlgorithmsVoice2Ginger)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE2_HOPALONG, OnUpdateAlgorithmsVoice2Hopalong)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE2_MARTIN, OnUpdateAlgorithmsVoice2Martin)
	ON_COMMAND(ID_ALGORITHMS_VOICE3_GINGER, OnAlgorithmsVoice3Ginger)
	ON_COMMAND(ID_ALGORITHMS_VOICE4_GINGER, OnAlgorithmsVoice4Ginger)
	ON_COMMAND(ID_ALGORITHMS_VOICE5_GINGER, OnAlgorithmsVoice5Ginger)
	ON_COMMAND(ID_ALGORITHMS_VOICE6_GINGER, OnAlgorithmsVoice6Ginger)
	ON_COMMAND(ID_ALGORITHMS_VOICE7_GINGER, OnAlgorithmsVoice7Ginger)
	ON_COMMAND(ID_ALGORITHMS_VOICE8_GINGER, OnAlgorithmsVoice8Ginger)
	ON_COMMAND(ID_ALGORITHMS_VOICE9_GINGER, OnAlgorithmsVoice9Ginger)
	ON_COMMAND(ID_ALGORITHMS_VOICE10_GINGER, OnAlgorithmsVoice10Ginger)
	ON_COMMAND(ID_ALGORITHMS_VOICE11_GINGER, OnAlgorithmsVoice11Ginger)
	ON_COMMAND(ID_ALGORITHMS_VOICE12_GINGER, OnAlgorithmsVoice12Ginger)
	ON_COMMAND(ID_ALGORITHMS_VOICE13_GINGER, OnAlgorithmsVoice13Ginger)
	ON_COMMAND(ID_ALGORITHMS_VOICE14_GINGER, OnAlgorithmsVoice14Ginger)
	ON_COMMAND(ID_ALGORITHMS_VOICE15_GINGER, OnAlgorithmsVoice15Ginger)
	ON_COMMAND(ID_ALGORITHMS_VOICE16_GINGER, OnAlgorithmsVoice16Ginger)
	ON_COMMAND(ID_ALGORITHMS_VOICE3_HOPALONG, OnAlgorithmsVoice3Hopalong)
	ON_COMMAND(ID_ALGORITHMS_VOICE4_HOPALONG, OnAlgorithmsVoice4Hopalong)
	ON_COMMAND(ID_ALGORITHMS_VOICE5_HOPALONG, OnAlgorithmsVoice5Hopalong)
	ON_COMMAND(ID_ALGORITHMS_VOICE6_HOPALONG, OnAlgorithmsVoice6Hopalong)
	ON_COMMAND(ID_ALGORITHMS_VOICE7_HOPALONG, OnAlgorithmsVoice7Hopalong)
	ON_COMMAND(ID_ALGORITHMS_VOICE8_HOPALONG, OnAlgorithmsVoice8Hopalong)
	ON_COMMAND(ID_ALGORITHMS_VOICE9_HOPALONG, OnAlgorithmsVoice9Hopalong)
	ON_COMMAND(ID_ALGORITHMS_VOICE10_HOPALONG, OnAlgorithmsVoice10Hopalong)
	ON_COMMAND(ID_ALGORITHMS_VOICE11_HOPALONG, OnAlgorithmsVoice11Hopalong)
	ON_COMMAND(ID_ALGORITHMS_VOICE12_HOPALONG, OnAlgorithmsVoice12Hopalong)
	ON_COMMAND(ID_ALGORITHMS_VOICE13_HOPALONG, OnAlgorithmsVoice13Hopalong)
	ON_COMMAND(ID_ALGORITHMS_VOICE14_HOPALONG, OnAlgorithmsVoice14Hopalong)
	ON_COMMAND(ID_ALGORITHMS_VOICE15_HOPALONG, OnAlgorithmsVoice15Hopalong)
	ON_COMMAND(ID_ALGORITHMS_VOICE16_HOPALONG, OnAlgorithmsVoice16Hopalong)
	ON_COMMAND(ID_ALGORITHMS_VOICE3_MARTIN, OnAlgorithmsVoice3Martin)
	ON_COMMAND(ID_ALGORITHMS_VOICE4_MARTIN, OnAlgorithmsVoice4Martin)
	ON_COMMAND(ID_ALGORITHMS_VOICE5_MARTIN, OnAlgorithmsVoice5Martin)
	ON_COMMAND(ID_ALGORITHMS_VOICE6_MARTIN, OnAlgorithmsVoice6Martin)
	ON_COMMAND(ID_ALGORITHMS_VOICE7_MARTIN, OnAlgorithmsVoice7Martin)
	ON_COMMAND(ID_ALGORITHMS_VOICE8_MARTIN, OnAlgorithmsVoice8Martin)
	ON_COMMAND(ID_ALGORITHMS_VOICE9_MARTIN, OnAlgorithmsVoice9Martin)
	ON_COMMAND(ID_ALGORITHMS_VOICE10_MARTIN, OnAlgorithmsVoice10Martin)
	ON_COMMAND(ID_ALGORITHMS_VOICE11_MARTIN, OnAlgorithmsVoice11Martin)
	ON_COMMAND(ID_ALGORITHMS_VOICE12_MARTIN, OnAlgorithmsVoice12Martin)
	ON_COMMAND(ID_ALGORITHMS_VOICE13_MARTIN, OnAlgorithmsVoice13Martin)
	ON_COMMAND(ID_ALGORITHMS_VOICE14_MARTIN, OnAlgorithmsVoice14Martin)
	ON_COMMAND(ID_ALGORITHMS_VOICE15_MARTIN, OnAlgorithmsVoice15Martin)
	ON_COMMAND(ID_ALGORITHMS_VOICE16_MARTIN, OnAlgorithmsVoice16Martin)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE3_GINGER, OnUpdateAlgorithmsVoice3Ginger)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE4_GINGER, OnUpdateAlgorithmsVoice4Ginger)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE5_GINGER, OnUpdateAlgorithmsVoice5Ginger)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE6_GINGER, OnUpdateAlgorithmsVoice6Ginger)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE7_GINGER, OnUpdateAlgorithmsVoice7Ginger)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE8_GINGER, OnUpdateAlgorithmsVoice8Ginger)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE9_GINGER, OnUpdateAlgorithmsVoice9Ginger)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE10_GINGER, OnUpdateAlgorithmsVoice10Ginger)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE11_GINGER, OnUpdateAlgorithmsVoice11Ginger)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE12_GINGER, OnUpdateAlgorithmsVoice12Ginger)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE13_GINGER, OnUpdateAlgorithmsVoice13Ginger)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE14_GINGER, OnUpdateAlgorithmsVoice14Ginger)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE15_GINGER, OnUpdateAlgorithmsVoice15Ginger)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE16_GINGER, OnUpdateAlgorithmsVoice16Ginger)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE3_HOPALONG, OnUpdateAlgorithmsVoice3Hopalong)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE4_HOPALONG, OnUpdateAlgorithmsVoice4Hopalong)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE5_HOPALONG, OnUpdateAlgorithmsVoice5Hopalong)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE6_HOPALONG, OnUpdateAlgorithmsVoice6Hopalong)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE7_HOPALONG, OnUpdateAlgorithmsVoice7Hopalong)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE8_HOPALONG, OnUpdateAlgorithmsVoice8Hopalong)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE9_HOPALONG, OnUpdateAlgorithmsVoice9Hopalong)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE10_HOPALONG, OnUpdateAlgorithmsVoice10Hopalong)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE11_HOPALONG, OnUpdateAlgorithmsVoice11Hopalong)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE12_HOPALONG, OnUpdateAlgorithmsVoice12Hopalong)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE13_HOPALONG, OnUpdateAlgorithmsVoice13Hopalong)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE14_HOPALONG, OnUpdateAlgorithmsVoice14Hopalong)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE15_HOPALONG, OnUpdateAlgorithmsVoice15Hopalong)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE16_HOPALONG, OnUpdateAlgorithmsVoice16Hopalong)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE3_MARTIN, OnUpdateAlgorithmsVoice3Martin)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE4_MARTIN, OnUpdateAlgorithmsVoice4Martin)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE5_MARTIN, OnUpdateAlgorithmsVoice5Martin)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE6_MARTIN, OnUpdateAlgorithmsVoice6Martin)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE7_MARTIN, OnUpdateAlgorithmsVoice7Martin)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE8_MARTIN, OnUpdateAlgorithmsVoice8Martin)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE9_MARTIN, OnUpdateAlgorithmsVoice9Martin)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE10_MARTIN, OnUpdateAlgorithmsVoice10Martin)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE11_MARTIN, OnUpdateAlgorithmsVoice11Martin)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE12_MARTIN, OnUpdateAlgorithmsVoice12Martin)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE13_MARTIN, OnUpdateAlgorithmsVoice13Martin)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE14_MARTIN, OnUpdateAlgorithmsVoice14Martin)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE15_MARTIN, OnUpdateAlgorithmsVoice15Martin)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE16_MARTIN, OnUpdateAlgorithmsVoice16Martin)
	ON_BN_CLICKED(IDC_PLAYFROM_BT, OnPlayfromBt)
	ON_COMMAND(ID_ALGORITHMS_VOICE1_LORENTZ, OnAlgorithmsVoice1Lorentz)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE1_LORENTZ, OnUpdateAlgorithmsVoice1Lorentz)
	ON_COMMAND(ID_ALGORITHMS_VOICE2_LORENTZ, OnAlgorithmsVoice2Lorentz)
	ON_COMMAND(ID_ALGORITHMS_VOICE3_LORENTZ, OnAlgorithmsVoice3Lorentz)
	ON_COMMAND(ID_ALGORITHMS_VOICE4_LORENTZ, OnAlgorithmsVoice4Lorentz)
	ON_COMMAND(ID_ALGORITHMS_VOICE5_LORENTZ, OnAlgorithmsVoice5Lorentz)
	ON_COMMAND(ID_ALGORITHMS_VOICE6_LORENTZ, OnAlgorithmsVoice6Lorentz)
	ON_COMMAND(ID_ALGORITHMS_VOICE7_LORENTZ, OnAlgorithmsVoice7Lorentz)
	ON_COMMAND(ID_ALGORITHMS_VOICE8_LORENTZ, OnAlgorithmsVoice8Lorentz)
	ON_COMMAND(ID_ALGORITHMS_VOICE9_LORENTZ, OnAlgorithmsVoice9Lorentz)
	ON_COMMAND(ID_ALGORITHMS_VOICE10_LORENTZ, OnAlgorithmsVoice10Lorentz)
	ON_COMMAND(ID_ALGORITHMS_VOICE11_LORENTZ, OnAlgorithmsVoice11Lorentz)
	ON_COMMAND(ID_ALGORITHMS_VOICE12_LORENTZ, OnAlgorithmsVoice12Lorentz)
	ON_COMMAND(ID_ALGORITHMS_VOICE13_LORENTZ, OnAlgorithmsVoice13Lorentz)
	ON_COMMAND(ID_ALGORITHMS_VOICE14_LORENTZ, OnAlgorithmsVoice14Lorentz)
	ON_COMMAND(ID_ALGORITHMS_VOICE15_LORENTZ, OnAlgorithmsVoice15Lorentz)
	ON_COMMAND(ID_ALGORITHMS_VOICE16_LORENTZ, OnAlgorithmsVoice16Lorentz)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE2_LORENTZ, OnUpdateAlgorithmsVoice2Lorentz)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE3_LORENTZ, OnUpdateAlgorithmsVoice3Lorentz)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE4_LORENTZ, OnUpdateAlgorithmsVoice4Lorentz)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE5_LORENTZ, OnUpdateAlgorithmsVoice5Lorentz)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE6_LORENTZ, OnUpdateAlgorithmsVoice6Lorentz)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE7_LORENTZ, OnUpdateAlgorithmsVoice7Lorentz)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE8_LORENTZ, OnUpdateAlgorithmsVoice8Lorentz)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE9_LORENTZ, OnUpdateAlgorithmsVoice9Lorentz)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE10_LORENTZ, OnUpdateAlgorithmsVoice10Lorentz)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE11_LORENTZ, OnUpdateAlgorithmsVoice11Lorentz)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE12_LORENTZ, OnUpdateAlgorithmsVoice12Lorentz)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE13_LORENTZ, OnUpdateAlgorithmsVoice13Lorentz)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE14_LORENTZ, OnUpdateAlgorithmsVoice14Lorentz)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE15_LORENTZ, OnUpdateAlgorithmsVoice15Lorentz)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE16_LORENTZ, OnUpdateAlgorithmsVoice16Lorentz)
	ON_COMMAND(IDC_OPTS_GRAPH, OnOptsGraph)
	ON_UPDATE_COMMAND_UI(IDC_OPTS_GRAPH, OnUpdateOptsGraph)
	ON_EN_KILLFOCUS(IDC_STARTCOUNTER_ED, OnKillfocusStartcounterEd)
	ON_COMMAND(IDC_OPTS_GETNOTE, OnOptsGetnote)
	ON_UPDATE_COMMAND_UI(IDC_OPTS_GETNOTE, OnUpdateOptsGetnote)
	ON_BN_CLICKED(IDC_GLOBALTEMPO_BT, OnGlobaltempoBt)
	ON_COMMAND(IDC_GLOBAL_CHANGES, OnGlobalChanges)
	ON_UPDATE_COMMAND_UI(IDC_GLOBAL_CHANGES, OnUpdateGlobalChanges)
	ON_COMMAND(ID_FILE_PLAYMIDI, OnFilePlaymidi)
	ON_UPDATE_COMMAND_UI(ID_FILE_PLAYMIDI, OnUpdateFilePlaymidi)
	ON_COMMAND(ID_CALCULATOR, OnCalculator)
	ON_COMMAND(IDC_OPTS_PERCUSSION, OnOptsPercussion)
	ON_UPDATE_COMMAND_UI(IDC_OPTS_PERCUSSION, OnUpdateOptsPercussion)
	ON_BN_CLICKED(IDC_GLOBALTIMESIG_BT, OnGlobaltimesigBt)
	ON_COMMAND(IDC_TRANSPOSER, OnTransposer)
	ON_UPDATE_COMMAND_UI(IDC_TRANSPOSER, OnUpdateTransposer)
	ON_COMMAND(IDC_CHANGE_PASSWORD, OnChangePassword)
	ON_UPDATE_COMMAND_UI(IDC_CHANGE_PASSWORD, OnUpdateChangePassword)
	ON_COMMAND(IDC_UNLOCK_DOCUMENT, OnUnlockDocument)
	ON_UPDATE_COMMAND_UI(IDC_UNLOCK_DOCUMENT, OnUpdateUnlockDocument)
	ON_COMMAND(IDC_ROUND_EVENTS, OnRoundEvents)
	ON_UPDATE_COMMAND_UI(IDC_ROUND_EVENTS, OnUpdateRoundEvents)
	ON_COMMAND(IDC_INSTRUMENTATION, OnInstrumentation)
	ON_UPDATE_COMMAND_UI(IDC_INSTRUMENTATION, OnUpdateInstrumentation)
	ON_COMMAND(ID_COMMENTS, OnComments)
	ON_COMMAND(ALGORITHMS_VOICE1_EARTHWORM, OnVoice1Earthworm)
	ON_COMMAND(ALGORITHMS_VOICE2_EARTHWORM, OnVoice2Earthworm)
	ON_COMMAND(ALGORITHMS_VOICE3_EARTHWORM, OnVoice3Earthworm)
	ON_COMMAND(ALGORITHMS_VOICE4_EARTHWORM, OnVoice4Earthworm)
	ON_COMMAND(ALGORITHMS_VOICE5_EARTHWORM, OnVoice5Earthworm)
	ON_COMMAND(ALGORITHMS_VOICE6_EARTHWORM, OnVoice6Earthworm)
	ON_COMMAND(ALGORITHMS_VOICE7_EARTHWORM, OnVoice7Earthworm)
	ON_COMMAND(ALGORITHMS_VOICE8_EARTHWORM, OnVoice8Earthworm)
	ON_COMMAND(ALGORITHMS_VOICE9_EARTHWORM, OnVoice9Earthworm)
	ON_COMMAND(ALGORITHMS_VOICE10_EARTHWORM, OnVoice10Earthworm)
	ON_COMMAND(ALGORITHMS_VOICE11_EARTHWORM, OnVoice11Earthworm)
	ON_COMMAND(ALGORITHMS_VOICE12_EARTHWORM, OnVoice12Earthworm)
	ON_COMMAND(ALGORITHMS_VOICE13_EARTHWORM, OnVoice13Earthworm)
	ON_COMMAND(ALGORITHMS_VOICE14_EARTHWORM, OnVoice14Earthworm)
	ON_COMMAND(ALGORITHMS_VOICE15_EARTHWORM, OnVoice15Earthworm)
	ON_COMMAND(ALGORITHMS_VOICE16_EARTHWORM, OnVoice16Earthworm)
	ON_UPDATE_COMMAND_UI(ALGORITHMS_VOICE1_EARTHWORM, OnUpdateVoice1Earthworm)
	ON_UPDATE_COMMAND_UI(ALGORITHMS_VOICE2_EARTHWORM, OnUpdateVoice2Earthworm)
	ON_UPDATE_COMMAND_UI(ALGORITHMS_VOICE3_EARTHWORM, OnUpdateVoice3Earthworm)
	ON_UPDATE_COMMAND_UI(ALGORITHMS_VOICE4_EARTHWORM, OnUpdateVoice4Earthworm)
	ON_UPDATE_COMMAND_UI(ALGORITHMS_VOICE5_EARTHWORM, OnUpdateVoice5Earthworm)
	ON_UPDATE_COMMAND_UI(ALGORITHMS_VOICE6_EARTHWORM, OnUpdateVoice6Earthworm)
	ON_UPDATE_COMMAND_UI(ALGORITHMS_VOICE7_EARTHWORM, OnUpdateVoice7Earthworm)
	ON_UPDATE_COMMAND_UI(ALGORITHMS_VOICE8_EARTHWORM, OnUpdateVoice8Earthworm)
	ON_UPDATE_COMMAND_UI(ALGORITHMS_VOICE9_EARTHWORM, OnUpdateVoice9Earthworm)
	ON_UPDATE_COMMAND_UI(ALGORITHMS_VOICE10_EARTHWORM, OnUpdateVoice10Earthworm)
	ON_UPDATE_COMMAND_UI(ALGORITHMS_VOICE11_EARTHWORM, OnUpdateVoice11Earthworm)
	ON_UPDATE_COMMAND_UI(ALGORITHMS_VOICE12_EARTHWORM, OnUpdateVoice12Earthworm)
	ON_UPDATE_COMMAND_UI(ALGORITHMS_VOICE13_EARTHWORM, OnUpdateVoice13Earthworm)
	ON_UPDATE_COMMAND_UI(ALGORITHMS_VOICE14_EARTHWORM, OnUpdateVoice14Earthworm)
	ON_UPDATE_COMMAND_UI(ALGORITHMS_VOICE15_EARTHWORM, OnUpdateVoice15Earthworm)
	ON_UPDATE_COMMAND_UI(ALGORITHMS_VOICE16_EARTHWORM, OnUpdateVoice16Earthworm)
	ON_COMMAND(ID_ALGORITHMS_VOICE1_RANDOM, OnAlgorithmsVoice1Random)
	ON_COMMAND(ID_ALGORITHMS_VOICE2_RANDOM, OnAlgorithmsVoice2Random)
	ON_COMMAND(ID_ALGORITHMS_VOICE3_RANDOM, OnAlgorithmsVoice3Random)
	ON_COMMAND(ID_ALGORITHMS_VOICE4_RANDOM, OnAlgorithmsVoice4Random)
	ON_COMMAND(ID_ALGORITHMS_VOICE5_RANDOM, OnAlgorithmsVoice5Random)
	ON_COMMAND(ID_ALGORITHMS_VOICE6_RANDOM, OnAlgorithmsVoice6Random)
	ON_COMMAND(ID_ALGORITHMS_VOICE7_RANDOM, OnAlgorithmsVoice7Random)
	ON_COMMAND(ID_ALGORITHMS_VOICE8_RANDOM, OnAlgorithmsVoice8Random)
	ON_COMMAND(ID_ALGORITHMS_VOICE9_RANDOM, OnAlgorithmsVoice9Random)
	ON_COMMAND(ID_ALGORITHMS_VOICE10_RANDOM, OnAlgorithmsVoice10Random)
	ON_COMMAND(ID_ALGORITHMS_VOICE11_RANDOM, OnAlgorithmsVoice11Random)
	ON_COMMAND(ID_ALGORITHMS_VOICE12_RANDOM, OnAlgorithmsVoice12Random)
	ON_COMMAND(ID_ALGORITHMS_VOICE13_RANDOM, OnAlgorithmsVoice13Random)
	ON_COMMAND(ID_ALGORITHMS_VOICE14_RANDOM, OnAlgorithmsVoice14Random)
	ON_COMMAND(ID_ALGORITHMS_VOICE15_RANDOM, OnAlgorithmsVoice15Random)
	ON_COMMAND(ID_ALGORITHMS_VOICE16_RANDOM, OnAlgorithmsVoice16Random)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE1_RANDOM, OnUpdateAlgorithmsVoice1Random)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE2_RANDOM, OnUpdateAlgorithmsVoice2Random)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE3_RANDOM, OnUpdateAlgorithmsVoice3Random)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE4_RANDOM, OnUpdateAlgorithmsVoice4Random)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE5_RANDOM, OnUpdateAlgorithmsVoice5Random)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE6_RANDOM, OnUpdateAlgorithmsVoice6Random)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE7_RANDOM, OnUpdateAlgorithmsVoice7Random)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE8_RANDOM, OnUpdateAlgorithmsVoice8Random)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE9_RANDOM, OnUpdateAlgorithmsVoice9Random)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE10_RANDOM, OnUpdateAlgorithmsVoice10Random)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE11_RANDOM, OnUpdateAlgorithmsVoice11Random)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE13_RANDOM, OnUpdateAlgorithmsVoice13Random)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE14_RANDOM, OnUpdateAlgorithmsVoice14Random)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE15_RANDOM, OnUpdateAlgorithmsVoice15Random)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE16_RANDOM, OnUpdateAlgorithmsVoice16Random)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE12_RANDOM, OnUpdateAlgorithmsVoice12Random)
	ON_COMMAND(ID_ALGORITHMS_VOICE1_HAILSTONE, OnAlgorithmsVoice1Hailstone)
	ON_COMMAND(ID_ALGORITHMS_VOICE2_HAILSTONE, OnAlgorithmsVoice2Hailstone)
	ON_COMMAND(ID_ALGORITHMS_VOICE3_HAILSTONE, OnAlgorithmsVoice3Hailstone)
	ON_COMMAND(ID_ALGORITHMS_VOICE4_HAILSTONE, OnAlgorithmsVoice4Hailstone)
	ON_COMMAND(ID_ALGORITHMS_VOICE5_HAILSTONE, OnAlgorithmsVoice5Hailstone)
	ON_COMMAND(ID_ALGORITHMS_VOICE6_HAILSTONE, OnAlgorithmsVoice6Hailstone)
	ON_COMMAND(ID_ALGORITHMS_VOICE7_HAILSTONE, OnAlgorithmsVoice7Hailstone)
	ON_COMMAND(ID_ALGORITHMS_VOICE8_HAILSTONE, OnAlgorithmsVoice8Hailstone)
	ON_COMMAND(ID_ALGORITHMS_VOICE9_HAILSTONE, OnAlgorithmsVoice9Hailstone)
	ON_COMMAND(ID_ALGORITHMS_VOICE10_HAILSTONE, OnAlgorithmsVoice10Hailstone)
	ON_COMMAND(ID_ALGORITHMS_VOICE11_HAILSTONE, OnAlgorithmsVoice11Hailstone)
	ON_COMMAND(ID_ALGORITHMS_VOICE12_HAILSTONE, OnAlgorithmsVoice12Hailstone)
	ON_COMMAND(ID_ALGORITHMS_VOICE13_HAILSTONE, OnAlgorithmsVoice13Hailstone)
	ON_COMMAND(ID_ALGORITHMS_VOICE14_HAILSTONE, OnAlgorithmsVoice14Hailstone)
	ON_COMMAND(ID_ALGORITHMS_VOICE15_HAILSTONE, OnAlgorithmsVoice15Hailstone)
	ON_COMMAND(ID_ALGORITHMS_VOICE16_HAILSTONE, OnAlgorithmsVoice16Hailstone)
	ON_COMMAND(ID_ALGORITHMS_VOICE1_WOLFRAM, OnAlgorithmsVoice1Wolfram)
	ON_COMMAND(ID_ALGORITHMS_VOICE2_WOLFRAM, OnAlgorithmsVoice2Wolfram)
	ON_COMMAND(ID_ALGORITHMS_VOICE3_WOLFRAM, OnAlgorithmsVoice3Wolfram)
	ON_COMMAND(ID_ALGORITHMS_VOICE4_WOLFRAM, OnAlgorithmsVoice4Wolfram)
	ON_COMMAND(ID_ALGORITHMS_VOICE5_WOLFRAM, OnAlgorithmsVoice5Wolfram)
	ON_COMMAND(ID_ALGORITHMS_VOICE6_WOLFRAM, OnAlgorithmsVoice6Wolfram)
	ON_COMMAND(ID_ALGORITHMS_VOICE7_WOLFRAM, OnAlgorithmsVoice7Wolfram)
	ON_COMMAND(ID_ALGORITHMS_VOICE8_WOLFRAM, OnAlgorithmsVoice8Wolfram)
	ON_COMMAND(ID_ALGORITHMS_VOICE9_WOLFRAM, OnAlgorithmsVoice9Wolfram)
	ON_COMMAND(ID_ALGORITHMS_VOICE10_WOLFRAM, OnAlgorithmsVoice10Wolfram)
	ON_COMMAND(ID_ALGORITHMS_VOICE11_WOLFRAM, OnAlgorithmsVoice11Wolfram)
	ON_COMMAND(ID_ALGORITHMS_VOICE12_WOLFRAM, OnAlgorithmsVoice12Wolfram)
	ON_COMMAND(ID_ALGORITHMS_VOICE13_WOLFRAM, OnAlgorithmsVoice13Wolfram)
	ON_COMMAND(ID_ALGORITHMS_VOICE14_WOLFRAM, OnAlgorithmsVoice14Wolfram)
	ON_COMMAND(ID_ALGORITHMS_VOICE15_WOLFRAM, OnAlgorithmsVoice15Wolfram)
	ON_COMMAND(ID_ALGORITHMS_VOICE16_WOLFRAM, OnAlgorithmsVoice16Wolfram)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE1_HAILSTONE, OnUpdateAlgorithmsVoice1Hailstone)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE2_HAILSTONE, OnUpdateAlgorithmsVoice2Hailstone)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE3_HAILSTONE, OnUpdateAlgorithmsVoice3Hailstone)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE4_HAILSTONE, OnUpdateAlgorithmsVoice4Hailstone)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE5_HAILSTONE, OnUpdateAlgorithmsVoice5Hailstone)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE6_HAILSTONE, OnUpdateAlgorithmsVoice6Hailstone)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE7_HAILSTONE, OnUpdateAlgorithmsVoice7Hailstone)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE8_HAILSTONE, OnUpdateAlgorithmsVoice8Hailstone)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE9_HAILSTONE, OnUpdateAlgorithmsVoice9Hailstone)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE10_HAILSTONE, OnUpdateAlgorithmsVoice10Hailstone)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE11_HAILSTONE, OnUpdateAlgorithmsVoice11Hailstone)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE12_HAILSTONE, OnUpdateAlgorithmsVoice12Hailstone)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE13_HAILSTONE, OnUpdateAlgorithmsVoice13Hailstone)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE14_HAILSTONE, OnUpdateAlgorithmsVoice14Hailstone)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE15_HAILSTONE, OnUpdateAlgorithmsVoice15Hailstone)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE16_HAILSTONE, OnUpdateAlgorithmsVoice16Hailstone)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE1_WOLFRAM, OnUpdateAlgorithmsVoice1Wolfram)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE2_WOLFRAM, OnUpdateAlgorithmsVoice2Wolfram)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE3_WOLFRAM, OnUpdateAlgorithmsVoice3Wolfram)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE4_WOLFRAM, OnUpdateAlgorithmsVoice4Wolfram)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE5_WOLFRAM, OnUpdateAlgorithmsVoice5Wolfram)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE6_WOLFRAM, OnUpdateAlgorithmsVoice6Wolfram)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE7_WOLFRAM, OnUpdateAlgorithmsVoice7Wolfram)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE8_WOLFRAM, OnUpdateAlgorithmsVoice8Wolfram)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE9_WOLFRAM, OnUpdateAlgorithmsVoice9Wolfram)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE10_WOLFRAM, OnUpdateAlgorithmsVoice10Wolfram)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE11_WOLFRAM, OnUpdateAlgorithmsVoice11Wolfram)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE12_WOLFRAM, OnUpdateAlgorithmsVoice12Wolfram)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE13_WOLFRAM, OnUpdateAlgorithmsVoice13Wolfram)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE14_WOLFRAM, OnUpdateAlgorithmsVoice14Wolfram)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE15_WOLFRAM, OnUpdateAlgorithmsVoice15Wolfram)
	ON_UPDATE_COMMAND_UI(ID_ALGORITHMS_VOICE16_WOLFRAM, OnUpdateAlgorithmsVoice16Wolfram)
	ON_COMMAND(ID_VIEW_TIPS, OnViewTips)
	ON_UPDATE_COMMAND_UI(ID_VIEW_TIPS, OnUpdateViewTips)
	ON_BN_CLICKED(IDC_LOCK_GENCOUNTER, OnLockGencounter)
	//}}AFX_MSG_MAP

	ON_NOTIFY_EX( TTN_NEEDTEXT, 0, OnToolTipNotify)


	END_MESSAGE_MAP()
	/////////////////////////////////////////////////////////////////////////////
// CFractMusView construction/destruction

CFractMusView::CFractMusView()
	: CFormView(CFractMusView::IDD)
{
	//{{AFX_DATA_INIT(CFractMusView)
	m_StartFrom = 0;
	//}}AFX_DATA_INIT
	// TODO: add construction code here

	m_pEvents = NULL;
	m_pGraph = NULL;
	m_TempoMarking = 96;
	GeneralCounter = 0;
	UpdateSettings = FALSE;
	IsDocumentLocked = FALSE;

	for(int i = VOICE1; i<MAXVOICES; i++)
	{
		m_StartNum[i] = 0;
	}

	m_IsActive[VOICE1]=TRUE;
	for (i=VOICE2; i<MAXVOICES; i++)
		m_IsActive[i]=FALSE;
	
	for (i=VOICE1; i<MAXVOICES; i++)
		m_Modulates[i]	= FALSE;

	for (i=VOICE1; i<MAXVOICES; i++)
		m_ScaleID[i]	= MAJOR;

	for (i=VOICE1; i<MAXVOICES; i++)
		m_NumOctsID[i]	= ONE_OCT;

	for (i=VOICE1; i<MAXVOICES; i++)
		m_Note[i]	= 48;

	for (i=VOICE1; i<MAXVOICES; i++)
		m_NoteValueID[i]= 2;

	for(i = 0; i<128; i++)
		m_randDlg.Instrument[i] = TRUE;

	for(i = 0; i<47; i++)
		m_randDlg.PercussionInstrument[i] = TRUE;

	for(i = 0; i<97; i++)
		m_randDlg.Notes[i] = TRUE;

	for(i = 0; i<15; i++)
		m_randDlg.Scales[i] = TRUE;

	m_changesDlg.UserScale = new int [Major[0] + 1];

	for (i=0; i<=Major[0]; i++)
		m_changesDlg.UserScale[i] = Major[i];


	for (i=VOICE1; i<MAXVOICES; i++)
	{
		WolframArray1[i] = NULL;
		WolframArray2[i] = NULL;
	}



	StartNumID[VOICE1] = IDC_VOICE1_STARTNUM_EDIT;
	StartNumID[VOICE2] = IDC_VOICE2_STARTNUM_EDIT;
	StartNumID[VOICE3] = IDC_VOICE3_STARTNUM_EDIT;
	StartNumID[VOICE4] = IDC_VOICE4_STARTNUM_EDIT;
	StartNumID[VOICE5] = IDC_VOICE5_STARTNUM_EDIT;
	StartNumID[VOICE6] = IDC_VOICE6_STARTNUM_EDIT;
	StartNumID[VOICE7] = IDC_VOICE7_STARTNUM_EDIT;
	StartNumID[VOICE8] = IDC_VOICE8_STARTNUM_EDIT;
	StartNumID[VOICE9] = IDC_VOICE9_STARTNUM_EDIT;
	StartNumID[VOICE10] = IDC_VOICE10_STARTNUM_EDIT;
	StartNumID[VOICE11] = IDC_VOICE11_STARTNUM_EDIT;
	StartNumID[VOICE12] = IDC_VOICE12_STARTNUM_EDIT;
	StartNumID[VOICE13] = IDC_VOICE13_STARTNUM_EDIT;
	StartNumID[VOICE14] = IDC_VOICE14_STARTNUM_EDIT;
	StartNumID[VOICE15] = IDC_VOICE15_STARTNUM_EDIT;
	StartNumID[VOICE16] = IDC_VOICE16_STARTNUM_EDIT;

	ScaleID[VOICE1] = IDC_VOICE1_SCALE_COMBO;
	ScaleID[VOICE2] = IDC_VOICE2_SCALE_COMBO;
	ScaleID[VOICE3] = IDC_VOICE3_SCALE_COMBO;
	ScaleID[VOICE4] = IDC_VOICE4_SCALE_COMBO;
	ScaleID[VOICE5] = IDC_VOICE5_SCALE_COMBO;
	ScaleID[VOICE6] = IDC_VOICE6_SCALE_COMBO;
	ScaleID[VOICE7] = IDC_VOICE7_SCALE_COMBO;
	ScaleID[VOICE8] = IDC_VOICE8_SCALE_COMBO;
	ScaleID[VOICE9] = IDC_VOICE9_SCALE_COMBO;
	ScaleID[VOICE10] = IDC_VOICE10_SCALE_COMBO;
	ScaleID[VOICE11] = IDC_VOICE11_SCALE_COMBO;
	ScaleID[VOICE12] = IDC_VOICE12_SCALE_COMBO;
	ScaleID[VOICE13] = IDC_VOICE13_SCALE_COMBO;
	ScaleID[VOICE14] = IDC_VOICE14_SCALE_COMBO;
	ScaleID[VOICE15] = IDC_VOICE15_SCALE_COMBO;
	ScaleID[VOICE16] = IDC_VOICE16_SCALE_COMBO;

	NumOctsID[VOICE1] = IDC_VOICE1_NUMOCTS_COMBO;
	NumOctsID[VOICE2] = IDC_VOICE2_NUMOCTS_COMBO;
	NumOctsID[VOICE3] = IDC_VOICE3_NUMOCTS_COMBO;
	NumOctsID[VOICE4] = IDC_VOICE4_NUMOCTS_COMBO;
	NumOctsID[VOICE5] = IDC_VOICE5_NUMOCTS_COMBO;
	NumOctsID[VOICE6] = IDC_VOICE6_NUMOCTS_COMBO;
	NumOctsID[VOICE7] = IDC_VOICE7_NUMOCTS_COMBO;
	NumOctsID[VOICE8] = IDC_VOICE8_NUMOCTS_COMBO;
	NumOctsID[VOICE9] = IDC_VOICE9_NUMOCTS_COMBO;
	NumOctsID[VOICE10] = IDC_VOICE10_NUMOCTS_COMBO;
	NumOctsID[VOICE11] = IDC_VOICE11_NUMOCTS_COMBO;
	NumOctsID[VOICE12] = IDC_VOICE12_NUMOCTS_COMBO;
	NumOctsID[VOICE13] = IDC_VOICE13_NUMOCTS_COMBO;
	NumOctsID[VOICE14] = IDC_VOICE14_NUMOCTS_COMBO;
	NumOctsID[VOICE15] = IDC_VOICE15_NUMOCTS_COMBO;
	NumOctsID[VOICE16] = IDC_VOICE16_NUMOCTS_COMBO;

	NoteID[VOICE1] = IDC_VOICE1_STARTNOTE_COMBO;
	NoteID[VOICE2] = IDC_VOICE2_STARTNOTE_COMBO;
	NoteID[VOICE3] = IDC_VOICE3_STARTNOTE_COMBO;
	NoteID[VOICE4] = IDC_VOICE4_STARTNOTE_COMBO;
	NoteID[VOICE5] = IDC_VOICE5_STARTNOTE_COMBO;
	NoteID[VOICE6] = IDC_VOICE6_STARTNOTE_COMBO;
	NoteID[VOICE7] = IDC_VOICE7_STARTNOTE_COMBO;
	NoteID[VOICE8] = IDC_VOICE8_STARTNOTE_COMBO;
	NoteID[VOICE9] = IDC_VOICE9_STARTNOTE_COMBO;
	NoteID[VOICE10] = IDC_VOICE10_STARTNOTE_COMBO;
	NoteID[VOICE11] = IDC_VOICE11_STARTNOTE_COMBO;
	NoteID[VOICE12] = IDC_VOICE12_STARTNOTE_COMBO;
	NoteID[VOICE13] = IDC_VOICE13_STARTNOTE_COMBO;
	NoteID[VOICE14] = IDC_VOICE14_STARTNOTE_COMBO;
	NoteID[VOICE15] = IDC_VOICE15_STARTNOTE_COMBO;
	NoteID[VOICE16] = IDC_VOICE16_STARTNOTE_COMBO;

	ActiveID[VOICE1] = IDC_VOICE1_ON_CHK;
	ActiveID[VOICE2] = IDC_VOICE2_ON_CHK;
	ActiveID[VOICE3] = IDC_VOICE3_ON_CHK;
	ActiveID[VOICE4] = IDC_VOICE4_ON_CHK;
	ActiveID[VOICE5] = IDC_VOICE5_ON_CHK;
	ActiveID[VOICE6] = IDC_VOICE6_ON_CHK;
	ActiveID[VOICE7] = IDC_VOICE7_ON_CHK;
	ActiveID[VOICE8] = IDC_VOICE8_ON_CHK;
	ActiveID[VOICE9] = IDC_VOICE9_ON_CHK;
	ActiveID[VOICE10] = IDC_VOICE10_ON_CHK;
	ActiveID[VOICE11] = IDC_VOICE11_ON_CHK;
	ActiveID[VOICE12] = IDC_VOICE12_ON_CHK;
	ActiveID[VOICE13] = IDC_VOICE13_ON_CHK;
	ActiveID[VOICE14] = IDC_VOICE14_ON_CHK;
	ActiveID[VOICE15] = IDC_VOICE15_ON_CHK;
	ActiveID[VOICE16] = IDC_VOICE16_ON_CHK;

	NoteValueID[VOICE1] = IDC_VOICE1_NOTEVALUE_COMBO;
	NoteValueID[VOICE2] = IDC_VOICE2_NOTEVALUE_COMBO;
	NoteValueID[VOICE3] = IDC_VOICE3_NOTEVALUE_COMBO;
	NoteValueID[VOICE4] = IDC_VOICE4_NOTEVALUE_COMBO;
	NoteValueID[VOICE5] = IDC_VOICE5_NOTEVALUE_COMBO;
	NoteValueID[VOICE6] = IDC_VOICE6_NOTEVALUE_COMBO;
	NoteValueID[VOICE7] = IDC_VOICE7_NOTEVALUE_COMBO;
	NoteValueID[VOICE8] = IDC_VOICE8_NOTEVALUE_COMBO;
	NoteValueID[VOICE9] = IDC_VOICE9_NOTEVALUE_COMBO;
	NoteValueID[VOICE10] = IDC_VOICE10_NOTEVALUE_COMBO;
	NoteValueID[VOICE11] = IDC_VOICE11_NOTEVALUE_COMBO;
	NoteValueID[VOICE12] = IDC_VOICE12_NOTEVALUE_COMBO;
	NoteValueID[VOICE13] = IDC_VOICE13_NOTEVALUE_COMBO;
	NoteValueID[VOICE14] = IDC_VOICE14_NOTEVALUE_COMBO;
	NoteValueID[VOICE15] = IDC_VOICE15_NOTEVALUE_COMBO;
	NoteValueID[VOICE16] = IDC_VOICE16_NOTEVALUE_COMBO;

	ModulatesID[VOICE1] = IDC_VOICE1_MODULATE_CHK;
	ModulatesID[VOICE2] = IDC_VOICE2_MODULATE_CHK;
	ModulatesID[VOICE3] = IDC_VOICE3_MODULATE_CHK;
	ModulatesID[VOICE4] = IDC_VOICE4_MODULATE_CHK;
	ModulatesID[VOICE5] = IDC_VOICE5_MODULATE_CHK;
	ModulatesID[VOICE6] = IDC_VOICE6_MODULATE_CHK;
	ModulatesID[VOICE7] = IDC_VOICE7_MODULATE_CHK;
	ModulatesID[VOICE8] = IDC_VOICE8_MODULATE_CHK;
	ModulatesID[VOICE9] = IDC_VOICE9_MODULATE_CHK;
	ModulatesID[VOICE10] = IDC_VOICE10_MODULATE_CHK;
	ModulatesID[VOICE11] = IDC_VOICE11_MODULATE_CHK;
	ModulatesID[VOICE12] = IDC_VOICE12_MODULATE_CHK;
	ModulatesID[VOICE13] = IDC_VOICE13_MODULATE_CHK;
	ModulatesID[VOICE14] = IDC_VOICE14_MODULATE_CHK;
	ModulatesID[VOICE15] = IDC_VOICE15_MODULATE_CHK;
	ModulatesID[VOICE16] = IDC_VOICE16_MODULATE_CHK;

	MidiButtonID[VOICE1] = IDC_VOICE1_MIDI_BUTTON;
	MidiButtonID[VOICE2] = IDC_VOICE2_MIDI_BUTTON;
	MidiButtonID[VOICE3] = IDC_VOICE3_MIDI_BUTTON;
	MidiButtonID[VOICE4] = IDC_VOICE4_MIDI_BUTTON;
	MidiButtonID[VOICE5] = IDC_VOICE5_MIDI_BUTTON;
	MidiButtonID[VOICE6] = IDC_VOICE6_MIDI_BUTTON;
	MidiButtonID[VOICE7] = IDC_VOICE7_MIDI_BUTTON;
	MidiButtonID[VOICE8] = IDC_VOICE8_MIDI_BUTTON;
	MidiButtonID[VOICE9] = IDC_VOICE9_MIDI_BUTTON;
	MidiButtonID[VOICE10] = IDC_VOICE10_MIDI_BUTTON;
	MidiButtonID[VOICE11] = IDC_VOICE11_MIDI_BUTTON;
	MidiButtonID[VOICE12] = IDC_VOICE12_MIDI_BUTTON;
	MidiButtonID[VOICE13] = IDC_VOICE13_MIDI_BUTTON;
	MidiButtonID[VOICE14] = IDC_VOICE14_MIDI_BUTTON;
	MidiButtonID[VOICE15] = IDC_VOICE15_MIDI_BUTTON;
	MidiButtonID[VOICE16] = IDC_VOICE16_MIDI_BUTTON;

	m_V1MidiSet[VOICE1].m_title = "Extra - voice 1";
	m_V1MidiSet[VOICE2].m_title = "Extra - voice 2";
	m_V1MidiSet[VOICE3].m_title = "Extra - voice 3";
	m_V1MidiSet[VOICE4].m_title = "Extra - voice 4";
	m_V1MidiSet[VOICE5].m_title = "Extra - voice 5";
	m_V1MidiSet[VOICE6].m_title = "Extra - voice 6";
	m_V1MidiSet[VOICE7].m_title = "Extra - voice 7";
	m_V1MidiSet[VOICE8].m_title = "Extra - voice 8";
	m_V1MidiSet[VOICE9].m_title = "Extra - voice 9";
	m_V1MidiSet[VOICE10].m_title = "Extra - voice 10";
	m_V1MidiSet[VOICE11].m_title = "Extra - voice 11";
	m_V1MidiSet[VOICE12].m_title = "Extra - voice 12";
	m_V1MidiSet[VOICE13].m_title = "Extra - voice 13";
	m_V1MidiSet[VOICE14].m_title = "Extra - voice 14";
	m_V1MidiSet[VOICE15].m_title = "Extra - voice 15";
	m_V1MidiSet[VOICE16].m_title = "Extra - voice 16";

	m_V1Modul[VOICE1].m_title = "Modulation - voice 1";
	m_V1Modul[VOICE2].m_title = "Modulation - voice 2";
	m_V1Modul[VOICE3].m_title = "Modulation - voice 3";
	m_V1Modul[VOICE4].m_title = "Modulation - voice 4";
	m_V1Modul[VOICE5].m_title = "Modulation - voice 5";
	m_V1Modul[VOICE6].m_title = "Modulation - voice 6";
	m_V1Modul[VOICE7].m_title = "Modulation - voice 7";
	m_V1Modul[VOICE8].m_title = "Modulation - voice 8";
	m_V1Modul[VOICE9].m_title = "Modulation - voice 9";
	m_V1Modul[VOICE10].m_title = "Modulation - voice 10";
	m_V1Modul[VOICE11].m_title = "Modulation - voice 11";
	m_V1Modul[VOICE12].m_title = "Modulation - voice 12";
	m_V1Modul[VOICE13].m_title = "Modulation - voice 13";
	m_V1Modul[VOICE14].m_title = "Modulation - voice 14";
	m_V1Modul[VOICE15].m_title = "Modulation - voice 15";
	m_V1Modul[VOICE16].m_title = "Modulation - voice 16";

	m_V1UsrScl[VOICE1].m_title = "User scale - voice 1";
	m_V1UsrScl[VOICE2].m_title = "User scale - voice 2";
	m_V1UsrScl[VOICE3].m_title = "User scale - voice 3";
	m_V1UsrScl[VOICE4].m_title = "User scale - voice 4";
	m_V1UsrScl[VOICE5].m_title = "User scale - voice 5";
	m_V1UsrScl[VOICE6].m_title = "User scale - voice 6";
	m_V1UsrScl[VOICE7].m_title = "User scale - voice 7";
	m_V1UsrScl[VOICE8].m_title = "User scale - voice 8";
	m_V1UsrScl[VOICE9].m_title = "User scale - voice 9";
	m_V1UsrScl[VOICE10].m_title = "User scale - voice 10";
	m_V1UsrScl[VOICE11].m_title = "User scale - voice 11";
	m_V1UsrScl[VOICE12].m_title = "User scale - voice 12";
	m_V1UsrScl[VOICE13].m_title = "User scale - voice 13";
	m_V1UsrScl[VOICE14].m_title = "User scale - voice 14";
	m_V1UsrScl[VOICE15].m_title = "User scale - voice 15";
	m_V1UsrScl[VOICE16].m_title = "User scale - voice 16";

	m_NoteVal[VOICE1].title = "Note Value - voice 1";
	m_NoteVal[VOICE2].title = "Note Value - voice 2";
	m_NoteVal[VOICE3].title = "Note Value - voice 3";
	m_NoteVal[VOICE4].title = "Note Value - voice 4";
	m_NoteVal[VOICE5].title = "Note Value - voice 5";
	m_NoteVal[VOICE6].title = "Note Value - voice 6";
	m_NoteVal[VOICE7].title = "Note Value - voice 7";
	m_NoteVal[VOICE8].title = "Note Value - voice 8";
	m_NoteVal[VOICE9].title = "Note Value - voice 9";
	m_NoteVal[VOICE10].title = "Note Value - voice 10";
	m_NoteVal[VOICE11].title = "Note Value - voice 11";
	m_NoteVal[VOICE12].title = "Note Value - voice 12";
	m_NoteVal[VOICE13].title = "Note Value - voice 13";
	m_NoteVal[VOICE14].title = "Note Value - voice 14";
	m_NoteVal[VOICE15].title = "Note Value - voice 15";
	m_NoteVal[VOICE16].title = "Note Value - voice 16";

}

CFractMusView::~CFractMusView()
{
	if(m_pGraph!=NULL)
	{
		m_pGraph->SendMessage(WM_CLOSE);
	}

	for (int i=VOICE1; i<MAXVOICES; i++)
	{
		if (WolframArray1[i])
		{
			delete WolframArray1[i];
			WolframArray1[i] = NULL;
		}
		if (WolframArray2[i])
		{
			delete WolframArray2[i];
			WolframArray2[i] = NULL;
		}
	}

}



void CFractMusView::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CFractMusView)
	DDX_Text(pDX, IDC_STARTFROM_BT, m_StartFrom);
	DDV_MinMaxLong(pDX, m_StartFrom, 0, 50000);
	//}}AFX_DATA_MAP

	DDX_Text(pDX, IDC_GENCOUNTER_ED, GeneralCounter);
	DDV_MinMaxLong(pDX, GeneralCounter, 0, 50000);

	DDX_Text(pDX, IDC_STARTCOUNTER_ED, StartCounter);
	DDV_MinMaxLong(pDX, StartCounter, 0, 50000);


	DDX_Text(pDX, IDC_VOICE1_STARTNUM_EDIT, m_StartNum[VOICE1]);
	DDV_MinMaxLong(pDX, m_StartNum[VOICE1], -20000, 20000);
	DDX_Check(pDX, IDC_VOICE1_ON_CHK, m_IsActive[VOICE1]);
	DDX_Check(pDX, IDC_VOICE1_MODULATE_CHK, m_Modulates[VOICE1]);

	DDX_Text(pDX, IDC_VOICE2_STARTNUM_EDIT, m_StartNum[VOICE2]);
	DDV_MinMaxLong(pDX, m_StartNum[VOICE2], -20000, 20000);
	DDX_Check(pDX, IDC_VOICE2_ON_CHK, m_IsActive[VOICE2]);
	DDX_Check(pDX, IDC_VOICE2_MODULATE_CHK, m_Modulates[VOICE2]);

	DDX_Text(pDX, IDC_VOICE3_STARTNUM_EDIT, m_StartNum[VOICE3]);
	DDV_MinMaxLong(pDX, m_StartNum[VOICE3], -20000, 20000);
	DDX_Check(pDX, IDC_VOICE3_ON_CHK, m_IsActive[VOICE3]);
	DDX_Check(pDX, IDC_VOICE3_MODULATE_CHK, m_Modulates[VOICE3]);

	DDX_Text(pDX, IDC_VOICE4_STARTNUM_EDIT, m_StartNum[VOICE4]);
	DDV_MinMaxLong(pDX, m_StartNum[VOICE4], -20000, 20000);
	DDX_Check(pDX, IDC_VOICE4_ON_CHK, m_IsActive[VOICE4]);
	DDX_Check(pDX, IDC_VOICE4_MODULATE_CHK, m_Modulates[VOICE4]);

	DDX_Text(pDX, IDC_VOICE5_STARTNUM_EDIT, m_StartNum[VOICE5]);
	DDV_MinMaxLong(pDX, m_StartNum[VOICE5], -20000, 20000);
	DDX_Check(pDX, IDC_VOICE5_ON_CHK, m_IsActive[VOICE5]);
	DDX_Check(pDX, IDC_VOICE5_MODULATE_CHK, m_Modulates[VOICE5]);

	DDX_Text(pDX, IDC_VOICE6_STARTNUM_EDIT, m_StartNum[VOICE6]);
	DDV_MinMaxLong(pDX, m_StartNum[VOICE6], -20000, 20000);
	DDX_Check(pDX, IDC_VOICE6_ON_CHK, m_IsActive[VOICE6]);
	DDX_Check(pDX, IDC_VOICE6_MODULATE_CHK, m_Modulates[VOICE6]);

	DDX_Text(pDX, IDC_VOICE7_STARTNUM_EDIT, m_StartNum[VOICE7]);
	DDV_MinMaxLong(pDX, m_StartNum[VOICE7], -20000, 20000);
	DDX_Check(pDX, IDC_VOICE7_ON_CHK, m_IsActive[VOICE7]);
	DDX_Check(pDX, IDC_VOICE7_MODULATE_CHK, m_Modulates[VOICE7]);

	DDX_Text(pDX, IDC_VOICE8_STARTNUM_EDIT, m_StartNum[VOICE8]);
	DDV_MinMaxLong(pDX, m_StartNum[VOICE8], -20000, 20000);
	DDX_Check(pDX, IDC_VOICE8_ON_CHK, m_IsActive[VOICE8]);
	DDX_Check(pDX, IDC_VOICE8_MODULATE_CHK, m_Modulates[VOICE8]);

	DDX_Text(pDX, IDC_VOICE9_STARTNUM_EDIT, m_StartNum[VOICE9]);
	DDV_MinMaxLong(pDX, m_StartNum[VOICE9], -20000, 20000);
	DDX_Check(pDX, IDC_VOICE9_ON_CHK, m_IsActive[VOICE9]);
	DDX_Check(pDX, IDC_VOICE9_MODULATE_CHK, m_Modulates[VOICE9]);

	DDX_Text(pDX, IDC_VOICE10_STARTNUM_EDIT, m_StartNum[VOICE10]);
	DDV_MinMaxLong(pDX, m_StartNum[VOICE10], -20000, 20000);
	DDX_Check(pDX, IDC_VOICE10_ON_CHK, m_IsActive[VOICE10]);
	DDX_Check(pDX, IDC_VOICE10_MODULATE_CHK, m_Modulates[VOICE10]);

	DDX_Text(pDX, IDC_VOICE11_STARTNUM_EDIT, m_StartNum[VOICE11]);
	DDV_MinMaxLong(pDX, m_StartNum[VOICE11], -20000, 20000);
	DDX_Check(pDX, IDC_VOICE11_ON_CHK, m_IsActive[VOICE11]);
	DDX_Check(pDX, IDC_VOICE11_MODULATE_CHK, m_Modulates[VOICE11]);

	DDX_Text(pDX, IDC_VOICE12_STARTNUM_EDIT, m_StartNum[VOICE12]);
	DDV_MinMaxLong(pDX, m_StartNum[VOICE12], -20000, 20000);
	DDX_Check(pDX, IDC_VOICE12_ON_CHK, m_IsActive[VOICE12]);
	DDX_Check(pDX, IDC_VOICE12_MODULATE_CHK, m_Modulates[VOICE12]);

	DDX_Text(pDX, IDC_VOICE13_STARTNUM_EDIT, m_StartNum[VOICE13]);
	DDV_MinMaxLong(pDX, m_StartNum[VOICE13], -20000, 20000);
	DDX_Check(pDX, IDC_VOICE13_ON_CHK, m_IsActive[VOICE13]);
	DDX_Check(pDX, IDC_VOICE13_MODULATE_CHK, m_Modulates[VOICE13]);

	DDX_Text(pDX, IDC_VOICE14_STARTNUM_EDIT, m_StartNum[VOICE14]);
	DDV_MinMaxLong(pDX, m_StartNum[VOICE14], -20000, 20000);
	DDX_Check(pDX, IDC_VOICE14_ON_CHK, m_IsActive[VOICE14]);
	DDX_Check(pDX, IDC_VOICE14_MODULATE_CHK, m_Modulates[VOICE14]);

	DDX_Text(pDX, IDC_VOICE15_STARTNUM_EDIT, m_StartNum[VOICE15]);
	DDV_MinMaxLong(pDX, m_StartNum[VOICE15], -20000, 20000);
	DDX_Check(pDX, IDC_VOICE15_ON_CHK, m_IsActive[VOICE15]);
	DDX_Check(pDX, IDC_VOICE15_MODULATE_CHK, m_Modulates[VOICE15]);

	DDX_Text(pDX, IDC_VOICE16_STARTNUM_EDIT, m_StartNum[VOICE16]);
	DDV_MinMaxLong(pDX, m_StartNum[VOICE16], -20000, 20000);
	DDX_Check(pDX, IDC_VOICE16_ON_CHK, m_IsActive[VOICE16]);
	DDX_Check(pDX, IDC_VOICE16_MODULATE_CHK, m_Modulates[VOICE16]);


}

BOOL CFractMusView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CFormView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CFractMusView diagnostics

#ifdef _DEBUG

void CFractMusView::AssertValid() const
{
	CFormView::AssertValid();
}

void CFractMusView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}

CFractMusDoc* CFractMusView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CFractMusDoc)));
	return (CFractMusDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CFractMusView message handlers




void CFractMusView::OnInitialUpdate() 
{
	CFormView::OnInitialUpdate();
	EnableToolTips(TRUE);
	CFractMusDoc* pDoc = GetDocument();
	CWnd* pW = AfxGetMainWnd();

	CString tit;

	tit = pDoc->GetTitle();

	tit+=" - FractMus 2000   "+pDoc->m_Copyright;

	pW->SetWindowText(tit);	


	int i;
	char s[20];
	CComboBox* pB;
	CButton* pButt;
	CEdit* pE;
	CScrollBar* pS = (CScrollBar*)GetDlgItem(IDC_TEMPO_SLIDER2);
	CStatic* pSt = (CStatic*)GetDlgItem(IDC_TEMPO_STATIC);

	m_TempoMarking = Tempo = pDoc->m_TempoMarking;
	GeneralCounter = pDoc->dGeneralCounter;
	StartCounter = pDoc->dStartCounter;
	m_StartFrom = pDoc->dStartCounter;
	LockGeneralCounter = FALSE;
	UpdateSettings = FALSE;
	GetDlgItem(IDC_LOCK_GENCOUNTER)->SetWindowText("Lock");
	GetDlgItem(IDC_ROUND)->EnableWindow(TRUE);
	GetDlgItem(IDC_RESET_GENCOUNTER)->EnableWindow(TRUE);
	GetDlgItem(IDC_GENCOUNTER_ED)->EnableWindow(TRUE);
	PlayChoice = CURRENT_SETTINGS;
	GetDlgItem(IDC_PLAYFROM_CMB)->EnableWindow(FALSE);
	GetDlgItem(IDC_PLAYFROM_BT)->EnableWindow(FALSE);


	pS->SetScrollRange(20, 340);
	pS->SetScrollPos(m_TempoMarking);
	itoa(m_TempoMarking, s ,10);

	pSt->SetWindowText(s);

	pB = (CComboBox*)GetDlgItem(IDC_PLAYFROM_CMB);

	int Bcount = pB->GetCount();

	if (Bcount)
	{
		for(i=Bcount; i; i--)
			pB->DeleteString(i);
		pB->DeleteString(0);
	}

	if(pDoc->m_NumEvents>-1)
	{
		for(i=0; i<=pDoc->m_NumEvents; i++)
		{
			itoa(i, s, 10);
			pB->AddString(s);
		}
		pB->SetCurSel(0);

		OnComposition();
	}

	pB = (CComboBox*)GetDlgItem(IDC_TIMESIGNUM_CMB);
	pB->SetCurSel((pDoc->m_TimeSigNum)-1);

	pB = (CComboBox*)GetDlgItem(IDC_TIMESIGDEN_CMB);

	switch (pDoc->m_TimeSigDen)
	{
		case 1:
			pB->SetCurSel(0);
			break;
		case 2:
			pB->SetCurSel(1);
			break;
		case 4:
			pB->SetCurSel(2);
			break;
		case 8:
			pB->SetCurSel(3);
			break;
		case 16:
			pB->SetCurSel(4);
			break;
		case 32:
			pB->SetCurSel(5);
			break;
	}

	for(i=VOICE1; i<MAXVOICES; i++)
	{
		m_IsActive[i] = pDoc->m_IsActive[i];
		m_Modulates[i] = pDoc->m_Modulates[i];
		m_StartNum[i] = pDoc->m_StartNum[i];
		m_ScaleID[i] = pDoc->m_ScaleID[i];
		m_NumOctsID[i] = pDoc->m_NumOctsID[i];
		m_Note[i] = pDoc->m_Note[i];
		m_NoteValueID[i] = pDoc->m_NoteValueID[i];
		LogMap[i] = 0.2f;
		LogMap2[i] = 0.2f;
		FNoise[i] = 0.0f;
		InitUserScale(i);
		if (pDoc->m_ScaleID[i] == USER_DEFINED)
			SetUserScale(i);
	}

	UpdateData(FALSE);

	for(i=VOICE1; i<MAXVOICES; i++)
	{
		m_V1Modul[i].m_V1ModulDlgNum = pDoc->m_ModulNum[i];
		m_V1Modul[i].m_V1ModulDlgChoice= pDoc->m_ModulRandom[i];
	}

	for(i=VOICE1; i<MAXVOICES; i++)
	{
		m_V1MidiSet[i].m_instrument = pDoc->m_Instrument[i];
		m_V1MidiSet[i].m_mode = pDoc->m_mode[i];
		m_V1MidiSet[i].m_channel = pDoc->m_Channel[i];
	}

	CString nval;

	for (i = VOICE1; i<MAXVOICES; i++)
	{
		nval.Format("Other (%d)", (long)pDoc->m_NoteValue[i]);
		pB = (CComboBox*)GetDlgItem(NoteValueID[i]);
		pB->DeleteString(OTHER);
		pB->AddString(nval);
	}

	IsDocumentLocked = FALSE;

	if (pDoc->m_Password != "")
	{
		IsDocumentLocked = TRUE;


		if (!FractMusREGISTERED)
		{
			CString msg2;
			CString msg;

			msg.Format("Copyright Notice: %s.\n\nThis composition has been password-protected.\nYou will need to enter the password to remove restrictions.", pDoc->m_Copyright);
			MessageBox(msg, "FractMus", MB_OK+MB_ICONINFORMATION);

			msg2.Format("You are using a version of FractMus which\ndoes NOT include the password-protection feature.\nPlease read de README.TXT or see Help if you would like to use this feature.");
			MessageBox(msg2, "FractMus", MB_OK+MB_ICONEXCLAMATION);
		}

	}


	if (FractMusREGISTERED&&(pDoc->m_Password != ""))
	{

		CString msg;

		msg.Format("Copyright Notice: %s.\n\nThis composition has been password-protected.\nYou will need to enter the password to remove restrictions.", pDoc->m_Copyright);
		MessageBox(msg, "FractMus", MB_OK+MB_ICONINFORMATION);
		OnUnlockDocument();

		if (IsDocumentLocked)
		{
			MessageBox("You will not be able to edit nor save this composition", "FractMus", MB_OK+MB_ICONEXCLAMATION);

		}

	}

	CWnd* pWin = AfxGetMainWnd();
	CMenu* pM = pWin->GetMenu();
	if (pM)
	{
		pM->EnableMenuItem(2, (IsDocumentLocked? MF_BYPOSITION|MF_DISABLED : MF_BYPOSITION|MF_ENABLED));
	}
	else
		MessageBox("Null pointer");


	for(i=VOICE1; i<MAXVOICES; i++)
	{
		pE = (CEdit*)GetDlgItem(StartNumID[i])->EnableWindow(m_IsActive[i]&&(!IsDocumentLocked));
		pButt = (CButton*)GetDlgItem(ModulatesID[i])->EnableWindow(m_IsActive[i]&&(!IsDocumentLocked));
		pB = (CComboBox*)GetDlgItem(NumOctsID[i]);
		pB->SetCurSel(m_NumOctsID[i]);
		pB = (CComboBox*)GetDlgItem(NumOctsID[i])->EnableWindow(m_IsActive[i]&&(!IsDocumentLocked));
		pB = (CComboBox*)GetDlgItem(NoteID[i]);
		pB->SetCurSel(m_Note[i]);
		pB = (CComboBox*)GetDlgItem(NoteID[i])->EnableWindow(m_IsActive[i]&&(!IsDocumentLocked));
		pB = (CComboBox*)GetDlgItem(ScaleID[i]);
		pB->SetCurSel(m_ScaleID[i]);
		pB = (CComboBox*)GetDlgItem(ScaleID[i])->EnableWindow(m_IsActive[i]&&(!IsDocumentLocked));
		pB = (CComboBox*)GetDlgItem(NoteValueID[i]);
		pB->SetCurSel(m_NoteValueID[i]);
		pB = (CComboBox*)GetDlgItem(NoteValueID[i])->EnableWindow(m_IsActive[i]&&(!IsDocumentLocked));
		pButt = (CButton*)GetDlgItem(MidiButtonID[i])->EnableWindow(m_IsActive[i]&&(!IsDocumentLocked));
		pButt = (CButton*)GetDlgItem(ActiveID[i]);
		if (pDoc->m_Channel[i] == PercussionChannel)
		{
			pButt->SetWindowText(PercussionNames[ pDoc->m_Percussion[i] ]);
		}
		else
			pButt->SetWindowText(InstrumentNames[ pDoc->m_Instrument[i] ]);
		pButt = (CButton*)GetDlgItem(ActiveID[i])->EnableWindow((!IsDocumentLocked));


	}


	if (m_pEvents)
	{
		m_pEvents->DestroyWindow();
		EventsDone();
	}


	if(m_pGraph)
		m_pGraph->SendMessage(WM_CLOSE);


	Seed = time(NULL);

	if (pDoc->m_Comment!="")
	{
		OnComments();
	}


}

void CFractMusView::OnVoice1MidiButton() 
{
	UpdateMidi(VOICE1);	
}

void CFractMusView::OnVoice1ModulateChk() 
{
	UpdateModulation(VOICE1);
}


void CFractMusView::OnCloseupVoice1NotevalueCombo() 
{
	UpdateNoteValueCombo(VOICE1);	
}

void CFractMusView::OnCloseupVoice1NumoctsCombo() 
{
	UpdateNumOctsCombo(VOICE1);	
}

void CFractMusView::OnVoice1OnChk() 
{
	UpdateActive(VOICE1);	
}

void CFractMusView::OnCloseupVoice1ScaleCombo() 
{
	UpdateScaleCombo(VOICE1);	
}

void CFractMusView::OnCloseupVoice1StartnoteCombo() 
{
	UpdateNoteCombo(VOICE1);	
}

void CFractMusView::OnKillfocusVoice1StartnumEdit() 
{
	UpdateStartNum(VOICE1);	
}


void CFractMusView::OnVoice2MidiButton() 
{
	UpdateMidi(VOICE2);	
}

void CFractMusView::OnVoice2ModulateChk() 
{
	UpdateModulation(VOICE2);
}


void CFractMusView::OnCloseupVoice2NotevalueCombo() 
{
	UpdateNoteValueCombo(VOICE2);	
}

void CFractMusView::OnCloseupVoice2NumoctsCombo() 
{
	UpdateNumOctsCombo(VOICE2);	
}

void CFractMusView::OnVoice2OnChk() 
{
	UpdateActive(VOICE2);	
}

void CFractMusView::OnCloseupVoice2ScaleCombo() 
{
	UpdateScaleCombo(VOICE2);	
}

void CFractMusView::OnCloseupVoice2StartnoteCombo() 
{
	UpdateNoteCombo(VOICE2);	
}

void CFractMusView::OnKillfocusVoice2StartnumEdit() 
{
	UpdateStartNum(VOICE2);	
}


void CFractMusView::OnVoice3MidiButton() 
{
	UpdateMidi(VOICE3);	
}

void CFractMusView::OnVoice3ModulateChk() 
{
	UpdateModulation(VOICE3);
}


void CFractMusView::OnCloseupVoice3NotevalueCombo() 
{
	UpdateNoteValueCombo(VOICE3);	
}

void CFractMusView::OnCloseupVoice3NumoctsCombo() 
{
	UpdateNumOctsCombo(VOICE3);	
}

void CFractMusView::OnVoice3OnChk() 
{
	UpdateActive(VOICE3);	
}

void CFractMusView::OnCloseupVoice3ScaleCombo() 
{
	UpdateScaleCombo(VOICE3);	
}

void CFractMusView::OnCloseupVoice3StartnoteCombo() 
{
	UpdateNoteCombo(VOICE3);	
}

void CFractMusView::OnKillfocusVoice3StartnumEdit() 
{
	UpdateStartNum(VOICE3);	
}


void CFractMusView::OnVoice4MidiButton() 
{
	UpdateMidi(VOICE4);	
}

void CFractMusView::OnVoice4ModulateChk() 
{
	UpdateModulation(VOICE4);
}


void CFractMusView::OnCloseupVoice4NotevalueCombo() 
{
	UpdateNoteValueCombo(VOICE4);	
}

void CFractMusView::OnCloseupVoice4NumoctsCombo() 
{
	UpdateNumOctsCombo(VOICE4);	
}

void CFractMusView::OnVoice4OnChk() 
{
	UpdateActive(VOICE4);	
}

void CFractMusView::OnCloseupVoice4ScaleCombo() 
{
	UpdateScaleCombo(VOICE4);	
}

void CFractMusView::OnCloseupVoice4StartnoteCombo() 
{
	UpdateNoteCombo(VOICE4);	
}

void CFractMusView::OnKillfocusVoice4StartnumEdit() 
{
	UpdateStartNum(VOICE4);	
}


void CFractMusView::OnVoice5MidiButton() 
{
	UpdateMidi(VOICE5);	
}

void CFractMusView::OnVoice5ModulateChk() 
{
	UpdateModulation(VOICE5);
}


void CFractMusView::OnCloseupVoice5NotevalueCombo() 
{
	UpdateNoteValueCombo(VOICE5);	
}

void CFractMusView::OnCloseupVoice5NumoctsCombo() 
{
	UpdateNumOctsCombo(VOICE5);	
}

void CFractMusView::OnVoice5OnChk() 
{
	UpdateActive(VOICE5);	
}

void CFractMusView::OnCloseupVoice5ScaleCombo() 
{
	UpdateScaleCombo(VOICE5);	
}

void CFractMusView::OnCloseupVoice5StartnoteCombo() 
{
	UpdateNoteCombo(VOICE5);	
}

void CFractMusView::OnKillfocusVoice5StartnumEdit() 
{
	UpdateStartNum(VOICE5);	
}


void CFractMusView::OnVoice6MidiButton() 
{
	UpdateMidi(VOICE6);	
}

void CFractMusView::OnVoice6ModulateChk() 
{
	UpdateModulation(VOICE6);
}


void CFractMusView::OnCloseupVoice6NotevalueCombo() 
{
	UpdateNoteValueCombo(VOICE6);	
}

void CFractMusView::OnCloseupVoice6NumoctsCombo() 
{
	UpdateNumOctsCombo(VOICE6);	
}

void CFractMusView::OnVoice6OnChk() 
{
	UpdateActive(VOICE6);	
}

void CFractMusView::OnCloseupVoice6ScaleCombo() 
{
	UpdateScaleCombo(VOICE6);	
}

void CFractMusView::OnCloseupVoice6StartnoteCombo() 
{
	UpdateNoteCombo(VOICE6);	
}

void CFractMusView::OnKillfocusVoice6StartnumEdit() 
{
	UpdateStartNum(VOICE6);	
}


void CFractMusView::OnVoice7MidiButton() 
{
	UpdateMidi(VOICE7);	
}

void CFractMusView::OnVoice7ModulateChk() 
{
	UpdateModulation(VOICE7);
}


void CFractMusView::OnCloseupVoice7NotevalueCombo() 
{
	UpdateNoteValueCombo(VOICE7);	
}

void CFractMusView::OnCloseupVoice7NumoctsCombo() 
{
	UpdateNumOctsCombo(VOICE7);	
}

void CFractMusView::OnVoice7OnChk() 
{
	UpdateActive(VOICE7);	
}

void CFractMusView::OnCloseupVoice7ScaleCombo() 
{
	UpdateScaleCombo(VOICE7);	
}

void CFractMusView::OnCloseupVoice7StartnoteCombo() 
{
	UpdateNoteCombo(VOICE7);	
}

void CFractMusView::OnKillfocusVoice7StartnumEdit() 
{
	UpdateStartNum(VOICE7);	
}


void CFractMusView::OnVoice8MidiButton() 
{
	UpdateMidi(VOICE8);	
}

void CFractMusView::OnVoice8ModulateChk() 
{
	UpdateModulation(VOICE8);
}

void CFractMusView::OnVoice9MidiButton() 
{
	UpdateMidi(VOICE9);	
}

void CFractMusView::OnVoice9ModulateChk() 
{
	UpdateModulation(VOICE9);
}



void CFractMusView::OnCloseupVoice8NotevalueCombo() 
{
	UpdateNoteValueCombo(VOICE8);	
}

void CFractMusView::OnCloseupVoice9NotevalueCombo() 
{
	UpdateNoteValueCombo(VOICE9);	
}


void CFractMusView::OnCloseupVoice8NumoctsCombo() 
{
	UpdateNumOctsCombo(VOICE8);
}

void CFractMusView::OnCloseupVoice9NumoctsCombo() 
{
	UpdateNumOctsCombo(VOICE9);	
}

void CFractMusView::OnVoice8OnChk() 
{
	UpdateActive(VOICE8);	
}

void CFractMusView::OnVoice9OnChk() 
{
	UpdateActive(VOICE9);	
}

void CFractMusView::OnCloseupVoice8ScaleCombo() 
{
	UpdateScaleCombo(VOICE8);	
}

void CFractMusView::OnCloseupVoice9ScaleCombo() 
{
	UpdateScaleCombo(VOICE9);	
}


void CFractMusView::OnCloseupVoice8StartnoteCombo() 
{
	UpdateNoteCombo(VOICE8);	
}

void CFractMusView::OnCloseupVoice9StartnoteCombo() 
{
	UpdateNoteCombo(VOICE9);	
}

void CFractMusView::OnKillfocusVoice8StartnumEdit() 
{
	UpdateStartNum(VOICE8);	
}

void CFractMusView::OnUpdateAppExit(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable((!m_Playing));
}

void CFractMusView::OnUpdateFileNew(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable((!m_Playing));
}

void CFractMusView::OnUpdateFileOpen(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable((!m_Playing));
}

void CFractMusView::OnUpdateFileSave(CCmdUI* pCmdUI) 
{

	pCmdUI->Enable((!m_Playing)&&(!IsDocumentLocked));

}


void CFractMusView::OnUpdateFileSaveAs(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable((!m_Playing)&&(!IsDocumentLocked));
}

void CFractMusView::OnUpdateFileSavemidi(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable((!m_Playing)&&(!IsDocumentLocked));
}



void CFractMusView::OnPlayPlay() 
{

	if (!UpdateData(TRUE))
		return;

	OnKillfocusGencounterEd();
	OnKillfocusStartcounterEd();

	UINT result = midiOutOpen(&midiOut, DevChoice, NULL, NULL, CALLBACK_NULL);

	if( result!=MMSYSERR_NOERROR )
	{
		switch(result)
		{

		case MMSYSERR_ALLOCATED:
			AfxMessageBox("The device is already in use, free it and try again", MB_OK+MB_ICONHAND);
			break;
		case MMSYSERR_NOMEM:
			AfxMessageBox("Could not allocate memory for the device", MB_OK+MB_ICONHAND);
			break;
		case MMSYSERR_INVALPARAM:
			AfxMessageBox("Invalid pointer!", MB_OK+MB_ICONHAND);
			break;
		}

		return;

	}

	CFractMusDoc* pDoc = GetDocument();

	CMenu* pM = AfxGetMainWnd()->GetSystemMenu(FALSE);
	pM->EnableMenuItem(SC_CLOSE, MF_GRAYED);
	CButton* pButt;

	if(m_pEvents)
	{
		m_pEvents->GetDlgItem(IDC_ADD)->EnableWindow(FALSE);
		m_pEvents->GetDlgItem(IDC_REMOVE)->EnableWindow(FALSE);
		m_pEvents->GetDlgItem(IDC_REPLACE_BT)->EnableWindow(FALSE);
		m_pEvents->GetDlgItem(IDC_PLAY)->EnableWindow(FALSE);
		m_pEvents->GetDlgItem(IDC_STOP)->EnableWindow(FALSE);
		m_pEvents->GetDlgItem(IDC_CLEAR)->EnableWindow(FALSE);		
		//m_pEvents->GetDlgItem(IDC_EVENTLIST)->EnableWindow(FALSE);
		m_pEvents->GetDlgItem(IDC_INSERT)->EnableWindow(FALSE);


	}


	GetDlgItem(IDC_TEMPO_SLIDER2)->EnableWindow(FALSE);
	GetDlgItem(IDC_PLAYFROM_CMB)->EnableWindow(FALSE);
	GetDlgItem(IDC_PLAYFROM_BT)->EnableWindow(FALSE);
	GetDlgItem(IDC_TIMESIGNUM_CMB)->EnableWindow(FALSE);
	GetDlgItem(IDC_TIMESIGDEN_CMB)->EnableWindow(FALSE);
	GetDlgItem(IDC_GLOBALTEMPO_BT)->EnableWindow(FALSE);
	GetDlgItem(IDC_GLOBALTIMESIG_BT)->EnableWindow(FALSE);


	for(int i =VOICE1; i<MAXVOICES; i++)
	{
		GetDlgItem(StartNumID[i])->EnableWindow(FALSE);
		GetDlgItem(ModulatesID[i])->EnableWindow(FALSE);
		GetDlgItem(NumOctsID[i])->EnableWindow(FALSE);
		GetDlgItem(NoteID[i])->EnableWindow(FALSE);
		GetDlgItem(ScaleID[i])->EnableWindow(FALSE);
		GetDlgItem(NoteValueID[i])->EnableWindow(FALSE);
		GetDlgItem(MidiButtonID[i])->EnableWindow(FALSE);
	}

	

	switch(PlayChoice)
	{
		case CURRENT_SETTINGS:
			m_Playing = TRUE;
			AfxBeginThread(PlayComposition, this, THREAD_PRIORITY_TIME_CRITICAL);
			break;
		case COMPOSITION:

			for (i=VOICE1; i<MAXVOICES; i++)
			{
				pButt = (CButton*)GetDlgItem(ActiveID[i]);
				pButt->EnableWindow(FALSE);
			}

			m_Playing = TRUE;
			AfxBeginThread(PlayScript, this, THREAD_PRIORITY_TIME_CRITICAL);
			break;
	}


}

void CFractMusView::OnUpdatePlayPlay(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable((!m_Playing)&&(!waiting));
}


void CFractMusView::OnPlayStop() 
{
	if (m_Playing)
		midiOutClose(midiOut);

	m_Playing = FALSE;

	CMenu* pM = AfxGetMainWnd()->GetSystemMenu(FALSE);
	pM->EnableMenuItem(SC_CLOSE, MF_ENABLED);
	CButton* pButt;

	GetDlgItem(IDC_TEMPO_SLIDER2)->EnableWindow(TRUE);
	GetDlgItem(IDC_PLAYFROM_CMB)->EnableWindow(PlayChoice == COMPOSITION);
	GetDlgItem(IDC_PLAYFROM_BT)->EnableWindow(PlayChoice == COMPOSITION);
	GetDlgItem(IDC_TIMESIGNUM_CMB)->EnableWindow(TRUE);
	GetDlgItem(IDC_TIMESIGDEN_CMB)->EnableWindow(TRUE);
	GetDlgItem(IDC_GLOBALTEMPO_BT)->EnableWindow(TRUE);
	GetDlgItem(IDC_GLOBALTIMESIG_BT)->EnableWindow(TRUE);


	for(int i =VOICE1; i<MAXVOICES; i++)
	{
		GetDlgItem(StartNumID[i])->EnableWindow(m_IsActive[i]&&(!IsDocumentLocked));
		GetDlgItem(ModulatesID[i])->EnableWindow(m_IsActive[i]&&(!IsDocumentLocked));
		GetDlgItem(NumOctsID[i])->EnableWindow(m_IsActive[i]&&(!IsDocumentLocked));
		GetDlgItem(NoteID[i])->EnableWindow(m_IsActive[i]&&(!IsDocumentLocked));
		GetDlgItem(ScaleID[i])->EnableWindow(m_IsActive[i]&&(!IsDocumentLocked));
		GetDlgItem(NoteValueID[i])->EnableWindow(m_IsActive[i]&&(!IsDocumentLocked));
		GetDlgItem(MidiButtonID[i])->EnableWindow(m_IsActive[i]&&(!IsDocumentLocked));
	}

	if(m_pEvents)
	{
		m_pEvents->GetDlgItem(IDC_ADD)->EnableWindow((!IsDocumentLocked));
		m_pEvents->GetDlgItem(IDC_REMOVE)->EnableWindow((!IsDocumentLocked));
		m_pEvents->GetDlgItem(IDC_REPLACE_BT)->EnableWindow((!IsDocumentLocked));
		m_pEvents->GetDlgItem(IDC_PLAY)->EnableWindow(TRUE);
		m_pEvents->GetDlgItem(IDC_STOP)->EnableWindow(TRUE);
		m_pEvents->GetDlgItem(IDC_CLEAR)->EnableWindow((!IsDocumentLocked));
		m_pEvents->GetDlgItem(IDC_FUCKIT)->EnableWindow(TRUE);
		m_pEvents->GetDlgItem(IDC_EVENTLIST)->EnableWindow(TRUE);
		m_pEvents->GetDlgItem(IDC_INSERT)->EnableWindow((!IsDocumentLocked));
	}

	if(PlayChoice == COMPOSITION)
		for (i=VOICE1; i<MAXVOICES; i++)
		{
			pButt = (CButton*)GetDlgItem(ActiveID[i]);
			pButt->EnableWindow((!IsDocumentLocked));
		}

	AfxBeginThread(Wait, this);

}


void CFractMusView::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	CScrollBar* pSC = (CScrollBar*)GetDlgItem(IDC_TEMPO_SLIDER2);
	CFractMusDoc* pDoc = GetDocument();
	CStatic* pSt = (CStatic*)GetDlgItem(IDC_TEMPO_STATIC);
	char s[20];

	if (pScrollBar != (CScrollBar*)pSC)
	{
		CFormView::OnHScroll(nSBCode, nPos, pScrollBar);
		return;
	}

	int pos = pSC->GetScrollPos();

	switch(nSBCode)
	{
		case SB_LINERIGHT:
			pos++;
			if(pos>340) pos = 340;
			break;
		case SB_LINELEFT:
			pos--;
			if(pos<20) pos = 20;
			break;
		case SB_THUMBPOSITION:
		case SB_THUMBTRACK:
			pos = nPos;
			break;
		case SB_PAGERIGHT:
			pos +=5;
			if(pos>340) pos = 340;
			break;
		case SB_PAGELEFT:
			pos -= 5;
			if(pos<20) pos = 20;
			break;
	}

	pSC->SetScrollPos(pos);

	m_TempoMarking = pos;
	itoa(m_TempoMarking, s ,10);
	pSt->SetWindowText(s);
	pDoc->m_TempoMarking = Tempo = m_TempoMarking;
	pDoc->SetModifiedFlag();
		
	CFormView::OnHScroll(nSBCode, nPos, pScrollBar);
}


void CFractMusView::EventsDone()
{
	m_pEvents = NULL;
}



void CFractMusView::UpdateActive(int voice)
{
	CFractMusDoc* pDoc = GetDocument();
	if (!UpdateData(TRUE))
		return;
	pDoc->m_IsActive[voice] = m_IsActive[voice];

	GetDlgItem(StartNumID[voice])->EnableWindow(m_IsActive[voice]&&(!m_Playing));
	GetDlgItem(ModulatesID[voice])->EnableWindow(m_IsActive[voice]&&(!m_Playing));
	GetDlgItem(NumOctsID[voice])->EnableWindow(m_IsActive[voice]&&(!m_Playing));
	GetDlgItem(NoteID[voice])->EnableWindow(m_IsActive[voice]&&(!m_Playing));
	GetDlgItem(ScaleID[voice])->EnableWindow(m_IsActive[voice]&&(!m_Playing));
	GetDlgItem(NoteValueID[voice])->EnableWindow(m_IsActive[voice]&&(!m_Playing));
	GetDlgItem(MidiButtonID[voice])->EnableWindow(m_IsActive[voice]&&(!m_Playing));

	pDoc->SetModifiedFlag();
}


void CFractMusView::UpdateStartNum(int voice)
{
	CFractMusDoc* pDoc = GetDocument();

	
	if (!UpdateData(TRUE))
		return;

	pDoc->m_StartNum[voice] = m_StartNum[voice];
	pDoc->SetModifiedFlag();
}

void CFractMusView::UpdateModulation(int voice)
{
	CFractMusDoc* pDoc = GetDocument();
	if (!UpdateData(TRUE))
		return;
	pDoc->m_Modulates[voice] = m_Modulates[voice];
	pDoc->SetModifiedFlag();

	if (m_Modulates[voice])
	{
		m_V1Modul[voice].m_V1ModulDlgNum = pDoc->m_ModulNum[voice];
		m_V1Modul[voice].m_V1ModulDlgChoice = pDoc->m_ModulRandom[voice];

		if (m_V1Modul[voice].DoModal() == IDOK)
		{
			pDoc->m_ModulNum[voice] = m_V1Modul[voice].m_V1ModulDlgNum;
			pDoc->m_ModulRandom[voice] = m_V1Modul[voice].m_V1ModulDlgChoice;
		}
		else
		{
			pDoc->m_Modulates[voice] = m_Modulates[voice] = FALSE;
			UpdateData(FALSE);
		}
	}

}

void CFractMusView::UpdateNumOctsCombo(int voice)
{
	if (!UpdateData(TRUE))
		return;

	CComboBox* pB = (CComboBox*)GetDlgItem(NumOctsID[voice]);
	int sel = pB->GetCurSel();
	if (sel == m_NumOctsID[voice])
		return;

	m_NumOctsID[voice] = sel;

	SetNumOcts(voice, sel);
}

void CFractMusView::UpdateNoteCombo(int voice)
{
	if (!UpdateData(TRUE))
		return;

	CComboBox* pB = (CComboBox*)GetDlgItem(NoteID[voice]);
	CFractMusDoc* pDoc = GetDocument();
	int sel = pB->GetCurSel();
	if (sel == m_Note[voice])
		return;

	m_Note[voice] = sel;
	pDoc->m_Note[voice] = sel;
	pDoc->SetModifiedFlag();
}

void CFractMusView::UpdateScaleCombo(int voice)
{
	if (!UpdateData(TRUE))
		return;

	CComboBox* pB = (CComboBox*)GetDlgItem(ScaleID[voice]);
	int sel = pB->GetCurSel();
	if ((sel == m_ScaleID[voice])&&(sel != USER_DEFINED))
		return;

	m_ScaleID[voice] = sel;
	
	SetScale(voice, sel);

}

void CFractMusView::UpdateNoteValueCombo(int voice)
{
	if (!UpdateData(TRUE))
		return;

	CComboBox* pB = (CComboBox*)GetDlgItem(NoteValueID[voice]);
	int sel = pB->GetCurSel();
	if ((sel == m_NoteValueID[voice])&&(sel!=OTHER))
		return;

	m_NoteValueID[voice] = sel;

	SetNoteValue(voice, sel);

}

void CFractMusView::UpdateMidi(int voice)
{
	CFractMusDoc* pDoc = GetDocument();
	if (!UpdateData(TRUE))
		return;

	m_V1MidiSet[voice].m_voice = voice;

	m_V1MidiSet[voice].m_dynchoice = pDoc->m_DynChoice[voice];
	m_V1MidiSet[voice].m_inv = pDoc->m_Inverted[voice];
	m_V1MidiSet[voice].m_matchscl = pDoc->m_MatchScale[voice];
	m_V1MidiSet[voice].m_channel = pDoc->m_Channel[voice];
	m_V1MidiSet[voice].m_pan = pDoc->m_Pan[voice];
	m_V1MidiSet[voice].m_instrument = pDoc->m_Instrument[voice];
	m_V1MidiSet[voice].m_perc = pDoc->m_Percussion[voice];
	m_V1MidiSet[voice].m_mode = pDoc->m_mode[voice];




	
	if (m_V1MidiSet[voice].DoModal() == IDOK)
	{

		CButton* pButt = (CButton*)GetDlgItem(ActiveID[voice]);

		pDoc->m_Instrument[voice] = m_V1MidiSet[voice].m_instrument;
		pDoc->m_mode[voice] = m_V1MidiSet[voice].m_mode;
		pDoc->m_Channel[voice] = m_V1MidiSet[voice].m_channel;
		pDoc->m_Inverted[voice] = m_V1MidiSet[voice].m_inv;
		pDoc->m_MatchScale[voice] = m_V1MidiSet[voice].m_matchscl;
		pDoc->m_DynChoice[voice] = m_V1MidiSet[voice].m_dynchoice;
		pDoc->m_Pan[voice] = m_V1MidiSet[voice].m_pan;
		pDoc->m_Percussion[voice] = m_V1MidiSet[voice].m_perc;

		if (pDoc->m_Channel[voice] == PercussionChannel)
		{
			pButt->SetWindowText(PercussionNames[ pDoc->m_Percussion[voice] ]);
		}
		else
			pButt->SetWindowText(InstrumentNames[ pDoc->m_Instrument[voice] ]);

		int i;

		switch(m_V1MidiSet[voice].m_dynchoice)
		{
			case PP:
				pDoc->m_Dynamic[voice] = PIANISSIMO;
				break;
			case P:
				pDoc->m_Dynamic[voice] = PIANO;
				break;
			case MP:
				pDoc->m_Dynamic[voice] = MEZZO_PIANO;
				break;
			case MF:
				pDoc->m_Dynamic[voice] = MEZZO_FORTE;
				break;
			case F:
				pDoc->m_Dynamic[voice] = FORTE;
				break;
			case FF:
				pDoc->m_Dynamic[voice] = FORTISSIMO;
				break;
		}

		if (pDoc->m_pScale[voice])
		{
			delete pDoc->m_pScale[voice];
			pDoc->m_pScale[voice] = NULL;
		}


		switch(pDoc->m_ScaleID[voice])
		{
			case MAJOR:
				if ((m_V1MidiSet[voice].m_inv)&&(m_V1MidiSet[voice].m_matchscl))
				{
					pDoc->m_pScale[voice] = new int[Phrygian[0] + 1];
					for(i = 0; i<=Phrygian[0]; i++)
						pDoc->m_pScale[voice][i] = Phrygian[i];
					break;
				}
				else
				{
					pDoc->m_pScale[voice] = new int[Major[0] + 1];
					for(i = 0; i<=Major[0]; i++)
						pDoc->m_pScale[voice][i] = Major[i];
					break;
				}
			case MINOR_NATURAL:
				if ((m_V1MidiSet[voice].m_inv)&&(m_V1MidiSet[voice].m_matchscl))
				{
					pDoc->m_pScale[voice] = new int[Mixolydian[0] + 1];
					for(i = 0; i<=Mixolydian[0]; i++)
						pDoc->m_pScale[voice][i] = Mixolydian[i];
					break;
				}
				else
				{		
					pDoc->m_pScale[voice] = new int[MinorNatural[0] + 1];
					for(i = 0; i<=MinorNatural[0]; i++)
						pDoc->m_pScale[voice][i] = MinorNatural[i];
					break;
				}
			case MINOR_HARMONIC:
				if ((m_V1MidiSet[voice].m_inv)&&(m_V1MidiSet[voice].m_matchscl))
				{
					pDoc->m_pScale[voice] = new int[7 + 1];
					pDoc->m_pScale[voice][0] = 7;
					pDoc->m_pScale[voice][1] = DO;
					pDoc->m_pScale[voice][2] = REb;
					pDoc->m_pScale[voice][3] = MI;
					pDoc->m_pScale[voice][4] = FA;
					pDoc->m_pScale[voice][5] = SOL;
					pDoc->m_pScale[voice][6] = LA;
					pDoc->m_pScale[voice][7] = SIb;

					break;
				}
				else
				{			
					pDoc->m_pScale[voice] = new int[MinorHarmonic[0] + 1];
					for(i = 0; i<=MinorHarmonic[0]; i++)
						pDoc->m_pScale[voice][i] = MinorHarmonic[i];
					break;
				}
			case MINOR_MELODIC:
				if ((m_V1MidiSet[voice].m_inv)&&(m_V1MidiSet[voice].m_matchscl))
				{
					pDoc->m_pScale[voice] = new int[7 + 1];
					pDoc->m_pScale[voice][0] = 7;
					pDoc->m_pScale[voice][1] = DO;
					pDoc->m_pScale[voice][2] = REb;
					pDoc->m_pScale[voice][3] = MIb;
					pDoc->m_pScale[voice][4] = FA;
					pDoc->m_pScale[voice][5] = SOL;
					pDoc->m_pScale[voice][6] = LA;
					pDoc->m_pScale[voice][7] = SIb;

					break;
				}
				else
				{			
					pDoc->m_pScale[voice] = new int[MinorMelodic[0] + 1];
					for(i = 0; i<=MinorMelodic[0]; i++)
						pDoc->m_pScale[voice][i] = MinorMelodic[i];
					break;
				}
			case LYDIAN:
				if ((m_V1MidiSet[voice].m_inv)&&(m_V1MidiSet[voice].m_matchscl))
				{
					pDoc->m_pScale[voice] = new int[Locrian[0] + 1];
					for(i = 0; i<=Locrian[0]; i++)
						pDoc->m_pScale[voice][i] = Locrian[i];
					break;
				}
				else
				{
					pDoc->m_pScale[voice] = new int[Lydian[0] + 1];
					for(i = 0; i<=Lydian[0]; i++)
						pDoc->m_pScale[voice][i] = Lydian[i];
					break;
				}		
			case MIXOLYDIAN:
				if ((m_V1MidiSet[voice].m_inv)&&(m_V1MidiSet[voice].m_matchscl))
				{
					pDoc->m_pScale[voice] = new int[MinorNatural[0] + 1];
					for(i = 0; i<=MinorNatural[0]; i++)
						pDoc->m_pScale[voice][i] = MinorNatural[i];
					break;
				}
				else
				{
					pDoc->m_pScale[voice] = new int[Mixolydian[0] + 1];
					for(i = 0; i<=Mixolydian[0]; i++)
						pDoc->m_pScale[voice][i] = Mixolydian[i];
					break;
				}		
			case DORIAN:
				pDoc->m_pScale[voice] = new int[Dorian[0] + 1];
				for(i = 0; i<=Dorian[0]; i++)
					pDoc->m_pScale[voice][i] = Dorian[i];
				break;
			case PHRYGIAN:
				if ((m_V1MidiSet[voice].m_inv)&&(m_V1MidiSet[voice].m_matchscl))
				{
					pDoc->m_pScale[voice] = new int[Major[0] + 1];
					for(i = 0; i<=Major[0]; i++)
						pDoc->m_pScale[voice][i] = Major[i];
					break;
				}
				else
				{
					pDoc->m_pScale[voice] = new int[Phrygian[0] + 1];
					for(i = 0; i<=Phrygian[0]; i++)
						pDoc->m_pScale[voice][i] = Phrygian[i];
					break;
				}		
			case LOCRIAN:
				if ((m_V1MidiSet[voice].m_inv)&&(m_V1MidiSet[voice].m_matchscl))
				{
					pDoc->m_pScale[voice] = new int[Lydian[0] + 1];
					for(i = 0; i<=Lydian[0]; i++)
						pDoc->m_pScale[voice][i] = Lydian[i];
					break;
				}
				else
				{
					pDoc->m_pScale[voice] = new int[Locrian[0] + 1];
					for(i = 0; i<=Locrian[0]; i++)
						pDoc->m_pScale[voice][i] = Locrian[i];
					break;
				}		
			case OCTATONIC1:
				if ((m_V1MidiSet[voice].m_inv)&&(m_V1MidiSet[voice].m_matchscl))
				{
					pDoc->m_pScale[voice] = new int[Octatonic2[0] + 1];
					for(i = 0; i<=Octatonic2[0]; i++)
						pDoc->m_pScale[voice][i] = Octatonic2[i];
					break;
				}
				else
				{
					pDoc->m_pScale[voice] = new int[Octatonic1[0] + 1];
					for(i = 0; i<=Octatonic1[0]; i++)
						pDoc->m_pScale[voice][i] = Octatonic1[i];
					break;
				}		
			case OCTATONIC2:
				if ((m_V1MidiSet[voice].m_inv)&&(m_V1MidiSet[voice].m_matchscl))
				{
					pDoc->m_pScale[voice] = new int[Octatonic1[0] + 1];
					for(i = 0; i<=Octatonic1[0]; i++)
						pDoc->m_pScale[voice][i] = Octatonic1[i];
					break;
				}
				else
				{
					pDoc->m_pScale[voice] = new int[Octatonic2[0] + 1];
					for(i = 0; i<=Octatonic2[0]; i++)
						pDoc->m_pScale[voice][i] = Octatonic2[i];
					break;
				}		
			case ACCOUSTIC:
				if ((m_V1MidiSet[voice].m_inv)&&(m_V1MidiSet[voice].m_matchscl))
				{
					pDoc->m_pScale[voice] = new int[7 + 1];
					pDoc->m_pScale[voice][0] = 7;
					pDoc->m_pScale[voice][1] = DO;
					pDoc->m_pScale[voice][2] = RE;
					pDoc->m_pScale[voice][3] = MIb;
					pDoc->m_pScale[voice][4] = FA;
					pDoc->m_pScale[voice][5] = SOLb;
					pDoc->m_pScale[voice][6] = LAb;
					pDoc->m_pScale[voice][7] = SIb;

					break;
				}
				else
				{			
					pDoc->m_pScale[voice] = new int[Accoustic[0] + 1];
					for(i = 0; i<=Accoustic[0]; i++)
						pDoc->m_pScale[voice][i] = Accoustic[i];
					break;
				}
			case WHOLETONE:
				pDoc->m_pScale[voice] = new int[Wholetone[0] + 1];
				for(i = 0; i<=Wholetone[0]; i++)
					pDoc->m_pScale[voice][i] = Wholetone[i];
				break;
			case PENTATONIC:
				if ((m_V1MidiSet[voice].m_inv)&&(m_V1MidiSet[voice].m_matchscl))
				{
					pDoc->m_pScale[voice] = new int[5 + 1];
					pDoc->m_pScale[voice][0] = 5;
					pDoc->m_pScale[voice][1] = DO;
					pDoc->m_pScale[voice][2] = MIb;
					pDoc->m_pScale[voice][3] = FA;
					pDoc->m_pScale[voice][4] = LAb;
					pDoc->m_pScale[voice][5] = SIb;
					break;
				}
				else
				{			
					pDoc->m_pScale[voice] = new int[Pentatonic[0] + 1];
					for(i = 0; i<=Pentatonic[0]; i++)
						pDoc->m_pScale[voice][i] = Pentatonic[i];
					break;
				}
			case CHROMATIC:
				pDoc->m_pScale[voice] = new int[Chromatic[0] + 1];
				for(i = 0; i<=Chromatic[0]; i++)
					pDoc->m_pScale[voice][i] = Chromatic[i];
				break;
			case USER_DEFINED:
				if ((m_V1MidiSet[voice].m_inv)&&(m_V1MidiSet[voice].m_matchscl))
				{
					pDoc->m_pScale[voice] = new int[pDoc->m_pUserScale[voice][0] + 1];
					pDoc->m_pScale[voice][0] = pDoc->m_pUserScale[voice][0];
					for(i = 1; i<=pDoc->m_pUserScale[voice][0]; i++)
						pDoc->m_pScale[voice][i] = (12-pDoc->m_pUserScale[voice][i])*(pDoc->m_pUserScale[voice][i]!=0);
					SortScale(pDoc->m_pScale[voice], pDoc->m_pScale[voice][0]);
					break;
				}
				else
				{
					pDoc->m_pScale[voice] = new int[pDoc->m_pUserScale[voice][0] + 1];
					pDoc->m_pScale[voice][0] = pDoc->m_pUserScale[voice][0];
					for(i = 1; i<=pDoc->m_pUserScale[voice][0]; i++)
						pDoc->m_pScale[voice][i] = pDoc->m_pUserScale[voice][i];
					break;
				}
		}

		pDoc->SetModifiedFlag();
	}

}

void CFractMusView::SetNumOcts(int voice, int choice)
{
	CFractMusDoc* pDoc = GetDocument();

	pDoc->m_NumOctsID[voice] = choice;

	switch(choice)
	{
		case ONE_OCT:
			pDoc->m_NumOcts[voice] = 1;
			break;
		case TWO_OCT:
			pDoc->m_NumOcts[voice] = 2;
			break;
		case THREE_OCT:
			pDoc->m_NumOcts[voice] = 3;
			break;
		case FOUR_OCT:
			pDoc->m_NumOcts[voice] = 4;
			break;
	}

	pDoc->SetModifiedFlag();
}

void CFractMusView::SetNoteValue(int voice, int choice)
{
	CFractMusDoc* pDoc = GetDocument();

	pDoc->m_NoteValueID[voice] = choice;

	switch(choice)
	{
		case WHOLE:
			pDoc->m_NoteValue[voice] = 32.0f;
			break;
		case HALF:
			pDoc->m_NoteValue[voice] = 16.0f;
			break;
		case QUARTER:
			pDoc->m_NoteValue[voice] = 8.0f;
			break;
		case EIGHT:
			pDoc->m_NoteValue[voice] = 4.0f;
			break;
		case SIXTEENTH:
			pDoc->m_NoteValue[voice] = 2.0f;
			break;
		case THIRTYSECOND:
			pDoc->m_NoteValue[voice] = 1.0f;
			break;
		case WHOLE_DOTTED:
			pDoc->m_NoteValue[voice] = 48.00f;
			break;
		case HALF_DOTTED:
			pDoc->m_NoteValue[voice] = 24.00f;
			break;
		case QUARTER_DOTTED:
			pDoc->m_NoteValue[voice] = 12.00f;
			break;
		case EIGHTH_DOTTED:
			pDoc->m_NoteValue[voice] = 6.0f;
			break;
		case SIXTEENTH_DOTTED:
			pDoc->m_NoteValue[voice] = 3.0f;
			break;
		case OTHER:
			CComboBox* pB = (CComboBox*)GetDlgItem(NoteValueID[voice]);
			CString nval;
			m_NoteVal[voice].m_NewVal = (long)pDoc->m_NoteValue[voice];
			m_NoteVal[voice].DoModal();
			nval.Format("Other (%d)", m_NoteVal[voice].m_NewVal);
			pB->DeleteString(OTHER);
			pB->AddString(nval);
			pDoc->m_NoteValue[voice] = (float)m_NoteVal[voice].m_NewVal;
			pB->SetCurSel(OTHER);
			break;
	}
	pDoc->SetModifiedFlag();
}

void CFractMusView::SetScale(int voice, int choice)
{
	CFractMusDoc* pDoc = GetDocument();
	int i;

	pDoc->m_ScaleID[voice] = choice;
	
	if (pDoc->m_pScale[voice])
	{
		delete pDoc->m_pScale[voice];
		pDoc->m_pScale[voice] = NULL;
	}


	switch(choice)
	{
		case MAJOR:
			if ((m_V1MidiSet[voice].m_inv)&&(m_V1MidiSet[voice].m_matchscl))
			{
				pDoc->m_pScale[voice] = new int[Phrygian[0] + 1];
				for(i = 0; i<=Phrygian[0]; i++)
					pDoc->m_pScale[voice][i] = Phrygian[i];
				break;
			}
			else
			{
				pDoc->m_pScale[voice] = new int[Major[0] + 1];
				for(i = 0; i<=Major[0]; i++)
					pDoc->m_pScale[voice][i] = Major[i];
				break;
			}
		case MINOR_NATURAL:
			if ((m_V1MidiSet[voice].m_inv)&&(m_V1MidiSet[voice].m_matchscl))
			{
				pDoc->m_pScale[voice] = new int[Mixolydian[0] + 1];
				for(i = 0; i<=Mixolydian[0]; i++)
					pDoc->m_pScale[voice][i] = Mixolydian[i];
				break;
			}
			else
			{		
				pDoc->m_pScale[voice] = new int[MinorNatural[0] + 1];
				for(i = 0; i<=MinorNatural[0]; i++)
					pDoc->m_pScale[voice][i] = MinorNatural[i];
				break;
			}
		case MINOR_HARMONIC:
			if ((m_V1MidiSet[voice].m_inv)&&(m_V1MidiSet[voice].m_matchscl))
			{
				pDoc->m_pScale[voice] = new int[7 + 1];
				pDoc->m_pScale[voice][0] = 7;
				pDoc->m_pScale[voice][1] = DO;
				pDoc->m_pScale[voice][2] = REb;
				pDoc->m_pScale[voice][3] = MI;
				pDoc->m_pScale[voice][4] = FA;
				pDoc->m_pScale[voice][5] = SOL;
				pDoc->m_pScale[voice][6] = LA;
				pDoc->m_pScale[voice][7] = SIb;

				break;
			}
			else
			{			
				pDoc->m_pScale[voice] = new int[MinorHarmonic[0] + 1];
				for(i = 0; i<=MinorHarmonic[0]; i++)
					pDoc->m_pScale[voice][i] = MinorHarmonic[i];
				break;
			}
		case MINOR_MELODIC:
			if ((m_V1MidiSet[voice].m_inv)&&(m_V1MidiSet[voice].m_matchscl))
			{
				pDoc->m_pScale[voice] = new int[7 + 1];
				pDoc->m_pScale[voice][0] = 7;
				pDoc->m_pScale[voice][1] = DO;
				pDoc->m_pScale[voice][2] = REb;
				pDoc->m_pScale[voice][3] = MIb;
				pDoc->m_pScale[voice][4] = FA;
				pDoc->m_pScale[voice][5] = SOL;
				pDoc->m_pScale[voice][6] = LA;
				pDoc->m_pScale[voice][7] = SIb;

				break;
			}
			else
			{			
				pDoc->m_pScale[voice] = new int[MinorMelodic[0] + 1];
				for(i = 0; i<=MinorMelodic[0]; i++)
					pDoc->m_pScale[voice][i] = MinorMelodic[i];
				break;
			}
		case LYDIAN:
			if ((m_V1MidiSet[voice].m_inv)&&(m_V1MidiSet[voice].m_matchscl))
			{
				pDoc->m_pScale[voice] = new int[Locrian[0] + 1];
				for(i = 0; i<=Locrian[0]; i++)
					pDoc->m_pScale[voice][i] = Locrian[i];
				break;
			}
			else
			{
				pDoc->m_pScale[voice] = new int[Lydian[0] + 1];
				for(i = 0; i<=Lydian[0]; i++)
					pDoc->m_pScale[voice][i] = Lydian[i];
				break;
			}		
		case MIXOLYDIAN:
			if ((m_V1MidiSet[voice].m_inv)&&(m_V1MidiSet[voice].m_matchscl))
			{
				pDoc->m_pScale[voice] = new int[MinorNatural[0] + 1];
				for(i = 0; i<=MinorNatural[0]; i++)
					pDoc->m_pScale[voice][i] = MinorNatural[i];
				break;
			}
			else
			{
				pDoc->m_pScale[voice] = new int[Mixolydian[0] + 1];
				for(i = 0; i<=Mixolydian[0]; i++)
					pDoc->m_pScale[voice][i] = Mixolydian[i];
				break;
			}		
		case DORIAN:
			pDoc->m_pScale[voice] = new int[Dorian[0] + 1];
			for(i = 0; i<=Dorian[0]; i++)
				pDoc->m_pScale[voice][i] = Dorian[i];
			break;
		case PHRYGIAN:
			if ((m_V1MidiSet[voice].m_inv)&&(m_V1MidiSet[voice].m_matchscl))
			{
				pDoc->m_pScale[voice] = new int[Major[0] + 1];
				for(i = 0; i<=Major[0]; i++)
					pDoc->m_pScale[voice][i] = Major[i];
				break;
			}
			else
			{
				pDoc->m_pScale[voice] = new int[Phrygian[0] + 1];
				for(i = 0; i<=Phrygian[0]; i++)
					pDoc->m_pScale[voice][i] = Phrygian[i];
				break;
			}		
		case LOCRIAN:
			if ((m_V1MidiSet[voice].m_inv)&&(m_V1MidiSet[voice].m_matchscl))
			{
				pDoc->m_pScale[voice] = new int[Lydian[0] + 1];
				for(i = 0; i<=Lydian[0]; i++)
					pDoc->m_pScale[voice][i] = Lydian[i];
				break;
			}
			else
			{
				pDoc->m_pScale[voice] = new int[Locrian[0] + 1];
				for(i = 0; i<=Locrian[0]; i++)
					pDoc->m_pScale[voice][i] = Locrian[i];
				break;
			}		
		case OCTATONIC1:
			if ((m_V1MidiSet[voice].m_inv)&&(m_V1MidiSet[voice].m_matchscl))
			{
				pDoc->m_pScale[voice] = new int[Octatonic2[0] + 1];
				for(i = 0; i<=Octatonic2[0]; i++)
					pDoc->m_pScale[voice][i] = Octatonic2[i];
				break;
			}
			else
			{
				pDoc->m_pScale[voice] = new int[Octatonic1[0] + 1];
				for(i = 0; i<=Octatonic1[0]; i++)
					pDoc->m_pScale[voice][i] = Octatonic1[i];
				break;
			}		
		case OCTATONIC2:
			if ((m_V1MidiSet[voice].m_inv)&&(m_V1MidiSet[voice].m_matchscl))
			{
				pDoc->m_pScale[voice] = new int[Octatonic1[0] + 1];
				for(i = 0; i<=Octatonic1[0]; i++)
					pDoc->m_pScale[voice][i] = Octatonic1[i];
				break;
			}
			else
			{
				pDoc->m_pScale[voice] = new int[Octatonic2[0] + 1];
				for(i = 0; i<=Octatonic2[0]; i++)
					pDoc->m_pScale[voice][i] = Octatonic2[i];
				break;
			}		
		case ACCOUSTIC:
			if ((m_V1MidiSet[voice].m_inv)&&(m_V1MidiSet[voice].m_matchscl))
			{
				pDoc->m_pScale[voice] = new int[7 + 1];
				pDoc->m_pScale[voice][0] = 7;
				pDoc->m_pScale[voice][1] = DO;
				pDoc->m_pScale[voice][2] = RE;
				pDoc->m_pScale[voice][3] = MIb;
				pDoc->m_pScale[voice][4] = FA;
				pDoc->m_pScale[voice][5] = SOLb;
				pDoc->m_pScale[voice][6] = LAb;
				pDoc->m_pScale[voice][7] = SIb;

				break;
			}
			else
			{			
				pDoc->m_pScale[voice] = new int[Accoustic[0] + 1];
				for(i = 0; i<=Accoustic[0]; i++)
					pDoc->m_pScale[voice][i] = Accoustic[i];
				break;
			}
		case WHOLETONE:
			pDoc->m_pScale[voice] = new int[Wholetone[0] + 1];
			for(i = 0; i<=Wholetone[0]; i++)
				pDoc->m_pScale[voice][i] = Wholetone[i];
			break;
		case PENTATONIC:
			if ((m_V1MidiSet[voice].m_inv)&&(m_V1MidiSet[voice].m_matchscl))
			{
				pDoc->m_pScale[voice] = new int[5 + 1];
				pDoc->m_pScale[voice][0] = 5;
				pDoc->m_pScale[voice][1] = DO;
				pDoc->m_pScale[voice][2] = MIb;
				pDoc->m_pScale[voice][3] = FA;
				pDoc->m_pScale[voice][4] = LAb;
				pDoc->m_pScale[voice][5] = SIb;
				break;
			}
			else
			{			
				pDoc->m_pScale[voice] = new int[Pentatonic[0] + 1];
				for(i = 0; i<=Pentatonic[0]; i++)
					pDoc->m_pScale[voice][i] = Pentatonic[i];
				break;
			}
		case CHROMATIC:
			pDoc->m_pScale[voice] = new int[Chromatic[0] + 1];
			for(i = 0; i<=Chromatic[0]; i++)
				pDoc->m_pScale[voice][i] = Chromatic[i];
			break;
		case USER_DEFINED:
			int j = 1;

			InitUserScale(voice);
			SetUserScale(voice);
			
			m_V1UsrScl[voice].DoModal();

			if (pDoc->m_pUserScale[voice])
			{
				delete pDoc->m_pUserScale[voice];
				pDoc->m_pUserScale[voice] = NULL;
			}
			pDoc->m_pUserScale[voice] = new int[m_V1UsrScl[voice].m_numnotes + 1];
			pDoc->m_pUserScale[voice][0] = m_V1UsrScl[voice].m_numnotes;
			for(i = 0; i<12; i++)
				if (m_V1UsrScl[voice].m_bUserScale[i])
					pDoc->m_pUserScale[voice][j++] = i;
			SortScale(pDoc->m_pUserScale[voice], pDoc->m_pUserScale[voice][0]);
			TransposeToC(pDoc->m_pUserScale[voice], pDoc->m_pUserScale[voice][0]);

			if ((m_V1MidiSet[voice].m_inv)&&(m_V1MidiSet[voice].m_matchscl))
			{
				pDoc->m_pScale[voice] = new int[pDoc->m_pUserScale[voice][0] + 1];
				pDoc->m_pScale[voice][0] = pDoc->m_pUserScale[voice][0];
				for(i = 1; i<=pDoc->m_pUserScale[voice][0]; i++)
					pDoc->m_pScale[voice][i] = (12-pDoc->m_pUserScale[voice][i])*(pDoc->m_pUserScale[voice][i]!=0);
				SortScale(pDoc->m_pScale[voice], pDoc->m_pScale[voice][0]);
				break;
			}
			else
			{
				pDoc->m_pScale[voice] = new int[pDoc->m_pUserScale[voice][0] + 1];
				pDoc->m_pScale[voice][0] = pDoc->m_pUserScale[voice][0];
				for(i = 1; i<=pDoc->m_pUserScale[voice][0]; i++)
					pDoc->m_pScale[voice][i] = pDoc->m_pUserScale[voice][i];
				break;
			}
	}

	pDoc->SetModifiedFlag();
}

void CFractMusView::SortScale(int* inscale, int limit)
{
	int top, search, temp;
	++limit;
	for (top=1; top<limit-1; top++)
		for (search=top+1; search<limit; search++)
			if (inscale[search]<inscale[top]) {
				temp=inscale[search];
				inscale[search]=inscale[top];
				inscale[top]=temp;
			}
}

void CFractMusView::TransposeToC(int* inscale, int num)
{
	int value=inscale[1];
	int i;
	for (i=1; i<=num; i++)
		inscale[i] -=value;
}


void CFractMusView::SetUserScale(int voice)
{
	CFractMusDoc* pDoc = GetDocument();

	m_V1UsrScl[voice].m_numnotes = pDoc->m_pUserScale[voice][0];

	for(int i=1; i<=pDoc->m_pUserScale[voice][0]; i++)
		m_V1UsrScl[voice].m_bUserScale[pDoc->m_pUserScale[voice][i]] = TRUE;

	m_V1UsrScl[voice].m_C_ON = m_V1UsrScl[voice].m_bUserScale[0];
	m_V1UsrScl[voice].m_Db_ON = m_V1UsrScl[voice].m_bUserScale[1];
	m_V1UsrScl[voice].m_D_ON = m_V1UsrScl[voice].m_bUserScale[2];
	m_V1UsrScl[voice].m_Eb_ON = m_V1UsrScl[voice].m_bUserScale[3];
	m_V1UsrScl[voice].m_E_ON = m_V1UsrScl[voice].m_bUserScale[4];
	m_V1UsrScl[voice].m_F_ON = m_V1UsrScl[voice].m_bUserScale[5];
	m_V1UsrScl[voice].m_Gb_ON = m_V1UsrScl[voice].m_bUserScale[6];
	m_V1UsrScl[voice].m_G_ON = m_V1UsrScl[voice].m_bUserScale[7];
	m_V1UsrScl[voice].m_Ab_ON = m_V1UsrScl[voice].m_bUserScale[8];
	m_V1UsrScl[voice].m_A_ON = m_V1UsrScl[voice].m_bUserScale[9];
	m_V1UsrScl[voice].m_Bb_ON = m_V1UsrScl[voice].m_bUserScale[10];
	m_V1UsrScl[voice].m_B_ON = m_V1UsrScl[voice].m_bUserScale[11];


}



void CFractMusView::InitUserScale(int voice)
{
	m_V1UsrScl[voice].m_numnotes = 0;
	for(int i=0; i<12; i++)
		m_V1UsrScl[voice].m_bUserScale[i] = FALSE;

	m_V1UsrScl[voice].m_C_ON = FALSE;
	m_V1UsrScl[voice].m_Db_ON = FALSE;
	m_V1UsrScl[voice].m_D_ON = FALSE;
	m_V1UsrScl[voice].m_Eb_ON = FALSE;
	m_V1UsrScl[voice].m_E_ON = FALSE;
	m_V1UsrScl[voice].m_F_ON = FALSE;
	m_V1UsrScl[voice].m_Gb_ON = FALSE;
	m_V1UsrScl[voice].m_G_ON = FALSE;
	m_V1UsrScl[voice].m_Ab_ON = FALSE;
	m_V1UsrScl[voice].m_A_ON = FALSE;
	m_V1UsrScl[voice].m_Bb_ON = FALSE;
	m_V1UsrScl[voice].m_B_ON = FALSE;
}

int __fastcall Morse_Thue(int voice, long counter, long test, const int* scale, 
				int numocts,long base, long startnum, 
				int startnote,BOOL modulate, BOOL modulrand, 
				int modulnum, BOOL inverted, float logmapvar, float oneoverf,
				float henonA, float henonB, float hopalong,
				float martin, float lorentz, unsigned long earthwormvar,
				unsigned long earthwormmulti, unsigned earthwormtruncator,
				double ranmin, double ranmax, BYTE wolframrule,
				int wolframsize, long hailstonestart)
{
	long Total;
	int oct;
	static int modulresult[MAXVOICES];
//	static int count1[MAXVOICES];

	ComputeTotal(test, base, &Total);


	if (modulate&&(!modulrand)&&(counter>=modulnum))
	{
		count1[voice] = counter%modulnum;

		if(!count1[voice])
		{
			modulresult[voice] = Total%12;
			count1[voice] = modulnum;
		}
	
	}
	else if (modulate&&modulrand)
	{
		if(count1[voice]<1)
		{
			modulresult[voice] = Total%12;
			count1[voice] = (rand()%50)+1;
		}
		count1[voice]--;
	}
	else
		count1[voice] = modulresult[voice] = 0;

	Total %=scale[0]*numocts;
	if (Total<scale[0]) oct=0;
	if ((Total>=scale[0])&&(Total<(scale[0]*2))) 
	{
		Total -=scale[0];
		oct=1;
	}
	if((Total>=(scale[0]*2))&&(Total<(scale[0]*3)))
	{
		Total -= scale[0]*2;
		oct=2;
	}
	if((Total>=(scale[0]*3))&&(Total<(scale[0]*4)))
	{
		Total -= scale[0]*3;
		oct=3;
	}

	int note = scale[Total+1]+startnote+(12*oct)+(modulresult[voice]*modulate);

	if(inverted)
	{
		int top = startnote + (12*numocts);
		note = (top-note)+startnote;
	}
	return note;


}

int __fastcall LogisticMap(int voice, long counter, long test, const int* scale, 
				int numocts,long base, long startnum, 
				int startnote,BOOL modulate, BOOL modulrand, 
				int modulnum, BOOL inverted, float logmapvar, float oneoverf,
				float henonA, float henonB, float hopalong,
				float martin, float lorentz, unsigned long earthwormvar,
				unsigned long earthwormmulti, unsigned earthwormtruncator,
				double ranmin, double ranmax, BYTE wolframrule,
				int wolframsize, long hailstonestart)

{
	int note, oct;
	long Total;
	static int modulresult[MAXVOICES];
//	static int count2[MAXVOICES];


	if (modulate&&(!modulrand)&&(counter>=modulnum))
	{
		count2[voice] = counter%modulnum;

		if(!count2[voice])
		{
			modulresult[voice] = counter%12;
			count2[voice] = modulnum;
		}
	
	}
	else if (modulate&&modulrand)
	{
		if(count2[voice]<1)
		{
			modulresult[voice] = counter%12;
			count2[voice] = (rand()%50)+1;
		}
		count2[voice]--;
	}
	else
		count2[voice] = modulresult[voice] = 0;

	LogMap[voice] = (LogMap[voice])*logmapvar*(1.0f-LogMap[voice]);

	Total = long((LogMap[voice])*float(scale[0]*numocts));

	if (Total<scale[0]) oct=0;
	if ((Total>=scale[0])&&(Total<(scale[0]*2))) 
	{
		Total -=scale[0];
		oct=1;
	}
	if((Total>=(scale[0]*2))&&(Total<(scale[0]*3)))
	{
		Total -= scale[0]*2;
		oct=2;
	}
	if((Total>=(scale[0]*3))&&(Total<(scale[0]*4)))
	{
		Total -= scale[0]*3;
		oct=3;
	}

	note = scale[Total+1]+startnote+(12*oct)+(modulresult[voice]*modulate);

	if(inverted)
	{
		int top = startnote + (12*numocts);
		note = (top-note)+startnote;
	}


	return note;


}

int __fastcall OneOverF(int voice, long counter, long test, const int* scale, 
				int numocts,long base, long startnum, 
				int startnote,BOOL modulate, BOOL modulrand, 
				int modulnum, BOOL inverted, float logmapvar, float oneoverf,
				float henonA, float henonB, float hopalong,
				float martin, float lorentz, unsigned long earthwormvar,
				unsigned long earthwormmulti, unsigned earthwormtruncator,
				double ranmin, double ranmax, BYTE wolframrule,
				int wolframsize, long hailstonestart)

{
	int note, oct;
	long Total;
	static int modulresult[MAXVOICES];
//	static int count3[MAXVOICES];
	

	if (modulate&&(!modulrand)&&(counter>=modulnum))
	{
		count3[voice] = counter%modulnum;

		if(!count3[voice])
		{
			modulresult[voice] = counter%12;
			count3[voice] = modulnum;
		}
	
	}
	else if (modulate&&modulrand)
	{
		if(count3[voice]<1)
		{
			modulresult[voice] = counter%12;
			count3[voice] = (rand()%50)+1;
		}
		count3[voice]--;
	}
	else
		count3[voice] = modulresult[voice] = 0;

	LogMap2[voice] = (LogMap2[voice])*3.999f*(1.0f-LogMap2[voice]);

	FNoise[voice] = (FNoise[voice]*oneoverf)+(float(sqrt(1-(oneoverf*oneoverf)))*LogMap2[voice]);


	Total = long(float(FNoise[voice])*float((scale[0]*numocts)));

	Total %= (scale[0]*numocts); // ?

	if (Total<scale[0]) oct=0;
	if ((Total>=scale[0])&&(Total<(scale[0]*2))) 
	{
		Total -=scale[0];
		oct=1;
	}
	if((Total>=(scale[0]*2))&&(Total<(scale[0]*3)))
	{
		Total -= scale[0]*2;
		oct=2;
	}
	if((Total>=(scale[0]*3))&&(Total<(scale[0]*4)))
	{
		Total -= scale[0]*3;
		oct=3;
	}

	note = scale[Total+1]+startnote+(12*oct)+(modulresult[voice]*modulate);

	if(inverted)
	{
		int top = startnote + (12*numocts);
		note = (top-note)+startnote;
	}


	return note;

}


int __fastcall Henon(int voice, long counter, long test, const int* scale, 
				int numocts,long base, long startnum, 
				int startnote,BOOL modulate, BOOL modulrand, 
				int modulnum, BOOL inverted, float logmapvar, float oneoverf,
				float henonA, float henonB, float hopalong,
				float martin, float lorentz, unsigned long earthwormvar,
				unsigned long earthwormmulti, unsigned earthwormtruncator,
				double ranmin, double ranmax, BYTE wolframrule,
				int wolframsize, long hailstonestart)
{
	int note, oct;
	long Total;
	static int modulresult[MAXVOICES];
//	static int count4[MAXVOICES];
	float res;


	if (modulate&&(!modulrand)&&(counter>=modulnum))
	{
		count4[voice] = counter%modulnum;

		if(!count4[voice])
		{
			modulresult[voice] = counter%12;
			count4[voice] = modulnum;
		}
	
	}
	else if (modulate&&modulrand)
	{
		if(count4[voice]<1)
		{
			modulresult[voice] = counter%12;
			count4[voice] = (rand()%50)+1;
		}
		count4[voice]--;
	}
	else
		count4[voice] = modulresult[voice] = 0;

	HenonX[voice] = 1 + HenonY[voice] - (henonA*HenonX[voice]*HenonX[voice]);
	HenonY[voice] = HenonX[voice]*henonB;

	res = sqrt((HenonX[voice]*HenonX[voice])+(HenonY[voice]*HenonY[voice]));

	Total = long(float(res)*float((scale[0]*numocts)));

	Total %= (scale[0]*numocts); // ?

	if (Total<scale[0]) oct=0;
	if ((Total>=scale[0])&&(Total<(scale[0]*2))) 
	{
		Total -=scale[0];
		oct=1;
	}
	if((Total>=(scale[0]*2))&&(Total<(scale[0]*3)))
	{
		Total -= scale[0]*2;
		oct=2;
	}
	if((Total>=(scale[0]*3))&&(Total<(scale[0]*4)))
	{
		Total -= scale[0]*3;
		oct=3;
	}

	note = scale[Total+1]+startnote+(12*oct)+(modulresult[voice]*modulate);

	if(inverted)
	{
		int top = startnote + (12*numocts);
		note = (top-note)+startnote;
	}


	return note;

}


//////////////////////////////////


int __fastcall Hopalong(int voice, long counter, long test, const int* scale, 
				int numocts,long base, long startnum, 
				int startnote,BOOL modulate, BOOL modulrand, 
				int modulnum, BOOL inverted, float logmapvar, float oneoverf,
				float henonA, float henonB, float hopalong,
				float martin, float lorentz, unsigned long earthwormvar,
				unsigned long earthwormmulti, unsigned earthwormtruncator,
				double ranmin, double ranmax, BYTE wolframrule,
				int wolframsize, long hailstonestart)

{
	int note, oct;
	long Total;
	static int modulresult[MAXVOICES];
//	static int count5[MAXVOICES];
	float res, sign, tempX, tempY;


	if (modulate&&(!modulrand)&&(counter>=modulnum))
	{
		count5[voice] = counter%modulnum;

		if(!count5[voice])
		{
			modulresult[voice] = counter%12;
			count5[voice] = modulnum;
		}
	
	}
	else if (modulate&&modulrand)
	{
		if(count5[voice]<1)
		{
			modulresult[voice] = counter%12;
			count5[voice] = (rand()%50)+1;
		}
		count5[voice]--;
	}
	else
		count5[voice] = modulresult[voice] = 0;

	sign = HopalongX[voice] >= 0.0f ? 1.0f : -1.0f;

	tempX = HopalongY[voice] - sign * (sqrt(fabs(sin(hopalong) * HopalongX[voice] - cos(hopalong))));
	tempY = hopalong - HopalongX[voice];

	HopalongX[voice] = tempX;
	HopalongY[voice] = tempY;


	res = sqrt((HopalongX[voice]*HopalongX[voice])+(HopalongY[voice]*HopalongY[voice]));

	Total = long(float(res)*float((scale[0]*numocts)));

	Total %= (scale[0]*numocts); // ?

	if (Total<scale[0]) oct=0;
	if ((Total>=scale[0])&&(Total<(scale[0]*2))) 
	{
		Total -=scale[0];
		oct=1;
	}
	if((Total>=(scale[0]*2))&&(Total<(scale[0]*3)))
	{
		Total -= scale[0]*2;
		oct=2;
	}
	if((Total>=(scale[0]*3))&&(Total<(scale[0]*4)))
	{
		Total -= scale[0]*3;
		oct=3;
	}

	note = scale[Total+1]+startnote+(12*oct)+(modulresult[voice]*modulate);

	if(inverted)
	{
		int top = startnote + (12*numocts);
		note = (top-note)+startnote;
	}


	return note;

}






///////////////////////////////


int __fastcall Martin(int voice, long counter, long test, const int* scale, 
				int numocts,long base, long startnum, 
				int startnote,BOOL modulate, BOOL modulrand, 
				int modulnum, BOOL inverted, float logmapvar, float oneoverf,
				float henonA, float henonB, float hopalong,
				float martin, float lorentz, unsigned long earthwormvar,
				unsigned long earthwormmulti, unsigned earthwormtruncator,
				double ranmin, double ranmax, BYTE wolframrule,
				int wolframsize, long hailstonestart)

{
	int note, oct;
	long Total;
	static int modulresult[MAXVOICES];
//	static int count6[MAXVOICES];
	float res, tempX, tempY;


	if (modulate&&(!modulrand)&&(counter>=modulnum))
	{
		count6[voice] = counter%modulnum;

		if(!count6[voice])
		{
			modulresult[voice] = counter%12;
			count6[voice] = modulnum;
		}
	
	}
	else if (modulate&&modulrand)
	{
		if(count6[voice]<1)
		{
			modulresult[voice] = counter%12;
			count6[voice] = (rand()%50)+1;
		}
		count6[voice]--;
	}
	else
		count6[voice] = modulresult[voice] = 0;


	tempX = MartinY[voice] - sin(MartinX[voice]);
	tempY = martin - MartinX[voice];

	MartinX[voice] = tempX;
	MartinY[voice] = tempY;


	res = sqrt((MartinX[voice]*MartinX[voice])+(MartinY[voice]*MartinY[voice]));

	Total = long(float(res)*float((scale[0]*numocts)));

	Total %= (scale[0]*numocts); // ?

	if (Total<scale[0]) oct=0;
	if ((Total>=scale[0])&&(Total<(scale[0]*2))) 
	{
		Total -=scale[0];
		oct=1;
	}
	if((Total>=(scale[0]*2))&&(Total<(scale[0]*3)))
	{
		Total -= scale[0]*2;
		oct=2;
	}
	if((Total>=(scale[0]*3))&&(Total<(scale[0]*4)))
	{
		Total -= scale[0]*3;
		oct=3;
	}

	note = scale[Total+1]+startnote+(12*oct)+(modulresult[voice]*modulate);

	if(inverted)
	{
		int top = startnote + (12*numocts);
		note = (top-note)+startnote;
	}


	return note;

}






//////////////////////////



int __fastcall Gingerbreadman(int voice, long counter, long test, const int* scale, 
				int numocts,long base, long startnum, 
				int startnote,BOOL modulate, BOOL modulrand, 
				int modulnum, BOOL inverted, float logmapvar, float oneoverf,
				float henonA, float henonB, float hopalong,
				float martin, float lorentz, unsigned long earthwormvar,
				unsigned long earthwormmulti, unsigned earthwormtruncator,
				double ranmin, double ranmax, BYTE wolframrule,
				int wolframsize, long hailstonestart)

{
	int note, oct;
	long Total;
	static int modulresult[MAXVOICES];
//	static int count7[MAXVOICES];
	float res, tempX, tempY;


	if (modulate&&(!modulrand)&&(counter>=modulnum))
	{
		count7[voice] = counter%modulnum;

		if(!count7[voice])
		{
			modulresult[voice] = counter%12;
			count7[voice] = modulnum;
		}
	
	}
	else if (modulate&&modulrand)
	{
		if(count7[voice]<1)
		{
			modulresult[voice] = counter%12;
			count7[voice] = (rand()%50)+1;
		}
		count7[voice]--;
	}
	else
		count7[voice] = modulresult[voice] = 0;

	tempX = 1.0f - GingerY[voice] + fabs(GingerX[voice]);
	tempY = GingerX[voice];

	GingerX[voice] = tempX;
	GingerY[voice] = tempY;


	res = sqrt((GingerX[voice]*GingerX[voice])+(GingerY[voice]*GingerY[voice]));

	Total = long(float(res)*float((scale[0]*numocts)));

	Total %= (scale[0]*numocts); // ?

	if (Total<scale[0]) oct=0;
	if ((Total>=scale[0])&&(Total<(scale[0]*2))) 
	{
		Total -=scale[0];
		oct=1;
	}
	if((Total>=(scale[0]*2))&&(Total<(scale[0]*3)))
	{
		Total -= scale[0]*2;
		oct=2;
	}
	if((Total>=(scale[0]*3))&&(Total<(scale[0]*4)))
	{
		Total -= scale[0]*3;
		oct=3;
	}

	note = scale[Total+1]+startnote+(12*oct)+(modulresult[voice]*modulate);

	if(inverted)
	{
		int top = startnote + (12*numocts);
		note = (top-note)+startnote;
	}


	return note;

}

/////////////////////////////////



int __fastcall Lorentz(int voice, long counter, long test, const int* scale, 
				int numocts,long base, long startnum, 
				int startnote,BOOL modulate, BOOL modulrand, 
				int modulnum, BOOL inverted, float logmapvar, float oneoverf,
				float henonA, float henonB, float hopalong,
				float martin, float lorentz, unsigned long earthwormvar,
				unsigned long earthwormmulti, unsigned earthwormtruncator,
				double ranmin, double ranmax, BYTE wolframrule,
				int wolframsize, long hailstonestart)

{
	int note, oct;
	long Total;
	static int modulresult[MAXVOICES];
//	static int count8[MAXVOICES];


	if (modulate&&(!modulrand)&&(counter>=modulnum))
	{
		count8[voice] = counter%modulnum;

		if(!count8[voice])
		{
			modulresult[voice] = counter%12;
			count8[voice] = modulnum;
		}
	
	}
	else if (modulate&&modulrand)
	{
		if(count8[voice]<1)
		{
			modulresult[voice] = counter%12;
			count8[voice] = (rand()%50)+1;
		}
		count8[voice]--;
	}
	else
		count8[voice] = modulresult[voice] = 0;

	LorentzX[voice] = fabs(lorentz*((3*LorentzX[voice])-(4*LorentzX[voice]*LorentzX[voice]*LorentzX[voice])));

	Total = long((LorentzX[voice])*float(scale[0]*numocts));

	Total %= (scale[0]*numocts); // ?

	if (Total<scale[0]) oct=0;
	if ((Total>=scale[0])&&(Total<(scale[0]*2))) 
	{
		Total -=scale[0];
		oct=1;
	}
	if((Total>=(scale[0]*2))&&(Total<(scale[0]*3)))
	{
		Total -= scale[0]*2;
		oct=2;
	}
	if((Total>=(scale[0]*3))&&(Total<(scale[0]*4)))
	{
		Total -= scale[0]*3;
		oct=3;
	}

	note = scale[Total+1]+startnote+(12*oct)+(modulresult[voice]*modulate);

	if(inverted)
	{
		int top = startnote + (12*numocts);
		note = (top-note)+startnote;
	}


	return note;


}



int __fastcall Earthworm(int voice, long counter, long test, const int* scale, 
				int numocts,long base, long startnum, 
				int startnote,BOOL modulate, BOOL modulrand, 
				int modulnum, BOOL inverted, float logmapvar, float oneoverf,
				float henonA, float henonB, float hopalong,
				float martin, float lorentz, unsigned long earthwormvar,
				unsigned long earthwormmulti, unsigned earthwormtruncator,
				double ranmin, double ranmax, BYTE wolframrule,
				int wolframsize, long hailstonestart)

{
	int note, oct;
	unsigned long Total;
	static int modulresult[MAXVOICES];
//	static int count7[MAXVOICES];
	unsigned long res = 0;


	if (modulate&&(!modulrand)&&(counter>=modulnum))
	{
		count9[voice] = counter%modulnum;

		if(!count9[voice])
		{
			modulresult[voice] = counter%12;
			count9[voice] = modulnum;
		}
	
	}
	else if (modulate&&modulrand)
	{
		if(count9[voice]<1)
		{
			modulresult[voice] = counter%12;
			count9[voice] = (rand()%50)+1;
		}
		count9[voice]--;
	}
	else
		count9[voice] = modulresult[voice] = 0;


	EarthWorm[voice] = EarthWorm[voice]*earthwormmulti;

	int cut = 1;

	for (int y = 0; y<earthwormtruncator; y++)
		cut *= 10;

	res = (EarthWorm[voice])%cut;


	if (res == 0)
		res = 1;

	EarthWorm[voice] = res;

	Total = res;

	Total %= (scale[0]*numocts); // ?

	if (Total<scale[0]) oct=0;
	if ((Total>=scale[0])&&(Total<(scale[0]*2))) 
	{
		Total -=scale[0];
		oct=1;
	}
	if((Total>=(scale[0]*2))&&(Total<(scale[0]*3)))
	{
		Total -= scale[0]*2;
		oct=2;
	}
	if((Total>=(scale[0]*3))&&(Total<(scale[0]*4)))
	{
		Total -= scale[0]*3;
		oct=3;
	}

	note = scale[Total+1]+startnote+(12*oct)+(modulresult[voice]*modulate);

	if(inverted)
	{
		int top = startnote + (12*numocts);
		note = (top-note)+startnote;
	}


	return note;

}



int __fastcall RandomAlgorithm(int voice, long counter, long test, const int* scale, 
				int numocts,long base, long startnum, 
				int startnote,BOOL modulate, BOOL modulrand, 
				int modulnum, BOOL inverted, float logmapvar, float oneoverf,
				float henonA, float henonB, float hopalong,
				float martin, float lorentz, unsigned long earthwormvar,
				unsigned long earthwormmulti, unsigned earthwormtruncator,
				double ranmin, double ranmax, BYTE wolframrule,
				int wolframsize, long hailstonestart)

{
	int note, oct;
	unsigned long Total;
	static int modulresult[MAXVOICES];
	double res = 0;


	if (modulate&&(!modulrand)&&(counter>=modulnum))
	{
		count10[voice] = counter%modulnum;

		if(!count10[voice])
		{
			modulresult[voice] = counter%12;
			count10[voice] = modulnum;
		}
	
	}
	else if (modulate&&modulrand)
	{
		if(count10[voice]<1)
		{
			modulresult[voice] = counter%12;
			count10[voice] = (rand()%50)+1;
		}
		count10[voice]--;
	}
	else
		count10[voice] = modulresult[voice] = 0;


	res = (RanDev()*(ranmax-ranmin))+ranmin;

	Total = unsigned long(res*double(scale[0]*numocts));

	Total %= (scale[0]*numocts); // ?

	if (Total<scale[0]) oct=0;
	if ((Total>=scale[0])&&(Total<(scale[0]*2))) 
	{
		Total -=scale[0];
		oct=1;
	}
	if((Total>=(scale[0]*2))&&(Total<(scale[0]*3)))
	{
		Total -= scale[0]*2;
		oct=2;
	}
	if((Total>=(scale[0]*3))&&(Total<(scale[0]*4)))
	{
		Total -= scale[0]*3;
		oct=3;
	}

	note = scale[Total+1]+startnote+(12*oct)+(modulresult[voice]*modulate);

	if(inverted)
	{
		int top = startnote + (12*numocts);
		note = (top-note)+startnote;
	}


	return note;

}




int __fastcall Hailstone(int voice, long counter, long test, const int* scale, 
				int numocts,long base, long startnum, 
				int startnote,BOOL modulate, BOOL modulrand, 
				int modulnum, BOOL inverted, float logmapvar, float oneoverf,
				float henonA, float henonB, float hopalong,
				float martin, float lorentz, unsigned long earthwormvar,
				unsigned long earthwormmulti, unsigned earthwormtruncator,
				double ranmin, double ranmax, BYTE wolframrule,
				int wolframsize, long hailstonestart)

{
	int note, oct;
	__int64 Total =0 ;
	static int modulresult[MAXVOICES];
//	static int count7[MAXVOICES];


	if (modulate&&(!modulrand)&&(counter>=modulnum))
	{
		count11[voice] = counter%modulnum;

		if(!count11[voice])
		{
			modulresult[voice] = counter%12;
			count11[voice] = modulnum;
		}
	
	}
	else if (modulate&&modulrand)
	{
		if(count11[voice]<1)
		{
			modulresult[voice] = counter%12;
			count11[voice] = (rand()%50)+1;
		}
		count11[voice]--;
	}
	else
		count11[voice] = modulresult[voice] = 0;


	Total = HailStoneNumber[voice];

	if (Total<0)
		Total = -Total;

	Total %= (scale[0]*numocts); // ?

	if (Total<scale[0]) oct=0;
	if ((Total>=scale[0])&&(Total<(scale[0]*2))) 
	{
		Total -=scale[0];
		oct=1;
	}
	if((Total>=(scale[0]*2))&&(Total<(scale[0]*3)))
	{
		Total -= scale[0]*2;
		oct=2;
	}
	if((Total>=(scale[0]*3))&&(Total<(scale[0]*4)))
	{
		Total -= scale[0]*3;
		oct=3;
	}

	note = scale[Total+1]+startnote+(12*oct)+(modulresult[voice]*modulate);

	if(inverted)
	{
		int top = startnote + (12*numocts);
		note = (top-note)+startnote;
	}

	if (HailStoneNumber[voice]%2)
		HailStoneNumber[voice] = (HailStoneNumber[voice]*3)+1;
	else
		HailStoneNumber[voice] = HailStoneNumber[voice]/2;

	if (HailStoneNumber[voice] == 1)
		HailStoneNumber[voice] = hailstonestart;


	return note;

}



int __fastcall Wolfram(int voice, long counter, long test, const int* scale, 
				int numocts,long base, long startnum, 
				int startnote,BOOL modulate, BOOL modulrand, 
				int modulnum, BOOL inverted, float logmapvar, float oneoverf,
				float henonA, float henonB, float hopalong,
				float martin, float lorentz, unsigned long earthwormvar,
				unsigned long earthwormmulti, unsigned earthwormtruncator,
				double ranmin, double ranmax, BYTE wolframrule,
				int wolframsize, long hailstonestart)

{
	int note, oct, type;
	unsigned long Total =0 ;
	static int modulresult[MAXVOICES];
	unsigned char cellLEFT, cellRIGHT, cell;
	unsigned left, right;


	if (modulate&&(!modulrand)&&(counter>=modulnum))
	{
		count12[voice] = counter%modulnum;

		if(!count12[voice])
		{
			modulresult[voice] = counter%12;
			count12[voice] = modulnum;
		}
	
	}
	else if (modulate&&modulrand)
	{
		if(count12[voice]<1)
		{
			modulresult[voice] = counter%12;
			count12[voice] = (rand()%50)+1;
		}
		count12[voice]--;
	}
	else
		count12[voice] = modulresult[voice] = 0;

	//////////////////////



			for (int i=0; i<wolframsize; i++)
			{
				cell = WolframArray1[voice][i];
				
				right = (i+1)<wolframsize ? i+1: 0;
				left  = (i-1)>-1 ?  i-1: wolframsize-1;

				cellLEFT = WolframArray1[voice][left];
				cellRIGHT = WolframArray1[voice][right];

				if (!cellLEFT&&!cell&&!cellRIGHT)
					type = 0;
				if (!cellLEFT&&!cell&&cellRIGHT)
					type = 1;
				if (!cellLEFT&&cell&&!cellRIGHT)
					type = 2;
				if (!cellLEFT&&cell&&cellRIGHT)
					type = 3;
				if (cellLEFT&&!cell&&!cellRIGHT)
					type = 4;
				if (cellLEFT&&!cell&&cellRIGHT)
					type = 5;
				if (cellLEFT&&cell&&!cellRIGHT)
					type = 6;
				if (cellLEFT&&cell&&cellRIGHT)
					type = 7;

				switch (type)

				{
					case 0:
						WolframArray2[voice][i] = wolframrule&0x01? 1 : 0;
						break;
					case 1:
						WolframArray2[voice][i] = wolframrule&0x02? 1 : 0;
						break;
					case 2:
						WolframArray2[voice][i] = wolframrule&0x04? 1 : 0;
						break;
					case 3:
						WolframArray2[voice][i] = wolframrule&0x08? 1 : 0;
						break;
					case 4:
						WolframArray2[voice][i] = wolframrule&0x10? 1 : 0;
						break;
					case 5:
						WolframArray2[voice][i] = wolframrule&0x20? 1 : 0;
						break;
					case 6:
						WolframArray2[voice][i] = wolframrule&0x40? 1 : 0;
						break;
					case 7:
						WolframArray2[voice][i] = wolframrule&0x80? 1 : 0;
						break;
				}
			}

			for (i = 0; i<wolframsize; i++)
				Total += WolframArray2[voice][i]? i : 0;

			for (i = 0; i<wolframsize; i++)
				WolframArray1[voice][i] = WolframArray2[voice][i];


	///////////////

	if (Total<0)
		Total = -Total;



	Total %= (scale[0]*numocts); // ?

	if (Total<scale[0]) oct=0;
	if ((Total>=scale[0])&&(Total<(scale[0]*2))) 
	{
		Total -=scale[0];
		oct=1;
	}
	if((Total>=(scale[0]*2))&&(Total<(scale[0]*3)))
	{
		Total -= scale[0]*2;
		oct=2;
	}
	if((Total>=(scale[0]*3))&&(Total<(scale[0]*4)))
	{
		Total -= scale[0]*3;
		oct=3;
	}

	note = scale[Total+1]+startnote+(12*oct)+(modulresult[voice]*modulate);

	if(inverted)
	{
		int top = startnote + (12*numocts);
		note = (top-note)+startnote;
	}



	return note;

}






void ComputeTotal(long test, long base, long* total)
{
	unsigned long reminder, quotient;
	*total=0;
	do {
		quotient=test/base;
		reminder=test%base;
		*total +=reminder;
		test=quotient;
	}
	while(quotient>(unsigned long)base);
	*total +=(test/base)+(test%base);

}


UINT Wait(LPVOID p)
{
	waiting = TRUE;
	Sleep(delay);
	waiting = FALSE;
	delay = 0L;
	return 0;
}


void CFractMusView::OnFileSavemidi() 
{


	CFractMusDoc* pDoc = GetDocument();

	OnKillfocusGencounterEd();
	OnKillfocusStartcounterEd();

	if (pDoc->m_NumEvents<0)
	{
		MessageBox("Cannot save midi file, you have not created a composition yet!", "Error",MB_OK+MB_ICONEXCLAMATION);
		return;
	}


	/*if ((PlayChoice == CURRENT_SETTINGS)&&(pDoc->dStartCounter>=pDoc->dGeneralCounter))
	{
		MessageBox("Start counter must be less then End Counter","Error", MB_OK+MB_ICONEXCLAMATION);
		return;
	}*/

	long test[MAXVOICES];
	int note[MAXVOICES];
	int lastnote[MAXVOICES];
	int prevnote[MAXVOICES];
	register int i, j;
	int dracula[MAXVOICES];
	register long count;

	char FileName[500];
	char FileTitle[100];

	UINT saveinfo = MessageBox("Do you want to write Algorithm info to the file?", "FractMus", MB_YESNO+MB_ICONQUESTION);


	OPENFILENAME ofn;
	memset(&ofn, 0, sizeof(ofn));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = NULL;
	ofn.hInstance = NULL;

	ofn.lpstrFilter = 
		TEXT("Midi files *.mid\0*.mid\0\0");

	ofn.lpstrCustomFilter = NULL;
	ofn.nMaxCustFilter = 0;
	ofn.nFilterIndex = 1;
	ofn.lpstrFile = FileName;
	ofn.nMaxFile = 500;
	ofn.lpstrFileTitle = FileTitle;
	ofn.nMaxFileTitle = 99;
	ofn.lpstrInitialDir = NULL;
	ofn.lpstrTitle="Save Midi File";
	ofn.Flags = OFN_OVERWRITEPROMPT|OFN_LONGNAMES;
	ofn.lpstrDefExt = "mid";
	ofn.lCustData = NULL;
	ofn.lpfnHook = NULL;
	ofn.lpTemplateName = NULL;

	FileName[0] = '\0';

	if(!GetSaveFileName(&ofn))
		return;

	if(FileName[0] == '\0')
		return;

	for (i=VOICE1; i<MAXVOICES; i++) 
	{
		count1[i] = 0;
		count2[i] = 0;
		count3[i] = 0;
		count4[i] = 0;
		count5[i] = 0;
		count6[i] = 0;
		count7[i] = 0;
		count8[i] = 0;
		count9[i] = 0;
		count10[i] = 0;
		count11[i] = 0;
		count12[i] = 0;
		EarthWorm[i] = 0;
	}


	if(1)
	{

	

		unsigned char data[4];
		register int k;
		CWaitCursor wait;
		BOOL WillBeActive[MAXVOICES];
		for(i=VOICE1; i<MAXVOICES; i++)
			WillBeActive[i] = FALSE;

		for(j=0; j<=pDoc->m_NumEvents; j++)
			for(i=VOICE1; i<MAXVOICES; i++)
				if(pDoc->Event[j].IsActive[i])
					WillBeActive[i] = TRUE;

		for(i=VOICE1; i<MAXVOICES; i++)
		{
			lastnote[i] = -1;
			prevnote[i] = -1;
		}



		if ((SaveMidiFile = fopen(FileName, "wb"))!=NULL)
		{

			unsigned long giveGuevo = 0;
			unsigned long giveMoreGuevo = 0;
			//unsigned long offdur=0;
			BOOL done[MAXVOICES];
			//BOOL prevnoteisoff = TRUE;

			mf_write_header_chunk(0,1,192);


			unsigned long trkhdr,trklength;
			long offset, place_marker;
			//int offnote = 0;
			
			trkhdr = MTrk;
			trklength = 0;

			offset = ftell(SaveMidiFile); 

			write32bit(trkhdr);
			write32bit(trklength);

			Mf_numbyteswritten = 0L;


			for(j=0; j<=pDoc->m_NumEvents; j++)
			{
				count = pDoc->Event[j].count;

				for(i=VOICE1; i<MAXVOICES; i++)
					dracula[i] = int(pDoc->Event[j].NoteValue[i]);

				for (i=VOICE1; i<MAXVOICES; i++) 
				{
					count1[i] = 0;
					count2[i] = 0;
					count3[i] = 0;
					count4[i] = 0;
					count5[i] = 0;
					count6[i] = 0;
					count7[i] = 0;
					count8[i] = 0;
					count9[i] = 0;
					count10[i] = 0;
					count11[i] = 0;
					count12[i] = 0;
					EarthWorm[i] = 0;
					done[i] = FALSE;
				}



				mf_write_tempo(60000000/(pDoc->Event[j].Tempo),0L);



  				data[0] = (char)pDoc->Event[j].TimeSigNum;
				data[1] = (char)(log10(double(pDoc->Event[j].TimeSigDen))/log10((double)2));
				data[2] = (char)8;
				data[3] = (char)8;

				mf_write_meta_event(0L, time_signature, data, 4L);




				for(i = VOICE1; i<MAXVOICES; i++)
					if(/*WillBeActive[i]*/pDoc->Event[j].IsActive[i])
					{

  						data[0] = pDoc->Event[j].Instrument[i];
						int v = pDoc->Event[j].Channel[i]==PercussionChannel? PercussionChannel : i;

						mf_write_midi_event(0L, program_chng, /*pDoc->Event[j].Channel[i]*/v, data, 1L);

					}

				for(i = VOICE1; i<MAXVOICES; i++)

					if(/*WillBeActive[i]*/pDoc->Event[j].IsActive[i])
					{
						data[0] = 0x0a;
						data[1] = 64+pDoc->Event[j].Pan[i];
						int v = pDoc->Event[j].Channel[i]==PercussionChannel? PercussionChannel : i;

						mf_write_midi_event(0L, control_change, /*pDoc->Event[j].Channel[i]*/v, data, 2L);

					}

				/*for(i = VOICE1; i<MAXVOICES; i++)
				{
					if(j>0)
					{
						if (pDoc->Event[j-1].IsActive[i])
						{
							data[0] = prevnote[i];
							data[1] = 0;

							mf_write_midi_event(0L, note_off, i, data, 2);
						}


					}
				}*/


			if (saveinfo == IDYES)
			{
				for(i = VOICE1; i<MAXVOICES; i++)
					if(pDoc->Event[j].IsActive[i])
					{
						CString al;
						al.Format("Chan %d: %s", pDoc->Event[j].Channel[i], AlgorithmNames[ pDoc->Event[j].AlgorithmNum[i] ]);
						CString coeff;
						switch (pDoc->Event[j].AlgorithmNum[i])
						{
							case MORSE_THUE:
								coeff.Format(" (%d, %d)", pDoc->Event[j].Base[i], pDoc->Event[j].Multi[i]);
								break;
							case LOGMAP:
								coeff.Format(" (%f)", pDoc->Event[j].LogMapVar[i]);
								break;
							case ONE_OVER_F_NOISE:
								coeff.Format(" (%f)", pDoc->Event[j].FNoiseVar[i]);
								break;
							case HENON:
								coeff.Format(" (%f,%f)", pDoc->Event[j].HenonA[i], pDoc->Event[j].HenonB[i]);
								break;
							case MARTIN:
								coeff.Format(" (%f)", pDoc->Event[j].MartinVar[i]);
								break;
							case HOPALONG:
								coeff.Format(" (%f)", pDoc->Event[j].HopalongVar[i]);
								break;
							case GINGERBREADMAN:
								coeff.Format(" (%f,%f)", pDoc->Event[j].GingerX[i], pDoc->Event[j].GingerY[i]);
								break;
							case LORENTZ:
								coeff.Format(" (%f)", pDoc->Event[j].LorentzVar[i]);
								break;
							case EARTHWORM:
								coeff.Format(" (%d, %d, %d)", pDoc->Event[j].EarthWormVar[i], pDoc->Event[j].EarthWormMulti[i], pDoc->Event[j].EarthWormTruncator[i]);
								break;
							case RANDOM:
								coeff.Format(" (%f, %f)", pDoc->Event[j].RandomMin[i], pDoc->Event[j].RandomMax[i]);
								break;
							case HAILSTONE:
								coeff.Format(" (%d)", pDoc->m_Hailstone[i]);
								break;
							case WOLFRAM:
								coeff.Format(" (Rule:%d, Size%d)", pDoc->m_WolframRule[i], pDoc->m_WolframSize[i]);
								break;

						}

						CString ms = al+coeff;

						mf_write_meta_event(0L, marker, (unsigned char*)ms.GetBuffer(1), ms.GetLength());
					}
			}


				for (i=VOICE1; i<MAXVOICES; i++)
				{

					if(WillBeActive[i]/*pDoc->Event[j].IsActive[i]*/)
					{

						if(pDoc->Event[j].AlgorithmNum[i]==MORSE_THUE)
						{

							for(k = 0; k<count+(pDoc->Event[j].StartNum[i]*dracula[i]); k++)
							{

								if(!(((pDoc->Event[j].StartNum[i]*dracula[i])+k)%dracula[i]))
								{

								test[i] = ((((pDoc->Event[j].StartNum[i]*dracula[i])+k)/dracula[i])*pDoc->Event[j].Multi[i]);

								(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, ((pDoc->Event[j].StartNum[i]*dracula[i])+k)/dracula[i], test[i],
													pDoc->Event[j].pScale[i], 
													pDoc->Event[j].NumOcts[i], 
													pDoc->Event[j].Base[i], 
													pDoc->Event[j].StartNum[i],
													pDoc->Event[j].Note[i], 
													pDoc->Event[j].Modulates[i],
													pDoc->Event[j].ModulRandom[i], 
													pDoc->Event[j].ModulNum[i],
													pDoc->Event[j].Inverted[i],
													pDoc->Event[j].LogMapVar[i],
													pDoc->Event[j].FNoiseVar[i],
													pDoc->Event[j].HenonA[i],
													pDoc->Event[j].HenonB[i],
													pDoc->Event[j].HopalongVar[i],
													pDoc->Event[j].MartinVar[i],
													pDoc->Event[j].LorentzVar[i],
													pDoc->Event[j].EarthWormVar[i],
													pDoc->Event[j].EarthWormMulti[i],
													pDoc->Event[j].EarthWormTruncator[i],
													pDoc->Event[j].RandomMin[i],
													pDoc->Event[j].RandomMax[i],
													pDoc->Event[j].WolframRule[i],
													pDoc->Event[j].WolframSize[i],
													pDoc->Event[j].Hailstone[i]


									);

								}
							}
						}


						if(pDoc->Event[j].AlgorithmNum[i]==LOGMAP)
						{
							LogMap[i] = 0.2f;

							for(k = 0; k<count+(pDoc->Event[j].StartNum[i]*dracula[i]); k++)
							{

								if(!(((pDoc->Event[j].StartNum[i]*dracula[i])+k)%dracula[i]))
								{

								(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, ((pDoc->Event[j].StartNum[i]*dracula[i])+k)/dracula[i], 0,
													pDoc->Event[j].pScale[i], 
													pDoc->Event[j].NumOcts[i], 
													pDoc->Event[j].Base[i], 
													pDoc->Event[j].StartNum[i],
													pDoc->Event[j].Note[i], 
													pDoc->Event[j].Modulates[i],
													pDoc->Event[j].ModulRandom[i], 
													pDoc->Event[j].ModulNum[i],
													pDoc->Event[j].Inverted[i],
													pDoc->Event[j].LogMapVar[i],
													pDoc->Event[j].FNoiseVar[i],
													pDoc->Event[j].HenonA[i],
													pDoc->Event[j].HenonB[i],
													pDoc->Event[j].HopalongVar[i],
													pDoc->Event[j].MartinVar[i],
													pDoc->Event[j].LorentzVar[i],
													pDoc->Event[j].EarthWormVar[i],
													pDoc->Event[j].EarthWormMulti[i],
													pDoc->Event[j].EarthWormTruncator[i],
													pDoc->Event[j].RandomMin[i],
													pDoc->Event[j].RandomMax[i],
													pDoc->Event[j].WolframRule[i],
													pDoc->Event[j].WolframSize[i],
													pDoc->Event[j].Hailstone[i]


									);

								}
							}
						}

						if(pDoc->Event[j].AlgorithmNum[i]==ONE_OVER_F_NOISE)
						{
							LogMap2[i] = 0.2f;
							FNoise[i] = 0.0f;

							for(k = 0; k<count+(pDoc->Event[j].StartNum[i]*dracula[i]); k++)
							{

								if(!(((pDoc->Event[j].StartNum[i]*dracula[i])+k)%dracula[i]))
								{

								(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, ((pDoc->Event[j].StartNum[i]*dracula[i])+k)/dracula[i], 0,
													pDoc->Event[j].pScale[i], 
													pDoc->Event[j].NumOcts[i], 
													pDoc->Event[j].Base[i], 
													pDoc->Event[j].StartNum[i],
													pDoc->Event[j].Note[i], 
													pDoc->Event[j].Modulates[i],
													pDoc->Event[j].ModulRandom[i], 
													pDoc->Event[j].ModulNum[i],
													pDoc->Event[j].Inverted[i],
													pDoc->Event[j].LogMapVar[i],
													pDoc->Event[j].FNoiseVar[i],
													pDoc->Event[j].HenonA[i],
													pDoc->Event[j].HenonB[i],
													pDoc->Event[j].HopalongVar[i],
													pDoc->Event[j].MartinVar[i],
													pDoc->Event[j].LorentzVar[i],
													pDoc->Event[j].EarthWormVar[i],
													pDoc->Event[j].EarthWormMulti[i],
													pDoc->Event[j].EarthWormTruncator[i],
													pDoc->Event[j].RandomMin[i],
													pDoc->Event[j].RandomMax[i],
													pDoc->Event[j].WolframRule[i],
													pDoc->Event[j].WolframSize[i],
													pDoc->Event[j].Hailstone[i]


									);

								}
							}
						}

						if(pDoc->Event[j].AlgorithmNum[i]==HENON)
						{
							HenonX[i]  = HenonY[i] = 0.0f;

							for(k = 0; k<count+(pDoc->Event[j].StartNum[i]*dracula[i]); k++)
							{

								if(!(((pDoc->Event[j].StartNum[i]*dracula[i])+k)%dracula[i]))
								{

								(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, ((pDoc->Event[j].StartNum[i]*dracula[i])+k)/dracula[i], 0,
													pDoc->Event[j].pScale[i], 
													pDoc->Event[j].NumOcts[i], 
													pDoc->Event[j].Base[i], 
													pDoc->Event[j].StartNum[i],
													pDoc->Event[j].Note[i], 
													pDoc->Event[j].Modulates[i],
													pDoc->Event[j].ModulRandom[i], 
													pDoc->Event[j].ModulNum[i],
													pDoc->Event[j].Inverted[i],
													pDoc->Event[j].LogMapVar[i],
													pDoc->Event[j].FNoiseVar[i],
													pDoc->Event[j].HenonA[i],
													pDoc->Event[j].HenonB[i],
													pDoc->Event[j].HopalongVar[i],
													pDoc->Event[j].MartinVar[i],
													pDoc->Event[j].LorentzVar[i],
													pDoc->Event[j].EarthWormVar[i],
													pDoc->Event[j].EarthWormMulti[i],
													pDoc->Event[j].EarthWormTruncator[i],
													pDoc->Event[j].RandomMin[i],
													pDoc->Event[j].RandomMax[i],
													pDoc->Event[j].WolframRule[i],
													pDoc->Event[j].WolframSize[i],
													pDoc->Event[j].Hailstone[i]


									);

								}
							}
						}


						if(pDoc->Event[j].AlgorithmNum[i]==HOPALONG)
						{
							HopalongX[i]  = HopalongY[i] = 0.0f;

							for(k = 0; k<count+(pDoc->Event[j].StartNum[i]*dracula[i]); k++)
							{

								if(!(((pDoc->Event[j].StartNum[i]*dracula[i])+k)%dracula[i]))
								{

								(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, ((pDoc->Event[j].StartNum[i]*dracula[i])+k)/dracula[i], 0,
													pDoc->Event[j].pScale[i], 
													pDoc->Event[j].NumOcts[i], 
													pDoc->Event[j].Base[i], 
													pDoc->Event[j].StartNum[i],
													pDoc->Event[j].Note[i], 
													pDoc->Event[j].Modulates[i],
													pDoc->Event[j].ModulRandom[i], 
													pDoc->Event[j].ModulNum[i],
													pDoc->Event[j].Inverted[i],
													pDoc->Event[j].LogMapVar[i],
													pDoc->Event[j].FNoiseVar[i],
													pDoc->Event[j].HenonA[i],
													pDoc->Event[j].HenonB[i],
													pDoc->Event[j].HopalongVar[i],
													pDoc->Event[j].MartinVar[i],
													pDoc->Event[j].LorentzVar[i],
													pDoc->Event[j].EarthWormVar[i],
													pDoc->Event[j].EarthWormMulti[i],
													pDoc->Event[j].EarthWormTruncator[i],
													pDoc->Event[j].RandomMin[i],
													pDoc->Event[j].RandomMax[i],
													pDoc->Event[j].WolframRule[i],
													pDoc->Event[j].WolframSize[i],
													pDoc->Event[j].Hailstone[i]


									);

								}
							}
						}


						if(pDoc->Event[j].AlgorithmNum[i]==MARTIN)
						{
							MartinX[i]  = MartinY[i] = 0.0f;

							for(k = 0; k<count+(pDoc->Event[j].StartNum[i]*dracula[i]); k++)
							{

								if(!(((pDoc->Event[j].StartNum[i]*dracula[i])+k)%dracula[i]))
								{

								(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, ((pDoc->Event[j].StartNum[i]*dracula[i])+k)/dracula[i], 0,
													pDoc->Event[j].pScale[i], 
													pDoc->Event[j].NumOcts[i], 
													pDoc->Event[j].Base[i], 
													pDoc->Event[j].StartNum[i],
													pDoc->Event[j].Note[i], 
													pDoc->Event[j].Modulates[i],
													pDoc->Event[j].ModulRandom[i], 
													pDoc->Event[j].ModulNum[i],
													pDoc->Event[j].Inverted[i],
													pDoc->Event[j].LogMapVar[i],
													pDoc->Event[j].FNoiseVar[i],
													pDoc->Event[j].HenonA[i],
													pDoc->Event[j].HenonB[i],
													pDoc->Event[j].HopalongVar[i],
													pDoc->Event[j].MartinVar[i],
													pDoc->Event[j].LorentzVar[i],
													pDoc->Event[j].EarthWormVar[i],
													pDoc->Event[j].EarthWormMulti[i],
													pDoc->Event[j].EarthWormTruncator[i],
													pDoc->Event[j].RandomMin[i],
													pDoc->Event[j].RandomMax[i],
													pDoc->Event[j].WolframRule[i],
													pDoc->Event[j].WolframSize[i],
													pDoc->Event[j].Hailstone[i]


									);

								}
							}
						}


						if(pDoc->Event[j].AlgorithmNum[i]==GINGERBREADMAN)
						{
							GingerX[i] = pDoc->Event[j].GingerX[i];
							GingerY[i] = pDoc->Event[j].GingerY[i];

							for(k = 0; k<count+(pDoc->Event[j].StartNum[i]*dracula[i]); k++)
							{

								if(!(((pDoc->Event[j].StartNum[i]*dracula[i])+k)%dracula[i]))
								{

								(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, ((pDoc->Event[j].StartNum[i]*dracula[i])+k)/dracula[i], 0,
													pDoc->Event[j].pScale[i], 
													pDoc->Event[j].NumOcts[i], 
													pDoc->Event[j].Base[i], 
													pDoc->Event[j].StartNum[i],
													pDoc->Event[j].Note[i], 
													pDoc->Event[j].Modulates[i],
													pDoc->Event[j].ModulRandom[i], 
													pDoc->Event[j].ModulNum[i],
													pDoc->Event[j].Inverted[i],
													pDoc->Event[j].LogMapVar[i],
													pDoc->Event[j].FNoiseVar[i],
													pDoc->Event[j].HenonA[i],
													pDoc->Event[j].HenonB[i],
													pDoc->Event[j].HopalongVar[i],
													pDoc->Event[j].MartinVar[i],
													pDoc->Event[j].LorentzVar[i],
													pDoc->Event[j].EarthWormVar[i],
													pDoc->Event[j].EarthWormMulti[i],
													pDoc->Event[j].EarthWormTruncator[i],
													pDoc->Event[j].RandomMin[i],
													pDoc->Event[j].RandomMax[i],
													pDoc->Event[j].WolframRule[i],
													pDoc->Event[j].WolframSize[i],
													pDoc->Event[j].Hailstone[i]


									);

								}
							}
						}



						if(pDoc->Event[j].AlgorithmNum[i]==LORENTZ)
						{
							LorentzX[i] = 1.0f;

							for(k = 0; k<count+(pDoc->Event[j].StartNum[i]*dracula[i]); k++)
							{

								if(!(((pDoc->Event[j].StartNum[i]*dracula[i])+k)%dracula[i]))
								{

								(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, ((pDoc->Event[j].StartNum[i]*dracula[i])+k)/dracula[i], 0,
													pDoc->Event[j].pScale[i], 
													pDoc->Event[j].NumOcts[i], 
													pDoc->Event[j].Base[i], 
													pDoc->Event[j].StartNum[i],
													pDoc->Event[j].Note[i], 
													pDoc->Event[j].Modulates[i],
													pDoc->Event[j].ModulRandom[i], 
													pDoc->Event[j].ModulNum[i],
													pDoc->Event[j].Inverted[i],
													pDoc->Event[j].LogMapVar[i],
													pDoc->Event[j].FNoiseVar[i],
													pDoc->Event[j].HenonA[i],
													pDoc->Event[j].HenonB[i],
													pDoc->Event[j].HopalongVar[i],
													pDoc->Event[j].MartinVar[i],
													pDoc->Event[j].LorentzVar[i],
													pDoc->Event[j].EarthWormVar[i],
													pDoc->Event[j].EarthWormMulti[i],
													pDoc->Event[j].EarthWormTruncator[i],
													pDoc->Event[j].RandomMin[i],
													pDoc->Event[j].RandomMax[i],
													pDoc->Event[j].WolframRule[i],
													pDoc->Event[j].WolframSize[i],
													pDoc->Event[j].Hailstone[i]


									);

								}
							}
						}



						if(pDoc->Event[j].AlgorithmNum[i]==EARTHWORM)
						{
							EarthWorm[i] = pDoc->Event[j].EarthWormVar[i];

							for(k = 0; k<count+(pDoc->Event[j].StartNum[i]*dracula[i]); k++)
							{

								if(!(((pDoc->Event[j].StartNum[i]*dracula[i])+k)%dracula[i]))
								{

								(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, ((pDoc->Event[j].StartNum[i]*dracula[i])+k)/dracula[i], 0,
													pDoc->Event[j].pScale[i], 
													pDoc->Event[j].NumOcts[i], 
													pDoc->Event[j].Base[i], 
													pDoc->Event[j].StartNum[i],
													pDoc->Event[j].Note[i], 
													pDoc->Event[j].Modulates[i],
													pDoc->Event[j].ModulRandom[i], 
													pDoc->Event[j].ModulNum[i],
													pDoc->Event[j].Inverted[i],
													pDoc->Event[j].LogMapVar[i],
													pDoc->Event[j].FNoiseVar[i],
													pDoc->Event[j].HenonA[i],
													pDoc->Event[j].HenonB[i],
													pDoc->Event[j].HopalongVar[i],
													pDoc->Event[j].MartinVar[i],
													pDoc->Event[j].LorentzVar[i],
													pDoc->Event[j].EarthWormVar[i],
													pDoc->Event[j].EarthWormMulti[i],
													pDoc->Event[j].EarthWormTruncator[i],
													pDoc->Event[j].RandomMin[i],
													pDoc->Event[j].RandomMax[i],
													pDoc->Event[j].WolframRule[i],
													pDoc->Event[j].WolframSize[i],
													pDoc->Event[j].Hailstone[i]


									);

								}
							}
						}


						if(pDoc->Event[j].AlgorithmNum[i]==HAILSTONE)
						{
							HailStoneNumber[i] = pDoc->Event[j].Hailstone[i];

							for(k = 0; k<count+(pDoc->Event[j].StartNum[i]*dracula[i]); k++)
							{

								if(!(((pDoc->Event[j].StartNum[i]*dracula[i])+k)%dracula[i]))
								{

								(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, ((pDoc->Event[j].StartNum[i]*dracula[i])+k)/dracula[i], 0,
													pDoc->Event[j].pScale[i], 
													pDoc->Event[j].NumOcts[i], 
													pDoc->Event[j].Base[i], 
													pDoc->Event[j].StartNum[i],
													pDoc->Event[j].Note[i], 
													pDoc->Event[j].Modulates[i],
													pDoc->Event[j].ModulRandom[i], 
													pDoc->Event[j].ModulNum[i],
													pDoc->Event[j].Inverted[i],
													pDoc->Event[j].LogMapVar[i],
													pDoc->Event[j].FNoiseVar[i],
													pDoc->Event[j].HenonA[i],
													pDoc->Event[j].HenonB[i],
													pDoc->Event[j].HopalongVar[i],
													pDoc->Event[j].MartinVar[i],
													pDoc->Event[j].LorentzVar[i],
													pDoc->Event[j].EarthWormVar[i],
													pDoc->Event[j].EarthWormMulti[i],
													pDoc->Event[j].EarthWormTruncator[i],
													pDoc->Event[j].RandomMin[i],
													pDoc->Event[j].RandomMax[i],
													pDoc->Event[j].WolframRule[i],
													pDoc->Event[j].WolframSize[i],
													pDoc->Event[j].Hailstone[i]


									);

								}
							}
						}



						if(pDoc->Event[j].AlgorithmNum[i]==WOLFRAM)
						{
							if (WolframArray1[i])
							{
								delete WolframArray1[i];
								WolframArray1[i] = NULL;
							}
							if (WolframArray2[i])
							{
								delete WolframArray2[i];
								WolframArray2[i] = NULL;
							}

							WolframArray1[i] = new BYTE[pDoc->Event[j].WolframSize[i]];
							WolframArray2[i] = new BYTE[pDoc->Event[j].WolframSize[i]];

							for (int idx = 0; idx<32; idx++)
									WolframArray1[i][idx] = pDoc->Event[j].WolframInit[i] & unsigned long (pow(2, idx)) ? 1 : 0;

							if (pDoc->Event[j].WolframSize[i]>32)
							{
								for (idx = 32; idx<pDoc->Event[j].WolframSize[i]; idx++)
									WolframArray1[i][idx] = 0;
							}

							for(k = 0; k<count+(pDoc->Event[j].StartNum[i]*dracula[i]); k++)
							{

								if(!(((pDoc->Event[j].StartNum[i]*dracula[i])+k)%dracula[i]))
								{

								(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, ((pDoc->Event[j].StartNum[i]*dracula[i])+k)/dracula[i], 0,
													pDoc->Event[j].pScale[i], 
													pDoc->Event[j].NumOcts[i], 
													pDoc->Event[j].Base[i], 
													pDoc->Event[j].StartNum[i],
													pDoc->Event[j].Note[i], 
													pDoc->Event[j].Modulates[i],
													pDoc->Event[j].ModulRandom[i], 
													pDoc->Event[j].ModulNum[i],
													pDoc->Event[j].Inverted[i],
													pDoc->Event[j].LogMapVar[i],
													pDoc->Event[j].FNoiseVar[i],
													pDoc->Event[j].HenonA[i],
													pDoc->Event[j].HenonB[i],
													pDoc->Event[j].HopalongVar[i],
													pDoc->Event[j].MartinVar[i],
													pDoc->Event[j].LorentzVar[i],
													pDoc->Event[j].EarthWormVar[i],
													pDoc->Event[j].EarthWormMulti[i],
													pDoc->Event[j].EarthWormTruncator[i],
													pDoc->Event[j].RandomMin[i],
													pDoc->Event[j].RandomMax[i],
													pDoc->Event[j].WolframRule[i],
													pDoc->Event[j].WolframSize[i],
													pDoc->Event[j].Hailstone[i]


									);

								}
							}
						}


					}
				}

				do
				{
					for (i=VOICE1; i<MAXVOICES; i++)
					{


						if(WillBeActive[i]/*pDoc->Event[j].IsActive[i]*/)
						{

							if(((pDoc->Event[j].StartNum[i]*dracula[i])+count)>=0)
							{

								if(!(((pDoc->Event[j].StartNum[i]*dracula[i])+count)%dracula[i]))
								{
									test[i] = ((((pDoc->Event[j].StartNum[i]*dracula[i])+count)/dracula[i])*pDoc->Event[j].Multi[i]);
									note[i] = (*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, ((pDoc->Event[j].StartNum[i]*dracula[i])+count)/dracula[i], test[i],
														pDoc->Event[j].pScale[i], 
														pDoc->Event[j].NumOcts[i], 
														pDoc->Event[j].Base[i], 
														pDoc->Event[j].StartNum[i],
														pDoc->Event[j].Note[i], 
														pDoc->Event[j].Modulates[i],
														pDoc->Event[j].ModulRandom[i], 
														pDoc->Event[j].ModulNum[i],
														pDoc->Event[j].Inverted[i],
														pDoc->Event[j].LogMapVar[i],
														pDoc->Event[j].FNoiseVar[i],
														pDoc->Event[j].HenonA[i],
														pDoc->Event[j].HenonB[i],
														pDoc->Event[j].HopalongVar[i],
														pDoc->Event[j].MartinVar[i],
														pDoc->Event[j].LorentzVar[i],
														pDoc->Event[j].EarthWormVar[i],
														pDoc->Event[j].EarthWormMulti[i],
														pDoc->Event[j].EarthWormTruncator[i],
														pDoc->Event[j].RandomMin[i],
														pDoc->Event[j].RandomMax[i],
														pDoc->Event[j].WolframRule[i],
														pDoc->Event[j].WolframSize[i],
														pDoc->Event[j].Hailstone[i]

														
												);

									BOOL flag[MAXVOICES];


									if ((lastnote[i]!=note[i])/*||(!pDoc->Event[j].IsActive[i])*/)
									{

										if (flag[i])
										{
											data[0] = prevnote[i];
											data[1] = 0;

											mf_write_midi_event(0L, note_off, /*pDoc->Event[j].Channel[i]*/i, data, 2);
										}

										int v = pDoc->Event[j].Channel[i]==PercussionChannel? PercussionChannel : i;
										data[0] = pDoc->Event[j].Channel[i] == PercussionChannel? pDoc->Event[j].Percussion[i]+35 : note[i];
										data[1] = pDoc->Event[j].Dynamic[i]*(pDoc->Event[j].IsActive[i]);

										if (pDoc->Event[j].IsActive[i])
										{
											mf_write_midi_event(giveGuevo*24, note_on, /*pDoc->Event[j].Channel[i]*/v, data, 2);

											//giveGuevo =0;
										}
										else
										{
											mf_write_midi_event(giveGuevo*24, note_off, /*pDoc->Event[j].Channel[i]*/v, data, 2);

											//giveGuevo =0;
										}

										if (prevnote[i]!=note[i])
										{
											data[0] = prevnote[i];
											data[1] = 0;

											mf_write_midi_event(0L, note_off, /*pDoc->Event[j].Channel[i]*/i, data, 2);
											//prevnoteisoff = TRUE;
											flag[i] = FALSE;
										}
										else
										{
											flag[i]=TRUE;
										}
										lastnote[i] = pDoc->Event[j].Mode[i]? note[i] : -1;
										
										if (!pDoc->Event[j].IsActive[i])
											lastnote[i] = -1;
										prevnote[i] = pDoc->Event[j].Channel[i] == PercussionChannel? pDoc->Event[j].Percussion[i]+35 : note[i];
										//offdur = giveGuevo;
										//offnote = prevnote[i];


										giveGuevo=0;

									}

								}

							}

							if(j>0&&(!done[i]))
							{
								if (!pDoc->Event[j].IsActive[i]&&(pDoc->Event[j-1].IsActive[i]))
								{
										data[0] = 0x7b;
										data[1] = 0;

										int v = pDoc->Event[j-1].Channel[i]==PercussionChannel? PercussionChannel : i;

										mf_write_midi_event(0L, control_change, v, data, 2L);
								}
								if((pDoc->Event[j].IsActive[i])&&(pDoc->Event[j-1].IsActive[i])&&(pDoc->Event[j].Channel[i]!=pDoc->Event[j-1].Channel[i]))
								{
										data[0] = 0x7b;
										data[1] = 0;

										int v = pDoc->Event[j-1].Channel[i]==PercussionChannel? PercussionChannel : i;

										mf_write_midi_event(0L, control_change, v, data, 2L);
								}
								/*if((pDoc->Event[j].IsActive[i])&&(pDoc->Event[j-1].IsActive[i])&&(pDoc->Event[j].Channel[i]==pDoc->Event[j-1].Channel[i]))
								{
										data[0] = prevnote[i];
										data[1] = 0;

										int v = pDoc->Event[j].Channel[i]==PercussionChannel? PercussionChannel : i;
										mf_write_midi_event(0L, note_off, v, data, 2);


								}*/

								done[i] = TRUE;

							}
							///////////////

						}
					}

					count++;
					giveGuevo++;
				}
				while(count<pDoc->Event[j].genCount);



				
			}

			for (i = VOICE1; i<MAXVOICES; i++)
				if(WillBeActive[i])
				{
						data[0] = 0x7b;
						data[1] = 0;

						mf_write_midi_event(0L, control_change, i, data, 2L);
				}

			if (pDoc->m_Copyright !="")
			{
				CString tit = pDoc->GetTitle();
				tit = tit + " ";


				mf_write_meta_event(0L,copyright_notice, (unsigned char*)tit.GetBuffer(1), tit.GetLength());
				mf_write_meta_event(0L,copyright_notice, (unsigned char*)pDoc->m_Copyright.GetBuffer(1), pDoc->m_Copyright.GetLength());

			}
			mf_write_meta_event(giveGuevo, end_of_track, NULL, 0L);

			place_marker = ftell(SaveMidiFile);
			

 			if(fseek(SaveMidiFile,offset,0) < 0)
				MessageBox("error");

			trklength = Mf_numbyteswritten;


			write32bit(trkhdr);
			write32bit(trklength);

			fseek(SaveMidiFile,place_marker,0);

			fclose(SaveMidiFile);

		}
		else
		{
			MessageBox("Could not create file");
			return;
		}


		

	}
}

	

WORD noteOn(BYTE Stat, BYTE Note, BYTE Vel)
{

	union 
	{
		DWORD dwData;
		BYTE bData[4];
	}Midi;

	Midi.bData[0] = Stat;
	Midi.bData[1] = Note;
	Midi.bData[2] = Vel;
	Midi.bData[3] = 0;

	return midiOutShortMsg(midiOut, Midi.dwData);
}

WORD noteOff(BYTE Stat, BYTE Note, BYTE Vel)
{

	union 
	{
		DWORD dwData;
		BYTE bData[4];
	}Midi;

	Midi.bData[0] = Stat;
	Midi.bData[1] = Note;
	Midi.bData[2] = Vel;
	Midi.bData[3] = 0;

	return midiOutShortMsg(midiOut, Midi.dwData);
}

WORD progChange(BYTE Stat, BYTE Instr)
{

	union 
	{
		DWORD dwData;
		BYTE bData[4];
	}Midi;

	Midi.bData[0] = Stat;
	Midi.bData[1] = Instr;
	Midi.bData[2] = 0;
	Midi.bData[3] = 0;

	return midiOutShortMsg(midiOut, Midi.dwData);
}

WORD modeChange(BYTE Stat, BYTE type, BYTE val)
{
	union 
	{
		DWORD dwData;
		BYTE bData[4];
	}Midi;

	Midi.bData[0] = Stat;
	Midi.bData[1] = type;
	Midi.bData[2] = val;
	Midi.bData[3] = 0;

	return midiOutShortMsg(midiOut, Midi.dwData);

}



UINT PlayComposition(LPVOID pObj)
{
	CFractMusView* pView = (CFractMusView*)pObj;
	CFractMusDoc* pDoc = pView->GetDocument();
	CEdit* pE = (CEdit*)(pView->GetDlgItem(IDC_GENCOUNTER_ED));
	pView->GetDlgItem(IDC_RESET_GENCOUNTER)->EnableWindow(FALSE);
	pView->GetDlgItem(IDC_LOCK_GENCOUNTER)->EnableWindow(FALSE);
	pView->GetDlgItem(IDC_GENCOUNTER_ED)->EnableWindow(FALSE);
	pView->GetDlgItem(IDC_STARTCOUNTER_ED)->EnableWindow(FALSE);
	pView->GetDlgItem(IDC_ROUND)->EnableWindow(FALSE);


	long test[MAXVOICES];
	int note[MAXVOICES];
	int lastnote[MAXVOICES];
	int prevnote[MAXVOICES];
	int mode[MAXVOICES];
	int Instrument[MAXVOICES];
	long Base[MAXVOICES];
	long Multi[MAXVOICES];
	long StartNum[MAXVOICES];
	BOOL Modulates[MAXVOICES];
	BOOL ModulRandom[MAXVOICES];
	int ModulNum[MAXVOICES];
	int NumOcts[MAXVOICES];
	int Note[MAXVOICES];
	int Pan[MAXVOICES];
	BOOL Inverted[MAXVOICES];
	int Dynamic[MAXVOICES];
	float LogMapVar[MAXVOICES];
	float FNoiseVar[MAXVOICES];
	float HenonA[MAXVOICES];
	float HenonB[MAXVOICES];
	float HopalongVar[MAXVOICES];
	float MartinVar[MAXVOICES];
	float LorentzVar[MAXVOICES];
	unsigned long EarthWormVar[MAXVOICES];
	unsigned long EarthWormMulti[MAXVOICES];
	unsigned EarthWormTruncator[MAXVOICES];
	double RandomMin[MAXVOICES];
	double RandomMax[MAXVOICES];
	BYTE WolframRule[MAXVOICES];
	int  WolframSize[MAXVOICES];
	long HailStone[MAXVOICES];

	const int* pScale[MAXVOICES];
	int Channel[MAXVOICES];
	int i;
	register int j;
	int dracula[MAXVOICES];
	int Percussion[MAXVOICES];
	long duration; 
	long count;
	char s[55];

	count = pDoc->dStartCounter;

	if (!pView->LockGeneralCounter)
	{

		pDoc->dGeneralCounter = pView->GeneralCounter = pDoc->dStartCounter;
		ltoa(pView->GeneralCounter, s, 10);
		pE->SetWindowText(s);
	}


	delay = duration = 60000/(8*Tempo);


	for (i=VOICE1; i<MAXVOICES; i++) 
		dracula[i] = int(pDoc->m_NoteValue[i]);

	for (i=VOICE1; i<MAXVOICES; i++) 
	{
		count1[i] = 0;
		count2[i] = 0;
		count3[i] = 0;
		count4[i] = 0;
		count5[i] = 0;
		count6[i] = 0;
		count7[i] = 0;
		count8[i] = 0;
		count9[i] = 0;
		count10[i] = 0;
		count11[i] = 0;
		count12[i] = 0;
		EarthWorm[i] = 0;
	}

	for (i=VOICE1; i<MAXVOICES; i++)
	{
		lastnote[i] = -1;
		prevnote[i] = -1;
		mode[i] = pDoc->m_mode[i];
		Instrument[i] = pDoc->m_Instrument[i];
		Base[i] = pDoc->m_Base[i];
		Multi[i] = pDoc->m_Multi[i];
		StartNum[i] = pDoc->m_StartNum[i];
		Modulates[i] = pDoc->m_Modulates[i];
		ModulRandom[i] = pDoc->m_ModulRandom[i];
		ModulNum[i] = pDoc->m_ModulNum[i];
		NumOcts[i] = pDoc->m_NumOcts[i];
		Note[i] = pDoc->m_Note[i];
		Channel[i] = pDoc->m_Channel[i];
		pScale[i] = pDoc->m_pScale[i];
		Inverted[i] = pDoc->m_Inverted[i];
		Dynamic[i] = pDoc->m_Dynamic[i];
		LogMapVar[i] = pDoc->m_LogMapVar[i];
		FNoiseVar[i] = pDoc->m_FNoiseVar[i];
		LorentzVar[i] = pDoc->m_LorentzVar[i];
		HenonA[i] = pDoc->m_HenonA[i];
		HenonB[i] = pDoc->m_HenonB[i];
		HopalongVar[i] = pDoc->m_HopalongVar[i];
		MartinVar[i] = pDoc->m_MartinVar[i];
		EarthWormVar[i] = pDoc->m_EarthWormVar[i];
		EarthWormMulti[i] = pDoc->m_EarthWormMulti[i];
		EarthWormTruncator[i] = pDoc->m_EarthWormTruncator[i];
		RandomMin[i] = pDoc->m_RandomMin[i];
		RandomMax[i] = pDoc->m_RandomMax[i];
		WolframRule[i] = pDoc->m_WolframRule[i];
		WolframSize[i] = pDoc->m_WolframSize[i];
		HailStone[i] = pDoc->m_Hailstone[i];


		Pan[i] = pDoc->m_Pan[i]; 
		Percussion[i] = pDoc->m_Percussion[i];

	}

	for (i=VOICE1; i<MAXVOICES; i++)
		progChange(0xC0+Channel[i], (BYTE)Instrument[i]);

	for (i=VOICE1; i<MAXVOICES; i++)
		modeChange(0xb0+Channel[i], (BYTE)0x0a, 64+Pan[i]);


	for (i=VOICE1; i<MAXVOICES; i++)
	{

		if(pDoc->AlgorithmNum[i]==MORSE_THUE)
		{

			for(j = 0; j<count+(StartNum[i]*dracula[i]); j++)
			{
				if(!((StartNum[i]*dracula[i]+j)%dracula[i]))
				{


				test[i] = (((StartNum[i]*dracula[i]+j)/dracula[i])*Multi[i]);
				(*(Algorithm[ pDoc->AlgorithmNum[i] ]))(i, (StartNum[i]*dracula[i]+j)/dracula[i], test[i],
									pScale[i], 
									NumOcts[i], 
									Base[i], 
									StartNum[i],
									Note[i], 
									Modulates[i],
									ModulRandom[i], 
									ModulNum[i],
									Inverted[i],
									LogMapVar[i],
									FNoiseVar[i],
									HenonA[i],
									HenonB[i],
									HopalongVar[i],
									MartinVar[i],
									LorentzVar[i],
									EarthWormVar[i],
									EarthWormMulti[i],
									EarthWormTruncator[i],
									RandomMin[i],
									RandomMax[i],
									WolframRule[i],
									WolframSize[i],
									HailStone[i]
									
					);
				}
			}
		}


		if(pDoc->AlgorithmNum[i]==LOGMAP)
		{
			LogMap[i] = 0.2f;

			for(j = 0; j<count+(StartNum[i]*dracula[i]); j++)
			{
				if(!((StartNum[i]*dracula[i]+j)%dracula[i]))
				{

				(*(Algorithm[ pDoc->AlgorithmNum[i] ]))(i, (StartNum[i]*dracula[i]+j)/dracula[i], 0,
									pScale[i], 
									NumOcts[i], 
									Base[i], 
									StartNum[i],
									Note[i], 
									Modulates[i],
									ModulRandom[i], 
									ModulNum[i],
									Inverted[i],
									LogMapVar[i],
									FNoiseVar[i],
									HenonA[i],
									HenonB[i],
									HopalongVar[i],
									MartinVar[i],
									LorentzVar[i],
									EarthWormVar[i],
									EarthWormMulti[i],
									EarthWormTruncator[i],
									RandomMin[i],
									RandomMax[i],
									WolframRule[i],
									WolframSize[i],
									HailStone[i]

									
					);
				}
			}
		}

		if(pDoc->AlgorithmNum[i]==ONE_OVER_F_NOISE)
		{
			LogMap2[i] = 0.2f;
			FNoise[i] = 0.0f;

			for(j = 0; j<count+(StartNum[i]*dracula[i]); j++)
			{
				if(!((StartNum[i]*dracula[i]+j)%dracula[i]))
				{

				(*(Algorithm[ pDoc->AlgorithmNum[i] ]))(i, (StartNum[i]*dracula[i]+j)/dracula[i], 0,
									pScale[i], 
									NumOcts[i], 
									Base[i], 
									StartNum[i],
									Note[i], 
									Modulates[i],
									ModulRandom[i], 
									ModulNum[i],
									Inverted[i],
									LogMapVar[i],
									FNoiseVar[i],
									HenonA[i],
									HenonB[i],
									HopalongVar[i],
									MartinVar[i],
									LorentzVar[i],
											EarthWormVar[i],
											EarthWormMulti[i],
											EarthWormTruncator[i],
									RandomMin[i],
									RandomMax[i],
									WolframRule[i],
									WolframSize[i],
									HailStone[i]

									
					);
				}
			}
		}

		if(pDoc->AlgorithmNum[i]==HENON)
		{
			HenonX[i] = HenonY[i] = 0.0f;

			for(j = 0; j<count+(StartNum[i]*dracula[i]); j++)
			{
				if(!((StartNum[i]*dracula[i]+j)%dracula[i]))
				{

				(*(Algorithm[ pDoc->AlgorithmNum[i] ]))(i, (StartNum[i]*dracula[i]+j)/dracula[i], 0,
									pScale[i], 
									NumOcts[i], 
									Base[i], 
									StartNum[i],
									Note[i], 
									Modulates[i],
									ModulRandom[i], 
									ModulNum[i],
									Inverted[i],
									LogMapVar[i],
									FNoiseVar[i],
									HenonA[i],
									HenonB[i],
									HopalongVar[i],
									MartinVar[i],
									LorentzVar[i],
											EarthWormVar[i],
											EarthWormMulti[i],
											EarthWormTruncator[i],
									RandomMin[i],
									RandomMax[i],
									WolframRule[i],
									WolframSize[i],
									HailStone[i]

									
					);
				}
			}
		}

		if(pDoc->AlgorithmNum[i]==HOPALONG)
		{
			HopalongX[i] = HopalongY[i] = 0.0f;

			for(j = 0; j<count+(StartNum[i]*dracula[i]); j++)
			{
				if(!((StartNum[i]*dracula[i]+j)%dracula[i]))
				{

				(*(Algorithm[ pDoc->AlgorithmNum[i] ]))(i, (StartNum[i]*dracula[i]+j)/dracula[i], 0,
									pScale[i], 
									NumOcts[i], 
									Base[i], 
									StartNum[i],
									Note[i], 
									Modulates[i],
									ModulRandom[i], 
									ModulNum[i],
									Inverted[i],
									LogMapVar[i],
									FNoiseVar[i],
									HenonA[i],
									HenonB[i],
									HopalongVar[i],
									MartinVar[i],
									LorentzVar[i],
											EarthWormVar[i],
											EarthWormMulti[i],
											EarthWormTruncator[i],
									RandomMin[i],
									RandomMax[i],
									WolframRule[i],
									WolframSize[i],
									HailStone[i]

									
					);
				}
			}
		}

		if(pDoc->AlgorithmNum[i]==MARTIN)
		{
			MartinX[i] = MartinY[i] = 0.0f;

			for(j = 0; j<count+(StartNum[i]*dracula[i]); j++)
			{
				if(!((StartNum[i]*dracula[i]+j)%dracula[i]))
				{

				(*(Algorithm[ pDoc->AlgorithmNum[i] ]))(i, (StartNum[i]*dracula[i]+j)/dracula[i], 0,
									pScale[i], 
									NumOcts[i], 
									Base[i], 
									StartNum[i],
									Note[i], 
									Modulates[i],
									ModulRandom[i], 
									ModulNum[i],
									Inverted[i],
									LogMapVar[i],
									FNoiseVar[i],
									HenonA[i],
									HenonB[i],
									HopalongVar[i],
									MartinVar[i],
									LorentzVar[i],
											EarthWormVar[i],
											EarthWormMulti[i],
											EarthWormTruncator[i],
									RandomMin[i],
									RandomMax[i],
									WolframRule[i],
									WolframSize[i],
									HailStone[i]

									
					);
				}
			}
		}

		if(pDoc->AlgorithmNum[i]==GINGERBREADMAN)
		{
			GingerX[i] = pDoc->m_GingerX[i];
			GingerY[i] = pDoc->m_GingerY[i];;

			for(j = 0; j<count+(StartNum[i]*dracula[i]); j++)
			{
				if(!((StartNum[i]*dracula[i]+j)%dracula[i]))
				{

				(*(Algorithm[ pDoc->AlgorithmNum[i] ]))(i, (StartNum[i]*dracula[i]+j)/dracula[i], 0,
									pScale[i], 
									NumOcts[i], 
									Base[i], 
									StartNum[i],
									Note[i], 
									Modulates[i],
									ModulRandom[i], 
									ModulNum[i],
									Inverted[i],
									LogMapVar[i],
									FNoiseVar[i],
									HenonA[i],
									HenonB[i],
									HopalongVar[i],
									MartinVar[i],
									LorentzVar[i],
											EarthWormVar[i],
											EarthWormMulti[i],
											EarthWormTruncator[i],
									RandomMin[i],
									RandomMax[i],
									WolframRule[i],
									WolframSize[i],
									HailStone[i]

									
					);
				}
			}
		}

		if(pDoc->AlgorithmNum[i]==LORENTZ)
		{
			LorentzX[i] = 1.0f;

			for(j = 0; j<count+(StartNum[i]*dracula[i]); j++)
			{
				if(!((StartNum[i]*dracula[i]+j)%dracula[i]))
				{

				(*(Algorithm[ pDoc->AlgorithmNum[i] ]))(i, (StartNum[i]*dracula[i]+j)/dracula[i], 0,
									pScale[i], 
									NumOcts[i], 
									Base[i], 
									StartNum[i],
									Note[i], 
									Modulates[i],
									ModulRandom[i], 
									ModulNum[i],
									Inverted[i],
									LogMapVar[i],
									FNoiseVar[i],
									HenonA[i],
									HenonB[i],
									HopalongVar[i],
									MartinVar[i],
									LorentzVar[i],
											EarthWormVar[i],
											EarthWormMulti[i],
											EarthWormTruncator[i],
									RandomMin[i],
									RandomMax[i],
									WolframRule[i],
									WolframSize[i],
									HailStone[i]

									
					);
				}
			}
		}


		if(pDoc->AlgorithmNum[i]==EARTHWORM)
		{
			EarthWorm[i] = EarthWormVar[i];

			for(j = 0; j<count+(StartNum[i]*dracula[i]); j++)
			{
				if(!((StartNum[i]*dracula[i]+j)%dracula[i]))
				{

				(*(Algorithm[ pDoc->AlgorithmNum[i] ]))(i, (StartNum[i]*dracula[i]+j)/dracula[i], 0,
									pScale[i], 
									NumOcts[i], 
									Base[i], 
									StartNum[i],
									Note[i], 
									Modulates[i],
									ModulRandom[i], 
									ModulNum[i],
									Inverted[i],
									LogMapVar[i],
									FNoiseVar[i],
									HenonA[i],
									HenonB[i],
									HopalongVar[i],
									MartinVar[i],
									LorentzVar[i],
									EarthWormVar[i],
									EarthWormMulti[i],
									EarthWormTruncator[i],
									RandomMin[i],
									RandomMax[i],
									WolframRule[i],
									WolframSize[i],
									HailStone[i]

									
					);
				}
			}
		}


		if(pDoc->AlgorithmNum[i]==HAILSTONE)
		{
			HailStoneNumber[i] = pDoc->m_Hailstone[i];

			for(j = 0; j<count+(StartNum[i]*dracula[i]); j++)
			{
				if(!((StartNum[i]*dracula[i]+j)%dracula[i]))
				{

				(*(Algorithm[ pDoc->AlgorithmNum[i] ]))(i, (StartNum[i]*dracula[i]+j)/dracula[i], 0,
									pScale[i], 
									NumOcts[i], 
									Base[i], 
									StartNum[i],
									Note[i], 
									Modulates[i],
									ModulRandom[i], 
									ModulNum[i],
									Inverted[i],
									LogMapVar[i],
									FNoiseVar[i],
									HenonA[i],
									HenonB[i],
									HopalongVar[i],
									MartinVar[i],
									LorentzVar[i],
											EarthWormVar[i],
											EarthWormMulti[i],
											EarthWormTruncator[i],
									RandomMin[i],
									RandomMax[i],
									WolframRule[i],
									WolframSize[i],
									HailStone[i]

									
					);
				}
			}
		}


				if(pDoc->AlgorithmNum[i]==WOLFRAM)
				{
					if (WolframArray1[i])
					{
						delete WolframArray1[i];
						WolframArray1[i] = NULL;
					}
					if (WolframArray2[i])
					{
						delete WolframArray2[i];
						WolframArray2[i] = NULL;
					}

					WolframArray1[i] = new BYTE[WolframSize[i]];
					WolframArray2[i] = new BYTE[WolframSize[i]];

					for (int idx = 0; idx<32; idx++)
							WolframArray1[i][idx] = pDoc->m_WolframInit[i] & unsigned long (pow(2, idx)) ? 1 : 0;

					if (WolframSize[i]>32)
					{
						for (idx = 32; idx<WolframSize[i]; idx++)
							WolframArray1[i][idx] = 0;
					}


					for(j = 0; j<pDoc->dStartCounter+(StartNum[i]*dracula[i]); j++)
					{
						if(!((StartNum[i]*dracula[i]+j)%dracula[i]))
						{

						(*(Algorithm[ pDoc->AlgorithmNum[i] ]))(i, (StartNum[i]*dracula[i]+j)/dracula[i], 0,
											pScale[i], 
											NumOcts[i], 
											Base[i], 
											StartNum[i],
											Note[i], 
											Modulates[i],
											ModulRandom[i], 
											ModulNum[i],
											Inverted[i],
											LogMapVar[i],
											FNoiseVar[i],
											HenonA[i],
											HenonB[i],
											HopalongVar[i],
											MartinVar[i],
											LorentzVar[i],
											EarthWormVar[i],
											EarthWormMulti[i],
											EarthWormTruncator[i],
											RandomMin[i],
											RandomMax[i],
											WolframRule[i],
											WolframSize[i],
											HailStone[i]
											
							);
						}
					}
				}


	}



	while(m_Playing)
	{

		if (pView->LockGeneralCounter&&(count>=pDoc->dGeneralCounter))
			break;
		for (i=VOICE1; i<MAXVOICES; i++)
		{
			if((StartNum[i]*dracula[i]+count)>=0)
			{
			if(!((StartNum[i]*dracula[i]+count)%dracula[i]))
			{
				test[i] = (((StartNum[i]*dracula[i]+count)/dracula[i])*Multi[i]);
				note[i] = (*(Algorithm[ pDoc->AlgorithmNum[i] ]))(i, (StartNum[i]*dracula[i]+count)/dracula[i], test[i],
									pScale[i], 
									NumOcts[i], 
									Base[i], 
									StartNum[i],
									Note[i], 
									Modulates[i],
									ModulRandom[i], 
									ModulNum[i],
									Inverted[i],
									LogMapVar[i],
									FNoiseVar[i],
									HenonA[i],
									HenonB[i],
									HopalongVar[i],
									MartinVar[i],
									LorentzVar[i],
									EarthWormVar[i],
									EarthWormMulti[i],
									EarthWormTruncator[i],
									RandomMin[i],
									RandomMax[i],
									WolframRule[i],
									WolframSize[i],
									HailStone[i]

					);
			}
			}
		}


		for (i=VOICE1; i<MAXVOICES; i++)
		{
			if((StartNum[i]*dracula[i]+count)>=0)
			{

				if(!((StartNum[i]*dracula[i]+count)%dracula[i]))
				{
					if ((lastnote[i]!=note[i]))
					{
						noteOff(0x80+Channel[i], (BYTE)prevnote[i], (BYTE)(Dynamic[i]*pDoc->m_IsActive[i]));
						if (Channel[i] == PercussionChannel)
							noteOn(0x90+Channel[i], (BYTE)Percussion[i]+35, (BYTE)(Dynamic[i]*pDoc->m_IsActive[i]));
						else
							noteOn(0x90+Channel[i], (BYTE)note[i], (BYTE)(Dynamic[i]*pDoc->m_IsActive[i]));
						lastnote[i] = mode[i]? note[i] : -1;
						prevnote[i] = Channel[i] == PercussionChannel? Percussion[i]+35 : note[i];
					}
				}

			}
		}

		count++;

		if((!pView->LockGeneralCounter)&&(pDoc->dGeneralCounter<50000))
			pDoc->dGeneralCounter++;

		Sleep(duration);

	}

	if (pView->LockGeneralCounter&&(count>=pDoc->dGeneralCounter)&&m_Playing)
			pView->OnPlayStop();

	for (i=VOICE1; i<MAXVOICES; i++)
		modeChange(0xB0+i, 0x7b, 0);



	pView->GetDlgItem(IDC_LOCK_GENCOUNTER)->EnableWindow(TRUE);
	pView->GetDlgItem(IDC_ROUND)->EnableWindow(!pView->LockGeneralCounter);
	pView->GetDlgItem(IDC_GENCOUNTER_ED)->EnableWindow(!pView->LockGeneralCounter);
	pView->GetDlgItem(IDC_STARTCOUNTER_ED)->EnableWindow(TRUE);
	pView->GetDlgItem(IDC_RESET_GENCOUNTER)->EnableWindow(TRUE);

	pView->GeneralCounter = pDoc->dGeneralCounter;

	ltoa(pView->GeneralCounter, s, 10);
	pE->SetWindowText(s);


	return 0;

}

UINT PlayScript(LPVOID pObj)
{
	CFractMusView* pView = (CFractMusView*)pObj;
	CFractMusDoc* pDoc = pView->GetDocument();
	CEdit* pE = (CEdit*)(pView->GetDlgItem(IDC_GENCOUNTER_ED));
	CComboBox* pCB = (CComboBox*)(pView->GetDlgItem(IDC_PLAYFROM_CMB));
	CButton* pButt;
	CEdit* pEdit;
	pView->GetDlgItem(IDC_RESET_GENCOUNTER)->EnableWindow(FALSE);
	pView->GetDlgItem(IDC_LOCK_GENCOUNTER)->EnableWindow(FALSE);
	pView->GetDlgItem(IDC_GENCOUNTER_ED)->EnableWindow(FALSE);
	pView->GetDlgItem(IDC_STARTCOUNTER_ED)->EnableWindow(FALSE);
	pView->GetDlgItem(IDC_ROUND)->EnableWindow(FALSE);

	long test[MAXVOICES];
	int note[MAXVOICES];
	int lastnote[MAXVOICES];
	int prevnote[MAXVOICES];
	int i, p;
	int j;
	register int k;
	int sel;
	int dracula[MAXVOICES];
	long duration; 
	long count;
	char ms[255];


	sel = pCB->GetCurSel();

	if( sel!=LB_ERR )
		j = sel;
	else
		j = 0;

	for (i=VOICE1; i<MAXVOICES; i++)
	{
		lastnote[i] = -1;
		prevnote[i] = -1;
	}

		BOOL firstime = TRUE;



	while ((j<=pDoc->m_NumEvents)&&m_Playing)
	{

		for (i=VOICE1; i<MAXVOICES; i++) 
		{
			count1[i] = 0;
			count2[i] = 0;
			count3[i] = 0;
			count4[i] = 0;
			count5[i] = 0;
			count6[i] = 0;
			count7[i] = 0;
			count8[i] = 0;
			count9[i] = 0;
			count10[i] = 0;
			count11[i] =0;
			count12[i] =0;
			EarthWorm[i] = 0;
		}


		duration = 60000/(8*pDoc->Event[j].Tempo);

 		if (firstime)
		{
			if ( (pView->m_StartFrom>=pDoc->Event[j].count)&&(pView->m_StartFrom<=pDoc->Event[j].genCount) )
				count = pView->m_StartFrom;
			else
				count = pDoc->Event[j].count;
		}
		else
			count = pDoc->Event[j].count;

		pCB->SetCurSel(j);

		pEdit = (CEdit*)(pView->GetDlgItem(IDC_GENCOUNTER_ED));
		ltoa(pDoc->Event[j].genCount, ms, 10);
		pEdit->SetWindowText(ms);

		pEdit = (CEdit*)(pView->GetDlgItem(IDC_STARTCOUNTER_ED));
		ltoa(pDoc->Event[j].count, ms, 10);
		pEdit->SetWindowText(ms);

		pView->GeneralCounter = pDoc->dGeneralCounter = pDoc->Event[j].genCount;
		pView->StartCounter = pDoc->dStartCounter = pDoc->Event[j].count;



		/////////////////////////////////////////////////

		if (pView->UpdateSettings)
		{

			CComboBox* pB;
			CScrollBar* pS;
			CStatic* pSt = (CStatic*)(pView->GetDlgItem(IDC_TEMPO_STATIC));


			Tempo = pView->m_TempoMarking = pDoc->m_TempoMarking = pDoc->Event[j].Tempo;
			pView->GeneralCounter = pDoc->dGeneralCounter = pDoc->Event[j].genCount;
			pView->StartCounter = pDoc->dStartCounter = pDoc->Event[j].count;
			pS = (CScrollBar*)(pView->GetDlgItem(IDC_TEMPO_SLIDER2));
			pS->SetScrollPos(Tempo);



			char s[10];

			itoa(Tempo, s, 10);
			pSt->SetWindowText(s);

			pS->EnableWindow(FALSE);


			pDoc->m_TimeSigNum = pDoc->Event[j].TimeSigNum;
			pDoc->m_TimeSigDen = pDoc->Event[j].TimeSigDen;
			pB = (CComboBox*)(pView->GetDlgItem(IDC_TIMESIGNUM_CMB));
			pB->SetCurSel((pDoc->m_TimeSigNum)-1);
			pB = (CComboBox*)(pView->GetDlgItem(IDC_TIMESIGDEN_CMB));
			pB->SetCurSel(int(log10(double(pDoc->m_TimeSigDen))/log10((double)2)));


			for(p = VOICE1; p<MAXVOICES; p++)
			{
				pDoc->AlgorithmNum[p] = pDoc->Event[j].AlgorithmNum[p];
				pDoc->m_IsActive[p] = pView->m_IsActive[p] = pDoc->Event[j].IsActive[p];
				pDoc->m_Base[p] = pDoc->Event[j].Base[p];
				pDoc->m_Multi[p] = pDoc->Event[j].Multi[p];
				pDoc->m_StartNum[p] = pView->m_StartNum[p] = pDoc->Event[j].StartNum[p];
				pDoc->m_NumOcts[p] = pDoc->Event[j].NumOcts[p];
				pDoc->m_Modulates[p] = pView->m_Modulates[p] = pDoc->Event[j].Modulates[p] ;
				pDoc->m_ModulRandom[p] = pView->m_V1Modul[p].m_V1ModulDlgChoice = pDoc->Event[j].ModulRandom[p] ;
				pDoc->m_ModulNum[p] = pView->m_V1Modul[p].m_V1ModulDlgNum = pDoc->Event[j].ModulNum[p] ;
				pDoc->m_Note[p] = pView->m_Note[p] = pDoc->Event[j].Note[p] ;
				pDoc->m_NoteValue[p] = pDoc->Event[j].NoteValue[p] ;
				pDoc->m_Instrument[p] = pView->m_V1MidiSet[p].m_instrument  = pDoc->Event[j].Instrument[p];
				pDoc->m_Channel[p] = pDoc->Event[j].Channel[p];
				pDoc->m_mode[p] = pView->m_V1MidiSet[p].m_mode = pDoc->Event[j].Mode[p];
				pDoc->m_ScaleID[p] = pView->m_ScaleID[p] = pDoc->Event[j].ScaleID[p];
				pDoc->m_NumOctsID[p] = pView->m_NumOctsID[p] = pDoc->Event[j].NumOctsID[p];
				pDoc->m_NoteValueID[p] = pView->m_NoteValueID[p] = pDoc->Event[j].NoteValueID[p];
				pDoc->m_Inverted[p] = pDoc->Event[j].Inverted[p];
				pDoc->m_MatchScale[p] = pDoc->Event[j].MatchScale[p];
				pDoc->m_Dynamic[p] = pDoc->Event[j].Dynamic[p];
				pDoc->m_DynChoice[p] = pDoc->Event[j].DynChoice[p];
				pDoc->m_LogMapVar[p] = pDoc->Event[j].LogMapVar[p];
				pDoc->m_FNoiseVar[p] = pDoc->Event[j].FNoiseVar[p];
				pDoc->m_LorentzVar[p] = pDoc->Event[j].LorentzVar[p];
				pDoc->m_HenonA[p] = pDoc->Event[j].HenonA[p];
				pDoc->m_HenonB[p] = pDoc->Event[j].HenonB[p];
				pDoc->m_HopalongVar[p] = pDoc->Event[j].HopalongVar[p];
				pDoc->m_MartinVar[p] = pDoc->Event[j].MartinVar[p];
				pDoc->m_GingerX[p] = pDoc->Event[j].GingerX[p];
				pDoc->m_GingerY[p] = pDoc->Event[j].GingerY[p];
				pDoc->m_EarthWormVar[p] = pDoc->Event[j].EarthWormVar[p];
				pDoc->m_EarthWormMulti[p] = pDoc->Event[j].EarthWormMulti[p];
				pDoc->m_EarthWormTruncator[p] = pDoc->Event[j].EarthWormTruncator[p];
				pDoc->m_RandomMin[p] = pDoc->Event[j].RandomMin[p];
				pDoc->m_RandomMax[p] = pDoc->Event[j].RandomMax[p];
				pDoc->m_WolframRule[p] = pDoc->Event[j].WolframRule[p];
				pDoc->m_WolframInit[p] = pDoc->Event[j].WolframInit[p];
				pDoc->m_WolframSize[p] = pDoc->Event[j].WolframSize[p];
				pDoc->m_Hailstone[p] = pDoc->Event[j].Hailstone[p];

				pDoc->m_Pan[p] = pDoc->Event[j].Pan[p];
				pDoc->m_Percussion[p] = pDoc->Event[j].Percussion[p];

				pButt = (CButton*)(pView->GetDlgItem(pView->ActiveID[p]));

				if (pDoc->m_Channel[p] == PercussionChannel)
				{
					pButt->SetWindowText(PercussionNames[ pDoc->m_Percussion[p] ]);
				}
				else
					pButt->SetWindowText(InstrumentNames[ pDoc->m_Instrument[p] ]);

			}

			pView->UpdateData(FALSE);

			for(p = VOICE1; p<MAXVOICES; p++)
			{

				pB = (CComboBox*)(pView->GetDlgItem(pView->NumOctsID[p]));
				pB->SetCurSel(pView->m_NumOctsID[p]);
				pB = (CComboBox*)(pView->GetDlgItem(pView->NoteID[p]));
				pB->SetCurSel(pView->m_Note[p]);
				pB = (CComboBox*)(pView->GetDlgItem(pView->ScaleID[p]));
				pB->SetCurSel(pView->m_ScaleID[p]);
				pB = (CComboBox*)(pView->GetDlgItem(pView->NoteValueID[p]));
				pB->SetCurSel(pView->m_NoteValueID[p]);
			}


			for (p = VOICE1; p<MAXVOICES; p++)
			{
				CString nval;

				if (pView->m_NoteValueID[p] == OTHER)
				{
					nval.Format("Other (%d)", (long)pDoc->m_NoteValue[p]);
					pB = (CComboBox*)(pView->GetDlgItem(pView->NoteValueID[p]));
					pB->DeleteString(OTHER);
					pB->AddString(nval);
					pB->SetCurSel(OTHER);
				}
			}



			for(p = VOICE1; p<MAXVOICES; p++)
			{
				if (pDoc->m_pUserScale[p])
				{
					delete pDoc->m_pUserScale[p];
					pDoc->m_pUserScale[p] = NULL;
				}
				pDoc->m_pUserScale[p] = new int[pDoc->Event[j].pUserScale[p][0] + 1];
				pDoc->m_pUserScale[p][0] = pDoc->Event[j].pUserScale[p][0];
				for(int h = 1; h<=pDoc->Event[j].pUserScale[p][0]; h++)
					pDoc->m_pUserScale[p][h] = pDoc->Event[j].pUserScale[p][h];
			}


			for(p = VOICE1; p<MAXVOICES; p++)
			{
				if (pDoc->m_pScale[p])
				{
					delete pDoc->m_pScale[p];
					pDoc->m_pScale[p] = NULL;
				}
				pDoc->m_pScale[p] = new int[pDoc->Event[j].pScale[p][0] + 1];
				pDoc->m_pScale[p][0] = pDoc->Event[j].pScale[p][0];
				for(int h = 1; h<=pDoc->Event[j].pScale[p][0]; h++)
					pDoc->m_pScale[p][h] = pDoc->Event[j].pScale[p][h];
				if (pDoc->m_ScaleID[p] == USER_DEFINED)
				{
					pView->InitUserScale(p);
					pView->SetUserScale(p);
				}
			}

			pB = (CComboBox*)pView->GetDlgItem(IDC_PLAYFROM_CMB);
			pB->SetCurSel(j);


		}




		////////////////////////////////////////////////



		for (i=VOICE1; i<MAXVOICES; i++)
			dracula[i] = int(pDoc->Event[j].NoteValue[i]);


		for (i=VOICE1; i<MAXVOICES; i++)
		{
			if (pDoc->Event[j].IsActive[i])
			{
			int v = pDoc->Event[j].Channel[i]==PercussionChannel? PercussionChannel : i;
			progChange(0xC0+v, (BYTE)pDoc->Event[j].Instrument[i]);
			}
		}

		for (i=VOICE1; i<MAXVOICES; i++)
		{
			if (pDoc->Event[j].IsActive[i])
			{
			int v = pDoc->Event[j].Channel[i]==PercussionChannel? PercussionChannel : i;
			modeChange(0xb0+/*pDoc->Event[j].Channel[i]*/v, (BYTE)0x0a, 64+(pDoc->Event[j].Pan[i]));
			}
		}

				for(i = VOICE1; i<MAXVOICES; i++)
				{
					if(j>0)
					{
						if (!pDoc->Event[j].IsActive[i]&&(pDoc->Event[j-1].IsActive[i]))
						{
							int v = pDoc->Event[j].Channel[i] == PercussionChannel? PercussionChannel : i;

							modeChange(0xb0+v, (BYTE)0x7b, 0);

						}
						if((pDoc->Event[j-1].IsActive[i])&&(pDoc->Event[j].Channel[i]!=pDoc->Event[j-1].Channel[i]))
						{
							int v = pDoc->Event[j-1].Channel[i] == PercussionChannel? PercussionChannel : i;
							modeChange(0xb0+v, (BYTE)0x7b, 0);

						}
					}
				}


		for (i=VOICE1; i<MAXVOICES; i++)
		{


			if(pDoc->Event[j].AlgorithmNum[i]==MORSE_THUE)
			{

				for(k = 0; k<count+((pDoc->Event[j].StartNum[i])*dracula[i]); k++)
				{

					if(!((((pDoc->Event[j].StartNum[i])*dracula[i])+k)%dracula[i]))
					{

					test[i] = ((((pDoc->Event[j].StartNum[i])*dracula[i])+k)/dracula[i])*pDoc->Event[j].Multi[i];

					(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, (pDoc->Event[j].StartNum[i]*dracula[i]+k)/dracula[i], test[i],
										pDoc->Event[j].pScale[i], 
										pDoc->Event[j].NumOcts[i], 
										pDoc->Event[j].Base[i], 
										pDoc->Event[j].StartNum[i],
										pDoc->Event[j].Note[i], 
										pDoc->Event[j].Modulates[i],
										pDoc->Event[j].ModulRandom[i], 
										pDoc->Event[j].ModulNum[i],
										pDoc->Event[j].Inverted[i],
										pDoc->Event[j].LogMapVar[i],
										pDoc->Event[j].FNoiseVar[i],
										pDoc->Event[j].HenonA[i],
										pDoc->Event[j].HenonB[i],
										pDoc->Event[j].HopalongVar[i],
										pDoc->Event[j].MartinVar[i],
										pDoc->Event[j].LorentzVar[i],
										pDoc->Event[j].EarthWormVar[i],
										pDoc->Event[j].EarthWormMulti[i],
										pDoc->Event[j].EarthWormTruncator[i],
										pDoc->Event[j].RandomMin[i],
										pDoc->Event[j].RandomMax[i],
										pDoc->Event[j].WolframRule[i],
										pDoc->Event[j].WolframSize[i],
										pDoc->Event[j].Hailstone[i]



						);
					}
				}
			}

			if(pDoc->Event[j].AlgorithmNum[i]==LOGMAP)
			{
				LogMap[i] = 0.2f;

				for(k = 0; k<count+((pDoc->Event[j].StartNum[i])*dracula[i]); k++)
				{

					if(!((((pDoc->Event[j].StartNum[i])*dracula[i])+k)%dracula[i]))
					{

					(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, (pDoc->Event[j].StartNum[i]*dracula[i]+k)/dracula[i], 0,
										pDoc->Event[j].pScale[i], 
										pDoc->Event[j].NumOcts[i], 
										pDoc->Event[j].Base[i], 
										pDoc->Event[j].StartNum[i],
										pDoc->Event[j].Note[i], 
										pDoc->Event[j].Modulates[i],
										pDoc->Event[j].ModulRandom[i], 
										pDoc->Event[j].ModulNum[i],
										pDoc->Event[j].Inverted[i],
										pDoc->Event[j].LogMapVar[i],
										pDoc->Event[j].FNoiseVar[i],
										pDoc->Event[j].HenonA[i],
										pDoc->Event[j].HenonB[i],
										pDoc->Event[j].HopalongVar[i],
										pDoc->Event[j].MartinVar[i],
										pDoc->Event[j].LorentzVar[i],
										pDoc->Event[j].EarthWormVar[i],
										pDoc->Event[j].EarthWormMulti[i],
										pDoc->Event[j].EarthWormTruncator[i],
										pDoc->Event[j].RandomMin[i],
										pDoc->Event[j].RandomMax[i],
										pDoc->Event[j].WolframRule[i],
										pDoc->Event[j].WolframSize[i],
										pDoc->Event[j].Hailstone[i]



						);
					}
				}
			}

			if(pDoc->Event[j].AlgorithmNum[i]==ONE_OVER_F_NOISE)
			{
				LogMap2[i] = 0.2f;
				FNoise[i] = 0.0f;

				for(k = 0; k<count+((pDoc->Event[j].StartNum[i])*dracula[i]); k++)
				{

					if(!((((pDoc->Event[j].StartNum[i])*dracula[i])+k)%dracula[i]))
					{

					(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, (pDoc->Event[j].StartNum[i]*dracula[i]+k)/dracula[i], 0,
										pDoc->Event[j].pScale[i], 
										pDoc->Event[j].NumOcts[i], 
										pDoc->Event[j].Base[i], 
										pDoc->Event[j].StartNum[i],
										pDoc->Event[j].Note[i], 
										pDoc->Event[j].Modulates[i],
										pDoc->Event[j].ModulRandom[i], 
										pDoc->Event[j].ModulNum[i],
										pDoc->Event[j].Inverted[i],
										pDoc->Event[j].LogMapVar[i],
										pDoc->Event[j].FNoiseVar[i],
										pDoc->Event[j].HenonA[i],
										pDoc->Event[j].HenonB[i],
										pDoc->Event[j].HopalongVar[i],
										pDoc->Event[j].MartinVar[i],
										pDoc->Event[j].LorentzVar[i],
										pDoc->Event[j].EarthWormVar[i],
										pDoc->Event[j].EarthWormMulti[i],
										pDoc->Event[j].EarthWormTruncator[i],
										pDoc->Event[j].RandomMin[i],
										pDoc->Event[j].RandomMax[i],
										pDoc->Event[j].WolframRule[i],
										pDoc->Event[j].WolframSize[i],
										pDoc->Event[j].Hailstone[i]


						);
					}
				}
			}

			if(pDoc->Event[j].AlgorithmNum[i]==HENON)
			{
				HenonX[i] = HenonY[i] = 0.0f;

				for(k = 0; k<count+((pDoc->Event[j].StartNum[i])*dracula[i]); k++)
				{

					if(!((((pDoc->Event[j].StartNum[i])*dracula[i])+k)%dracula[i]))
					{

					(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, k, 0,
										pDoc->Event[j].pScale[i], 
										pDoc->Event[j].NumOcts[i], 
										pDoc->Event[j].Base[i], 
										pDoc->Event[j].StartNum[i],
										pDoc->Event[j].Note[i], 
										pDoc->Event[j].Modulates[i],
										pDoc->Event[j].ModulRandom[i], 
										pDoc->Event[j].ModulNum[i],
										pDoc->Event[j].Inverted[i],
										pDoc->Event[j].LogMapVar[i],
										pDoc->Event[j].FNoiseVar[i],
										pDoc->Event[j].HenonA[i],
										pDoc->Event[j].HenonB[i],
										pDoc->Event[j].HopalongVar[i],
										pDoc->Event[j].MartinVar[i],
										pDoc->Event[j].LorentzVar[i],
										pDoc->Event[j].EarthWormVar[i],
										pDoc->Event[j].EarthWormMulti[i],
										pDoc->Event[j].EarthWormTruncator[i],
										pDoc->Event[j].RandomMin[i],
										pDoc->Event[j].RandomMax[i],
										pDoc->Event[j].WolframRule[i],
										pDoc->Event[j].WolframSize[i],
										pDoc->Event[j].Hailstone[i]


						);
					}
				}
			}

			if(pDoc->Event[j].AlgorithmNum[i]==HOPALONG)
			{
				HopalongX[i] = HopalongY[i] = 0.0f;

				for(k = 0; k<count+((pDoc->Event[j].StartNum[i])*dracula[i]); k++)
				{

					if(!((((pDoc->Event[j].StartNum[i])*dracula[i])+k)%dracula[i]))
					{

					(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, (pDoc->Event[j].StartNum[i]*dracula[i]+k)/dracula[i], 0,
										pDoc->Event[j].pScale[i], 
										pDoc->Event[j].NumOcts[i], 
										pDoc->Event[j].Base[i], 
										pDoc->Event[j].StartNum[i],
										pDoc->Event[j].Note[i], 
										pDoc->Event[j].Modulates[i],
										pDoc->Event[j].ModulRandom[i], 
										pDoc->Event[j].ModulNum[i],
										pDoc->Event[j].Inverted[i],
										pDoc->Event[j].LogMapVar[i],
										pDoc->Event[j].FNoiseVar[i],
										pDoc->Event[j].HenonA[i],
										pDoc->Event[j].HenonB[i],
										pDoc->Event[j].HopalongVar[i],
										pDoc->Event[j].MartinVar[i],
										pDoc->Event[j].LorentzVar[i],
										pDoc->Event[j].EarthWormVar[i],
										pDoc->Event[j].EarthWormMulti[i],
										pDoc->Event[j].EarthWormTruncator[i],
										pDoc->Event[j].RandomMin[i],
										pDoc->Event[j].RandomMax[i],
										pDoc->Event[j].WolframRule[i],
										pDoc->Event[j].WolframSize[i],
										pDoc->Event[j].Hailstone[i]


						);
					}
				}
			}


			if(pDoc->Event[j].AlgorithmNum[i]==MARTIN)
			{
				MartinX[i] = MartinY[i] = 0.0f;

				for(k = 0; k<count+((pDoc->Event[j].StartNum[i])*dracula[i]); k++)
				{

					if(!((((pDoc->Event[j].StartNum[i])*dracula[i])+k)%dracula[i]))
					{

					(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, (pDoc->Event[j].StartNum[i]*dracula[i]+k)/dracula[i], 0,
										pDoc->Event[j].pScale[i], 
										pDoc->Event[j].NumOcts[i], 
										pDoc->Event[j].Base[i], 
										pDoc->Event[j].StartNum[i],
										pDoc->Event[j].Note[i], 
										pDoc->Event[j].Modulates[i],
										pDoc->Event[j].ModulRandom[i], 
										pDoc->Event[j].ModulNum[i],
										pDoc->Event[j].Inverted[i],
										pDoc->Event[j].LogMapVar[i],
										pDoc->Event[j].FNoiseVar[i],
										pDoc->Event[j].HenonA[i],
										pDoc->Event[j].HenonB[i],
										pDoc->Event[j].HopalongVar[i],
										pDoc->Event[j].MartinVar[i],
										pDoc->Event[j].LorentzVar[i],
										pDoc->Event[j].EarthWormVar[i],
										pDoc->Event[j].EarthWormMulti[i],
										pDoc->Event[j].EarthWormTruncator[i],
										pDoc->Event[j].RandomMin[i],
										pDoc->Event[j].RandomMax[i],
										pDoc->Event[j].WolframRule[i],
										pDoc->Event[j].WolframSize[i],
										pDoc->Event[j].Hailstone[i]


						);
					}
				}
			}

			if(pDoc->Event[j].AlgorithmNum[i]==GINGERBREADMAN)
			{
				GingerX[i] = pDoc->Event[j].GingerX[i];
				GingerY[i] = pDoc->Event[j].GingerY[i];

				for(k = 0; k<count+((pDoc->Event[j].StartNum[i])*dracula[i]); k++)
				{

					if(!((((pDoc->Event[j].StartNum[i])*dracula[i])+k)%dracula[i]))
					{

					(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, (pDoc->Event[j].StartNum[i]*dracula[i]+k)/dracula[i], 0,
										pDoc->Event[j].pScale[i], 
										pDoc->Event[j].NumOcts[i], 
										pDoc->Event[j].Base[i], 
										pDoc->Event[j].StartNum[i],
										pDoc->Event[j].Note[i], 
										pDoc->Event[j].Modulates[i],
										pDoc->Event[j].ModulRandom[i], 
										pDoc->Event[j].ModulNum[i],
										pDoc->Event[j].Inverted[i],
										pDoc->Event[j].LogMapVar[i],
										pDoc->Event[j].FNoiseVar[i],
										pDoc->Event[j].HenonA[i],
										pDoc->Event[j].HenonB[i],
										pDoc->Event[j].HopalongVar[i],
										pDoc->Event[j].MartinVar[i],
										pDoc->Event[j].LorentzVar[i],
										pDoc->Event[j].EarthWormVar[i],
										pDoc->Event[j].EarthWormMulti[i],
										pDoc->Event[j].EarthWormTruncator[i],
										pDoc->Event[j].RandomMin[i],
										pDoc->Event[j].RandomMax[i],
										pDoc->Event[j].WolframRule[i],
										pDoc->Event[j].WolframSize[i],
										pDoc->Event[j].Hailstone[i]


						);
					}
				}
			}


			if(pDoc->Event[j].AlgorithmNum[i]==LORENTZ)
			{
				LorentzX[i] = 1.0f;

				for(k = 0; k<count+((pDoc->Event[j].StartNum[i])*dracula[i]); k++)
				{

					if(!((((pDoc->Event[j].StartNum[i])*dracula[i])+k)%dracula[i]))
					{

					(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, (pDoc->Event[j].StartNum[i]*dracula[i]+k)/dracula[i], 0,
										pDoc->Event[j].pScale[i], 
										pDoc->Event[j].NumOcts[i], 
										pDoc->Event[j].Base[i], 
										pDoc->Event[j].StartNum[i],
										pDoc->Event[j].Note[i], 
										pDoc->Event[j].Modulates[i],
										pDoc->Event[j].ModulRandom[i], 
										pDoc->Event[j].ModulNum[i],
										pDoc->Event[j].Inverted[i],
										pDoc->Event[j].LogMapVar[i],
										pDoc->Event[j].FNoiseVar[i],
										pDoc->Event[j].HenonA[i],
										pDoc->Event[j].HenonB[i],
										pDoc->Event[j].HopalongVar[i],
										pDoc->Event[j].MartinVar[i],
										pDoc->Event[j].LorentzVar[i],
										pDoc->Event[j].EarthWormVar[i],
										pDoc->Event[j].EarthWormMulti[i],
										pDoc->Event[j].EarthWormTruncator[i],
										pDoc->Event[j].RandomMin[i],
										pDoc->Event[j].RandomMax[i],
										pDoc->Event[j].WolframRule[i],
										pDoc->Event[j].WolframSize[i],
										pDoc->Event[j].Hailstone[i]


						);
					}
				}
			}



			if(pDoc->Event[j].AlgorithmNum[i]==EARTHWORM)
			{
				EarthWorm[i] = pDoc->Event[j].EarthWormVar[i];

				for(k = 0; k<count+((pDoc->Event[j].StartNum[i])*dracula[i]); k++)
				{

					if(!((((pDoc->Event[j].StartNum[i])*dracula[i])+k)%dracula[i]))
					{

					(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, (pDoc->Event[j].StartNum[i]*dracula[i]+k)/dracula[i], 0,
										pDoc->Event[j].pScale[i], 
										pDoc->Event[j].NumOcts[i], 
										pDoc->Event[j].Base[i], 
										pDoc->Event[j].StartNum[i],
										pDoc->Event[j].Note[i], 
										pDoc->Event[j].Modulates[i],
										pDoc->Event[j].ModulRandom[i], 
										pDoc->Event[j].ModulNum[i],
										pDoc->Event[j].Inverted[i],
										pDoc->Event[j].LogMapVar[i],
										pDoc->Event[j].FNoiseVar[i],
										pDoc->Event[j].HenonA[i],
										pDoc->Event[j].HenonB[i],
										pDoc->Event[j].HopalongVar[i],
										pDoc->Event[j].MartinVar[i],
										pDoc->Event[j].LorentzVar[i],
										pDoc->Event[j].EarthWormVar[i],
										pDoc->Event[j].EarthWormMulti[i],
										pDoc->Event[j].EarthWormTruncator[i],
										pDoc->Event[j].RandomMin[i],
										pDoc->Event[j].RandomMax[i],
										pDoc->Event[j].WolframRule[i],
										pDoc->Event[j].WolframSize[i],
										pDoc->Event[j].Hailstone[i]


						);
					}
				}
			}


						if(pDoc->Event[j].AlgorithmNum[i]==HAILSTONE)
						{
							HailStoneNumber[i] = pDoc->Event[j].Hailstone[i];

							for(k = 0; k<count+(pDoc->Event[j].StartNum[i]*dracula[i]); k++)
							{

								if(!(((pDoc->Event[j].StartNum[i]*dracula[i])+k)%dracula[i]))
								{

								(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, ((pDoc->Event[j].StartNum[i]*dracula[i])+k)/dracula[i], 0,
													pDoc->Event[j].pScale[i], 
													pDoc->Event[j].NumOcts[i], 
													pDoc->Event[j].Base[i], 
													pDoc->Event[j].StartNum[i],
													pDoc->Event[j].Note[i], 
													pDoc->Event[j].Modulates[i],
													pDoc->Event[j].ModulRandom[i], 
													pDoc->Event[j].ModulNum[i],
													pDoc->Event[j].Inverted[i],
													pDoc->Event[j].LogMapVar[i],
													pDoc->Event[j].FNoiseVar[i],
													pDoc->Event[j].HenonA[i],
													pDoc->Event[j].HenonB[i],
													pDoc->Event[j].HopalongVar[i],
													pDoc->Event[j].MartinVar[i],
													pDoc->Event[j].LorentzVar[i],
													pDoc->Event[j].EarthWormVar[i],
													pDoc->Event[j].EarthWormMulti[i],
													pDoc->Event[j].EarthWormTruncator[i],
													pDoc->Event[j].RandomMin[i],
													pDoc->Event[j].RandomMax[i],
													pDoc->Event[j].WolframRule[i],
													pDoc->Event[j].WolframSize[i],
													pDoc->Event[j].Hailstone[i]


									);

								}
							}
						}


						if(pDoc->Event[j].AlgorithmNum[i]==WOLFRAM)
						{
							if (WolframArray1[i])
							{
								delete WolframArray1[i];
								WolframArray1[i] = NULL;
							}
							if (WolframArray2[i])
							{
								delete WolframArray2[i];
								WolframArray2[i] = NULL;
							}

							WolframArray1[i] = new BYTE[pDoc->Event[j].WolframSize[i]];
							WolframArray2[i] = new BYTE[pDoc->Event[j].WolframSize[i]];

							for (int idx = 0; idx<32; idx++)
									WolframArray1[i][idx] = pDoc->Event[j].WolframInit[i] & unsigned long (pow(2, idx)) ? 1 : 0;

							if (pDoc->Event[j].WolframSize[i]>32)
							{
								for (idx = 32; idx<pDoc->Event[j].WolframSize[i]; idx++)
									WolframArray1[i][idx] = 0;
							}

							for(k = 0; k<count+(pDoc->Event[j].StartNum[i]*dracula[i]); k++)
							{

								if(!(((pDoc->Event[j].StartNum[i]*dracula[i])+k)%dracula[i]))
								{

								(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, ((pDoc->Event[j].StartNum[i]*dracula[i])+k)/dracula[i], 0,
													pDoc->Event[j].pScale[i], 
													pDoc->Event[j].NumOcts[i], 
													pDoc->Event[j].Base[i], 
													pDoc->Event[j].StartNum[i],
													pDoc->Event[j].Note[i], 
													pDoc->Event[j].Modulates[i],
													pDoc->Event[j].ModulRandom[i], 
													pDoc->Event[j].ModulNum[i],
													pDoc->Event[j].Inverted[i],
													pDoc->Event[j].LogMapVar[i],
													pDoc->Event[j].FNoiseVar[i],
													pDoc->Event[j].HenonA[i],
													pDoc->Event[j].HenonB[i],
													pDoc->Event[j].HopalongVar[i],
													pDoc->Event[j].MartinVar[i],
													pDoc->Event[j].LorentzVar[i],
													pDoc->Event[j].EarthWormVar[i],
													pDoc->Event[j].EarthWormMulti[i],
													pDoc->Event[j].EarthWormTruncator[i],
													pDoc->Event[j].RandomMin[i],
													pDoc->Event[j].RandomMax[i],
													pDoc->Event[j].WolframRule[i],
													pDoc->Event[j].WolframSize[i],
													pDoc->Event[j].Hailstone[i]


									);

								}
							}
						}





		}



		while(m_Playing&&(count<pDoc->Event[j].genCount))
		{
							if (!pDoc->Event[j].IsActive[i])
								lastnote[i] = -1;

			if (pDoc->Event[j].IsActive[i])
			{

			for (i=VOICE1; i<MAXVOICES; i++)
			{

				if(((pDoc->Event[j].StartNum[i]*dracula[i])+count)>=0)
				{

					if(!((((pDoc->Event[j].StartNum[i])*dracula[i])+count)%dracula[i]))
					{
						test[i] = ((((pDoc->Event[j].StartNum[i])*dracula[i])+count)/dracula[i])*pDoc->Event[j].Multi[i];
						note[i] = (*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, ((pDoc->Event[j].StartNum[i]*dracula[i])+count)/dracula[i], test[i],
											pDoc->Event[j].pScale[i], 
											pDoc->Event[j].NumOcts[i], 
											pDoc->Event[j].Base[i], 
											pDoc->Event[j].StartNum[i],
											pDoc->Event[j].Note[i], 
											pDoc->Event[j].Modulates[i],
											pDoc->Event[j].ModulRandom[i], 
											pDoc->Event[j].ModulNum[i],
											pDoc->Event[j].Inverted[i],
											pDoc->Event[j].LogMapVar[i],
											pDoc->Event[j].FNoiseVar[i],
											pDoc->Event[j].HenonA[i],
											pDoc->Event[j].HenonB[i],
											pDoc->Event[j].HopalongVar[i],
											pDoc->Event[j].MartinVar[i],
											pDoc->Event[j].LorentzVar[i],
											pDoc->Event[j].EarthWormVar[i],
											pDoc->Event[j].EarthWormMulti[i],
											pDoc->Event[j].EarthWormTruncator[i],
											pDoc->Event[j].RandomMin[i],
											pDoc->Event[j].RandomMax[i],
											pDoc->Event[j].WolframRule[i],
											pDoc->Event[j].WolframSize[i],
											pDoc->Event[j].Hailstone[i]

							);	
					}

				}
			}


			for (i=VOICE1; i<MAXVOICES; i++)
			{

				/*if (!pDoc->Event[j].IsActive[i])
				{

					modeChange(0xb0+i, (BYTE)0x7b, 0);
				}*/

				if(((pDoc->Event[j].StartNum[i]*dracula[i])+count)>=0)
				{

					if(!(((pDoc->Event[j].StartNum[i]*dracula[i])+count)%dracula[i]))
					{
						if ((lastnote[i]!=note[i])||(!pDoc->Event[j].IsActive[i]))
						{
							noteOff(0x80+/*pDoc->Event[j].Channel[i]*/i, (BYTE)prevnote[i], (BYTE)0);
							if (pDoc->Event[j].Channel[i] == PercussionChannel)
								noteOn(0x90+/*pDoc->Event[j].Channel[i]*/PercussionChannel, (BYTE)pDoc->Event[j].Percussion[i]+35, (BYTE)(pDoc->Event[j].Dynamic[i]*pDoc->Event[j].IsActive[i]));
							else
								noteOn(0x90+/*pDoc->Event[j].Channel[i]*/i, (BYTE)note[i], (BYTE)(pDoc->Event[j].Dynamic[i]*pDoc->Event[j].IsActive[i]));
							lastnote[i] = pDoc->Event[j].Mode[i]? note[i] : -1;
							prevnote[i] = pDoc->Event[j].Channel[i] == PercussionChannel? pDoc->Event[j].Percussion[i]+35 : note[i];
						}

					}

				}
				//// else
			}
			}

			count++;
			pView->m_StartFrom = count;
			pEdit = (CEdit*)(pView->GetDlgItem(IDC_STARTFROM_BT));
			ltoa(pView->m_StartFrom, ms, 10);
			pEdit->SetWindowText(ms);
			Sleep(duration);
		}

		firstime=FALSE;


		++j;

	}


	for (i=VOICE1; i<MAXVOICES; i++)
		modeChange(0xB0+i, 0x7b, 0);


	pView->OnPlayStop();

	pView->GetDlgItem(IDC_LOCK_GENCOUNTER)->EnableWindow(TRUE);
	pView->GetDlgItem(IDC_GENCOUNTER_ED)->EnableWindow(!pView->LockGeneralCounter);
	pView->GetDlgItem(IDC_STARTCOUNTER_ED)->EnableWindow(TRUE);
	pView->GetDlgItem(IDC_RESET_GENCOUNTER)->EnableWindow(TRUE);
	pView->GetDlgItem(IDC_ROUND)->EnableWindow(!pView->LockGeneralCounter);


	return 0;
}
void CFractMusView::OnOptsEvents() 
{
	if (m_pEvents == NULL)
	{
		m_pEvents = new CEventsDlg(this);
		m_pEvents->Create();
	}
	
}

void CFractMusView::OnUpdateOptsEvents(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable((!m_pEvents));
	
}

void CFractMusView::OnResetGencounter() 
{
	CFractMusDoc* pDoc = GetDocument();

	if (!LockGeneralCounter)
		pDoc->dGeneralCounter = GeneralCounter = 0L;
	pDoc->dStartCounter = StartCounter = 0L;

	UpdateData(FALSE);

	pDoc->SetModifiedFlag();
	
}

void CFractMusView::OnKillfocusGencounterEd() 
{
	if(!UpdateData(TRUE))
		return;
	CFractMusDoc* pDoc = GetDocument();
	pDoc->dGeneralCounter = GeneralCounter;
	pDoc->SetModifiedFlag();
}


void CFractMusView::OnLockGencounter() 
{
	LockGeneralCounter = !LockGeneralCounter;

	CString s;
	s = LockGeneralCounter? "Unlock" : "Lock";

	GetDlgItem(IDC_LOCK_GENCOUNTER)->SetWindowText(s);
	GetDlgItem(IDC_GENCOUNTER_ED)->EnableWindow(!LockGeneralCounter);
	GetDlgItem(IDC_ROUND)->EnableWindow(!LockGeneralCounter);
}

void CFractMusView::OnAlgorithmsVoice1Morsethue() 
{
	UpdateAlgorithm(VOICE1, MORSE_THUE);
}

void CFractMusView::OnUpdateAlgorithmsVoice1Morsethue(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE1] == MORSE_THUE);
	pCmdUI->Enable(!m_Playing);
}

void CFractMusView::OnAlgorithmsVoice2Morsethue() 
{
	UpdateAlgorithm(VOICE2, MORSE_THUE);
}

void CFractMusView::OnUpdateAlgorithmsVoice2Morsethue(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE2] == MORSE_THUE);
	pCmdUI->Enable(!m_Playing);
}

void CFractMusView::OnAlgorithmsVoice3Morsethue() 
{
	UpdateAlgorithm(VOICE3, MORSE_THUE);
}

void CFractMusView::OnUpdateAlgorithmsVoice3Morsethue(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE3] == MORSE_THUE);
	pCmdUI->Enable(!m_Playing);
}

void CFractMusView::OnAlgorithmsVoice4Morsethue() 
{
	UpdateAlgorithm(VOICE4, MORSE_THUE);
}

void CFractMusView::OnUpdateAlgorithmsVoice4Morsethue(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE4] == MORSE_THUE);
	pCmdUI->Enable(!m_Playing);
}

void CFractMusView::OnAlgorithmsVoice5Morsethue() 
{
	UpdateAlgorithm(VOICE5, MORSE_THUE);
}

void CFractMusView::OnUpdateAlgorithmsVoice5Morsethue(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE5] == MORSE_THUE);
	pCmdUI->Enable(!m_Playing);
}

void CFractMusView::OnAlgorithmsVoice6Morsethue() 
{
	UpdateAlgorithm(VOICE6, MORSE_THUE);
}

void CFractMusView::OnUpdateAlgorithmsVoice6Morsethue(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE6] == MORSE_THUE);
	pCmdUI->Enable(!m_Playing);
}

void CFractMusView::OnAlgorithmsVoice7Morsethue() 
{
	UpdateAlgorithm(VOICE7, MORSE_THUE);
}

void CFractMusView::OnUpdateAlgorithmsVoice7Morsethue(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE7] == MORSE_THUE);
	pCmdUI->Enable(!m_Playing);
}

void CFractMusView::OnAlgorithmsVoice8Morsethue() 
{
	UpdateAlgorithm(VOICE8, MORSE_THUE);
}

void CFractMusView::OnUpdateAlgorithmsVoice8Morsethue(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE8] == MORSE_THUE);
	pCmdUI->Enable(!m_Playing);
}

void CFractMusView::UpdateAlgorithm(int voice, short alg)
{
	CFractMusDoc* pDoc = GetDocument();
	pDoc->AlgorithmNum[voice] = alg;

	if (alg == MORSE_THUE)
	{
		CMorseThueDlg dlg;
		CString Title;
		Title.Format("Morse-Thue Voice %d", voice+1);
		dlg.title = Title;
		dlg.m_base = pDoc->m_Base[voice];
		dlg.m_multi = pDoc->m_Multi[voice];
		dlg.pView = this;
		dlg.voicenum = voice;

		if(dlg.DoModal() == IDOK)
		{
			pDoc->m_Base[voice] = dlg.m_base;
			pDoc->m_Multi[voice] = dlg.m_multi;
		}
	}

	if (alg == LOGMAP)
	{
		CLogMapDlg dlg;
		CString Title;
		Title.Format("LogMap Voice %d", voice+1);
		dlg.title = Title;
		dlg.m_LogMapVar = pDoc->m_LogMapVar[voice];
		dlg.pView = this;
		dlg.voicenum = voice;

		if(dlg.DoModal() == IDOK)
		{
			pDoc->m_LogMapVar[voice] = dlg.m_LogMapVar;
		}
	}

	if (alg == ONE_OVER_F_NOISE)
	{
		CFNoiseDlg dlg;
		CString Title;
		Title.Format("1/f Voice %d", voice+1);
		dlg.title = Title;
		dlg.m_fnoisevar = pDoc->m_FNoiseVar[voice];
		dlg.pView = this;
		dlg.voicenum = voice;

		if(dlg.DoModal() == IDOK)
		{
			pDoc->m_FNoiseVar[voice] = dlg.m_fnoisevar;
		}
	}

	if (alg == HENON)
	{
		CHenonDlg dlg;
		CString Title;
		Title.Format("Henon Voice %d", voice+1);
		dlg.title = Title;
		dlg.m_henonA = pDoc->m_HenonA[voice];
		dlg.m_henonB = pDoc->m_HenonB[voice];
		dlg.pView = this;
		dlg.voicenum = voice;

		if(dlg.DoModal() == IDOK)
		{
			pDoc->m_HenonA[voice] = dlg.m_henonA;
			pDoc->m_HenonB[voice] = dlg.m_henonB;
		}
	}


	if (alg == HOPALONG)
	{
		CHopalongMartinDlg dlg;
		CString Title;
		Title.Format("Hopalong Voice %d", voice+1);
		dlg.title = Title;
		dlg.m_angle = pDoc->m_HopalongVar[voice];
		dlg.pView = this;
		dlg.voicenum = voice;

		if(dlg.DoModal() == IDOK)
		{
			pDoc->m_HopalongVar[voice] = dlg.m_angle;
		}
	}

	if (alg == MARTIN)
	{
		CMartinDlg dlg;
		CString Title;
		Title.Format("Martin Voice %d", voice+1);
		dlg.title = Title;
		dlg.m_angle = pDoc->m_MartinVar[voice];
		dlg.pView = this;
		dlg.voicenum = voice;


		if(dlg.DoModal() == IDOK)
		{
			pDoc->m_MartinVar[voice] = dlg.m_angle;
		}
	}

	if (alg == GINGERBREADMAN)
	{
		CGingerDlg dlg;
		CString Title;
		Title.Format("Ginger Voice %d", voice+1);
		dlg.title = Title;
		dlg.m_gingerx = pDoc->m_GingerX[voice];
		dlg.m_gingery = pDoc->m_GingerY[voice];
		dlg.pView = this;
		dlg.voicenum = voice;

		if(dlg.DoModal() == IDOK)
		{
			pDoc->m_GingerX[voice] = dlg.m_gingerx;
			pDoc->m_GingerY[voice] = dlg.m_gingery;
		}
	}


	if (alg == LORENTZ)
	{
		CLorentzDlg dlg;
		CString Title;
		Title.Format("Lorenz Voice %d", voice+1);
		dlg.title = Title;
		dlg.m_lorentz = pDoc->m_LorentzVar[voice];
		dlg.pView=this;
		dlg.voicenum = voice;


		if(dlg.DoModal() == IDOK)
		{
			pDoc->m_LorentzVar[voice] = dlg.m_lorentz;
		}
	}

	if (alg == EARTHWORM)
	{
		CEarthWormDlg dlg;
		CString Title;
		Title.Format("EarthWorm Sequence Voice %d", voice+1);
		dlg.title = Title;
		dlg.m_xvalue = pDoc->m_EarthWormVar[voice];
		dlg.m_multi = pDoc->m_EarthWormMulti[voice];
		dlg.m_truncator = pDoc->m_EarthWormTruncator[voice];
		dlg.pView=this;
		dlg.voicenum = voice;

		if(dlg.DoModal() == IDOK)
		{
			pDoc->m_EarthWormVar[voice] = dlg.m_xvalue;
			pDoc->m_EarthWormMulti[voice] = dlg.m_multi;
			pDoc->m_EarthWormTruncator[voice] = dlg.m_truncator;
		}
	}

	if (alg == RANDOM)
	{
		CRandomAlgorithmDlg dlg;
		CString Title;
		Title.Format("Random Voice %d", voice+1);
		dlg.title = Title;
		dlg.m_ranmin = pDoc->m_RandomMin[voice];
		dlg.m_ranmax = pDoc->m_RandomMax[voice];
		dlg.pView=this;
		dlg.voicenum = voice;


		if(dlg.DoModal() == IDOK)
		{
		pDoc->m_RandomMin[voice] = dlg.m_ranmin;
		pDoc->m_RandomMax[voice] = dlg.m_ranmax;
		}
	}

	if (alg == HAILSTONE)
	{
		CHailstoneDlg dlg;
		CString Title;
		Title.Format("3n+1 Numbers Voice %d", voice+1);
		dlg.title = Title;
		dlg.m_hailstone = pDoc->m_Hailstone[voice];
		dlg.pView = this;
		dlg.voicenum  = voice;
		if(dlg.DoModal() == IDOK)
		{
			pDoc->m_Hailstone[voice] = dlg.m_hailstone;
		}
	}

	if (alg == WOLFRAM)
	{
		CWolframDlg dlg;
		CString Title;
		Title.Format("Wolfram CA Voice %d", voice+1);
		dlg.title = Title;
		dlg.m_rules = pDoc->m_WolframRule[voice];
		dlg.m_initconfig = pDoc->m_WolframInit[voice];
		dlg.m_universesize = pDoc->m_WolframSize[voice];

		if(dlg.DoModal() == IDOK)
		{
		pDoc->m_WolframRule[voice] = dlg.m_rules;
		pDoc->m_WolframInit[voice] = dlg.m_initconfig;
		pDoc->m_WolframSize[voice] = dlg.m_universesize;
		}
	}


	pDoc->SetModifiedFlag();
}

void CFractMusView::OnCurrent() 
{
	PlayChoice = CURRENT_SETTINGS;	
	GetDlgItem(IDC_PLAYFROM_CMB)->EnableWindow(FALSE);
	GetDlgItem(IDC_PLAYFROM_BT)->EnableWindow(FALSE);


}

void CFractMusView::OnUpdateCurrent(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(PlayChoice == CURRENT_SETTINGS);
	pCmdUI->Enable(!m_Playing);
}

void CFractMusView::OnComposition() 
{
	CFractMusDoc* pDoc = GetDocument();

	PlayChoice = COMPOSITION;

	if ((pDoc->m_NumEvents>-1)&&(!m_Playing))
	{
		GetDlgItem(IDC_PLAYFROM_CMB)->EnableWindow(TRUE);
		GetDlgItem(IDC_PLAYFROM_BT)->EnableWindow(TRUE);
	}
	else
	{
		GetDlgItem(IDC_PLAYFROM_CMB)->EnableWindow(FALSE);
		GetDlgItem(IDC_PLAYFROM_BT)->EnableWindow(FALSE);
	}

}

void CFractMusView::OnUpdateComposition(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument();
	pCmdUI->SetCheck(PlayChoice == COMPOSITION);
	pCmdUI->Enable((pDoc->m_NumEvents>-1)&&(!m_Playing));
	
}

void CFractMusView::OnOptsGraph() 
{

	if (!UpdateData(TRUE))
		return;

	if (m_pGraph != NULL)
		return;

	CWaitCursor wait;

	CFractMusDoc* pDoc = GetDocument();
	int maxX = 500 + ::GetSystemMetrics(SM_CXSIZEFRAME)*2;
	int maxY = 500 + ::GetSystemMetrics(SM_CYCAPTION)+::GetSystemMetrics(SM_CXSIZEFRAME);



	long test[MAXVOICES], count;
	int note[MAXVOICES];
	int lastnote[MAXVOICES];
	int prevnote[MAXVOICES];
	BOOL active[MAXVOICES];
	int i, totvoices = 0;
	int j = 0;
	int dracula[MAXVOICES];
	unsigned long tot = 0, tot2 = 0;

	for (i=VOICE1; i<MAXVOICES; i++)
	{
		lastnote[i] = -1;
		prevnote[i] = -1;
	}

	for(i=VOICE1; i<MAXVOICES; i++)
		active[i] = FALSE;

	for(j=0; j<=pDoc->m_NumEvents; j++)
		for(i=VOICE1; i<MAXVOICES; i++)
			if(pDoc->Event[j].IsActive[i])
				active[i] = TRUE;

	for(i=VOICE1; i<MAXVOICES; i++)
		if (active[i])
			++totvoices;

	j =0;

	while (j<=pDoc->m_NumEvents)
	{

		count = pDoc->Event[j].count;

		for (i=VOICE1; i<MAXVOICES; i++)
		{
			count1[i] = 0;
			count2[i] = 0;
			count3[i] = 0;
			count4[i] = 0;
			count5[i] = 0;
			count6[i] = 0;
			count7[i] = 0;
			count8[i] = 0;
			count9[i] = 0;
			count10[i] = 0;
			count11[i] = 0;
			count12[i] = 0;
		}



		for (i=VOICE1; i<MAXVOICES; i++)
			dracula[i] = int(pDoc->Event[j].NoteValue[i]);


		for (i=VOICE1; i<MAXVOICES; i++)
		{
			if(pDoc->Event[j].AlgorithmNum[i]==LOGMAP)
			{
				LogMap[i] = 0.2f;

				for(int k = 0; k<count+((pDoc->Event[j].StartNum[i])*dracula[i]); k++)
				{

					if(!((((pDoc->Event[j].StartNum[i])*dracula[i])+k)%dracula[i]))
					{

					(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, ((pDoc->Event[j].StartNum[i]*dracula[i])+k)/dracula[i], 0,
										pDoc->Event[j].pScale[i], 
										pDoc->Event[j].NumOcts[i], 
										pDoc->Event[j].Base[i], 
										pDoc->Event[j].StartNum[i],
										pDoc->Event[j].Note[i], 
										pDoc->Event[j].Modulates[i],
										pDoc->Event[j].ModulRandom[i], 
										pDoc->Event[j].ModulNum[i],
										pDoc->Event[j].Inverted[i],
										pDoc->Event[j].LogMapVar[i],
										pDoc->Event[j].FNoiseVar[i],
										pDoc->Event[j].HenonA[i],
										pDoc->Event[j].HenonB[i],
										pDoc->Event[j].HopalongVar[i],
										pDoc->Event[j].MartinVar[i],
										pDoc->Event[j].LorentzVar[i],
										pDoc->Event[j].EarthWormVar[i],
										pDoc->Event[j].EarthWormMulti[i],
										pDoc->Event[j].EarthWormTruncator[i],
										pDoc->Event[j].RandomMin[i],
										pDoc->Event[j].RandomMax[i],
										pDoc->Event[j].WolframRule[i],
										pDoc->Event[j].WolframSize[i],
										pDoc->Event[j].Hailstone[i]



						);
					}
				}
			}

			if(pDoc->Event[j].AlgorithmNum[i]==ONE_OVER_F_NOISE)
			{
				LogMap2[i] = 0.2f;
				FNoise[i] = 0.0f;

				for(int k = 0; k<count+((pDoc->Event[j].StartNum[i])*dracula[i]); k++)
				{

					if(!((((pDoc->Event[j].StartNum[i])*dracula[i])+k)%dracula[i]))
					{

					(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, ((pDoc->Event[j].StartNum[i]*dracula[i])+k)/dracula[i], 0,
										pDoc->Event[j].pScale[i], 
										pDoc->Event[j].NumOcts[i], 
										pDoc->Event[j].Base[i], 
										pDoc->Event[j].StartNum[i],
										pDoc->Event[j].Note[i], 
										pDoc->Event[j].Modulates[i],
										pDoc->Event[j].ModulRandom[i], 
										pDoc->Event[j].ModulNum[i],
										pDoc->Event[j].Inverted[i],
										pDoc->Event[j].LogMapVar[i],
										pDoc->Event[j].FNoiseVar[i],
										pDoc->Event[j].HenonA[i],
										pDoc->Event[j].HenonB[i],
										pDoc->Event[j].HopalongVar[i],
										pDoc->Event[j].MartinVar[i],
										pDoc->Event[j].LorentzVar[i],
										pDoc->Event[j].EarthWormVar[i],
										pDoc->Event[j].EarthWormMulti[i],
										pDoc->Event[j].EarthWormTruncator[i],
										pDoc->Event[j].RandomMin[i],
										pDoc->Event[j].RandomMax[i],
										pDoc->Event[j].WolframRule[i],
										pDoc->Event[j].WolframSize[i],
										pDoc->Event[j].Hailstone[i]


						);
					}
				}
			}

			if(pDoc->Event[j].AlgorithmNum[i]==HENON)
			{
				HenonX[i] = HenonY[i] = 0.0f;

				for(int k = 0; k<count+((pDoc->Event[j].StartNum[i])*dracula[i]); k++)
				{

					if(!((((pDoc->Event[j].StartNum[i])*dracula[i])+k)%dracula[i]))
					{

					(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, ((pDoc->Event[j].StartNum[i]*dracula[i])+k)/dracula[i], 0,
										pDoc->Event[j].pScale[i], 
										pDoc->Event[j].NumOcts[i], 
										pDoc->Event[j].Base[i], 
										pDoc->Event[j].StartNum[i],
										pDoc->Event[j].Note[i], 
										pDoc->Event[j].Modulates[i],
										pDoc->Event[j].ModulRandom[i], 
										pDoc->Event[j].ModulNum[i],
										pDoc->Event[j].Inverted[i],
										pDoc->Event[j].LogMapVar[i],
										pDoc->Event[j].FNoiseVar[i],
										pDoc->Event[j].HenonA[i],
										pDoc->Event[j].HenonB[i],
										pDoc->Event[j].HopalongVar[i],
										pDoc->Event[j].MartinVar[i],
										pDoc->Event[j].LorentzVar[i],
										pDoc->Event[j].EarthWormVar[i],
										pDoc->Event[j].EarthWormMulti[i],
										pDoc->Event[j].EarthWormTruncator[i],
										pDoc->Event[j].RandomMin[i],
										pDoc->Event[j].RandomMax[i],
										pDoc->Event[j].WolframRule[i],
										pDoc->Event[j].WolframSize[i],
										pDoc->Event[j].Hailstone[i]


						);
					}
				}
			}

			if(pDoc->Event[j].AlgorithmNum[i]==HOPALONG)
			{
				HopalongX[i] = HopalongY[i] = 0.0f;

				for(int k = 0; k<count+((pDoc->Event[j].StartNum[i])*dracula[i]); k++)
				{

					if(!((((pDoc->Event[j].StartNum[i])*dracula[i])+k)%dracula[i]))
					{

					(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, ((pDoc->Event[j].StartNum[i]*dracula[i])+k)/dracula[i], 0,
										pDoc->Event[j].pScale[i], 
										pDoc->Event[j].NumOcts[i], 
										pDoc->Event[j].Base[i], 
										pDoc->Event[j].StartNum[i],
										pDoc->Event[j].Note[i], 
										pDoc->Event[j].Modulates[i],
										pDoc->Event[j].ModulRandom[i], 
										pDoc->Event[j].ModulNum[i],
										pDoc->Event[j].Inverted[i],
										pDoc->Event[j].LogMapVar[i],
										pDoc->Event[j].FNoiseVar[i],
										pDoc->Event[j].HenonA[i],
										pDoc->Event[j].HenonB[i],
										pDoc->Event[j].HopalongVar[i],
										pDoc->Event[j].MartinVar[i],
										pDoc->Event[j].LorentzVar[i],
										pDoc->Event[j].EarthWormVar[i],
										pDoc->Event[j].EarthWormMulti[i],
										pDoc->Event[j].EarthWormTruncator[i],
										pDoc->Event[j].RandomMin[i],
										pDoc->Event[j].RandomMax[i],
										pDoc->Event[j].WolframRule[i],
										pDoc->Event[j].WolframSize[i],
										pDoc->Event[j].Hailstone[i]


						);
					}
				}
			}


			if(pDoc->Event[j].AlgorithmNum[i]==MARTIN)
			{
				MartinX[i] = MartinY[i] = 0.0f;

				for(int k = 0; k<count+((pDoc->Event[j].StartNum[i])*dracula[i]); k++)
				{

					if(!((((pDoc->Event[j].StartNum[i])*dracula[i])+k)%dracula[i]))
					{

					(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, ((pDoc->Event[j].StartNum[i]*dracula[i])+k)/dracula[i], 0,
										pDoc->Event[j].pScale[i], 
										pDoc->Event[j].NumOcts[i], 
										pDoc->Event[j].Base[i], 
										pDoc->Event[j].StartNum[i],
										pDoc->Event[j].Note[i], 
										pDoc->Event[j].Modulates[i],
										pDoc->Event[j].ModulRandom[i], 
										pDoc->Event[j].ModulNum[i],
										pDoc->Event[j].Inverted[i],
										pDoc->Event[j].LogMapVar[i],
										pDoc->Event[j].FNoiseVar[i],
										pDoc->Event[j].HenonA[i],
										pDoc->Event[j].HenonB[i],
										pDoc->Event[j].HopalongVar[i],
										pDoc->Event[j].MartinVar[i],
										pDoc->Event[j].LorentzVar[i],
										pDoc->Event[j].EarthWormVar[i],
										pDoc->Event[j].EarthWormMulti[i],
										pDoc->Event[j].EarthWormTruncator[i],
										pDoc->Event[j].RandomMin[i],
										pDoc->Event[j].RandomMax[i],
										pDoc->Event[j].WolframRule[i],
										pDoc->Event[j].WolframSize[i],
										pDoc->Event[j].Hailstone[i]


						);
					}
				}
			}

			if(pDoc->Event[j].AlgorithmNum[i]==GINGERBREADMAN)
			{
				GingerX[i] = pDoc->Event[j].GingerX[i];
				GingerY[i] = pDoc->Event[j].GingerY[i];

				for(int k = 0; k<count+((pDoc->Event[j].StartNum[i])*dracula[i]); k++)
				{

					if(!((((pDoc->Event[j].StartNum[i])*dracula[i])+k)%dracula[i]))
					{

					(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, ((pDoc->Event[j].StartNum[i]*dracula[i])+k)/dracula[i], 0,
										pDoc->Event[j].pScale[i], 
										pDoc->Event[j].NumOcts[i], 
										pDoc->Event[j].Base[i], 
										pDoc->Event[j].StartNum[i],
										pDoc->Event[j].Note[i], 
										pDoc->Event[j].Modulates[i],
										pDoc->Event[j].ModulRandom[i], 
										pDoc->Event[j].ModulNum[i],
										pDoc->Event[j].Inverted[i],
										pDoc->Event[j].LogMapVar[i],
										pDoc->Event[j].FNoiseVar[i],
										pDoc->Event[j].HenonA[i],
										pDoc->Event[j].HenonB[i],
										pDoc->Event[j].HopalongVar[i],
										pDoc->Event[j].MartinVar[i],
										pDoc->Event[j].LorentzVar[i],
										pDoc->Event[j].EarthWormVar[i],
										pDoc->Event[j].EarthWormMulti[i],
										pDoc->Event[j].EarthWormTruncator[i],
										pDoc->Event[j].RandomMin[i],
										pDoc->Event[j].RandomMax[i],
										pDoc->Event[j].WolframRule[i],
										pDoc->Event[j].WolframSize[i],
										pDoc->Event[j].Hailstone[i]


						);
					}
				}
			}


			if(pDoc->Event[j].AlgorithmNum[i]==LORENTZ)
			{
				LorentzX[i] = 1.0f;

				for(int k = 0; k<count+((pDoc->Event[j].StartNum[i])*dracula[i]); k++)
				{

					if(!((((pDoc->Event[j].StartNum[i])*dracula[i])+k)%dracula[i]))
					{

					(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, ((pDoc->Event[j].StartNum[i]*dracula[i])+k)/dracula[i], 0,
										pDoc->Event[j].pScale[i], 
										pDoc->Event[j].NumOcts[i], 
										pDoc->Event[j].Base[i], 
										pDoc->Event[j].StartNum[i],
										pDoc->Event[j].Note[i], 
										pDoc->Event[j].Modulates[i],
										pDoc->Event[j].ModulRandom[i], 
										pDoc->Event[j].ModulNum[i],
										pDoc->Event[j].Inverted[i],
										pDoc->Event[j].LogMapVar[i],
										pDoc->Event[j].FNoiseVar[i],
										pDoc->Event[j].HenonA[i],
										pDoc->Event[j].HenonB[i],
										pDoc->Event[j].HopalongVar[i],
										pDoc->Event[j].MartinVar[i],
										pDoc->Event[j].LorentzVar[i],
										pDoc->Event[j].EarthWormVar[i],
										pDoc->Event[j].EarthWormMulti[i],
										pDoc->Event[j].EarthWormTruncator[i],
										pDoc->Event[j].RandomMin[i],
										pDoc->Event[j].RandomMax[i],
										pDoc->Event[j].WolframRule[i],
										pDoc->Event[j].WolframSize[i],
										pDoc->Event[j].Hailstone[i]


						);
					}
				}
			}


			if(pDoc->Event[j].AlgorithmNum[i]==EARTHWORM)
			{
				EarthWorm[i] = pDoc->Event[j].EarthWormVar[i];

				for(int k = 0; k<count+((pDoc->Event[j].StartNum[i])*dracula[i]); k++)
				{

					if(!((((pDoc->Event[j].StartNum[i])*dracula[i])+k)%dracula[i]))
					{

					(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, ((pDoc->Event[j].StartNum[i]*dracula[i])+k)/dracula[i], 0,
										pDoc->Event[j].pScale[i], 
										pDoc->Event[j].NumOcts[i], 
										pDoc->Event[j].Base[i], 
										pDoc->Event[j].StartNum[i],
										pDoc->Event[j].Note[i], 
										pDoc->Event[j].Modulates[i],
										pDoc->Event[j].ModulRandom[i], 
										pDoc->Event[j].ModulNum[i],
										pDoc->Event[j].Inverted[i],
										pDoc->Event[j].LogMapVar[i],
										pDoc->Event[j].FNoiseVar[i],
										pDoc->Event[j].HenonA[i],
										pDoc->Event[j].HenonB[i],
										pDoc->Event[j].HopalongVar[i],
										pDoc->Event[j].MartinVar[i],
										pDoc->Event[j].LorentzVar[i],
										pDoc->Event[j].EarthWormVar[i],
										pDoc->Event[j].EarthWormMulti[i],
										pDoc->Event[j].EarthWormTruncator[i],
										pDoc->Event[j].RandomMin[i],
										pDoc->Event[j].RandomMax[i],
										pDoc->Event[j].WolframRule[i],
										pDoc->Event[j].WolframSize[i],
										pDoc->Event[j].Hailstone[i]


						);
					}
				}
			}


						if(pDoc->Event[j].AlgorithmNum[i]==HAILSTONE)
						{
							HailStoneNumber[i] = pDoc->Event[j].Hailstone[i];

							for(int k = 0; k<count+(pDoc->Event[j].StartNum[i]*dracula[i]); k++)
							{

								if(!(((pDoc->Event[j].StartNum[i]*dracula[i])+k)%dracula[i]))
								{

								(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, ((pDoc->Event[j].StartNum[i]*dracula[i])+k)/dracula[i], 0,
													pDoc->Event[j].pScale[i], 
													pDoc->Event[j].NumOcts[i], 
													pDoc->Event[j].Base[i], 
													pDoc->Event[j].StartNum[i],
													pDoc->Event[j].Note[i], 
													pDoc->Event[j].Modulates[i],
													pDoc->Event[j].ModulRandom[i], 
													pDoc->Event[j].ModulNum[i],
													pDoc->Event[j].Inverted[i],
													pDoc->Event[j].LogMapVar[i],
													pDoc->Event[j].FNoiseVar[i],
													pDoc->Event[j].HenonA[i],
													pDoc->Event[j].HenonB[i],
													pDoc->Event[j].HopalongVar[i],
													pDoc->Event[j].MartinVar[i],
													pDoc->Event[j].LorentzVar[i],
													pDoc->Event[j].EarthWormVar[i],
													pDoc->Event[j].EarthWormMulti[i],
													pDoc->Event[j].EarthWormTruncator[i],
													pDoc->Event[j].RandomMin[i],
													pDoc->Event[j].RandomMax[i],
													pDoc->Event[j].WolframRule[i],
													pDoc->Event[j].WolframSize[i],
													pDoc->Event[j].Hailstone[i]


									);

								}
							}
						}



						if(pDoc->Event[j].AlgorithmNum[i]==WOLFRAM)
						{
							if (WolframArray1[i])
							{
								delete WolframArray1[i];
								WolframArray1[i] = NULL;
							}
							if (WolframArray2[i])
							{
								delete WolframArray2[i];
								WolframArray2[i] = NULL;
							}

							WolframArray1[i] = new BYTE[pDoc->Event[j].WolframSize[i]];
							WolframArray2[i] = new BYTE[pDoc->Event[j].WolframSize[i]];

							for (int idx = 0; idx<32; idx++)
									WolframArray1[i][idx] = pDoc->Event[j].WolframInit[i] & unsigned long (pow(2, idx)) ? 1 : 0;

							if (pDoc->Event[j].WolframSize[i]>32)
							{
								for (idx = 32; idx<pDoc->Event[j].WolframSize[i]; idx++)
									WolframArray1[i][idx] = 0;
							}

							for(int k = 0; k<count+(pDoc->Event[j].StartNum[i]*dracula[i]); k++)
							{

								if(!(((pDoc->Event[j].StartNum[i]*dracula[i])+k)%dracula[i]))
								{

								(*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, ((pDoc->Event[j].StartNum[i]*dracula[i])+k)/dracula[i], 0,
													pDoc->Event[j].pScale[i], 
													pDoc->Event[j].NumOcts[i], 
													pDoc->Event[j].Base[i], 
													pDoc->Event[j].StartNum[i],
													pDoc->Event[j].Note[i], 
													pDoc->Event[j].Modulates[i],
													pDoc->Event[j].ModulRandom[i], 
													pDoc->Event[j].ModulNum[i],
													pDoc->Event[j].Inverted[i],
													pDoc->Event[j].LogMapVar[i],
													pDoc->Event[j].FNoiseVar[i],
													pDoc->Event[j].HenonA[i],
													pDoc->Event[j].HenonB[i],
													pDoc->Event[j].HopalongVar[i],
													pDoc->Event[j].MartinVar[i],
													pDoc->Event[j].LorentzVar[i],
													pDoc->Event[j].EarthWormVar[i],
													pDoc->Event[j].EarthWormMulti[i],
													pDoc->Event[j].EarthWormTruncator[i],
													pDoc->Event[j].RandomMin[i],
													pDoc->Event[j].RandomMax[i],
													pDoc->Event[j].WolframRule[i],
													pDoc->Event[j].WolframSize[i],
													pDoc->Event[j].Hailstone[i]


									);

								}
							}
						}



		}


		while(count<pDoc->Event[j].genCount)
		{
			for (i=VOICE1; i<MAXVOICES; i++)
			{

				if (pDoc->Event[j].IsActive[i])
				{

					if(((pDoc->Event[j].StartNum[i]*dracula[i])+count)>=0)
					{

						if(!((((pDoc->Event[j].StartNum[i])*dracula[i])+count)%dracula[i]))
						{
							test[i] = ((((pDoc->Event[j].StartNum[i])*dracula[i])+count)/dracula[i])*pDoc->Event[j].Multi[i];
							note[i] = (*(Algorithm[ pDoc->Event[j].AlgorithmNum[i] ]))(i, ((pDoc->Event[j].StartNum[i]*dracula[i])+count)/dracula[i], test[i],
												pDoc->Event[j].pScale[i], 
												pDoc->Event[j].NumOcts[i], 
												pDoc->Event[j].Base[i], 
												pDoc->Event[j].StartNum[i],
												pDoc->Event[j].Note[i], 
												pDoc->Event[j].Modulates[i],
												pDoc->Event[j].ModulRandom[i], 
												pDoc->Event[j].ModulNum[i],
												pDoc->Event[j].Inverted[i],
												pDoc->Event[j].LogMapVar[i],
												pDoc->Event[j].FNoiseVar[i],
												pDoc->Event[j].HenonA[i],
												pDoc->Event[j].HenonB[i],
												pDoc->Event[j].HopalongVar[i],
												pDoc->Event[j].MartinVar[i],
												pDoc->Event[j].LorentzVar[i],
												pDoc->Event[j].EarthWormVar[i],
												pDoc->Event[j].EarthWormMulti[i],
												pDoc->Event[j].EarthWormTruncator[i],
												pDoc->Event[j].RandomMin[i],
												pDoc->Event[j].RandomMax[i],
										pDoc->Event[j].WolframRule[i],
										pDoc->Event[j].WolframSize[i],
										pDoc->Event[j].Hailstone[i]

										);	

								tot += note[i];
								tot2++;

						}

					}
				}
			}

			count++;
			tot++;

		} // inner while

		++j;
		tot += j;
		tot2 += j;

	} // outer while




	CClientDC dc(m_pGraph);
	m_pGraph = new CGraphWnd;
	CRect r(0,0,maxX,maxY);
	LOGBRUSH lb;

	lb.lbStyle = BS_SOLID;
	lb.lbColor = RGB(0,0,0);
	lb.lbHatch = HS_CROSS;


	m_pGraph->m_memDC.CreateCompatibleDC(&dc);
	m_pGraph->m_bmp.CreateCompatibleBitmap(&dc, 500, 500);
	m_pGraph->m_memDC.SelectObject(&(m_pGraph->m_bmp));
	m_pGraph->m_bkbrush.CreateBrushIndirect(&lb);
	m_pGraph->m_memDC.SelectObject(&(m_pGraph->m_bkbrush));
	m_pGraph->m_Pen.CreatePen(PS_SOLID, 0, tot);
	m_pGraph->m_memDC.SelectObject(&(m_pGraph->m_Pen));
	m_pGraph->m_memDC.PatBlt(0,0,maxX,maxY,PATCOPY);
	m_pGraph->m_pParent = this;
	m_pGraph->Create(	NULL, "Image from composition",
						WS_OVERLAPPED|WS_CAPTION|
						WS_SYSMENU|WS_MINIMIZEBOX|
						WS_VISIBLE, r
						   );

	int ii, jj, xx, yy, cx, cy;

	cx = 500/2;
	cy = 500/2;


	COLORREF color, red, green, blue;

	for ( xx=tot, ii =0; ii<250; xx+=tot2, ii++)
	{
		for( yy=xx, jj=ii; jj<250; yy+=tot2, jj++)
		{
			red = ((unsigned long(xx*xx)+unsigned long(yy*yy))>>5)%255;
			green = ((unsigned long(xx*xx)+unsigned long(yy*yy))>>7)%255;
			blue = ((unsigned long(xx*xx)+unsigned long(yy*yy))>>11)%255;

			color = RGB(red, green, blue);

			m_pGraph->m_memDC.SetPixel(cx+ii, cy+jj, color);
			m_pGraph->m_memDC.SetPixel(cx+jj, cy+ii, color);
			m_pGraph->m_memDC.SetPixel(cx+jj, cy-ii, color);
			m_pGraph->m_memDC.SetPixel(cx+ii, cy-jj, color);
			m_pGraph->m_memDC.SetPixel(cx-ii, cy-jj, color);
			m_pGraph->m_memDC.SetPixel(cx-jj, cy-ii, color);	
			m_pGraph->m_memDC.SetPixel(cx-jj, cy+ii, color);	
			m_pGraph->m_memDC.SetPixel(cx-ii, cy+jj, color);


		}
		//m_pGraph->RedrawWindow(NULL,NULL,RDW_INVALIDATE|RDW_UPDATENOW|RDW_NOERASE);
	}



}

void CFractMusView::OnUpdateOptsGraph(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument();

	pCmdUI->Enable((!m_pGraph)&&(!m_Playing)&&(pDoc->m_NumEvents>-1)&&(!IsDocumentLocked));
	
}


void CFractMusView::OnRound() 
{
	CFractMusDoc* pDoc = GetDocument();
	CRoundDlg round;
	for(int i = VOICE1; i<MAXVOICES; i++)
		round.m_IsActive[i] = pDoc->m_IsActive[i];

	if(round.DoModal()==IDOK)
	{
		switch(round.m_UpOrDown)
		{
			case ROUND_UP:
				while(pDoc->dGeneralCounter%(long)pDoc->m_NoteValue[round.m_RoundVoice])
					pDoc->dGeneralCounter++;
				while(pDoc->dStartCounter%(long)pDoc->m_NoteValue[round.m_RoundVoice])
					pDoc->dStartCounter++;
				break;
			case ROUND_DOWN:
				while(pDoc->dGeneralCounter%(long)pDoc->m_NoteValue[round.m_RoundVoice])
					pDoc->dGeneralCounter--;
				while(pDoc->dStartCounter%(long)pDoc->m_NoteValue[round.m_RoundVoice])
					pDoc->dStartCounter--;
				break;
		}

		GeneralCounter = pDoc->dGeneralCounter;
		StartCounter = pDoc->dStartCounter;
		pDoc->SetModifiedFlag();
		UpdateData(FALSE);
	}
		
}

void CFractMusView::OnAlgorithmsVoice1Logmap() 
{
	UpdateAlgorithm(VOICE1, LOGMAP);
}

void CFractMusView::OnUpdateAlgorithmsVoice1Logmap(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE1] == LOGMAP);
	pCmdUI->Enable(!m_Playing);
}

void CFractMusView::OnAlgorithmsVoice2Logmap() 
{
	UpdateAlgorithm(VOICE2, LOGMAP);	
}

void CFractMusView::OnUpdateAlgorithmsVoice2Logmap(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE2] == LOGMAP);
	pCmdUI->Enable(!m_Playing);
}

void CFractMusView::OnAlgorithmsVoice3Logmap() 
{
	UpdateAlgorithm(VOICE3, LOGMAP);	
}

void CFractMusView::OnUpdateAlgorithmsVoice3Logmap(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE3] == LOGMAP);
	pCmdUI->Enable(!m_Playing);
}

void CFractMusView::OnAlgorithmsVoice4Logmap() 
{
	UpdateAlgorithm(VOICE4, LOGMAP);
}

void CFractMusView::OnUpdateAlgorithmsVoice4Logmap(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE4] == LOGMAP);
	pCmdUI->Enable(!m_Playing);
}

void CFractMusView::OnAlgorithmsVoice5Logmap() 
{
	UpdateAlgorithm(VOICE5, LOGMAP);
}

void CFractMusView::OnUpdateAlgorithmsVoice5Logmap(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE5] == LOGMAP);
	pCmdUI->Enable(!m_Playing);
}

void CFractMusView::OnAlgorithmsVoice6Logmap() 
{
	UpdateAlgorithm(VOICE6, LOGMAP);
}

void CFractMusView::OnUpdateAlgorithmsVoice6Logmap(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE6] == LOGMAP);
	pCmdUI->Enable(!m_Playing);
}

void CFractMusView::OnAlgorithmsVoice7Logmap() 
{
	UpdateAlgorithm(VOICE7, LOGMAP);
}

void CFractMusView::OnUpdateAlgorithmsVoice7Logmap(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE7] == LOGMAP);
	pCmdUI->Enable(!m_Playing);
}

void CFractMusView::OnAlgorithmsVoice8Logmap() 
{
	UpdateAlgorithm(VOICE8, LOGMAP);
}

void CFractMusView::OnUpdateAlgorithmsVoice8Logmap(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE8] == LOGMAP);
	pCmdUI->Enable(!m_Playing);
}

void CFractMusView::OnAlgorithmsVoice1Oneoverf() 
{
	UpdateAlgorithm(VOICE1, ONE_OVER_F_NOISE);	
}

void CFractMusView::OnUpdateAlgorithmsVoice1Oneoverf(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE1] == ONE_OVER_F_NOISE);
	pCmdUI->Enable(!m_Playing);
}

float RanDev()
{
	long j,k;
	static long idum2 = 123456789l;
	static long iy = 0l;
	static long iv[32L];
	float temp;

	if (Seed<=0L)
	{
		if(-Seed<1L)
			Seed = 1L;
		else
			Seed = -Seed;

		idum2 = Seed;

		for(j = NTAB+7; j>=0; --j)
		{
			k = Seed/IQ1;
			Seed = IA1*(Seed-k*IQ1) - k * IR1;

			if(Seed<0L)
				Seed += IM1;
			if(j<NTAB)
				iv[size_t(j)] = Seed;
		}

		iy = iv[0];
	}

	k = Seed/IQ1;

	Seed = IA1*(Seed-k*IQ1)-k*IR1;

	if(Seed<0L)
		Seed +=IM1;

	k = idum2/IQ2;

	idum2 = IA2*(idum2-k*IQ2)-k*IR2;

	if(idum2<0L)
		idum2 +=IM2;

	j = iy/NDIV;
	iy = iv[size_t(j)] - idum2;
	iv[size_t(j)] = Seed;

	if(iy<1L)
		iy += IMM1;

	temp = AM*float(iy);

	if(temp>RNMX)
		return RNMX;
	else
		return temp;
}

void CFractMusView::OnAlgorithmsVoice2Oneoverf() 
{
	UpdateAlgorithm(VOICE2, ONE_OVER_F_NOISE);	
}

void CFractMusView::OnUpdateAlgorithmsVoice2Oneoverf(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE2] == ONE_OVER_F_NOISE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice3Oneoverf() 
{
	UpdateAlgorithm(VOICE3, ONE_OVER_F_NOISE);	
}

void CFractMusView::OnUpdateAlgorithmsVoice3Oneoverf(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE3] == ONE_OVER_F_NOISE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice4Oneoverf() 
{
	UpdateAlgorithm(VOICE4, ONE_OVER_F_NOISE);	
}

void CFractMusView::OnUpdateAlgorithmsVoice4Oneoverf(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE4] == ONE_OVER_F_NOISE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice5Oneoverf() 
{
	UpdateAlgorithm(VOICE5, ONE_OVER_F_NOISE);	
}

void CFractMusView::OnUpdateAlgorithmsVoice5Oneoverf(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE5] == ONE_OVER_F_NOISE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice6Oneoverf() 
{
	UpdateAlgorithm(VOICE6, ONE_OVER_F_NOISE);	
}

void CFractMusView::OnUpdateAlgorithmsVoice6Oneoverf(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE6] == ONE_OVER_F_NOISE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice7Oneoverf() 
{
	UpdateAlgorithm(VOICE7, ONE_OVER_F_NOISE);	
}

void CFractMusView::OnUpdateAlgorithmsVoice7Oneoverf(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE7] == ONE_OVER_F_NOISE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice8Oneoverf() 
{
	UpdateAlgorithm(VOICE8, ONE_OVER_F_NOISE);	
}

void CFractMusView::OnUpdateAlgorithmsVoice8Oneoverf(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE8] == ONE_OVER_F_NOISE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnOptsDevice() 
{
	CMidiDevDlg dlg;

	dlg.DoModal();

	if (dlg.m_midiDevChoice == DevChoice)
		return;

	DevChoice = dlg.m_midiDevChoice;

	//midiOutClose(midiOut);

	/*UINT result = midiOutOpen(&midiOut, DevChoice, NULL, NULL, CALLBACK_NULL);

	if( result!=MMSYSERR_NOERROR )
	{
		switch(result)
		{

		case MMSYSERR_ALLOCATED:
			AfxMessageBox("The device is already in use, free it and try again", MB_OK+MB_ICONHAND);
			break;
		case MMSYSERR_NOMEM:
			AfxMessageBox("Could not allocate memory for the device", MB_OK+MB_ICONHAND);
			break;
		case MMSYSERR_INVALPARAM:
			AfxMessageBox("Invalid pointer!", MB_OK+MB_ICONHAND);
			break;
		}

		AfxGetMainWnd()->PostMessage(WM_QUIT);

	}*/
	
}

void CFractMusView::OnUpdateOptsDevice(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!m_Playing);
}


void CFractMusView::OnVoice10MidiButton() 
{
	UpdateMidi(VOICE10);	
	
}

void CFractMusView::OnVoice10ModulateChk() 
{
	UpdateModulation(VOICE10);	
}


void CFractMusView::OnCloseupVoice10NotevalueCombo() 
{
	UpdateNoteValueCombo(VOICE10);	
	
}

void CFractMusView::OnCloseupVoice10NumoctsCombo() 
{
	UpdateNumOctsCombo(VOICE10);	
}

void CFractMusView::OnVoice10OnChk() 
{
	UpdateActive(VOICE10);	
	
}

void CFractMusView::OnCloseupVoice10ScaleCombo() 
{
	UpdateScaleCombo(VOICE10);	
	
}

void CFractMusView::OnCloseupVoice10StartnoteCombo() 
{
	UpdateNoteCombo(VOICE10);	
	
}

void CFractMusView::OnKillfocusVoice10StartnumEdit() 
{
	UpdateStartNum(VOICE10);	
	
}


void CFractMusView::OnVoice11MidiButton() 
{
	UpdateMidi(VOICE11);	
	
}

void CFractMusView::OnVoice11ModulateChk() 
{
	UpdateModulation(VOICE11);	
}



void CFractMusView::OnVoice13MidiButton() 
{
	UpdateMidi(VOICE13);	
	
}

void CFractMusView::OnVoice13ModulateChk() 
{
	UpdateModulation(VOICE13);	
	
}




void CFractMusView::OnCloseupVoice11NotevalueCombo() 
{
	UpdateNoteValueCombo(VOICE11);	
	
}

void CFractMusView::OnCloseupVoice11NumoctsCombo() 
{
	UpdateNumOctsCombo(VOICE11);	
}

void CFractMusView::OnCloseupVoice12NumoctsCombo() 
{
	UpdateNumOctsCombo(VOICE12);	
	
}

void CFractMusView::OnCloseupVoice13NumoctsCombo() 
{
	UpdateNumOctsCombo(VOICE13);	
	
}


void CFractMusView::OnVoice11OnChk() 
{
	UpdateActive(VOICE11);	
	
}

void CFractMusView::OnVoice12OnChk() 
{
	UpdateActive(VOICE12);	
	
}

void CFractMusView::OnVoice13OnChk() 
{
	UpdateActive(VOICE13);	
	
}

void CFractMusView::OnCloseupVoice11ScaleCombo() 
{
	UpdateScaleCombo(VOICE11);	
	
}

void CFractMusView::OnCloseupVoice12ScaleCombo() 
{
	UpdateScaleCombo(VOICE12);	
	
}


void CFractMusView::OnCloseupVoice13ScaleCombo() 
{
	UpdateScaleCombo(VOICE13);	
	
}


void CFractMusView::OnCloseupVoice11StartnoteCombo() 
{
	UpdateNoteCombo(VOICE11);	
	
}

void CFractMusView::OnCloseupVoice12StartnoteCombo() 
{
	UpdateNoteCombo(VOICE12);	
	
}


void CFractMusView::OnCloseupVoice13StartnoteCombo() 
{
	UpdateNoteCombo(VOICE13);	
	
}

void CFractMusView::OnKillfocusVoice11StartnumEdit() 
{
	UpdateStartNum(VOICE11);	
	
}


void CFractMusView::OnVoice12MidiButton() 
{
	UpdateMidi(VOICE12);	
	
}

void CFractMusView::OnVoice12ModulateChk() 
{
	UpdateModulation(VOICE12);	
	
}


void CFractMusView::OnCloseupVoice12NotevalueCombo() 
{
	UpdateNoteValueCombo(VOICE12);	
	
}

void CFractMusView::OnCloseupVoice13NotevalueCombo() 
{
	UpdateNoteValueCombo(VOICE13);	
	
}


void CFractMusView::OnVoice14MidiButton() 
{
	UpdateMidi(VOICE14);	
	
}

void CFractMusView::OnVoice14ModulateChk() 
{
	UpdateModulation(VOICE14);	
	
}


void CFractMusView::OnCloseupVoice14NotevalueCombo() 
{
	UpdateNoteValueCombo(VOICE14);	
	
}

void CFractMusView::OnCloseupVoice14NumoctsCombo() 
{
	UpdateNumOctsCombo(VOICE14);	
	
}

void CFractMusView::OnVoice14OnChk() 
{
	UpdateActive(VOICE14);	
	
}

void CFractMusView::OnCloseupVoice14ScaleCombo() 
{
	UpdateScaleCombo(VOICE14);	
	
}

void CFractMusView::OnCloseupVoice14StartnoteCombo() 
{
	UpdateNoteCombo(VOICE14);	
	
}

void CFractMusView::OnKillfocusVoice14StartnumEdit() 
{
	UpdateStartNum(VOICE14);	
	
}


void CFractMusView::OnVoice15MidiButton() 
{
	UpdateMidi(VOICE15);	
	
}

void CFractMusView::OnVoice15ModulateChk() 
{
	UpdateModulation(VOICE15);	
	
}


void CFractMusView::OnCloseupVoice15NotevalueCombo() 
{
	UpdateNoteValueCombo(VOICE15);	
	
}

void CFractMusView::OnCloseupVoice15NumoctsCombo() 
{
	UpdateNumOctsCombo(VOICE15);	
	
}

void CFractMusView::OnVoice15OnChk() 
{
	UpdateActive(VOICE15);	
	
}

void CFractMusView::OnCloseupVoice15ScaleCombo() 
{
	UpdateScaleCombo(VOICE15);	
	
}

void CFractMusView::OnCloseupVoice15StartnoteCombo() 
{
	UpdateNoteCombo(VOICE15);	
	
}

void CFractMusView::OnKillfocusVoice15StartnumEdit() 
{
	UpdateStartNum(VOICE15);	
	
}


void CFractMusView::OnVoice16MidiButton() 
{
	UpdateMidi(VOICE16);	
	
}

void CFractMusView::OnVoice16ModulateChk() 
{
	UpdateModulation(VOICE16);	
	
}


void CFractMusView::OnCloseupVoice16NotevalueCombo() 
{
	UpdateNoteValueCombo(VOICE16);	
	
}

void CFractMusView::OnCloseupVoice16NumoctsCombo() 
{
	UpdateNumOctsCombo(VOICE16);	
	
}

void CFractMusView::OnVoice16OnChk() 
{
	UpdateActive(VOICE16);	
	
}

void CFractMusView::OnCloseupVoice16ScaleCombo() 
{
	UpdateScaleCombo(VOICE16);	
	
}

void CFractMusView::OnCloseupVoice16StartnoteCombo() 
{
	UpdateNoteCombo(VOICE16);	
	
}

void CFractMusView::OnKillfocusVoice16StartnumEdit() 
{
	UpdateStartNum(VOICE16);	
	
}


void CFractMusView::OnKillfocusVoice12StartnumEdit() 
{
	UpdateStartNum(VOICE12);	
	
}


void CFractMusView::OnKillfocusVoice13StartnumEdit() 
{
	UpdateStartNum(VOICE13);	
	
}




void CFractMusView::OnKillfocusVoice9StartnumEdit() 
{
	UpdateStartNum(VOICE9);	
	
}




void CFractMusView::OnCloseupTimesignumCmb() 
{
	CFractMusDoc* pDoc = GetDocument();

	CComboBox* pB = (CComboBox*)GetDlgItem(IDC_TIMESIGNUM_CMB);
	short sel;

	sel = pB->GetCurSel();

	if (sel != LB_ERR)
		pDoc->m_TimeSigNum = sel+1;
	
}

void CFractMusView::OnCloseupTimesigdenCmb() 
{
	CFractMusDoc* pDoc = GetDocument();

	CComboBox* pB = (CComboBox*)GetDlgItem(IDC_TIMESIGDEN_CMB);
	short sel;

	sel = pB->GetCurSel();

	if (sel != LB_ERR)
	{

		switch (sel)
		{
			case 0:
				pDoc->m_TimeSigDen = 1;
				break;
			case 1:
				pDoc->m_TimeSigDen = 2;
				break;
			case 2:
				pDoc->m_TimeSigDen = 4;
				break;
			case 3:
				pDoc->m_TimeSigDen = 8;
				break;
			case 4:
				pDoc->m_TimeSigDen = 16;
				break;
			case 5:
				pDoc->m_TimeSigDen = 32;
				break;
		}

		pDoc->SetModifiedFlag();

	}

	
}


void CFractMusView::OnAlgorithmsVoice10Logmap() 
{
	UpdateAlgorithm(VOICE10, LOGMAP);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice10Logmap(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE10] == LOGMAP);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice10Morsethue() 
{
	UpdateAlgorithm(VOICE10, MORSE_THUE);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice10Morsethue(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE10] == MORSE_THUE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice10Oneoverf() 
{
	UpdateAlgorithm(VOICE10, ONE_OVER_F_NOISE);	
}

void CFractMusView::OnUpdateAlgorithmsVoice10Oneoverf(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE10] == ONE_OVER_F_NOISE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice11Logmap() 
{
	UpdateAlgorithm(VOICE11, LOGMAP);	
}

void CFractMusView::OnUpdateAlgorithmsVoice11Logmap(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE11] == LOGMAP);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice11Morsethue() 
{
	UpdateAlgorithm(VOICE11, MORSE_THUE);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice11Morsethue(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE11] == MORSE_THUE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice11Oneoverf() 
{
	UpdateAlgorithm(VOICE11, ONE_OVER_F_NOISE);	
	
}

void CFractMusView::OnUpdateAlgorithmsVoice11Oneoverf(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE11] == ONE_OVER_F_NOISE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice12Logmap() 
{
	UpdateAlgorithm(VOICE12, LOGMAP);	
	
}

void CFractMusView::OnUpdateAlgorithmsVoice12Logmap(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE12] == LOGMAP);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice12Morsethue() 
{
	UpdateAlgorithm(VOICE12, MORSE_THUE);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice12Morsethue(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE12] == MORSE_THUE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice12Oneoverf() 
{
	UpdateAlgorithm(VOICE12, ONE_OVER_F_NOISE);	
	
}

void CFractMusView::OnUpdateAlgorithmsVoice12Oneoverf(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE12] == ONE_OVER_F_NOISE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice13Logmap() 
{
	UpdateAlgorithm(VOICE13, LOGMAP);	
	
}

void CFractMusView::OnUpdateAlgorithmsVoice13Logmap(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE13] == LOGMAP);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice13Morsethue() 
{
	UpdateAlgorithm(VOICE13, MORSE_THUE);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice13Morsethue(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE13] == MORSE_THUE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice13Oneoverf() 
{
	UpdateAlgorithm(VOICE13, ONE_OVER_F_NOISE);	
	
}

void CFractMusView::OnUpdateAlgorithmsVoice13Oneoverf(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE13] == ONE_OVER_F_NOISE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice14Logmap() 
{
	UpdateAlgorithm(VOICE14, LOGMAP);	
	
}

void CFractMusView::OnUpdateAlgorithmsVoice14Logmap(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE14] == LOGMAP);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice14Morsethue() 
{
	UpdateAlgorithm(VOICE14, MORSE_THUE);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice14Morsethue(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE14] == MORSE_THUE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice14Oneoverf() 
{
	UpdateAlgorithm(VOICE14, ONE_OVER_F_NOISE);	
	
}

void CFractMusView::OnUpdateAlgorithmsVoice14Oneoverf(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE14] == ONE_OVER_F_NOISE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice15Logmap() 
{
	UpdateAlgorithm(VOICE15, LOGMAP);	
	
}

void CFractMusView::OnUpdateAlgorithmsVoice15Logmap(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE15] == LOGMAP);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice15Morsethue() 
{
	UpdateAlgorithm(VOICE15, MORSE_THUE);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice15Morsethue(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE15] == MORSE_THUE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice15Oneoverf() 
{
	UpdateAlgorithm(VOICE15, ONE_OVER_F_NOISE);	
	
}

void CFractMusView::OnUpdateAlgorithmsVoice15Oneoverf(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE15] == ONE_OVER_F_NOISE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice16Logmap() 
{
	UpdateAlgorithm(VOICE16, LOGMAP);	
	
}

void CFractMusView::OnUpdateAlgorithmsVoice16Logmap(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE16] == LOGMAP);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice16Morsethue() 
{
	UpdateAlgorithm(VOICE16, MORSE_THUE);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice16Morsethue(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE16] == MORSE_THUE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice16Oneoverf() 
{
	UpdateAlgorithm(VOICE16, ONE_OVER_F_NOISE);	
	
}

void CFractMusView::OnUpdateAlgorithmsVoice16Oneoverf(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE16] == ONE_OVER_F_NOISE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice9Logmap() 
{
	UpdateAlgorithm(VOICE9, LOGMAP);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice9Logmap(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE9] == LOGMAP);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice9Morsethue() 
{
	UpdateAlgorithm(VOICE9, MORSE_THUE);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice9Morsethue(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE9] == MORSE_THUE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice9Oneoverf() 
{
	UpdateAlgorithm(VOICE9, ONE_OVER_F_NOISE);	
	
}

void CFractMusView::OnUpdateAlgorithmsVoice9Oneoverf(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE9] == ONE_OVER_F_NOISE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdatesettings() 
{

#ifdef NOTREGISTERED

	MessageBox("This feature is disabled in the unregistered version of FractMus, please resgister.", "FractMus", MB_OK+MB_ICONEXCLAMATION);

	return;
#endif

#ifndef NOTREGISTERED

	UpdateSettings = !UpdateSettings;

#endif
		
}

void CFractMusView::OnUpdateUpdatesettings(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument();
	pCmdUI->SetCheck(UpdateSettings);
	pCmdUI->Enable((pDoc->m_NumEvents>-1)&&(PlayChoice == COMPOSITION));
	
}

void CFractMusView::OnCompInfo() 
{

	 CInfoDlg dlg;
	 dlg.m_pParent = this;
	 dlg.DoModal();
	
}

void CFractMusView::OnUpdateCompInfo(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument();
	pCmdUI->Enable((pDoc->m_NumEvents>-1)&&(!IsDocumentLocked));	
}

void CFractMusView::OnRandomize() 
{
	m_randDlg.m_Parent = this;

	m_randDlg.DoModal();
	
}

void CFractMusView::OnUpdateRandomize(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!m_Playing&&(!IsDocumentLocked));
	
}

void CFractMusView::OnAlgorithmsVoice1Henon() 
{
	UpdateAlgorithm(VOICE1, HENON);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice1Henon(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE1] == HENON);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice2Henon() 
{
	UpdateAlgorithm(VOICE2, HENON);
	
}

void CFractMusView::OnAlgorithmsVoice3Henon() 
{
	UpdateAlgorithm(VOICE3, HENON);
	
}

void CFractMusView::OnAlgorithmsVoice4Henon() 
{
	UpdateAlgorithm(VOICE4, HENON);
	
}

void CFractMusView::OnAlgorithmsVoice5Henon() 
{
	UpdateAlgorithm(VOICE5, HENON);
	
}

void CFractMusView::OnAlgorithmsVoice6Henon() 
{
	UpdateAlgorithm(VOICE6, HENON);
	
}

void CFractMusView::OnAlgorithmsVoice7Henon() 
{
	UpdateAlgorithm(VOICE7, HENON);
	
}

void CFractMusView::OnAlgorithmsVoice8Henon() 
{
	UpdateAlgorithm(VOICE8, HENON);
	
}

void CFractMusView::OnAlgorithmsVoice9Henon() 
{
	UpdateAlgorithm(VOICE9, HENON);
	
}

void CFractMusView::OnAlgorithmsVoice10Henon() 
{
	UpdateAlgorithm(VOICE10, HENON);
	
}

void CFractMusView::OnAlgorithmsVoice11Henon() 
{
	UpdateAlgorithm(VOICE11, HENON);
	
}

void CFractMusView::OnAlgorithmsVoice12Henon() 
{
	UpdateAlgorithm(VOICE12, HENON);
	
}

void CFractMusView::OnAlgorithmsVoice13Henon() 
{
	UpdateAlgorithm(VOICE13, HENON);
	
}

void CFractMusView::OnAlgorithmsVoice14Henon() 
{
	UpdateAlgorithm(VOICE14, HENON);
	
}

void CFractMusView::OnAlgorithmsVoice15Henon() 
{
	UpdateAlgorithm(VOICE15, HENON);
	
}

void CFractMusView::OnAlgorithmsVoice16Henon() 
{
	UpdateAlgorithm(VOICE16, HENON);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice2Henon(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE2] == HENON);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice3Henon(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE3] == HENON);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice4Henon(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE4] == HENON);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice5Henon(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE5] == HENON);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice6Henon(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE6] == HENON);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice7Henon(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE7] == HENON);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice8Henon(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE8] == HENON);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice9Henon(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE9] == HENON);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice10Henon(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE10] == HENON);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice11Henon(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE11] == HENON);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice12Henon(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE12] == HENON);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice13Henon(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE13] == HENON);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice14Henon(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE14] == HENON);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice15Henon(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE15] == HENON);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice16Henon(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE16] == HENON);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice1Ginger() 
{
	UpdateAlgorithm(VOICE1, GINGERBREADMAN);
	
}

void CFractMusView::OnAlgorithmsVoice1Hopalong() 
{
	UpdateAlgorithm(VOICE1, HOPALONG);
	
}

void CFractMusView::OnAlgorithmsVoice1Martin() 
{
	UpdateAlgorithm(VOICE1, MARTIN);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice1Ginger(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE1] == GINGERBREADMAN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice1Hopalong(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE1] == HOPALONG);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice1Martin(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE1] == MARTIN);
	pCmdUI->Enable(!m_Playing);
}

void CFractMusView::OnAlgorithmsVoice2Ginger() 
{
	UpdateAlgorithm(VOICE2, GINGERBREADMAN);
	
}

void CFractMusView::OnAlgorithmsVoice2Hopalong() 
{
	UpdateAlgorithm(VOICE2, HOPALONG);
	
}

void CFractMusView::OnAlgorithmsVoice2Martin() 
{
	UpdateAlgorithm(VOICE2, MARTIN);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice2Ginger(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE2] == GINGERBREADMAN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice2Hopalong(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE2] == HOPALONG);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice2Martin(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE2] == MARTIN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice3Ginger() 
{
	UpdateAlgorithm(VOICE3, GINGERBREADMAN);
	
}

void CFractMusView::OnAlgorithmsVoice4Ginger() 
{
	UpdateAlgorithm(VOICE4, GINGERBREADMAN);
	
}

void CFractMusView::OnAlgorithmsVoice5Ginger() 
{
	UpdateAlgorithm(VOICE5, GINGERBREADMAN);
	
}

void CFractMusView::OnAlgorithmsVoice6Ginger() 
{
	UpdateAlgorithm(VOICE6, GINGERBREADMAN);
	
}

void CFractMusView::OnAlgorithmsVoice7Ginger() 
{
	UpdateAlgorithm(VOICE7, GINGERBREADMAN);
	
}

void CFractMusView::OnAlgorithmsVoice8Ginger() 
{
	UpdateAlgorithm(VOICE8, GINGERBREADMAN);
	
}

void CFractMusView::OnAlgorithmsVoice9Ginger() 
{
	UpdateAlgorithm(VOICE9, GINGERBREADMAN);
	
}

void CFractMusView::OnAlgorithmsVoice10Ginger() 
{
	UpdateAlgorithm(VOICE10, GINGERBREADMAN);
	
}

void CFractMusView::OnAlgorithmsVoice11Ginger() 
{
	UpdateAlgorithm(VOICE11, GINGERBREADMAN);
	
}

void CFractMusView::OnAlgorithmsVoice12Ginger() 
{
	UpdateAlgorithm(VOICE12, GINGERBREADMAN);
	
}

void CFractMusView::OnAlgorithmsVoice13Ginger() 
{
	UpdateAlgorithm(VOICE13, GINGERBREADMAN);
	
}

void CFractMusView::OnAlgorithmsVoice14Ginger() 
{
	UpdateAlgorithm(VOICE14, GINGERBREADMAN);
	
}

void CFractMusView::OnAlgorithmsVoice15Ginger() 
{
	UpdateAlgorithm(VOICE15, GINGERBREADMAN);
	
}

void CFractMusView::OnAlgorithmsVoice16Ginger() 
{
	UpdateAlgorithm(VOICE16, GINGERBREADMAN);
	
}

void CFractMusView::OnAlgorithmsVoice3Hopalong() 
{
	UpdateAlgorithm(VOICE3, HOPALONG);
	
}

void CFractMusView::OnAlgorithmsVoice4Hopalong() 
{
	UpdateAlgorithm(VOICE4, HOPALONG);
	
}

void CFractMusView::OnAlgorithmsVoice5Hopalong() 
{
	UpdateAlgorithm(VOICE5, HOPALONG);
	
}

void CFractMusView::OnAlgorithmsVoice6Hopalong() 
{
	UpdateAlgorithm(VOICE6, HOPALONG);
	
}

void CFractMusView::OnAlgorithmsVoice7Hopalong() 
{
	UpdateAlgorithm(VOICE7, HOPALONG);
	
}

void CFractMusView::OnAlgorithmsVoice8Hopalong() 
{
	UpdateAlgorithm(VOICE8, HOPALONG);
	
}

void CFractMusView::OnAlgorithmsVoice9Hopalong() 
{
	UpdateAlgorithm(VOICE9, HOPALONG);
	
}

void CFractMusView::OnAlgorithmsVoice10Hopalong() 
{
	UpdateAlgorithm(VOICE10, HOPALONG);
	
}

void CFractMusView::OnAlgorithmsVoice11Hopalong() 
{
	UpdateAlgorithm(VOICE11, HOPALONG);
	
}

void CFractMusView::OnAlgorithmsVoice12Hopalong() 
{
	UpdateAlgorithm(VOICE12, HOPALONG);
	
}

void CFractMusView::OnAlgorithmsVoice13Hopalong() 
{
	UpdateAlgorithm(VOICE13, HOPALONG);
	
}

void CFractMusView::OnAlgorithmsVoice14Hopalong() 
{
	UpdateAlgorithm(VOICE14, HOPALONG);
	
}

void CFractMusView::OnAlgorithmsVoice15Hopalong() 
{
	UpdateAlgorithm(VOICE15, HOPALONG);
	
}

void CFractMusView::OnAlgorithmsVoice16Hopalong() 
{
	UpdateAlgorithm(VOICE16, HOPALONG);
	
}

void CFractMusView::OnAlgorithmsVoice3Martin() 
{
	UpdateAlgorithm(VOICE3, MARTIN);
	
}

void CFractMusView::OnAlgorithmsVoice4Martin() 
{
	UpdateAlgorithm(VOICE4, MARTIN);
	
}

void CFractMusView::OnAlgorithmsVoice5Martin() 
{
	UpdateAlgorithm(VOICE5, MARTIN);
	
}

void CFractMusView::OnAlgorithmsVoice6Martin() 
{
	UpdateAlgorithm(VOICE6, MARTIN);
	
}

void CFractMusView::OnAlgorithmsVoice7Martin() 
{
	UpdateAlgorithm(VOICE7, MARTIN);
	
}

void CFractMusView::OnAlgorithmsVoice8Martin() 
{
	UpdateAlgorithm(VOICE8, MARTIN);
	
}

void CFractMusView::OnAlgorithmsVoice9Martin() 
{
	UpdateAlgorithm(VOICE9, MARTIN);
	
}

void CFractMusView::OnAlgorithmsVoice10Martin() 
{
	UpdateAlgorithm(VOICE10, MARTIN);
	
}

void CFractMusView::OnAlgorithmsVoice11Martin() 
{
	UpdateAlgorithm(VOICE11, MARTIN);
	
}

void CFractMusView::OnAlgorithmsVoice12Martin() 
{
	UpdateAlgorithm(VOICE12, MARTIN);
	
}

void CFractMusView::OnAlgorithmsVoice13Martin() 
{
	UpdateAlgorithm(VOICE13, MARTIN);
	
}

void CFractMusView::OnAlgorithmsVoice14Martin() 
{
	UpdateAlgorithm(VOICE14, MARTIN);
	
}

void CFractMusView::OnAlgorithmsVoice15Martin() 
{
	UpdateAlgorithm(VOICE15, MARTIN);
	
}

void CFractMusView::OnAlgorithmsVoice16Martin() 
{
	UpdateAlgorithm(VOICE16, MARTIN);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice3Ginger(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE3] == GINGERBREADMAN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice4Ginger(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE4] == GINGERBREADMAN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice5Ginger(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE5] == GINGERBREADMAN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice6Ginger(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE6] == GINGERBREADMAN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice7Ginger(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE7] == GINGERBREADMAN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice8Ginger(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE8] == GINGERBREADMAN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice9Ginger(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE9] == GINGERBREADMAN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice10Ginger(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE10] == GINGERBREADMAN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice11Ginger(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE11] == GINGERBREADMAN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice12Ginger(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE12] == GINGERBREADMAN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice13Ginger(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE13] == GINGERBREADMAN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice14Ginger(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE14] == GINGERBREADMAN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice15Ginger(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE15] == GINGERBREADMAN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice16Ginger(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE16] == GINGERBREADMAN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice3Hopalong(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE3] == HOPALONG);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice4Hopalong(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE4] == HOPALONG);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice5Hopalong(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE5] == HOPALONG);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice6Hopalong(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE6] == HOPALONG);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice7Hopalong(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE7] == HOPALONG);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice8Hopalong(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE8] == HOPALONG);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice9Hopalong(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE9] == HOPALONG);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice10Hopalong(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE10] == HOPALONG);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice11Hopalong(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE11] == HOPALONG);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice12Hopalong(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE12] == HOPALONG);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice13Hopalong(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE13] == HOPALONG);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice14Hopalong(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE14] == HOPALONG);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice15Hopalong(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE15] == HOPALONG);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice16Hopalong(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE16] == HOPALONG);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice3Martin(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE3] == MARTIN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice4Martin(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE4] == MARTIN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice5Martin(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE5] == MARTIN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice6Martin(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE6] == MARTIN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice7Martin(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE7] == MARTIN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice8Martin(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE8] == MARTIN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice9Martin(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE9] == MARTIN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice10Martin(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE10] == MARTIN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice11Martin(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE11] == MARTIN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice12Martin(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE12] == MARTIN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice13Martin(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE13] == MARTIN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice14Martin(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE14] == MARTIN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice15Martin(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE15] == MARTIN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice16Martin(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE16] == MARTIN);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnPlayfromBt() 
{
	CFractMusDoc* pDoc = GetDocument();
	CComboBox* pB;
	CComboBox* pCB = (CComboBox*)GetDlgItem(IDC_PLAYFROM_CMB);
	CButton* pButt;
	CEdit* pE;
	CScrollBar* pS;
	CStatic* pSt = (CStatic*)(GetDlgItem(IDC_TEMPO_STATIC));


	int sel = pCB->GetCurSel();

	if (sel == LB_ERR)
		return;

	Tempo = m_TempoMarking = pDoc->m_TempoMarking = pDoc->Event[sel].Tempo;
	GeneralCounter = pDoc->dGeneralCounter = pDoc->Event[sel].genCount;
	m_StartFrom = StartCounter = pDoc->dStartCounter = pDoc->Event[sel].count;

	pS = (CScrollBar*)(GetDlgItem(IDC_TEMPO_SLIDER2));
	pS->SetScrollPos(Tempo);


	char s[10];

	itoa(Tempo, s, 10);
	pSt->SetWindowText(s);

	pDoc->m_TimeSigNum = pDoc->Event[sel].TimeSigNum;
	pDoc->m_TimeSigDen = pDoc->Event[sel].TimeSigDen;
	pB = (CComboBox*)(GetDlgItem(IDC_TIMESIGNUM_CMB));
	pB->SetCurSel((pDoc->m_TimeSigNum)-1);
	pB = (CComboBox*)(GetDlgItem(IDC_TIMESIGDEN_CMB));
	pB->SetCurSel(int(log10(double(pDoc->m_TimeSigDen))/log10((double)2)));

	for(int i = VOICE1; i<MAXVOICES; i++)
	{
		pDoc->AlgorithmNum[i] = pDoc->Event[sel].AlgorithmNum[i];
		pDoc->m_IsActive[i] = m_IsActive[i] = pDoc->Event[sel].IsActive[i];
		pDoc->m_Base[i] = pDoc->Event[sel].Base[i];
		pDoc->m_Multi[i] =  pDoc->Event[sel].Multi[i];
		pDoc->m_StartNum[i] = m_StartNum[i] = pDoc->Event[sel].StartNum[i];
		pDoc->m_NumOcts[i] = pDoc->Event[sel].NumOcts[i];
		pDoc->m_Modulates[i] = m_Modulates[i] = pDoc->Event[sel].Modulates[i] ;
		pDoc->m_ModulRandom[i] = m_V1Modul[i].m_V1ModulDlgChoice = pDoc->Event[sel].ModulRandom[i] ;
		pDoc->m_ModulNum[i] = m_V1Modul[i].m_V1ModulDlgNum = pDoc->Event[sel].ModulNum[i] ;
		pDoc->m_Note[i] = m_Note[i] = pDoc->Event[sel].Note[i] ;
		pDoc->m_NoteValue[i] = pDoc->Event[sel].NoteValue[i] ;
		pDoc->m_Instrument[i] = m_V1MidiSet[i].m_instrument  = pDoc->Event[sel].Instrument[i];
		pDoc->m_Channel[i] = pDoc->Event[sel].Channel[i];
		pDoc->m_mode[i] = m_V1MidiSet[i].m_mode = pDoc->Event[sel].Mode[i];
		pDoc->m_ScaleID[i] = m_ScaleID[i] = pDoc->Event[sel].ScaleID[i];
		pDoc->m_NumOctsID[i] = m_NumOctsID[i] = pDoc->Event[sel].NumOctsID[i];
		pDoc->m_NoteValueID[i] = m_NoteValueID[i] = pDoc->Event[sel].NoteValueID[i];
		pDoc->m_Inverted[i] = pDoc->Event[sel].Inverted[i];
		pDoc->m_MatchScale[i] = pDoc->Event[sel].MatchScale[i];
		pDoc->m_Dynamic[i] = pDoc->Event[sel].Dynamic[i];
		pDoc->m_DynChoice[i] = pDoc->Event[sel].DynChoice[i];
		pDoc->m_LogMapVar[i] = pDoc->Event[sel].LogMapVar[i];
		pDoc->m_FNoiseVar[i] = pDoc->Event[sel].FNoiseVar[i];
		pDoc->m_LorentzVar[i] = pDoc->Event[sel].LorentzVar[i];
		pDoc->m_HenonA[i] = pDoc->Event[sel].HenonA[i];
		pDoc->m_HenonB[i] = pDoc->Event[sel].HenonB[i];
		pDoc->m_HopalongVar[i] = pDoc->Event[sel].HopalongVar[i];
		pDoc->m_MartinVar[i] = pDoc->Event[sel].MartinVar[i];
		pDoc->m_GingerX[i] = pDoc->Event[sel].GingerX[i];
		pDoc->m_GingerY[i] = pDoc->Event[sel].GingerY[i];
		pDoc->m_EarthWormVar[i] = pDoc->Event[sel].EarthWormVar[i];
		pDoc->m_EarthWormMulti[i] = pDoc->Event[sel].EarthWormMulti[i];
		pDoc->m_EarthWormTruncator[i] = pDoc->Event[sel].EarthWormTruncator[i];
		pDoc->m_RandomMin[i] = pDoc->Event[sel].RandomMin[i];
		pDoc->m_RandomMax[i] = pDoc->Event[sel].RandomMax[i];
		pDoc->m_WolframRule[i] = pDoc->Event[sel].WolframRule[i];
		pDoc->m_WolframInit[i] = pDoc->Event[sel].WolframInit[i];
		pDoc->m_WolframSize[i] = pDoc->Event[sel].WolframSize[i];
		pDoc->m_Hailstone[i] = pDoc->Event[sel].Hailstone[i];
		pDoc->m_Pan[i] = pDoc->Event[sel].Pan[i];
		pDoc->m_Percussion[i] = pDoc->Event[sel].Percussion[i];
	}

	UpdateData(FALSE);

	
	for(i = VOICE1; i<MAXVOICES; i++)
	{
		pButt = (CButton*)GetDlgItem(ActiveID[i]);

		if (pDoc->m_Channel[i] == PercussionChannel)
		{
			pButt->SetWindowText(PercussionNames[ pDoc->m_Percussion[i] ]);
		}
		else
			pButt->SetWindowText(InstrumentNames[ pDoc->m_Instrument[i] ]);
	}



	for(i = VOICE1; i<MAXVOICES; i++)
	{

		pE = (CEdit*)(GetDlgItem(StartNumID[i]))->EnableWindow(m_IsActive[i]&&(!IsDocumentLocked));
		pButt = (CButton*)(GetDlgItem(ModulatesID[i]))->EnableWindow(m_IsActive[i]&&(!IsDocumentLocked));
		pB = (CComboBox*)(GetDlgItem(NumOctsID[i]));
		pB->SetCurSel(m_NumOctsID[i]);
		pB = (CComboBox*)(GetDlgItem(NumOctsID[i]))->EnableWindow(m_IsActive[i]&&(!IsDocumentLocked));
		pB = (CComboBox*)(GetDlgItem(NoteID[i]));
		pB->SetCurSel(m_Note[i]);
		pB = (CComboBox*)(GetDlgItem(NoteID[i]))->EnableWindow(m_IsActive[i]&&(!IsDocumentLocked));
		pB = (CComboBox*)(GetDlgItem(ScaleID[i]));
		pB->SetCurSel(m_ScaleID[i]);
		pB = (CComboBox*)(GetDlgItem(ScaleID[i]))->EnableWindow(m_IsActive[i]&&(!IsDocumentLocked));
		pB = (CComboBox*)(GetDlgItem(NoteValueID[i]));
		pB->SetCurSel(m_NoteValueID[i]);
		pB = (CComboBox*)(GetDlgItem(NoteValueID[i]))->EnableWindow(m_IsActive[i]&&(!IsDocumentLocked));
		pButt = (CButton*)(GetDlgItem(MidiButtonID[i]))->EnableWindow(m_IsActive[i]&&(!IsDocumentLocked));

	}


	for (i = VOICE1; i<MAXVOICES; i++)
	{
		CString nval;

		if (m_NoteValueID[i] == OTHER)
		{
			nval.Format("Other (%d)", (long)pDoc->m_NoteValue[i]);
			pB = (CComboBox*)GetDlgItem(NoteValueID[i]);
			pB->DeleteString(OTHER);
			pB->AddString(nval);
			pB->SetCurSel(OTHER);
		}
	}



	for(i = VOICE1; i<MAXVOICES; i++)
	{
		if (pDoc->m_pUserScale[i])
		{
			delete pDoc->m_pUserScale[i];
			pDoc->m_pUserScale[i] = NULL;
		}
		pDoc->m_pUserScale[i] = new int[pDoc->Event[sel].pUserScale[i][0] + 1];
		pDoc->m_pUserScale[i][0] = pDoc->Event[sel].pUserScale[i][0];
		for(int j = 1; j<=pDoc->Event[sel].pUserScale[i][0]; j++)
			pDoc->m_pUserScale[i][j] = pDoc->Event[sel].pUserScale[i][j];

	}

	for(i = VOICE1; i<MAXVOICES; i++)
	{
		if (pDoc->m_pScale[i])
		{
			delete pDoc->m_pScale[i];
			pDoc->m_pScale[i] = NULL;
		}
		pDoc->m_pScale[i] = new int[pDoc->Event[sel].pScale[i][0] + 1];
		pDoc->m_pScale[i][0] = pDoc->Event[sel].pScale[i][0];
		for(int j = 1; j<=pDoc->Event[sel].pScale[i][0]; j++)
			pDoc->m_pScale[i][j] = pDoc->Event[sel].pScale[i][j];
	/*	if (pDoc->m_ScaleID[i] == USER_DEFINED)
		{
			InitUserScale(i);
			SetUserScale(i);
		}*/
	}
	
}

void CFractMusView::OnAlgorithmsVoice1Lorentz() 
{
	UpdateAlgorithm(VOICE1, LORENTZ);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice1Lorentz(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE1] == LORENTZ);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice2Lorentz() 
{
	UpdateAlgorithm(VOICE2, LORENTZ);
	
}

void CFractMusView::OnAlgorithmsVoice3Lorentz() 
{
	UpdateAlgorithm(VOICE3, LORENTZ);
	
}

void CFractMusView::OnAlgorithmsVoice4Lorentz() 
{
	UpdateAlgorithm(VOICE4, LORENTZ);
	
}

void CFractMusView::OnAlgorithmsVoice5Lorentz() 
{
	UpdateAlgorithm(VOICE5, LORENTZ);
	
}

void CFractMusView::OnAlgorithmsVoice6Lorentz() 
{
	UpdateAlgorithm(VOICE6, LORENTZ);
	
}

void CFractMusView::OnAlgorithmsVoice7Lorentz() 
{
	UpdateAlgorithm(VOICE7, LORENTZ);
	
}

void CFractMusView::OnAlgorithmsVoice8Lorentz() 
{
	UpdateAlgorithm(VOICE8, LORENTZ);
	
}

void CFractMusView::OnAlgorithmsVoice9Lorentz() 
{
	UpdateAlgorithm(VOICE9, LORENTZ);
	
}

void CFractMusView::OnAlgorithmsVoice10Lorentz() 
{
	UpdateAlgorithm(VOICE10, LORENTZ);
	
}

void CFractMusView::OnAlgorithmsVoice11Lorentz() 
{
	UpdateAlgorithm(VOICE11, LORENTZ);
	
}

void CFractMusView::OnAlgorithmsVoice12Lorentz() 
{
	UpdateAlgorithm(VOICE12, LORENTZ);
	
}

void CFractMusView::OnAlgorithmsVoice13Lorentz() 
{
	UpdateAlgorithm(VOICE13, LORENTZ);
	
}

void CFractMusView::OnAlgorithmsVoice14Lorentz() 
{
	UpdateAlgorithm(VOICE14, LORENTZ);
	
}

void CFractMusView::OnAlgorithmsVoice15Lorentz() 
{
	UpdateAlgorithm(VOICE15, LORENTZ);
	
}

void CFractMusView::OnAlgorithmsVoice16Lorentz() 
{
	UpdateAlgorithm(VOICE16, LORENTZ);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice2Lorentz(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE2] == LORENTZ);
	pCmdUI->Enable(!m_Playing);	
}

void CFractMusView::OnUpdateAlgorithmsVoice3Lorentz(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE3] == LORENTZ);
	pCmdUI->Enable(!m_Playing);	
}

void CFractMusView::OnUpdateAlgorithmsVoice4Lorentz(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE4] == LORENTZ);
	pCmdUI->Enable(!m_Playing);	
}

void CFractMusView::OnUpdateAlgorithmsVoice5Lorentz(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE5] == LORENTZ);
	pCmdUI->Enable(!m_Playing);	
}

void CFractMusView::OnUpdateAlgorithmsVoice6Lorentz(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE6] == LORENTZ);
	pCmdUI->Enable(!m_Playing);	
}

void CFractMusView::OnUpdateAlgorithmsVoice7Lorentz(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE7] == LORENTZ);
	pCmdUI->Enable(!m_Playing);	
}

void CFractMusView::OnUpdateAlgorithmsVoice8Lorentz(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE8] == LORENTZ);
	pCmdUI->Enable(!m_Playing);	
}

void CFractMusView::OnUpdateAlgorithmsVoice9Lorentz(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE9] == LORENTZ);
	pCmdUI->Enable(!m_Playing);	
}

void CFractMusView::OnUpdateAlgorithmsVoice10Lorentz(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE10] == LORENTZ);
	pCmdUI->Enable(!m_Playing);	
}

void CFractMusView::OnUpdateAlgorithmsVoice11Lorentz(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE11] == LORENTZ);
	pCmdUI->Enable(!m_Playing);	
}

void CFractMusView::OnUpdateAlgorithmsVoice12Lorentz(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE12] == LORENTZ);
	pCmdUI->Enable(!m_Playing);	
}

void CFractMusView::OnUpdateAlgorithmsVoice13Lorentz(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE13] == LORENTZ);
	pCmdUI->Enable(!m_Playing);	
}

void CFractMusView::OnUpdateAlgorithmsVoice14Lorentz(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE14] == LORENTZ);
	pCmdUI->Enable(!m_Playing);	
}

void CFractMusView::OnUpdateAlgorithmsVoice15Lorentz(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE15] == LORENTZ);
	pCmdUI->Enable(!m_Playing);	
}

void CFractMusView::OnUpdateAlgorithmsVoice16Lorentz(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE16] == LORENTZ);
	pCmdUI->Enable(!m_Playing);	
}


void CFractMusView::OnKillfocusStartcounterEd() 
{
	if(!UpdateData(TRUE))
		return;
	CFractMusDoc* pDoc = GetDocument();
	pDoc->dStartCounter = StartCounter;
	pDoc->SetModifiedFlag();
	
}

void CFractMusView::OnOptsGetnote() 
{
	if(!UpdateData(TRUE))
		return;

	OnKillfocusGencounterEd();
	OnKillfocusStartcounterEd();

	OnKillfocusVoice1StartnumEdit();

	OnKillfocusVoice2StartnumEdit();
	OnKillfocusVoice3StartnumEdit();

	OnKillfocusVoice4StartnumEdit();

	OnKillfocusVoice5StartnumEdit();

	OnKillfocusVoice6StartnumEdit();

	OnKillfocusVoice7StartnumEdit();

	OnKillfocusVoice8StartnumEdit();

	OnKillfocusVoice9StartnumEdit();

	OnKillfocusVoice10StartnumEdit();

	OnKillfocusVoice11StartnumEdit();

	OnKillfocusVoice12StartnumEdit();

	OnKillfocusVoice13StartnumEdit();

	OnKillfocusVoice14StartnumEdit();

	OnKillfocusVoice15StartnumEdit();

	OnKillfocusVoice16StartnumEdit();


	CGetNoteDlg dlg;
	dlg.pView = this;

	dlg.DoModal();
}

void CFractMusView::OnUpdateOptsGetnote(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!m_Playing&&(!IsDocumentLocked));
	
}

void CFractMusView::OnGlobaltempoBt() 
{
	CFractMusDoc* pDoc = GetDocument();

	if (IsDocumentLocked)
	{
		MessageBox("You must first unlock the document", "FractMus", MB_OK+MB_ICONEXCLAMATION);
		return;
	}

	if (pDoc->m_NumEvents<0)
	{
		MessageBox("You have not created a composition yet", "Error",MB_OK+MB_ICONEXCLAMATION);
		return;
	}

	CString msg;

	msg.Format("Do you want to globally change your composition's tempo to %d?", pDoc->m_TempoMarking);


	if(MessageBox(msg, "FractMus", MB_YESNO+MB_ICONQUESTION) == IDYES)
	{
		for(int i = 0; i<=pDoc->m_NumEvents; i++)
			pDoc->Event[i].Tempo = pDoc->m_TempoMarking;
	}

	
}

void CFractMusView::OnGlobalChanges() 
{
	if (!UpdateData())
		return;


	m_changesDlg.pView = this;
	m_changesDlg.DoModal();
	
}

void CFractMusView::OnUpdateGlobalChanges(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!m_Playing&&(!IsDocumentLocked));
	
}



void CFractMusView::OnFilePlaymidi() 
{
	char FileName[500];
	char FileTitle[100];

	OPENFILENAME ofn;
	memset(&ofn, 0, sizeof(ofn));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = NULL;
	ofn.hInstance = NULL;

	ofn.lpstrFilter = 
		TEXT("Midi files *.mid\0*.mid\0\0");

	ofn.lpstrCustomFilter = NULL;
	ofn.nMaxCustFilter = 0;
	ofn.nFilterIndex = 1;
	ofn.lpstrFile = FileName;
	ofn.nMaxFile = 500;
	ofn.lpstrFileTitle = FileTitle;
	ofn.nMaxFileTitle = 99;
	ofn.lpstrInitialDir = NULL;
	ofn.lpstrTitle="Open midi file";
	ofn.Flags = OFN_FILEMUSTEXIST|OFN_LONGNAMES|OFN_PATHMUSTEXIST;
	ofn.lpstrDefExt = "mid";
	ofn.lCustData = NULL;
	ofn.lpfnHook = NULL;
	ofn.lpTemplateName = NULL;

	FileName[0] = '\0';

	if(!GetOpenFileName(&ofn))
		return;

	if(FileName[0] == '\0')
		return;

	if ( ShellExecute(NULL, "open", "mplayer.exe", FileName, NULL, SW_SHOW) == (HINSTANCE__ *)ERROR_FILE_NOT_FOUND)
	{
		MessageBox("mplayer.exe was not found", "FractMus", MB_OK+MB_ICONEXCLAMATION);
	}

	midiOutClose(midiOut);

	
}

void CFractMusView::OnUpdateFilePlaymidi(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!m_Playing);
	
}



void CFractMusView::OnCalculator() 
{
	if ( ShellExecute(NULL, "open", "calc.exe", NULL, NULL, SW_SHOW) == (HINSTANCE__ *)ERROR_FILE_NOT_FOUND)
	{
		MessageBox("calc.exe was not found", "FractMus", MB_OK+MB_ICONEXCLAMATION);
	}
	
}




void CFractMusView::OnOptsPercussion() 
{
	CPercussionDlg dlg;

	dlg.m_sel = PercussionChannel;

	dlg.DoModal();

	PercussionChannel = dlg.m_sel;
	
}

void CFractMusView::OnUpdateOptsPercussion(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!m_Playing);
	
}








////////////////////////////////








/* Functions to implement in order to write a MIDI file */





/* private stuff */




/*
 * mfwrite() - The only fuction you'll need to call to write out
 *             a midi file.
 *
 * format      0 - Single multi-channel track
 *             1 - Multiple simultaneous tracks
 *             2 - One or more sequentially independent
 *                 single track patterns                
 * ntracks     The number of tracks in the file.
 * division    This is kind of tricky, it can represent two
 *             things, depending on whether it is positive or negative
 *             (bit 15 set or not).  If  bit  15  of division  is zero,
 *             bits 14 through 0 represent the number of delta-time
 *             "ticks" which make up a quarter note.  If bit  15 of
 *             division  is  a one, delta-times in a file correspond to
 *             subdivisions of a second similiar to  SMPTE  and  MIDI
 *             time code.  In  this format bits 14 through 8 contain
 *             one of four values - 24, -25, -29, or -30,
 *             corresponding  to  the  four standard  SMPTE and MIDI
 *             time code frame per second formats, where  -29
 *             represents  30  drop  frame.   The  second  byte
 *             consisting  of  bits 7 through 0 corresponds the the
 *             resolution within a frame.  Refer the Standard MIDI
 *             Files 1.0 spec for more details.
 * fp          This should be the open file pointer to the file you
 *             want to write.  It will have be a global in order
 *             to work with Mf_putc.  
 */ 


/**
void mf_write_track_chunk(int which_track,FILE* fp)
{
	unsigned long trkhdr,trklength;
	long offset, place_marker;
	
	
	trkhdr = MTrk;
	trklength = 0;

	//Remember where the length was written, because we don't
//	   know how long it will be until we've finished writing
	offset = ftell(fp); 

#ifdef DEBUG
        printf("offset = %d\n",(int) offset);
#endif

	// Write the track chunk header 
	write32bit(trkhdr);
	write32bit(trklength);

	Mf_numbyteswritten = 0L;  //the header's length doesn't count 

	WriteTrack(which_track);

	// mf_write End of track meta event 
	mf_write_meta_event(0L, end_of_track, NULL, 0L);
	// It's impossible to know how long the track chunk will be beforehand,
     //      so the position of the track length data is kept so that it can
     //      be written after the chunk has been generated /
	place_marker = ftell(fp);
	

#ifdef DEBUG
printf("length = %d\n",(int) trklength);
#endif

 	if(fseek(fp,offset,0) < 0)
	    //mferror("error seeking during final stage of write");

	trklength = Mf_numbyteswritten;

	// Re-mf_write the track chunk header with right length 
	write32bit(trkhdr);
	write32bit(trklength);

	fseek(fp,place_marker,0);
} **/


void mf_write_header_chunk(int format,int ntracks,int division)
{
    unsigned long ident,length;
    
    ident = MThd;           /* Head chunk identifier                    */
    length = 6;             /* Chunk length                             */

    /* individual bytes of the header must be written separately
       to preserve byte order across cpu types :-( */
    write32bit(ident);
    write32bit(length);
    write16bit(format);
    write16bit(ntracks);
    write16bit(division);
} /* end gen_header_chunk() */


/*
 * mf_write_midi_event()
 * 
 * Library routine to mf_write a single MIDI track event in the standard MIDI
 * file format. The format is:
 *
 *                    <delta-time><event>
 *
 * In this case, event can be any multi-byte midi message, such as
 * "note on", "note off", etc.      
 *
 * delta_time - the time in ticks since the last event.
 * type - the type of meta event.
 * chan - The midi channel.
 * data - A pointer to a block of chars containing the META EVENT,
 *        data.
 * size - The length of the meta-event data.
 */
int mf_write_midi_event(unsigned long delta_time,
						unsigned int type,
						unsigned int chan,
						unsigned char *data,
						unsigned long size)
{
    int i;
    unsigned char c;

    WriteVarLen(delta_time);

    /* all MIDI events start with the type in the first four bits,
       and the channel in the lower four bits */
    c = type | chan;

    if(chan > 15)
        perror("error: MIDI channel greater than 16\n");

    eputc(c);

    /* write out the data bytes */
    for(i = 0; i < size; i++)
	eputc(data[i]);

    return(size);
} /* end mf_write MIDI event */

/*
 * mf_write_meta_event()
 *
 * Library routine to mf_write a single meta event in the standard MIDI
 * file format. The format of a meta event is:
 *
 *          <delta-time><FF><type><length><bytes>
 *
 * delta_time - the time in ticks since the last event.
 * type - the type of meta event.
 * data - A pointer to a block of chars containing the META EVENT,
 *        data.
 * size - The length of the meta-event data.
 */
int
mf_write_meta_event(unsigned long delta_time,
					unsigned char type,
					unsigned char *data,
					unsigned long size)
{
    int i;
    WriteVarLen(delta_time);

    /* This marks the fact we're writing a meta-event */
    eputc(meta_event);

    /* The type of meta event */
    eputc(type);

    /* The length of the data bytes to follow */
    WriteVarLen(size);

    for(i = 0; i < size; i++)
    {
	if(eputc(data[i]) != data[i])
	    return(-1);
    }
    return(size);
} /* end mf_write_meta_event */

void mf_write_tempo(unsigned long tempo, unsigned long delta_time)
{
    /* Write tempo */
    /* all tempos are written as 120 beats/minute, */
    /* expressed in microseconds/quarter note     */
    WriteVarLen(delta_time); ///??? eputc(0); //
    eputc(meta_event);
    eputc(set_tempo);

    eputc(3);
    eputc((unsigned)(0xff & (tempo >> 16)));
    eputc((unsigned)(0xff & (tempo >> 8)));
    eputc((unsigned)(0xff & tempo));
}


/*
 * Write multi-length bytes to MIDI format files
 */
void WriteVarLen(unsigned long value)
{
  unsigned long buffer;

  buffer = value & 0x7f;
  while((value >>= 7) > 0)
  {
	buffer <<= 8;
	buffer |= 0x80;
	buffer += (value & 0x7f);
  }
  while(1){
       eputc((unsigned)(buffer & 0xff));
       
	if(buffer & 0x80)
		buffer >>= 8;
	else
		return;
	}
}/* end of WriteVarLen */




/*
 * write32bit()
 * write16bit()
 *
 * These routines are used to make sure that the byte order of
 * the various data types remains constant between machines. This
 * helps make sure that the code will be portable from one system
 * to the next.  It is slightly dangerous that it assumes that longs
 * have at least 32 bits and ints have at least 16 bits, but this
 * has been true at least on PCs, UNIX machines, and Macintosh's.
 *
 */
void write32bit(unsigned long data)
{
    eputc((unsigned)((data >> 24) & 0xff));
    eputc((unsigned)((data >> 16) & 0xff));
    eputc((unsigned)((data >> 8 ) & 0xff));
    eputc((unsigned)(data & 0xff));
}

void write16bit(int data)
{
    eputc((unsigned)((data & 0xff00) >> 8));
    eputc((unsigned)(data & 0xff));
}

/* write a single character and abort on error */
int eputc(unsigned char c)			
{
	int return_val;
	
	return_val = myputc(c);

	if ( return_val == EOF )
		AfxMessageBox("error");
		
	++Mf_numbyteswritten;
	return(return_val);
}


/*void CFractMusView::OnComposers() 
{
	CComposersCreditsDlg dlg;

	dlg.DoModal();
	
}*/

void CFractMusView::OnGlobaltimesigBt() 
{
	CFractMusDoc* pDoc = GetDocument();


	if (IsDocumentLocked)
	{
		MessageBox("You must first unlock the document", "FractMus", MB_OK+MB_ICONEXCLAMATION);
		return;
	}

	if (pDoc->m_NumEvents<0)
	{
		MessageBox("You have not created a composition yet", "Error",MB_OK+MB_ICONEXCLAMATION);
		return;
	}

	CString msg;

	msg.Format("Do you want to globally change your composition's time signature to %d/%d?", pDoc->m_TimeSigNum, pDoc->m_TimeSigDen);


	if(MessageBox(msg, "FractMus", MB_YESNO+MB_ICONQUESTION) == IDYES)
	{
		for(int i = 0; i<=pDoc->m_NumEvents; i++)
		{
			pDoc->Event[i].TimeSigNum = pDoc->m_TimeSigNum;
			pDoc->Event[i].TimeSigDen = pDoc->m_TimeSigDen;
		}

	}
	
}

void CFractMusView::OnTransposer() 
{
	if (!UpdateData())
		return;

	m_transposerDlg.pView = this;
	m_transposerDlg.DoModal();
}

void CFractMusView::OnUpdateTransposer(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!m_Playing&&(!IsDocumentLocked));
	
}

void CFractMusView::OnChangePassword() 
{
	if (!FractMusREGISTERED)
	{
		CString msg;

		msg.Format("You are using a version of FractMus which\ndoes NOT include the password-protection feature.\nPlease read de README.TXT or see Help if you would like to use this feature.");
		MessageBox(msg, "FractMus", MB_OK+MB_ICONEXCLAMATION);
		return;
	}

	CChangePassword dlg;

	dlg.pView = this;

	if (dlg.DoModal() == IDOK)
	{
		CFractMusDoc* pDoc = GetDocument();
		CWnd* pW = AfxGetMainWnd();

		CString tit;

		tit = pDoc->GetTitle();

		tit+=" - FractMus 2000   "+pDoc->m_Copyright;

		pW->SetWindowText(tit);
	}





	
}

void CFractMusView::OnUpdateChangePassword(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!m_Playing&&(!IsDocumentLocked));
	
}

void CFractMusView::OnUnlockDocument() 
{
	if (!FractMusREGISTERED)
	{
		CString msg;

		msg.Format("You are using a version of FractMus which\ndoes NOT include the password-protection feature.\nPlease read de README.TXT or see Help if you would like to use this feature.");
		MessageBox(msg, "FractMus", MB_OK+MB_ICONEXCLAMATION);
		return;
	}

	CFractMusDoc* pDoc = GetDocument();
	CUnlockDocumentDlg dlg;

	dlg.pView = this;

	dlg.DoModal();

	if (IsDocumentLocked == FALSE)
	{

		CEdit* pE;
		CComboBox* pB;
		CButton* pButt;

		for(int i=VOICE1; i<MAXVOICES; i++)
		{
			pE = (CEdit*)GetDlgItem(StartNumID[i])->EnableWindow(m_IsActive[i]&&(!IsDocumentLocked));
			pButt = (CButton*)GetDlgItem(ModulatesID[i])->EnableWindow(m_IsActive[i]&&(!IsDocumentLocked));
			pB = (CComboBox*)GetDlgItem(NumOctsID[i]);
			pB->SetCurSel(m_NumOctsID[i]);
			pB = (CComboBox*)GetDlgItem(NumOctsID[i])->EnableWindow(m_IsActive[i]&&(!IsDocumentLocked));
			pB = (CComboBox*)GetDlgItem(NoteID[i]);
			pB->SetCurSel(m_Note[i]);
			pB = (CComboBox*)GetDlgItem(NoteID[i])->EnableWindow(m_IsActive[i]&&(!IsDocumentLocked));
			pB = (CComboBox*)GetDlgItem(ScaleID[i]);
			pB->SetCurSel(m_ScaleID[i]);
			pB = (CComboBox*)GetDlgItem(ScaleID[i])->EnableWindow(m_IsActive[i]&&(!IsDocumentLocked));
			pB = (CComboBox*)GetDlgItem(NoteValueID[i]);
			pB->SetCurSel(m_NoteValueID[i]);
			pB = (CComboBox*)GetDlgItem(NoteValueID[i])->EnableWindow(m_IsActive[i]&&(!IsDocumentLocked));
			pButt = (CButton*)GetDlgItem(MidiButtonID[i])->EnableWindow(m_IsActive[i]&&(!IsDocumentLocked));
			pButt = (CButton*)GetDlgItem(ActiveID[i])->EnableWindow((!IsDocumentLocked));

			if (m_pEvents)
			{
				m_pEvents->GetDlgItem(IDC_ADD)->EnableWindow(!IsDocumentLocked);
				m_pEvents->GetDlgItem(IDC_INSERT)->EnableWindow(!IsDocumentLocked);
				m_pEvents->GetDlgItem(IDC_REMOVE)->EnableWindow(!IsDocumentLocked);
				m_pEvents->GetDlgItem(IDC_REPLACE_BT)->EnableWindow(!IsDocumentLocked);
				m_pEvents->GetDlgItem(IDC_CLEAR)->EnableWindow(!IsDocumentLocked);
			}

		}

		CWnd* pWin = AfxGetMainWnd();
		CMenu* pM = pWin->GetMenu();
		if (pM)
		{
			pM->EnableMenuItem(2, (IsDocumentLocked? MF_BYPOSITION|MF_DISABLED : MF_BYPOSITION|MF_ENABLED));
		}
		else
			MessageBox("Null pointer");


	}

	
}

void CFractMusView::OnUpdateUnlockDocument(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!m_Playing&&(IsDocumentLocked));
	
}

void CFractMusView::OnRoundEvents() 
{
	CRoundEventCountersDlg dlg;
	dlg.pView = this;
	dlg.DoModal();
}

void CFractMusView::OnUpdateRoundEvents(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument();

	pCmdUI->Enable((!m_Playing)&&(pDoc->m_NumEvents>-1)&&(!IsDocumentLocked));
	
}

void CFractMusView::OnInstrumentation() 
{
	if (!UpdateData())
		return;

	m_instrumentationDlg.pView = this;
	m_instrumentationDlg.DoModal();
	
}

void CFractMusView::OnUpdateInstrumentation(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument();

	pCmdUI->Enable((!m_Playing)&&(!IsDocumentLocked));
	
}

BOOL CFractMusView::OnToolTipNotify( UINT id, NMHDR * pNMHDR, LRESULT * pResult )
{
	if (!ShowTips)
		return FALSE;

    TOOLTIPTEXT *pTTT = (TOOLTIPTEXT *)pNMHDR;
    
	UINT nID =pNMHDR->idFrom;
    if (pTTT->uFlags & TTF_IDISHWND)    
	{
        // idFrom is actually the HWND of the tool
        nID = ::GetDlgCtrlID((HWND)nID);        
		if(nID)        
		{
			CFractMusDoc* pDoc = GetDocument();
			CString algo, coeff, dyna, inv, matchscl, mode, modul, pan, scl;
			CString msg[16];
			CString msg2[16];
			CString msg3[16];
			CString msg4[16];

			for (int i = VOICE1; i<MAXVOICES; i++)
			{
				scl = "";

				algo.Format("Algorithm: %s", AlgorithmNames[ pDoc->AlgorithmNum[i] ]);

				switch (pDoc->AlgorithmNum[i])
				{
					case MORSE_THUE:
						coeff.Format(" (%d, %d)", pDoc->m_Base[i], pDoc->m_Multi[i]);
						break;
					case LOGMAP:
						coeff.Format(" (%f)", pDoc->m_LogMapVar[i]);
						break;
					case ONE_OVER_F_NOISE:
						coeff.Format(" (%f)", pDoc->m_FNoiseVar[i]);
						break;
					case HENON:
						coeff.Format(" (%f,%f)", pDoc->m_HenonA[i], pDoc->m_HenonB[i]);
						break;
					case MARTIN:
						coeff.Format(" (%f)", pDoc->m_MartinVar[i]);
						break;
					case HOPALONG:
						coeff.Format(" (%f)", pDoc->m_HopalongVar[i]);
						break;
					case GINGERBREADMAN:
						coeff.Format(" (%f,%f)", pDoc->m_GingerX[i], pDoc->m_GingerY[i]);
						break;
					case LORENTZ:
						coeff.Format(" (%f)", pDoc->m_LorentzVar[i]);
						break;
					case EARTHWORM:
						coeff.Format(" (%d, %d, %d)", pDoc->m_EarthWormVar[i], pDoc->m_EarthWormMulti[i], pDoc->m_EarthWormTruncator[i]);
						break;
					case RANDOM:
						coeff.Format(" (%f, %f)", pDoc->m_RandomMin[i], pDoc->m_RandomMax[i]);
						break;
					case HAILSTONE:
						coeff.Format(" (%d)", pDoc->m_Hailstone[i]);
						break;
					case WOLFRAM:
						coeff.Format(" (Rule:%d, Size:%d, Init:%d)", pDoc->m_WolframRule[i], pDoc->m_WolframSize[i], pDoc->m_WolframInit[i]);
						break;

				}


				dyna.Format("Dynamic: %s   ", dyn[pDoc->m_DynChoice[i]]);

				if (pDoc->m_mode[i] == 0)
				{
					mode.Format("Mode: All notes   ");
				}
				if (pDoc->m_mode[i] == 1)
				{
					mode.Format("Mode: Tied   ");
				}

				if (pDoc->m_Inverted[i])
				{
					inv.Format("Inverted: Yes   ");
				}
				else
				{
					inv.Format("Inverted: No   ");

				}
				if(pDoc->m_MatchScale[i])
				{
					matchscl.Format("Match scale: Yes   ");
				}
				else
				{
					matchscl.Format("Match scale: No   ");

				}

				pan.Format("Pan: %d", pDoc->m_Pan[i]);

				if (pDoc->m_Modulates[i])
				{
					if(pDoc->m_ModulRandom[i])
						modul.Format("Modulates Randomly");
					else
						modul.Format("Modulates every %d", pDoc->m_ModulNum[i]);
				}
				else
					modul = "";

				if (pDoc->m_ScaleID[i] == USER_DEFINED)
				{

				    for(int v=1; v<=pDoc->m_pUserScale[i][0]; v++)
					{
						scl +=Pitches[ pDoc->m_pUserScale[i][v] ];
						scl +=" ";
					}
				}
				else
					scl = "";



				msg[i] = IsDocumentLocked ? "This document is protected" : algo+coeff;
				msg2[i] = IsDocumentLocked ? "This document is protected" : modul;
				msg3[i] = IsDocumentLocked ? "This document is protected" : dyna+mode+inv+matchscl+pan;
				msg4[i] = IsDocumentLocked ? "This document is protected" : scl;

			}//for

			switch (nID)
			{
				case IDC_VOICE1_ON_CHK:
					pTTT->lpszText = new char[msg[0].GetLength() + 1];
					strcpy(pTTT->lpszText, msg[0].GetBuffer(1));
					break;
				case IDC_VOICE2_ON_CHK:
					pTTT->lpszText = new char[msg[1].GetLength() + 1];
					strcpy(pTTT->lpszText, msg[1].GetBuffer(1));
					break;
				case IDC_VOICE3_ON_CHK:
					pTTT->lpszText = new char[msg[2].GetLength() + 1];
					strcpy(pTTT->lpszText, msg[2].GetBuffer(1));
					break;
				case IDC_VOICE4_ON_CHK:
					pTTT->lpszText = new char[msg[3].GetLength() + 1];
					strcpy(pTTT->lpszText, msg[3].GetBuffer(1));
					break;
				case IDC_VOICE5_ON_CHK:
					pTTT->lpszText = new char[msg[4].GetLength() + 1];
					strcpy(pTTT->lpszText, msg[4].GetBuffer(1));
					break;
				case IDC_VOICE6_ON_CHK:
					pTTT->lpszText = new char[msg[5].GetLength() + 1];
					strcpy(pTTT->lpszText, msg[5].GetBuffer(1));
					break;
				case IDC_VOICE7_ON_CHK:
					pTTT->lpszText = new char[msg[6].GetLength() + 1];
					strcpy(pTTT->lpszText, msg[6].GetBuffer(1));
					break;
				case IDC_VOICE8_ON_CHK:
					pTTT->lpszText = new char[msg[7].GetLength() + 1];
					strcpy(pTTT->lpszText, msg[7].GetBuffer(1));
					break;
				case IDC_VOICE9_ON_CHK:
					pTTT->lpszText = new char[msg[8].GetLength() + 1];
					strcpy(pTTT->lpszText, msg[8].GetBuffer(1));
					break;
				case IDC_VOICE10_ON_CHK:
					pTTT->lpszText = new char[msg[9].GetLength() + 1];
					strcpy(pTTT->lpszText, msg[9].GetBuffer(1));
					break;
				case IDC_VOICE11_ON_CHK:
					pTTT->lpszText = new char[msg[10].GetLength() + 1];
					strcpy(pTTT->lpszText, msg[10].GetBuffer(1));
					break;
				case IDC_VOICE12_ON_CHK:
					pTTT->lpszText = new char[msg[11].GetLength() + 1];
					strcpy(pTTT->lpszText, msg[11].GetBuffer(1));
					break;
				case IDC_VOICE13_ON_CHK:
					pTTT->lpszText = new char[msg[12].GetLength() + 1];
					strcpy(pTTT->lpszText, msg[12].GetBuffer(1));
					break;
				case IDC_VOICE14_ON_CHK:
					pTTT->lpszText = new char[msg[13].GetLength() + 1];
					strcpy(pTTT->lpszText, msg[13].GetBuffer(1));
					break;
				case IDC_VOICE15_ON_CHK:
					pTTT->lpszText = new char[msg[14].GetLength() + 1];
					strcpy(pTTT->lpszText, msg[14].GetBuffer(1));
					break;
				case IDC_VOICE16_ON_CHK:
					pTTT->lpszText = new char[msg[15].GetLength() + 1];
					strcpy(pTTT->lpszText, msg[15].GetBuffer(1));
					break;
				case IDC_VOICE1_MODULATE_CHK:
					pTTT->lpszText = new char[msg2[0].GetLength() + 1];
					strcpy(pTTT->lpszText, msg2[0].GetBuffer(1));
					break;
				case IDC_VOICE2_MODULATE_CHK:
					pTTT->lpszText = new char[msg2[1].GetLength() + 1];
					strcpy(pTTT->lpszText, msg2[1].GetBuffer(1));
					break;
				case IDC_VOICE3_MODULATE_CHK:
					pTTT->lpszText = new char[msg2[2].GetLength() + 1];
					strcpy(pTTT->lpszText, msg2[2].GetBuffer(1));
					break;
				case IDC_VOICE4_MODULATE_CHK:
					pTTT->lpszText = new char[msg2[3].GetLength() + 1];
					strcpy(pTTT->lpszText, msg2[3].GetBuffer(1));
					break;
				case IDC_VOICE5_MODULATE_CHK:
					pTTT->lpszText = new char[msg2[4].GetLength() + 1];
					strcpy(pTTT->lpszText, msg2[4].GetBuffer(1));
					break;
				case IDC_VOICE6_MODULATE_CHK:
					pTTT->lpszText = new char[msg2[5].GetLength() + 1];
					strcpy(pTTT->lpszText, msg2[5].GetBuffer(1));
					break;
				case IDC_VOICE7_MODULATE_CHK:
					pTTT->lpszText = new char[msg2[6].GetLength() + 1];
					strcpy(pTTT->lpszText, msg2[6].GetBuffer(1));
					break;
				case IDC_VOICE8_MODULATE_CHK:
					pTTT->lpszText = new char[msg2[7].GetLength() + 1];
					strcpy(pTTT->lpszText, msg2[7].GetBuffer(1));
					break;
				case IDC_VOICE9_MODULATE_CHK:
					pTTT->lpszText = new char[msg2[8].GetLength() + 1];
					strcpy(pTTT->lpszText, msg2[8].GetBuffer(1));
					break;
				case IDC_VOICE10_MODULATE_CHK:
					pTTT->lpszText = new char[msg2[9].GetLength() + 1];
					strcpy(pTTT->lpszText, msg2[9].GetBuffer(1));
					break;
				case IDC_VOICE11_MODULATE_CHK:
					pTTT->lpszText = new char[msg2[10].GetLength() + 1];
					strcpy(pTTT->lpszText, msg2[10].GetBuffer(1));
					break;
				case IDC_VOICE12_MODULATE_CHK:
					pTTT->lpszText = new char[msg2[11].GetLength() + 1];
					strcpy(pTTT->lpszText, msg2[11].GetBuffer(1));
					break;
				case IDC_VOICE13_MODULATE_CHK:
					pTTT->lpszText = new char[msg2[12].GetLength() + 1];
					strcpy(pTTT->lpszText, msg2[12].GetBuffer(1));
					break;
				case IDC_VOICE14_MODULATE_CHK:
					pTTT->lpszText = new char[msg2[13].GetLength() + 1];
					strcpy(pTTT->lpszText, msg2[13].GetBuffer(1));
					break;
				case IDC_VOICE15_MODULATE_CHK:
					pTTT->lpszText = new char[msg2[14].GetLength() + 1];
					strcpy(pTTT->lpszText, msg2[14].GetBuffer(1));
					break;
				case IDC_VOICE16_MODULATE_CHK:
					pTTT->lpszText = new char[msg2[15].GetLength() + 1];
					strcpy(pTTT->lpszText, msg2[15].GetBuffer(1));
					break;
				case IDC_VOICE1_MIDI_BUTTON:
					pTTT->lpszText = new char[msg3[0].GetLength() + 1];
					strcpy(pTTT->lpszText, msg3[0].GetBuffer(1));
					break;
				case IDC_VOICE2_MIDI_BUTTON:
					pTTT->lpszText = new char[msg3[1].GetLength() + 1];
					strcpy(pTTT->lpszText, msg3[1].GetBuffer(1));
					break;
				case IDC_VOICE3_MIDI_BUTTON:
					pTTT->lpszText = new char[msg3[2].GetLength() + 1];
					strcpy(pTTT->lpszText, msg3[2].GetBuffer(1));
					break;
				case IDC_VOICE4_MIDI_BUTTON:
					pTTT->lpszText = new char[msg3[3].GetLength() + 1];
					strcpy(pTTT->lpszText, msg3[3].GetBuffer(1));
					break;
				case IDC_VOICE5_MIDI_BUTTON:
					pTTT->lpszText = new char[msg3[4].GetLength() + 1];
					strcpy(pTTT->lpszText, msg3[4].GetBuffer(1));
					break;
				case IDC_VOICE6_MIDI_BUTTON:
					pTTT->lpszText = new char[msg3[5].GetLength() + 1];
					strcpy(pTTT->lpszText, msg3[5].GetBuffer(1));
					break;
				case IDC_VOICE7_MIDI_BUTTON:
					pTTT->lpszText = new char[msg3[6].GetLength() + 1];
					strcpy(pTTT->lpszText, msg3[6].GetBuffer(1));
					break;
				case IDC_VOICE8_MIDI_BUTTON:
					pTTT->lpszText = new char[msg3[7].GetLength() + 1];
					strcpy(pTTT->lpszText, msg3[7].GetBuffer(1));
					break;
				case IDC_VOICE9_MIDI_BUTTON:
					pTTT->lpszText = new char[msg3[8].GetLength() + 1];
					strcpy(pTTT->lpszText, msg3[8].GetBuffer(1));
					break;
				case IDC_VOICE10_MIDI_BUTTON:
					pTTT->lpszText = new char[msg3[9].GetLength() + 1];
					strcpy(pTTT->lpszText, msg3[9].GetBuffer(1));
					break;
				case IDC_VOICE11_MIDI_BUTTON:
					pTTT->lpszText = new char[msg3[10].GetLength() + 1];
					strcpy(pTTT->lpszText, msg3[10].GetBuffer(1));
					break;
				case IDC_VOICE12_MIDI_BUTTON:
					pTTT->lpszText = new char[msg3[11].GetLength() + 1];
					strcpy(pTTT->lpszText, msg3[11].GetBuffer(1));
					break;
				case IDC_VOICE13_MIDI_BUTTON:
					pTTT->lpszText = new char[msg3[12].GetLength() + 1];
					strcpy(pTTT->lpszText, msg3[12].GetBuffer(1));
					break;
				case IDC_VOICE14_MIDI_BUTTON:
					pTTT->lpszText = new char[msg3[13].GetLength() + 1];
					strcpy(pTTT->lpszText, msg3[13].GetBuffer(1));
					break;
				case IDC_VOICE15_MIDI_BUTTON:
					pTTT->lpszText = new char[msg3[14].GetLength() + 1];
					strcpy(pTTT->lpszText, msg3[14].GetBuffer(1));
					break;
				case IDC_VOICE16_MIDI_BUTTON:
					pTTT->lpszText = new char[msg3[15].GetLength() + 1];
					strcpy(pTTT->lpszText, msg3[15].GetBuffer(1));
					break;
				case IDC_VOICE1_SCALE_COMBO:
					pTTT->lpszText = new char[msg4[0].GetLength() + 1];
					strcpy(pTTT->lpszText, msg4[0].GetBuffer(1));
					break;
				case IDC_VOICE2_SCALE_COMBO:
					pTTT->lpszText = new char[msg4[1].GetLength() + 1];
					strcpy(pTTT->lpszText, msg4[1].GetBuffer(1));
					break;
				case IDC_VOICE3_SCALE_COMBO:
					pTTT->lpszText = new char[msg4[2].GetLength() + 1];
					strcpy(pTTT->lpszText, msg4[2].GetBuffer(1));
					break;
				case IDC_VOICE4_SCALE_COMBO:
					pTTT->lpszText = new char[msg4[3].GetLength() + 1];
					strcpy(pTTT->lpszText, msg4[3].GetBuffer(1));
					break;
				case IDC_VOICE5_SCALE_COMBO:
					pTTT->lpszText = new char[msg4[4].GetLength() + 1];
					strcpy(pTTT->lpszText, msg4[4].GetBuffer(1));
					break;
				case IDC_VOICE6_SCALE_COMBO:
					pTTT->lpszText = new char[msg4[5].GetLength() + 1];
					strcpy(pTTT->lpszText, msg4[5].GetBuffer(1));
					break;
				case IDC_VOICE7_SCALE_COMBO:
					pTTT->lpszText = new char[msg4[6].GetLength() + 1];
					strcpy(pTTT->lpszText, msg4[6].GetBuffer(1));
					break;
				case IDC_VOICE8_SCALE_COMBO:
					pTTT->lpszText = new char[msg4[7].GetLength() + 1];
					strcpy(pTTT->lpszText, msg4[7].GetBuffer(1));
					break;
				case IDC_VOICE9_SCALE_COMBO:
					pTTT->lpszText = new char[msg4[8].GetLength() + 1];
					strcpy(pTTT->lpszText, msg4[8].GetBuffer(1));
					break;
				case IDC_VOICE10_SCALE_COMBO:
					pTTT->lpszText = new char[msg4[9].GetLength() + 1];
					strcpy(pTTT->lpszText, msg4[9].GetBuffer(1));
					break;
				case IDC_VOICE11_SCALE_COMBO:
					pTTT->lpszText = new char[msg4[10].GetLength() + 1];
					strcpy(pTTT->lpszText, msg4[10].GetBuffer(1));
					break;
				case IDC_VOICE12_SCALE_COMBO:
					pTTT->lpszText = new char[msg4[11].GetLength() + 1];
					strcpy(pTTT->lpszText, msg4[11].GetBuffer(1));
					break;
				case IDC_VOICE13_SCALE_COMBO:
					pTTT->lpszText = new char[msg4[12].GetLength() + 1];
					strcpy(pTTT->lpszText, msg4[12].GetBuffer(1));
					break;
				case IDC_VOICE14_SCALE_COMBO:
					pTTT->lpszText = new char[msg4[13].GetLength() + 1];
					strcpy(pTTT->lpszText, msg4[13].GetBuffer(1));
					break;
				case IDC_VOICE15_SCALE_COMBO:
					pTTT->lpszText = new char[msg4[14].GetLength() + 1];
					strcpy(pTTT->lpszText, msg4[14].GetBuffer(1));
					break;
				case IDC_VOICE16_SCALE_COMBO:
					pTTT->lpszText = new char[msg4[15].GetLength() + 1];
					strcpy(pTTT->lpszText, msg4[15].GetBuffer(1));
					break;
				default:
					pTTT->lpszText = NULL;
					break;

			}
			pTTT->hinst = NULL;
			return(TRUE);
        }    
	}
	return(FALSE);
}

void CFractMusView::OnComments() 
{
	CCommentsDlg dlg;

	dlg.pView = this;

	if (IsDocumentLocked)
		dlg.m_canedit = FALSE;
	else
		dlg.m_canedit = TRUE;

	dlg.DoModal();
	

	
}

void CFractMusView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	CFractMusDoc* pDoc = GetDocument();
	CWnd* pW = AfxGetMainWnd();

	CString tit;

	tit = pDoc->GetTitle();

	tit+=" - FractMus 2000   "+pDoc->m_Copyright;

	pW->SetWindowText(tit);	
	
	
}

void CFractMusView::OnVoice1Earthworm() 
{
	UpdateAlgorithm(VOICE1, EARTHWORM);
	
}

void CFractMusView::OnVoice2Earthworm() 
{
	UpdateAlgorithm(VOICE2, EARTHWORM);
	
}

void CFractMusView::OnVoice3Earthworm() 
{
	UpdateAlgorithm(VOICE3, EARTHWORM);
	
}

void CFractMusView::OnVoice4Earthworm() 
{
	UpdateAlgorithm(VOICE4, EARTHWORM);
	
}

void CFractMusView::OnVoice5Earthworm() 
{
	UpdateAlgorithm(VOICE5, EARTHWORM);
	
}

void CFractMusView::OnVoice6Earthworm() 
{
	UpdateAlgorithm(VOICE6, EARTHWORM);
	
}

void CFractMusView::OnVoice7Earthworm() 
{
	UpdateAlgorithm(VOICE7, EARTHWORM);
	
}

void CFractMusView::OnVoice8Earthworm() 
{
	UpdateAlgorithm(VOICE8, EARTHWORM);
	
}

void CFractMusView::OnVoice9Earthworm() 
{
	UpdateAlgorithm(VOICE9, EARTHWORM);
	
}

void CFractMusView::OnVoice10Earthworm() 
{
	UpdateAlgorithm(VOICE10, EARTHWORM);
	
}

void CFractMusView::OnVoice11Earthworm() 
{
	UpdateAlgorithm(VOICE11, EARTHWORM);
	
}

void CFractMusView::OnVoice12Earthworm() 
{
	UpdateAlgorithm(VOICE12, EARTHWORM);
	
}

void CFractMusView::OnVoice13Earthworm() 
{
	UpdateAlgorithm(VOICE13, EARTHWORM);
	
}

void CFractMusView::OnVoice14Earthworm() 
{
	UpdateAlgorithm(VOICE14, EARTHWORM);
	
}

void CFractMusView::OnVoice15Earthworm() 
{
	UpdateAlgorithm(VOICE15, EARTHWORM);
	
}

void CFractMusView::OnVoice16Earthworm() 
{
	UpdateAlgorithm(VOICE16, EARTHWORM);
	
}

void CFractMusView::OnUpdateVoice1Earthworm(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE1] == EARTHWORM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateVoice2Earthworm(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE2] == EARTHWORM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateVoice3Earthworm(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE3] == EARTHWORM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateVoice4Earthworm(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE4] == EARTHWORM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateVoice5Earthworm(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE5] == EARTHWORM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateVoice6Earthworm(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE6] == EARTHWORM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateVoice7Earthworm(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE7] == EARTHWORM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateVoice8Earthworm(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE8] == EARTHWORM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateVoice9Earthworm(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE9] == EARTHWORM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateVoice10Earthworm(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE10] == EARTHWORM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateVoice11Earthworm(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE11] == EARTHWORM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateVoice12Earthworm(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE12] == EARTHWORM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateVoice13Earthworm(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE13] == EARTHWORM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateVoice14Earthworm(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE14] == EARTHWORM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateVoice15Earthworm(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE15] == EARTHWORM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateVoice16Earthworm(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE16] == EARTHWORM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnAlgorithmsVoice1Random() 
{
	UpdateAlgorithm(VOICE1, RANDOM);
	
}

void CFractMusView::OnAlgorithmsVoice2Random() 
{
	UpdateAlgorithm(VOICE2, RANDOM);
	
}

void CFractMusView::OnAlgorithmsVoice3Random() 
{
	UpdateAlgorithm(VOICE3, RANDOM);
}

void CFractMusView::OnAlgorithmsVoice4Random() 
{
	UpdateAlgorithm(VOICE4, RANDOM);
	
}

void CFractMusView::OnAlgorithmsVoice5Random() 
{
	UpdateAlgorithm(VOICE5, RANDOM);
	
}

void CFractMusView::OnAlgorithmsVoice6Random() 
{
	UpdateAlgorithm(VOICE6, RANDOM);
	
}

void CFractMusView::OnAlgorithmsVoice7Random() 
{
	UpdateAlgorithm(VOICE7, RANDOM);
	
}

void CFractMusView::OnAlgorithmsVoice8Random() 
{
	UpdateAlgorithm(VOICE8, RANDOM);
	
}

void CFractMusView::OnAlgorithmsVoice9Random() 
{
	UpdateAlgorithm(VOICE9, RANDOM);
	
}

void CFractMusView::OnAlgorithmsVoice10Random() 
{
	UpdateAlgorithm(VOICE10, RANDOM);
	
}

void CFractMusView::OnAlgorithmsVoice11Random() 
{
	UpdateAlgorithm(VOICE11, RANDOM);
	
}

void CFractMusView::OnAlgorithmsVoice12Random() 
{
	UpdateAlgorithm(VOICE12, RANDOM);
	
}

void CFractMusView::OnAlgorithmsVoice13Random() 
{
	UpdateAlgorithm(VOICE13, RANDOM);
	
}

void CFractMusView::OnAlgorithmsVoice14Random() 
{
	UpdateAlgorithm(VOICE14, RANDOM);
	
}

void CFractMusView::OnAlgorithmsVoice15Random() 
{
	UpdateAlgorithm(VOICE15, RANDOM);
	
}

void CFractMusView::OnAlgorithmsVoice16Random() 
{
	UpdateAlgorithm(VOICE16, RANDOM);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice1Random(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE1] == RANDOM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice2Random(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE2] == RANDOM);
	pCmdUI->Enable(!m_Playing);
	
	
}

void CFractMusView::OnUpdateAlgorithmsVoice3Random(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE3] == RANDOM);
	pCmdUI->Enable(!m_Playing);
	
	
}

void CFractMusView::OnUpdateAlgorithmsVoice4Random(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE4] == RANDOM);
	pCmdUI->Enable(!m_Playing);
	
	
}

void CFractMusView::OnUpdateAlgorithmsVoice5Random(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE5] == RANDOM);
	pCmdUI->Enable(!m_Playing);
	
	
}

void CFractMusView::OnUpdateAlgorithmsVoice6Random(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE6] == RANDOM);
	pCmdUI->Enable(!m_Playing);
	
	
}

void CFractMusView::OnUpdateAlgorithmsVoice7Random(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE7] == RANDOM);
	pCmdUI->Enable(!m_Playing);
	
	
}

void CFractMusView::OnUpdateAlgorithmsVoice8Random(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE8] == RANDOM);
	pCmdUI->Enable(!m_Playing);
	
	
}

void CFractMusView::OnUpdateAlgorithmsVoice9Random(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE9] == RANDOM);
	pCmdUI->Enable(!m_Playing);
	
	
}

void CFractMusView::OnUpdateAlgorithmsVoice10Random(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE10] == RANDOM);
	pCmdUI->Enable(!m_Playing);
	
	
}

void CFractMusView::OnUpdateAlgorithmsVoice11Random(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE11] == RANDOM);
	pCmdUI->Enable(!m_Playing);
	
	
}

void CFractMusView::OnUpdateAlgorithmsVoice12Random(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE12] == RANDOM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice13Random(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE13] == RANDOM);
	pCmdUI->Enable(!m_Playing);
	
	
}

void CFractMusView::OnUpdateAlgorithmsVoice14Random(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE14] == RANDOM);
	pCmdUI->Enable(!m_Playing);
	
	
}

void CFractMusView::OnUpdateAlgorithmsVoice15Random(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE15] == RANDOM);
	pCmdUI->Enable(!m_Playing);
	
	
}

void CFractMusView::OnUpdateAlgorithmsVoice16Random(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE16] == RANDOM);
	pCmdUI->Enable(!m_Playing);
	
	
}


void CFractMusView::OnAlgorithmsVoice1Hailstone() 
{
	UpdateAlgorithm(VOICE1, HAILSTONE);
	
}

void CFractMusView::OnAlgorithmsVoice2Hailstone() 
{
	UpdateAlgorithm(VOICE2, HAILSTONE);
	
}

void CFractMusView::OnAlgorithmsVoice3Hailstone() 
{
	UpdateAlgorithm(VOICE3, HAILSTONE);
	
}

void CFractMusView::OnAlgorithmsVoice4Hailstone() 
{
	UpdateAlgorithm(VOICE4, HAILSTONE);
	
}

void CFractMusView::OnAlgorithmsVoice5Hailstone() 
{
	UpdateAlgorithm(VOICE5, HAILSTONE);
	
}

void CFractMusView::OnAlgorithmsVoice6Hailstone() 
{
	UpdateAlgorithm(VOICE6, HAILSTONE);
	
}

void CFractMusView::OnAlgorithmsVoice7Hailstone() 
{
	UpdateAlgorithm(VOICE7, HAILSTONE);
	
}

void CFractMusView::OnAlgorithmsVoice8Hailstone() 
{
	UpdateAlgorithm(VOICE8, HAILSTONE);
	
}

void CFractMusView::OnAlgorithmsVoice9Hailstone() 
{
	UpdateAlgorithm(VOICE9, HAILSTONE);
	
}

void CFractMusView::OnAlgorithmsVoice10Hailstone() 
{
	UpdateAlgorithm(VOICE10, HAILSTONE);
	
}

void CFractMusView::OnAlgorithmsVoice11Hailstone() 
{
	UpdateAlgorithm(VOICE11, HAILSTONE);
	
}

void CFractMusView::OnAlgorithmsVoice12Hailstone() 
{
	UpdateAlgorithm(VOICE12, HAILSTONE);
	
}

void CFractMusView::OnAlgorithmsVoice13Hailstone() 
{
	UpdateAlgorithm(VOICE13, HAILSTONE);
	
}

void CFractMusView::OnAlgorithmsVoice14Hailstone() 
{
	UpdateAlgorithm(VOICE14, HAILSTONE);
	
}

void CFractMusView::OnAlgorithmsVoice15Hailstone() 
{
	UpdateAlgorithm(VOICE15, HAILSTONE);
	
}

void CFractMusView::OnAlgorithmsVoice16Hailstone() 
{
	UpdateAlgorithm(VOICE16, HAILSTONE);
	
}

void CFractMusView::OnAlgorithmsVoice1Wolfram() 
{
	UpdateAlgorithm(VOICE1, WOLFRAM);
	
}

void CFractMusView::OnAlgorithmsVoice2Wolfram() 
{
	UpdateAlgorithm(VOICE2, WOLFRAM);
	
}

void CFractMusView::OnAlgorithmsVoice3Wolfram() 
{
	UpdateAlgorithm(VOICE3, WOLFRAM);
	
}

void CFractMusView::OnAlgorithmsVoice4Wolfram() 
{
	UpdateAlgorithm(VOICE4, WOLFRAM);
	
}

void CFractMusView::OnAlgorithmsVoice5Wolfram() 
{
	UpdateAlgorithm(VOICE5, WOLFRAM);
	
}

void CFractMusView::OnAlgorithmsVoice6Wolfram() 
{
	UpdateAlgorithm(VOICE6, WOLFRAM);
	
}

void CFractMusView::OnAlgorithmsVoice7Wolfram() 
{
	UpdateAlgorithm(VOICE7, WOLFRAM);
	
}

void CFractMusView::OnAlgorithmsVoice8Wolfram() 
{
	UpdateAlgorithm(VOICE8, WOLFRAM);
	
}

void CFractMusView::OnAlgorithmsVoice9Wolfram() 
{
	UpdateAlgorithm(VOICE9, WOLFRAM);
	
}

void CFractMusView::OnAlgorithmsVoice10Wolfram() 
{
	UpdateAlgorithm(VOICE10, WOLFRAM);
	
}

void CFractMusView::OnAlgorithmsVoice11Wolfram() 
{
	UpdateAlgorithm(VOICE11, WOLFRAM);
	
}

void CFractMusView::OnAlgorithmsVoice12Wolfram() 
{
	UpdateAlgorithm(VOICE12, WOLFRAM);
	
}

void CFractMusView::OnAlgorithmsVoice13Wolfram() 
{
	UpdateAlgorithm(VOICE13, WOLFRAM);
	
}

void CFractMusView::OnAlgorithmsVoice14Wolfram() 
{
	UpdateAlgorithm(VOICE14, WOLFRAM);
	
}

void CFractMusView::OnAlgorithmsVoice15Wolfram() 
{
	UpdateAlgorithm(VOICE15, WOLFRAM);
	
}

void CFractMusView::OnAlgorithmsVoice16Wolfram() 
{
	UpdateAlgorithm(VOICE16, WOLFRAM);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice1Hailstone(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE1] == HAILSTONE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice2Hailstone(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE2] == HAILSTONE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice3Hailstone(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE3] == HAILSTONE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice4Hailstone(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE4] == HAILSTONE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice5Hailstone(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE5] == HAILSTONE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice6Hailstone(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE6] == HAILSTONE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice7Hailstone(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE7] == HAILSTONE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice8Hailstone(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE8] == HAILSTONE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice9Hailstone(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE9] == HAILSTONE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice10Hailstone(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE10] == HAILSTONE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice11Hailstone(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE11] == HAILSTONE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice12Hailstone(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE12] == HAILSTONE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice13Hailstone(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE13] == HAILSTONE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice14Hailstone(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE14] == HAILSTONE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice15Hailstone(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE15] == HAILSTONE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice16Hailstone(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE16] == HAILSTONE);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice1Wolfram(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE1] == WOLFRAM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice2Wolfram(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE2] == WOLFRAM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice3Wolfram(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE3] == WOLFRAM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice4Wolfram(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE4] == WOLFRAM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice5Wolfram(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE5] == WOLFRAM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice6Wolfram(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE6] == WOLFRAM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice7Wolfram(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE7] == WOLFRAM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice8Wolfram(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE8] == WOLFRAM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice9Wolfram(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE9] == WOLFRAM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice10Wolfram(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE10] == WOLFRAM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice11Wolfram(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE11] == WOLFRAM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice12Wolfram(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE12] == WOLFRAM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice13Wolfram(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE13] == WOLFRAM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice14Wolfram(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE14] == WOLFRAM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice15Wolfram(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE15] == WOLFRAM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnUpdateAlgorithmsVoice16Wolfram(CCmdUI* pCmdUI) 
{
	CFractMusDoc* pDoc = GetDocument(); 
	pCmdUI->SetCheck(pDoc->AlgorithmNum[VOICE16] == WOLFRAM);
	pCmdUI->Enable(!m_Playing);
	
}

void CFractMusView::OnViewTips() 
{
	ShowTips = !ShowTips;
	
}

void CFractMusView::OnUpdateViewTips(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(ShowTips);
}
